part of firebase_dart.api;


class DataSnapshot {

  final Node _node;
  final Query ref;
  final Index _index;

  DataSnapshot(this._node, this.ref, this._index);

  get val => _node.val();

  get exportVal => _node.val(true);

  bool get exists => _node.isEmpty;

  DataSnapshot child(String childPathString) {
    validation.validatePathString("Firebase.DataSnapshot.child", 1, childPathString, false);
    var childPath = new util.Path.fromString(childPathString);
    var childRef = this.ref.child(childPath);
    return new DataSnapshot(this._node.getChild(childPath), childRef, priorityIndex);
  }

  bool hasChild(String childPathString) {
    validation.validatePathString("Firebase.DataSnapshot.hasChild", 1, childPathString, false);
    var childPath = new util.Path.fromString(childPathString);
    return!this._node.getChild(childPath).isEmpty;
  }

  get priority => _node.priority.val();

  void forEach(void f(DataSnapshot v)) {
    if (this._node.isLeafNode) {
      return;
    }
    ChildrenNode childrenNode = (this._node);
    childrenNode.forEachChild(this._index, (String key, Node node) {
      return f(new DataSnapshot._(node, ref.child(key), priorityIndex));
    });
  }

  bool get hasChildren => !_node.isLeafNode&&!_node.isEmpty;


  String get key => ref.key;

  int get numChildren => _node.numChildren;

}

