
library firebase_dart.core.rest;

import 'view.dart';
import 'repo.dart';
import 'package:firebase_dart/src/dart/api.dart';

class ReadonlyRestClient {
  final RepoInfo _repoInfo;
  final _onDataUpdate;
  var _credential;
  var _listens = {};
  final _log = util.logWrapper("p:rest");

  ReadonlyRestClient(this.repoInfo, this.onDataUpdate);

  listen(Query query, currentHashFn, tag, onComplete) {
    var pathString = query.path.toString();
    this._log("Listen called for " + pathString + " " + query.queryIdentifier());
    var listenId = _getListenId(query, tag);
    var thisListen = {};
    this._listens[listenId] = thisListen;
    var queryStringParamaters = query.queryParams.toRestQueryStringParameters();
    var self = this;
    this._restRequest(pathString + ".json", queryStringParamaters, (error, result) {
      var data = result;
      if (error == 404) {
        data = null;
        error = null;
      }
      if (error == null) {
        _onDataUpdate(pathString, data, false, tag);
      }
      if (fb.util.obj.get(self._listens, listenId) == thisListen) {
        var status;
        if (!error) {
          status = "ok";
        } else {
          if (error == 401) {
            status = "permission_denied";
          } else {
            status = "rest_error:" + error;
          }
        }
        onComplete(status, null);
      }
    });
  }

  unlisten(query, tag) {
    var listenId = fb.core.ReadonlyRestClient.getListenId_(query, tag);
    delete this._listens[listenId];
  }

  auth(cred, opt_callback, opt_cancelCallback) {
    this._credential = cred;
    var res = fb.util.jwt.decode(cred);
    var auth = res.data;
    var expires = res.claims && res.claims["exp"];
    if (opt_callback) {
      opt_callback("ok", {"auth":auth, "expires":expires});
    }
  }

  unauth(onComplete) {
    this._credential = null;
    onComplete("ok", null);
  }

  onDisconnectPut(pathString, data, opt_onComplete) {
  }

  onDisconnectMerge(pathString, data, opt_onComplete) {
  }

  onDisconnectCancel(pathString, opt_onComplete) {
  }

  put(pathString, data, opt_onComplete, opt_hash) {
  }

  merge(pathString, data, onComplete, opt_hash) {
  }

  reportStats(stats) {
  }

  _restRequest(String pathString, Map<String,String> queryStringParameters, callback) {
    queryStringParameters = queryStringParameters ?? {};
    queryStringParameters["format"] = "export";
    if (this._credential) {
      queryStringParameters["auth"] = this._credential;
    }
    var url = new Uri(
        scheme: this._repoInfo.secure ? "https" : "http",
        host: _repoInfo.host,
        path: pathString,
        queryParameters: queryStringParameters);
    this._log("Sending REST request for " + url);
    var xhr = new XMLHttpRequest();
    var self = this;
    xhr.onreadystatechange = () {
      if (callback && xhr.readyState == 4) {
        self._log("REST Response for " + url + " received. status:", xhr.status, "response:", xhr.responseText);
        var res = null;
        if (xhr.status >= 200 && xhr.status < 300) {
          try {
            res = fb.util.json.eval(xhr.responseText);
          } catch (e) {
            fb.core.util.warn("Failed to parse JSON response for " + url + ": " + xhr.responseText);
          }
          callback(null, res);
        } else {
          if (xhr.status != 401 && xhr.status != 404) {
            fb.core.util.warn("Got unsuccessful REST response for " + url + " Status: " + xhr.status);
          }
          callback(xhr.status);
        }
        callback = null;
      }
    };
    xhr.open("GET", url, true);
    xhr.send();
  }

  static _getListenId(query, [String tag]) {
    if (tag!=null) return r"tag$"+tag;

    assert(query.getQueryParams().isDefault());//, "should have a tag if it's not a default query.");
    return query.path.toString();

  }
}

