
part of firebase_dart.core.snap;


/*
fb.core.snap.Index.FallbackType;
fb.core.snap.Index.Fallback = {};
*/

abstract class Index {

  static const fallback = const {};

  const Index();

  int compare(NamedNode a, NamedNode b);

  bool isDefinedOn(Node node);

  bool indexedValueChanged(Node oldNode, Node newNode) {
    var oldWrapped = new NamedNode(util.MIN_NAME, oldNode);
    var newWrapped = new NamedNode(util.MIN_NAME, newNode);
    return this.compare(oldWrapped, newWrapped) != 0;
  }

  NamedNode get minPost => NamedNode.MIN;

  NamedNode get maxPost;

  NamedNode makePost(String indexValue, String name);

  String toString();


}

class PathIndex extends Index {

  util.Path _indexPath;

  PathIndex(this._indexPath) {
    assert(!_indexPath.isEmpty && _indexPath.front != ".priority");
    //"Can't create PathIndex with empty path or .priority key");
  }

  Node extractChild(Node snap) => snap.getChild(_indexPath);

  bool isDefinedOn(Node node) => !node.getChild(this._indexPath).isEmpty;

  int compare(NamedNode a, NamedNode b) {
    var aChild = this.extractChild(a.node);
    var bChild = this.extractChild(b.node);
    var indexCmp = aChild.compareTo(bChild);
    if (indexCmp == 0) {
      return util.nameCompare(a.name, b.name);
    } else {
      return indexCmp;
    }
  }

  NamedNode makePost(String indexValue, String name) {
    var valueNode = new Node.fromJson(indexValue);
    var node = EMPTY_NODE.updateChild(this._indexPath, valueNode);
    return new NamedNode(name, node);
  }


  NamedNode get maxPost {
    var node = EMPTY_NODE.updateChild(this._indexPath, MAX_NODE);
    return new NamedNode(util.MAX_NAME, node);
  }

  String toString() => _indexPath.join("/");

}

class _PriorityIndex extends Index {

  const _PriorityIndex();

  int compare(NamedNode a, NamedNode b) {
    var aPriority = a.node.priority;
    var bPriority = b.node.priority;
    var indexCmp = aPriority.compareTo(bPriority);
    if (indexCmp == 0) {
      return util.nameCompare(a.name, b.name);
    } else {
      return indexCmp;
    }
  }


  bool isDefinedOn(Node node) => !node.priority.isEmpty;


  bool indexedValueChanged(Node oldNode, Node newNode) =>
      oldNode.priority!=newNode.priority;

  get minPost => NamedNode.MIN;
  get maxPost => const NamedNode(util.MAX_NAME, const LeafNode._("[PRIORITY-POST]", MAX_NODE));

  NamedNode makePost(String indexValue, String name) {
    var priorityNode = new Node.fromJson(indexValue);
    return new NamedNode(name, new LeafNode("[PRIORITY-POST]", priorityNode));
  }

  toString() => ".priority";


}

const priorityIndex = const _PriorityIndex();

class _KeyIndex extends Index {

  const _KeyIndex();

  int compare(NamedNode a, NamedNode b) => util.nameCompare(a.name, b.name);

  bool isDefinedOn(Node node) =>
    throw new Exception("KeyIndex.isDefinedOn not expected to be called.");

  bool indexedValueChanged(Node oldNode, Node newNode) => false;

  get minPost => NamedNode.MIN;

  get maxPost => const NamedNode(util.MAX_NAME, EMPTY_NODE);

  NamedNode makePost(String indexValue, String name) {
    return new NamedNode((indexValue), EMPTY_NODE);
  }

  toString() => ".key";

}

const keyIndex = const _KeyIndex();

class _ValueIndex extends Index {

  const _ValueIndex();

  int compare(NamedNode a, NamedNode b) {
    var indexCmp = a.node.compareTo(b.node);
    if (indexCmp == 0) {
      return util.nameCompare(a.name, b.name);
    } else {
      return indexCmp;
    }
  }

  bool isDefinedOn(Node node) => true;

  bool indexedValueChanged(Node oldNode, Node newNode) => oldNode!=newNode;

  get minPost => NamedNode.MIN;

  get maxPost => NamedNode.MAX;


  NamedNode makePost(indexValue, String name) {
    var valueNode = new Node.fromJson(indexValue);
    return new NamedNode(name, valueNode);
  }

  toString() => ".value";


}

const valueIndex = const _ValueIndex();
