part of firebase_dart.core.snap;


class IndexMap {

  static const Default = const IndexMap(
      const {".priority":Index.fallback}, const {".priority":priorityIndex});

  final Map<Index,SortedMap> _indexes;
  final Set<String> _indexSet;

  const IndexMap(this._indexes, this._indexSet);


  operator[](indexKey) {
    var sortedMap = this._indexes[indexKey];
    if (sortedMap==null) {
      throw new Exception("No index defined for $indexKey");
    }
    if (sortedMap == Index.fallback) {
      return null;
    } else {
      return sortedMap;
    }
  }

  bool hasIndex(Index indexDefinition) {
    return this._indexSet.contains(indexDefinition.toString());
  }

  addIndex(Index indexDefinition, existingChildren) {

    assert(indexDefinition != keyIndex);//, "KeyIndex always exists and isn't meant to be added to the IndexMap.");
    var childList = [];
    var sawIndexedValue = false;
    var iter = existingChildren.getIterator(NamedNode.wrap);
    var next = iter.getNext();
    while (next) {
      sawIndexedValue = sawIndexedValue || indexDefinition.isDefinedOn(next.node);
      childList.add(next);
      next = iter.getNext();
    }
    var newIndex;
    if (sawIndexedValue) {
      newIndex = buildChildSet(childList, indexDefinition.getCompare());
    } else {
      newIndex = Index.fallback;
    }
    var indexName = indexDefinition.toString();
    var newIndexSet = new Set.from(_indexSet);

    newIndexSet[indexName] = indexDefinition;
    var newIndexes = goog.object.clone(this._indexes);
    newIndexes[indexName] = newIndex;
    return new IndexMap(newIndexes, newIndexSet);
  }

  addToIndexes(NamedNode namedNode, Map<String,Node> existingChildren) {
    var self = this;
    var newIndexes = this._indexes.map( (index, indexedChildren) {
      if (indexedChildren == Index.fallback) {
        if (index.isDefinedOn(namedNode.node)) {
          var childList = <Node>[];
          var iter = existingChildren.getIterator(fb.core.snap.NamedNode.Wrap);
          var next = iter.getNext();
          while (next) {
            if (next.name != namedNode.name) {
              childList.add(next);
            }
            next = iter.getNext();
          }
          childList.add(namedNode);
          return buildChildSet(childList, index.getCompare());
        } else {
          return fb.core.snap.Index.Fallback;
        }
      } else {
        var existingSnap = existingChildren[namedNode.name];
        var newChildren = indexedChildren;
        if (existingSnap!=null) {
          newChildren = newChildren.remove(new fb.core.snap.NamedNode(namedNode.name, existingSnap));
        }
        return newChildren.insert(namedNode, namedNode.node);
      }
    });
    return new fb.core.snap.IndexMap(newIndexes, this._indexSet);
  }

  removeFromIndexes(NamedNode namedNode, SortedMap<String,Node> existingChildren) {
    var newIndexes = new Map.fromIterables(_indexes.keys,
        _indexes.values.map((SortedMap<String,Node> indexedChildren) {

          if (indexedChildren == Index.fallback) {
            return indexedChildren;
          } else {
            var existingSnap = existingChildren[namedNode.name];
            if (existingSnap!=null) {
              return indexedChildren.withKeyRemoved(namedNode.name);
            } else {
              return indexedChildren;
            }
          }
    }));
    return new IndexMap(newIndexes, this._indexSet);
  }

}

