part of firebase_dart.core.snap;

class LeafNode extends Node {

  final value;
  final Node priority;

  const LeafNode._(this.value, this.priority);

  factory LeafNode(value, [priorityNode = Node.empty]) {
    assert(value !=
        null); //, "LeafNode shouldn't be created with null/undefined value.");
    validatePriorityNode(priorityNode);
  }

  static const VALUE_TYPE_ORDER = const ["object", "boolean", "number", "string"];

  get isLeafNode => true;

  updatePriority(newPriorityNode) {
    return new LeafNode(this.value, newPriorityNode);
  }

  getImmediateChild(childName) {
    if (childName == ".priority") {
      return this.priority;
    } else {
      return EMPTY_NODE;
    }
  }

  getChild(path) {
    if (path.isEmpty()) {
      return this;
    } else {
      if (path.getFront() == ".priority") {
        return this.priority;
      } else {
        return EMPTY_NODE;
      }
    }
  }

  bool hasChild(String childName) => false;

  String getPredecessorChildName(String childName, Node childNode, Index index) => null;

  Node updateImmediateChild(String childName, Node newChildNode) {
    if (childName == ".priority") {
      return this.updatePriority(newChildNode);
    } else {
      if (newChildNode.isEmpty && childName != ".priority") {
        return this;
      } else {
        return Node.empty.updateImmediateChild(childName, newChildNode)
            .updatePriority(this.priority);
      }
    }
  }

  Node updateChild(util.Path path, Node newChildNode) {
    var front = path.front;
    if (front == null) {
      return newChildNode;
    } else {
      if (newChildNode.isEmpty && front != ".priority") {
        return this;
      } else {
        assert(front != ".priority" || path.length == 1) ;//, ".priority must be the last token in a path");
        return this.updateImmediateChild(front, EMPTY_NODE.updateChild(path.popFront(), newChildNode));
      }
    }
  }

  bool get isEmpty => false;

  int get numChildren => 0;


  forEachChild(index, action) => false;


  val([bool exportFormat = false]) {
    if (exportFormat && !this.priority.isEmpty) {
      return {
        ".value":value,
        ".priority":this.priority.val()
      };
    } else {
      return this.value;
    }
  }

  String get _valueType {
    if (value is String) return "string";
    if (value is num) return "number";
    if (value is bool) return "boolean";
    return "object";
  }

  int get hashCode {
    if (this._lazyHash == null) {
      var toHash = "";
      if (!this.priority.isEmpty) {
        toHash += "priority:" + priorityHashText((this.priority.val())) + ":";
      }
      var type = _valueType;
      toHash += type + ":";
      if (value is num) {
        toHash += value.toDouble().toString();
      } else {
        toHash += this.value;
      }
      this._lazyHash = toHash.hashCode;
    }
    return(this._lazyHash);
  }


  int compareTo(Node other) {
    if (other == EMPTY_NODE) {
      return 1;
    } else {
      if (other is ChildrenNode) {
        return-1;
      } else {
        assert(other.isLeafNode);//, "Unknown node type");
        return this._compareToLeafNode((other));
      }
    }
  }

  _compareToLeafNode(LeafNode otherLeaf) {
    var otherLeafType = otherLeaf._valueType;
    var thisLeafType = _valueType;
    var otherIndex = VALUE_TYPE_ORDER.indexOf(otherLeafType);
    var thisIndex = VALUE_TYPE_ORDER.indexOf(thisLeafType);
    assert(otherIndex >= 0); //, "Unknown leaf type: " + otherLeafType);
    assert(thisIndex >= 0); //, "Unknown leaf type: " + thisLeafType);
    if (otherIndex == thisIndex) {
      if (thisLeafType == "object") {
        return 0;
      } else {
        if (this.value < otherLeaf.value) {
          return -1;
        } else {
          if (this.value == otherLeaf.value) {
            return 0;
          } else {
            return 1;
          }
        }
      }
    } else {
      return thisIndex - otherIndex;
    }
  }

  Node withIndex(Index indexDefinition) => this;

  bool isIndexed(Index index) => true;

  bool operator==(Node other) {
    if (identical(other,this)) {
      return true;
    } else {
      if (other.isLeafNode) {
        var otherLeaf = (other);
        return this.value == otherLeaf.value && this.priority==otherLeaf.priority;
      } else {
        return false;
      }
    }
  }

  toString() => JSON.encode(this.val(true));
}

