part of firebase_dart.core.snap;

abstract class Node implements Comparable<Node> {

  static const empty = EMPTY_NODE;

  const Node();

  factory Node.fromJson(json, [priority]) {
    if (json == null) {
      return empty;
    }
    if (json is Map && json.containsKey(".priority")) {
      priority = json[".priority"];
    }
    assert(priority == null || priority is String || priority is num ||
        (priority is Map && priority.containsKey(".sv"))); //"Invalid priority type found: " + typeof priority);

    if (json is Map && json.containsKey(".value") && json[".value"] != null) {
      json = json[".value"];
    }

    if (json is !Map || json.containsKey(".sv") ) {
      return new LeafNode(json, new Node.fromJson(priority));
    }


    if (!(json is List) && USE_HINZE) {
      var children = <NamedNode>[];
      var childrenHavePriority = false;
      Map<String, dynamic> hinzeJsonObj = (json);

      hinzeJsonObj.forEach((key, child) {
        if (key is !String || (key as String).substring(0,1) != ".") {
          var childNode = new Node.fromJson(hinzeJsonObj[key]);
          if (!childNode.isEmpty) {
            childrenHavePriority = childrenHavePriority || !childNode.priority.isEmpty;
            children.add(new NamedNode(key, childNode));
          }
        }
      });
      if (children.length == 0) {
        return empty;
      }
      var childSet = (buildChildSet(children, NAME_ONLY_COMPARATOR, (namedNode) {
        return namedNode.name;
      }, NAME_COMPARATOR));
      if (childrenHavePriority) {
        var sortedChildSet = buildChildSet(children, PriorityIndex.getCompare());
        return new ChildrenNode(childSet, new Node.fromJson(priority),
            new IndexMap({".priority":sortedChildSet}, {".priority":PriorityIndex}));
      } else {
        return new ChildrenNode(childSet, new Node.fromJson(priority), IndexMap.Default);
      }
    } else {
      var node = empty;
      Map<String, dynamic> jsonObj = (json);
      jsonObj.forEach((String key, childData) {
        if (key.substring(0, 1) != ".") {
          var childNode = new Node.fromJson(childData);
          if (childNode.isLeafNode || !childNode.isEmpty) {
            node = node.updateImmediateChild(key, childNode);
          }
        }
      });
      return node.updatePriority(new Node.fromJson(priority));
    }
  }

  static final Expando<int> _lazyHashes = new Expando();
  int get _lazyHash => _lazyHashes[this];
  set _lazyHash(int v) => _lazyHashes[this] = v;


  bool get isLeafNode;


  Node get priority;
  Node updatePriority(Node newPriorityNode);
  Node getImmediateChild(String childName);
  Node getChild(util.Path path);
  getPredecessorChildName(String childName, Node childNode, Index index);
  Node updateImmediateChild(String childName, Node childNode);
  Node updateChild(path, Node newChildNode);
  bool hasChild(String childName);
  bool get isEmpty;
  int get numChildren;
  void forEachChild(Index index, void action(String,Node));
  val([exportFormat]);

  int get hashCode;
  operator==(other);


  Node withIndex(Index indexDefinition);

  bool isIndexed(Index index);

}


class NamedNode {
  static const MIN = const NamedNode(util.MIN_NAME, EMPTY_NODE);
  static const MAX = const NamedNode(util.MAX_NAME, MAX_NODE);

  final String name;
  final Node node;

  const NamedNode(this.name, this.node);

  static NamedNode wrap(String name, Node node) => new NamedNode(name, node);
}

var USE_HINZE = true;


var LOG_2 = math.log(2);

class Base12Num {

  var count;
  var _current;
  var _bits;

  Base12Num(length) {
    var logBase2 = (num) {
      return int.parse("${math.log(num) / LOG_2}");
    };
    var bitMask = (bits) {
      return int.parse(new List.filled(bits + 1, "1").join(), radix: 2);
    };
    this.count = logBase2(length + 1);
    this._current = this.count - 1;
    var mask = bitMask(this.count);
    this._bits = length + 1 & mask;
  }

  nextBitIsOne() {
    var result = !(this._bits & 1 << this._current);
    this._current--;
    return result;
  }

}

buildChildSet(List<NamedNode> childList, int cmp(NamedNode a, NamedNode b), keyFn, mapSortFn) {
  childList.sort(cmp);
  var buildBalancedTree = (low, high) {
    var length = high - low;
    if (length == 0) {
      return null;
    } else {
      if (length == 1) {
        var namedNode = childList[low];
        var key = keyFn ? keyFn(namedNode) : namedNode;
        return new fb.LLRBNode(
            key, namedNode.node, fb.LLRBNode.BLACK, null, null);
      } else {
        var middle = int.parse(length / 2) + low;
        var left = buildBalancedTree(low, middle);
        var right = buildBalancedTree(middle + 1, high);
        namedNode = childList[middle];
        key = keyFn ? keyFn(namedNode) : namedNode;
        return new fb.LLRBNode(
            key, namedNode.node, fb.LLRBNode.BLACK, left, right);
      }
    }
  };
  var buildFrom12Array = (base12) {
    var node = null;
    var root = null;
    var index = childList.length;
    var buildPennant = (chunkSize, color) {
      var low = index - chunkSize;
      var high = index;
      index -= chunkSize;
      var childTree = buildBalancedTree(low + 1, high);
      var namedNode = childList[low];
      var key = keyFn ? keyFn(namedNode) : namedNode;
      attachPennant(
          new fb.LLRBNode(key, namedNode.node, color, null, childTree));
    };
    var attachPennant = (pennant) {
      if (node) {
        node.left = pennant;
        node = pennant;
      } else {
        root = pennant;
        node = pennant;
      }
    };
    for (var i = 0; i < base12.count; ++i) {
      var isOne = base12.nextBitIsOne();
      var chunkSize = math.pow(2, base12.count - (i + 1));
      if (isOne) {
        buildPennant(chunkSize, fb.LLRBNode.BLACK);
      } else {
        buildPennant(chunkSize, fb.LLRBNode.BLACK);
        buildPennant(chunkSize, fb.LLRBNode.RED);
      }
    }
    return root;
  };
  var base12 = new Base12Num(childList.length);
  var root = buildFrom12Array(base12);
  if (root != null) {
    return new util.SortedMap(mapSortFn || cmp, root);
  } else {
    return new util.SortedMap(mapSortFn || cmp);
  }
}

priorityHashText(priority) => priority is num ? "number:$priority" : "string:$priority";

void validatePriorityNode(Node priorityNode) {
  if (priorityNode.isLeafNode) {
    var val = priorityNode.val();
    assert(val is String || val is num || (val is Map && val.containsKey(".sv")));
    // "Priority must be a string or number.");
  } else {
    assert(priorityNode == MAX_NODE || priorityNode.isEmpty); //, "priority of unexpected type.");
  }

  assert(priorityNode == MAX_NODE || priorityNode.priority.isEmpty);
  //, "Priority nodes can't have a priority of their own.");
}

const EMPTY_NODE = const ChildrenNode._(const SortedMap(NAME_COMPARATOR), null, IndexMap.Default);

class _MaxNode extends Node {

  const _MaxNode() : super();

  int compareTo(Node other) => identical(this,other) ? 0 : 1;

  bool operator==(other) => identical(this,other);

  Node get priority => this;


  getImmediateChild(String childName) => EMPTY_NODE;

  get isEmpty => false;

}

const MAX_NODE = const _MaxNode();

