
import 'dart:math' as math;
import 'dart:async';
import 'repo.dart';

final _random = new math.Random();

class StatsListener {
  final _collection;
  var _last = null;
  StatsListener(this._collection);

  get() {
    var newStats = this._collection.get();
    var delta = new Map.from(newStats);
    if (this._last) {
      for (var stat in this._last) {
        delta[stat] = delta[stat] - this._last[stat];
      }
    }
    this._last = newStats;
    return delta;
  }
}

class StatsReporter {
  final _statsToReport = {};
  final _statsListener;
  final _server;

  StatsReporter(collection,this._server) : _statsListener = new StatsListener(collection) {
    var timeout = FIRST_STATS_MIN_TIME + (FIRST_STATS_MAX_TIME - FIRST_STATS_MIN_TIME) * _random.nextDouble();
    new Future.delayed(new Duration(milliseconds: timeout.floor()))
    .then(_reportStats);
  }

  includeStat(stat) {
    this._statsToReport[stat] = true;
  }

  _reportStats(_) {
    var stats = this._statsListener.get();
    var reportedStats = {};
    var haveStatsToReport = false;
    for (var stat in stats) {
      if (stats[stat] > 0 && _statsToReport.containsKey(stat)) {
        reportedStats[stat] = stats[stat];
        haveStatsToReport = true;
      }
    }
    if (haveStatsToReport) {
      this._server.reportStats(reportedStats);
    }
    var timeout = _random.nextDouble() * 2 * REPORT_STATS_INTERVAL;
    new Future.delayed(new Duration(milliseconds: timeout.floor()))
    .then(_reportStats);
  }

}

const FIRST_STATS_MIN_TIME = 10 * 1E3;
const FIRST_STATS_MAX_TIME = 30 * 1E3;
const REPORT_STATS_INTERVAL = 5 * 60 * 1E3;

class StatsCollection {
  final _counters = <String,int>{};


  incrementCounter(String name, [int amount = 1]) =>
      _counters[name] = (_counters[name] ?? 0) +1;

  get() => new Map<String,int>.from(_counters);

}

class StatsManager {

  static final _collections = <Repo,StatsCollection>{};
  static final _reporters = <RepoInfo,StatsReporter>{};

  static getCollection(repoInfo) =>
    _collections.putIfAbsent(repoInfo, ()=>new StatsCollection());

  static getOrCreateReporter(repoInfo, StatsReporter creatorFunction()) =>
    _reporters.putIfAbsent(repoInfo, creatorFunction);

}

