

library firebase_dart.core.util;

import 'dart:collection';
import 'package:quiver/collection.dart';
import 'snap.dart';

part 'util/path.dart';
part 'util/server_values.dart';

const MIN_NAME = "[MIN_NAME]";
const MAX_NAME = "[MAX_NAME]";

int nameCompare(String a, String b) {
  if (a == b) {
    return 0;
  } else {
    if (a == MIN_NAME || b == MAX_NAME) {
      return-1;
    }
    if (b == MIN_NAME || a == MAX_NAME) {
      return 1;
    }

    var aAsInt = tryParseInt(a), bAsInt = tryParseInt(b);
    if (aAsInt != null) {
      if (bAsInt != null) {
        return aAsInt - bAsInt == 0 ? a.length - b.length : aAsInt - bAsInt;
      } else {
        return-1;
      }
    } else {
      if (bAsInt != null) {
        return 1;
      } else {
        return a.compareTo(b);
      }
    }
  }
}


final _INTEGER_REGEXP = new RegExp(r"^-?\\d{1,10}$");

tryParseInt(String str) {
  if (_INTEGER_REGEXP.hasMatch(str)) {
    var intVal = int.parse(str);
    if (intVal >= -2147483648 && intVal <= 2147483647) {
      return intVal;
    }
  }
  return null;
}


Exception errorForServerCode(code) {
  var reason = "Unknown Error";
  if (code == "too_big") {
    reason = "The data requested exceeds the maximum size " + "that can be accessed with a single request.";
  } else {
    if (code == "permission_denied") {
      reason = "Client doesn't have permission to access the desired data.";
    } else {
      if (code == "unavailable") {
        reason = "The service is unavailable";
      }
    }
  }
  var error = new Exception(code + ": " + reason);
  error.code = code.toUpperCase();
  return error;
}
