part of firebase_dart.core.util;

class ServerValues {

  static Map<String, dynamic> generateWithValues(values) {
    values = values ?? {};
    values["timestamp"] = values["timestamp"] ?? new DateTime.now().millisecondsSinceEpoch;
    return values;
  }


  static dynamic resolveDeferredValue(dynamic value, Map<String, dynamic> serverValues) {
    if (value is !Map) {
      return value;
    } else {
      assert(value.containsKey(".sv"));// "Unexpected leaf node or priority contents");
      return serverValues[value[".sv"]];
    }
  }

  static SparseSnapshotTree resolveDeferredValueTree(tree, Map<String, dynamic> serverValues) {
    var resolvedTree = new SparseSnapshotTree();
    tree.forEachTree(Path.empty, (path, node) {
      resolvedTree.remember(path, resolveDeferredValueSnapshot(node, serverValues));
    });
    return resolvedTree;
  }

  static Node resolveDeferredValueSnapshot(Node node, Map<String, dynamic> serverValues) {
    var rawPri = (node.priority.val());
    var priority = resolveDeferredValue(rawPri, serverValues);
    var newNode;
    if (node is LeafNode) {
      var leafNode = (node);
      var value = resolveDeferredValue(leafNode.value, serverValues);
      if (value != leafNode.value || priority != leafNode.priority.val()) {
        return new LeafNode(value, new Node.fromJson(priority));
      } else {
        return node;
      }
    } else {
      var childrenNode = (node);
      newNode = childrenNode;
      if (priority != childrenNode.priority.val()) {
        newNode = newNode.updatePriority(new LeafNode(priority));
      }
      childrenNode.forEachChild(priorityIndex, (childName, childNode) {
        var newChildNode = resolveDeferredValueSnapshot(childNode, serverValues);
        if (newChildNode != childNode) {
          newNode = newNode.updateImmediateChild(childName, newChildNode);
        }
      });
      return newNode;
    }
  }

}
