
import 'llrb.dart';
import 'dart:collection';

class SortedMap<K,V> extends UnmodifiableMapBase<K,V> {
  
  final LLRBNode<K,V> _root;


  const SortedMap._(this._root);
  
  factory SortedMap({Comparator<K> comparator}) {
    return new SortedMap._(new LLRBNode.nil(comparator: comparator));
  }

  SortedMap<K,V> withKeyValueAdded(K key, V value) => new SortedMap<K,V>._(
        _root.insert(key, value).copy(color: LLRBNode.BLACK));


  SortedMap<K,V> withKeyRemoved(K key) => new SortedMap<K,V>._(
        _root.remove(key).copy(color: LLRBNode.BLACK));


  V operator[](K key) {
    var cmp;
    var node = this._root;
    while (!node.isEmpty) {
      cmp = _root.comparator(key, node.key);
      if (cmp == 0) {
        return node.value;
      } else {
        if (cmp < 0) {
          node = node.left;
        } else {
          if (cmp > 0) {
            node = node.right;
          }
        }
      }
    }
    return null;
  }

  @override
  bool containsKey(K key) {
    var node = this._root;
    while (!node.isEmpty) {
      var cmp = _root.comparator(key, node.key);
      if (cmp == 0) {
        return true;
      } else {
        if (cmp < 0) {
          node = node.left;
        } else {
          if (cmp > 0) {
            node = node.right;
          }
        }
      }
    }
  }

  /**
   * Get the last key in the map that is strictly smaller than key.
   * Returns null if no key was not found.
   */
  K lastKeyBefore(K key) {
    var node = this._root, rightParent = null;
    while (!node.isEmpty) {
      var cmp = _root.comparator(key, node.key);
      if (cmp == 0) {
        if (!node.left.isEmpty) {
          node = node.left;
          while (!node.right.isEmpty) {
            node = node.right;
          }
          return node.key;
        } else {
          if (rightParent!=null) {
            return rightParent.key;
          } else {
            return null;
          }
        }
      } else {
        if (cmp < 0) {
          node = node.left;
        } else {
          if (cmp > 0) {
            rightParent = node;
            node = node.right;
          }
        }
      }
    }
    return null;
  }

  bool get isEmpty => _root.isEmpty;

  int get length => _root.length;

  K firstKey() => _root.firstKey();

  K lastKey() => _root.lastKey();

  forEach(void f(K key, V value)) => _root.forEach((p)=>f(p.key,p.value));

  Iterable<K> get keys => _root.map((p)=>p.k);

  Iterator<Pair<K,V>> get iterator => _root.iterator;

  Iterator<Pair<K,V>> get reversedIterator => _root.reversedIterator;

  Iterator<Pair<K, V>> iteratorFrom(K key) => _root.iteratorFrom(key);

  Iterator<Pair<K, V>> reversedIteratorFrom(K key) => _root.reversedIteratorFrom(key);

}


