part of firebase_dart.core.view;

class CacheNode {
  final Node node;
  final bool isFullyInitialized;
  final bool isFiltered;

  const CacheNode(this.node, this.isFullyInitialized, this.isFiltered);

  bool isCompleteForPath(util.Path path) {
    if (path.isEmpty) {
      return this.isFullyInitialized && !this.isFiltered;
    } else {
      var childKey = path.front;
      return this.isCompleteForChild(childKey);
    }
  }

  bool isCompleteForChild(String key) {
    return this.isFullyInitialized && !this.isFiltered || this.node.hasChild(key);
  }

}

class ViewCache {

  final CacheNode eventCache;
  final CacheNode serverCache;

  const ViewCache(this.eventCache, this.serverCache);

  static const empty = const ViewCache(const CacheNode(EMPTY_NODE, false, false),
      const CacheNode(EMPTY_NODE, false, false));

  ViewCache updateEventSnap(Node eventSnap, bool complete, bool filtered) {
    return new ViewCache(new CacheNode(eventSnap, complete, filtered), this.serverCache);
  }

  updateServerSnap(Node serverSnap, bool complete, bool filtered) {
    return new ViewCache(this.eventCache, new CacheNode(serverSnap, complete, filtered));
  }

  Node get completeEventSnap => eventCache.isFullyInitialized ? eventCache.node : null;

  Node get completeServerSnap => serverCache.isFullyInitialized ? serverCache.node : null;

}

