part of firebase_dart.core.view;

enum ChangeType {CHILD_ADDED, CHILD_CHANGED, CHILD_MOVED, CHILD_REMOVED,VALUE}

class Change {

  final ChangeType type;
  final Node snapshotNode;
  final String childName;
  final Node oldSnap;
  final String prevName;

  const Change(this.type, this.snapshotNode, {this.childName, this.oldSnap, this.prevName});

  const Change.value(Node snapshot) : this(ChangeType.VALUE, snapshot);
  const Change.childAdded(String childKey, Node snapshot) :
        this(ChangeType.CHILD_ADDED, snapshot, childName: childKey);

  const Change.childRemoved(String childKey, Node snapshot) :
      this(ChangeType.CHILD_REMOVED, snapshot);

  const Change.childChanged(String childKey, Node newSnapshot, Node oldSnapshot) :
      this(ChangeType.CHILD_CHANGED, newSnapshot, childName: childKey, oldSnap: oldSnapshot);

  const Change.childMoved(String childKey, Node snapshot) :
      this(ChangeType.CHILD_MOVED, snapshot, childName: childKey);

  Change changeWithPrevName(String prevName) {
    return new Change(this.type, this.snapshotNode, childName: this.childName,
        oldSnap: this.oldSnap, prevName: prevName);
  }
}

class ChildChangeAccumulator {

  final _changeMap = <String,Change>{};

  trackChildChange(Change change) {
    var type = change.type;
    var childKey = (change.childName);
    assert(type == ChangeType.CHILD_ADDED || type == ChangeType.CHILD_CHANGED ||
        type == ChangeType.CHILD_REMOVED);//, "Only child changes supported for tracking");

    assert(childKey != ".priority");//, "Only non-priority child changes can be tracked.");

    var oldChange = _changeMap[childKey];
    if (oldChange!=null) {
      var oldType = oldChange.type;
      if (type == ChangeType.CHILD_ADDED && oldType == ChangeType.CHILD_REMOVED) {
        this._changeMap[childKey] = new Change.childChanged(childKey, change.snapshotNode, oldChange.snapshotNode);
      } else {
        if (type == ChangeType.CHILD_REMOVED && oldType == ChangeType.CHILD_ADDED) {
          this._changeMap.remove(childKey);
        } else {
          if (type == ChangeType.CHILD_REMOVED && oldType == ChangeType.CHILD_CHANGED) {
            this._changeMap[childKey] = new Change.childRemoved(childKey, (oldChange.oldSnap));
          } else {
            if (type == ChangeType.CHILD_CHANGED && oldType == ChangeType.CHILD_ADDED) {
              this._changeMap[childKey] = new Change.childAdded(childKey, change.snapshotNode);
            } else {
              if (type == ChangeType.CHILD_CHANGED && oldType == ChangeType.CHILD_CHANGED) {
                this._changeMap[childKey] = new Change.childChanged(childKey, change.snapshotNode, (oldChange.oldSnap));
              } else {
                throw new Exception("Illegal combination of changes: $change occurred after $oldChange");
              }
            }
          }
        }
      }
    } else {
      this._changeMap[childKey] = change;
    }
  }

  Iterable<Change> get changes => _changeMap.values;

}

