
part of firebase_dart.core.view;

abstract class CompleteChildSource {

  getCompleteChild(String childKey);

  getChildAfterChild(index, child, bool reverse);



}


class _NoCompleteChildSource implements CompleteChildSource {
  const _NoCompleteChildSource();

  getCompleteChild(String childKey) => null;

  getChildAfterChild(index, child, bool reverse) => null;

}

const NO_COMPLETE_CHILD_SOURCE = const _NoCompleteChildSource();

class WriteTreeCompleteChildSource implements CompleteChildSource {
  final WriteTreeRef _writes;
  final ViewCache _viewCache;
  final Node _completeServerCache;

  WriteTreeCompleteChildSource(this._writes, this._viewCache, [this._completeServerCache]);

  Node getCompleteChild(String childKey) {
    var node = _viewCache.eventCache;
    if (node.isCompleteForChild(childKey)) {
      return node.node.getImmediateChild(childKey);
    } else {
      var serverNode = this._completeServerCache != null ?
      new CacheNode(this._completeServerCache, true, false) : this._viewCache.serverCache;
      return this._writes.calcCompleteChild(childKey, serverNode);
    }
  }

  getChildAfterChild(index, child, bool reverse) {
    var completeServerData = this._completeServerCache != null
        ? this._completeServerCache : this._viewCache.completeServerSnap;
    var nodes = this._writes.calcIndexedSlice(completeServerData, child, 1, reverse, index);
    if (nodes.length == 0) {
      return null;
    } else {
      return nodes[0];
    }
  }

}



