
part of firebase_dart.core.view;

class _WIRE_PROTOCOL_CONSTANTS {
  static const INDEX_START_VALUE="sp";
  static const INDEX_START_NAME="sn";
  static const INDEX_END_VALUE="ep";
  static const INDEX_END_NAME="en";
  static const LIMIT="l";
  static const VIEW_FROM="vf";
  static const VIEW_FROM_LEFT="l";
  static const VIEW_FROM_RIGHT="r";
  static const INDEX="i";

}

class _REST_QUERY_CONSTANTS {
  static const ORDER_BY="orderBy";
  static const PRIORITY_INDEX=r"$priority";
  static const VALUE_INDEX=r"$value";
  static const KEY_INDEX=r"$key";
  static const START_AT="startAt";
  static const END_AT="endAt";
  static const LIMIT_TO_FIRST="limitToFirst";
  static const LIMIT_TO_LAST="limitToLast";
}

class QueryParams {

  bool _limitSet = false;
  bool _startSet = false;
  bool _startNameSet = false;
  bool _endSet = false;
  bool _endNameSet = false;
  int _limit = 0;
  String _viewFrom = "";
  String _indexStartValue = null;
  String _indexStartName = "";
  String _indexEndValue = null;
  String _indexEndName = "";
  Index _index = priorityIndex;

  QueryParams();

  static final DEFAULT = new QueryParams();

  bool get hasStart => _startSet;

  bool get isViewFromLeft {
    if (this._viewFrom == "") {
      return this._startSet;
    } else {
      return this._viewFrom == _WIRE_PROTOCOL_CONSTANTS.VIEW_FROM_LEFT;

    }
  }

  String get indexStartValue {
    if (!hasStart)
      throw new Exception("Only valid if start has been set");
    return _indexStartValue;
  }

  String get indexStartName {
    if (!hasStart)
      throw new Exception("Only valid if start has been set");
    if (this._startNameSet) {
      return this._indexStartName;
    } else {
      return util.MIN_NAME;
    }
  }

  bool get hasEnd => _endSet;

  String get indexEndValue {
    if (!hasEnd)
      throw new Exception("Only valid if end has been set");
    return this._indexEndValue;
  }

  String get indexEndName {
    if (!hasEnd)
      throw new Exception("Only valid if end has been set");
    if (this._endNameSet) {
      return this._indexEndName;
    } else {
      return util.MAX_NAME;
    }
  }

  bool get hasLimit => _limitSet;
  
  bool get hasAnchoredLimit => hasLimit && _viewFrom != "";

  int get limit {
    if (!hasLimit) 
      throw new Exception("Only valid if limit has been set");
    return this._limit;
  }
  
  Index get index => _index;

  QueryParams _copy() {
    var copy = new QueryParams();
    copy._limitSet = this._limitSet;
    copy._limit = this._limit;
    copy._startSet = this._startSet;
    copy._indexStartValue = this._indexStartValue;
    copy._startNameSet = this._startNameSet;
    copy._indexStartName = this._indexStartName;
    copy._endSet = this._endSet;
    copy._indexEndValue = this._indexEndValue;
    copy._endNameSet = this._endNameSet;
    copy._indexEndName = this._indexEndName;
    copy._index = this._index;
    return copy;
  }

  QueryParams limitToFirst(int newLimit) {
    var newParams = this._copy();
    newParams._limitSet = true;
    newParams._limit = newLimit;
    newParams._viewFrom = _WIRE_PROTOCOL_CONSTANTS.VIEW_FROM_LEFT;
    return newParams;
  }


  QueryParams limitToLast(int newLimit) {
    var newParams = this._copy();
    newParams._limitSet = true;
    newParams._limit = newLimit;
    newParams._viewFrom = _WIRE_PROTOCOL_CONSTANTS.VIEW_FROM_RIGHT;
    return newParams;
  }

  QueryParams startAt(String indexValue, String key) {
    var newParams = this._copy();
    newParams._startSet = true;
    newParams._indexStartValue = indexValue;
    if (key != null) {
      newParams._startNameSet = true;
      newParams._indexStartName = key;
    } else {
      newParams._startNameSet = false;
      newParams._indexStartName = "";
    }
    return newParams;
  }

  QueryParams endAt(String indexValue, String key) {
    var newParams = this._copy();
    newParams._endSet = true;
    newParams._indexEndValue = indexValue;
    if (key!=null) {
      newParams._endNameSet = true;
      newParams._indexEndName = key;
    } else {
      newParams._endNameSet = false;
      newParams._indexEndName = "";
    }
    return newParams;
  }
  
  QueryParams orderBy(String index) {
    var newParams = this._copy();
    newParams._index = index;
    return newParams;
  }

  Map<String,dynamic> getQueryObject() {
    final Map<String,dynamic> obj = {};
    if (this._startSet) {
      obj[_WIRE_PROTOCOL_CONSTANTS.INDEX_START_VALUE] = this._indexStartValue;
      if (this._startNameSet) {
        obj[_WIRE_PROTOCOL_CONSTANTS.INDEX_START_NAME] = this._indexStartName;
      }
    }
    if (this._endSet) {
      obj[_WIRE_PROTOCOL_CONSTANTS.INDEX_END_VALUE] = this._indexEndValue;
      if (this._endNameSet) {
        obj[_WIRE_PROTOCOL_CONSTANTS.INDEX_END_NAME] = this._indexEndName;
      }
    }
    if (this._limitSet) {
      obj[_WIRE_PROTOCOL_CONSTANTS.LIMIT] = this._limit;
      var viewFrom = this._viewFrom;
      if (viewFrom == "") {
        if (this.isViewFromLeft) {
          viewFrom = _WIRE_PROTOCOL_CONSTANTS.VIEW_FROM_LEFT;
        } else {
          viewFrom = _WIRE_PROTOCOL_CONSTANTS.VIEW_FROM_RIGHT;
        }
      }
      obj[_WIRE_PROTOCOL_CONSTANTS.VIEW_FROM] = viewFrom;
    }
    if (this._index != priorityIndex) {
      obj[_WIRE_PROTOCOL_CONSTANTS.INDEX] = this._index.toString();
    }
    return obj;
  }

  bool get loadsAllData => !(this._startSet || this._endSet || this._limitSet);
  
  
  bool get isDefault => loadsAllData && this._index == priorityIndex;

  NodeFilter get nodeFilter {
    if (this.loadsAllData) {
      return new IndexedFilter(this.index);
    } else {
      if (this.hasLimit) {
        return new LimitedFilter(this);
      } else {
        return new RangedFilter(this);
      }
    }
  }

  Map<String,String> toRestQueryStringParameters() {
    final qs = <String,String>{};
    if (this.isDefault) {
      return qs;
    }
    var orderBy;
    if (this._index == priorityIndex) {
      orderBy = _REST_QUERY_CONSTANTS.PRIORITY_INDEX;
    } else {
      if (this._index == valueIndex) {
        orderBy = _REST_QUERY_CONSTANTS.VALUE_INDEX;
      } else {
        if (this._index == keyIndex) {
          orderBy = _REST_QUERY_CONSTANTS.KEY_INDEX;
        } else {
          if (_index is !PathIndex)
            throw new Exception("Unrecognized index type!");
          orderBy = this._index.toString();
        }
      }
    }
    qs[_REST_QUERY_CONSTANTS.ORDER_BY] = JSON.encode(orderBy);
    if (this._startSet) {
      qs[_REST_QUERY_CONSTANTS.START_AT] = JSON.encode(this._indexStartValue);
      if (this._startNameSet) {
        qs[_REST_QUERY_CONSTANTS.START_AT] += "," + JSON.encode(this._indexStartName);
      }
    }
    if (this._endSet) {
      qs[_REST_QUERY_CONSTANTS.END_AT] = JSON.encode(this._indexEndValue);
      if (this._endNameSet) {
        qs[_REST_QUERY_CONSTANTS.END_AT] += "," + JSON.encode(this._indexEndName);
      }
    }
    if (this._limitSet) {
      if (this.isViewFromLeft) {
        qs[_REST_QUERY_CONSTANTS.LIMIT_TO_FIRST] = this._limit;
      } else {
        qs[_REST_QUERY_CONSTANTS.LIMIT_TO_LAST] = this._limit;
      }
    }
    return qs;
  }

  toString() {
    return JSON.encode(this.getQueryObject());
  }
}
