goog.provide("fb.api.INTERNAL");
goog.require("fb.core.PersistentConnection");
goog.require("fb.realtime.Connection");
goog.require("fb.login.transports.PopupReceiver");
goog.require("fb.login.Constants");
fb.api.INTERNAL = {};
fb.api.INTERNAL.forceLongPolling = function() {
    fb.realtime.WebSocketConnection.forceDisallow();
    fb.realtime.BrowserPollConnection.forceAllow();
};
goog.exportProperty(fb.api.INTERNAL, "forceLongPolling", fb.api.INTERNAL.forceLongPolling);
fb.api.INTERNAL.forceWebSockets = function() {
    fb.realtime.BrowserPollConnection.forceDisallow();
};
goog.exportProperty(fb.api.INTERNAL, "forceWebSockets", fb.api.INTERNAL.forceWebSockets);
fb.api.INTERNAL.setSecurityDebugCallback = function(ref, callback) {
    ref.repo.persistentConnection_.securityDebugCallback_ = callback;
};
goog.exportProperty(fb.api.INTERNAL, "setSecurityDebugCallback", fb.api.INTERNAL.setSecurityDebugCallback);
fb.api.INTERNAL.stats = function(ref, showDelta) {
    ref.repo.stats(showDelta);
};
goog.exportProperty(fb.api.INTERNAL, "stats", fb.api.INTERNAL.stats);
fb.api.INTERNAL.statsIncrementCounter = function(ref, metric) {
    ref.repo.statsIncrementCounter(metric);
};
goog.exportProperty(fb.api.INTERNAL, "statsIncrementCounter", fb.api.INTERNAL.statsIncrementCounter);
fb.api.INTERNAL.dataUpdateCount = function(ref) {
    return ref.repo.dataUpdateCount;
};
goog.exportProperty(fb.api.INTERNAL, "dataUpdateCount", fb.api.INTERNAL.dataUpdateCount);
fb.api.INTERNAL.interceptServerData = function(ref, callback) {
    return ref.repo.interceptServerData_(callback);
};
goog.exportProperty(fb.api.INTERNAL, "interceptServerData", fb.api.INTERNAL.interceptServerData);
fb.api.INTERNAL.onLoginPopupOpen = function(callback) {
    new fb.login.transports.PopupReceiver(callback);
};
goog.exportProperty(fb.api.INTERNAL, "onPopupOpen", fb.api.INTERNAL.onLoginPopupOpen);
fb.api.INTERNAL.setAuthenticationServer = function(host) {
    fb.login.Constants.SERVER_HOST = host;
};
goog.exportProperty(fb.api.INTERNAL, "setAuthenticationServer", fb.api.INTERNAL.setAuthenticationServer);
goog.provide("fb.api.TransactionResult");
fb.api.TransactionResult = function(committed, snapshot) {
    this["committed"] = committed;
    this["snapshot"] = snapshot;
};
goog.provide("fb.api.OnDisconnect");
goog.require("fb.core.util");
goog.require("fb.core.util.validation");
goog.require("fb.util.promise.Deferred");
goog.require("fb.util.validation");
fb.api.OnDisconnect = function(repo, path) {
    this.repo_ = repo;
    this.path_ = path;
};
fb.api.OnDisconnect.prototype.cancel = function(opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.onDisconnect().cancel", 0, 1, arguments.length);
    fb.util.validation.validateCallback("Firebase.onDisconnect().cancel", 1, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo_.onDisconnectCancel(this.path_, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
};
goog.exportProperty(fb.api.OnDisconnect.prototype, "cancel", fb.api.OnDisconnect.prototype.cancel);
fb.api.OnDisconnect.prototype.remove = function(opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.onDisconnect().remove", 0, 1, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.onDisconnect().remove", this.path_);
    fb.util.validation.validateCallback("Firebase.onDisconnect().remove", 1, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo_.onDisconnectSet(this.path_, null, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
};
goog.exportProperty(fb.api.OnDisconnect.prototype, "remove", fb.api.OnDisconnect.prototype.remove);
fb.api.OnDisconnect.prototype.set = function(value, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.onDisconnect().set", 1, 2, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.onDisconnect().set", this.path_);
    fb.core.util.validation.validateFirebaseDataArg("Firebase.onDisconnect().set", 1, value, this.path_, false);
    fb.util.validation.validateCallback("Firebase.onDisconnect().set", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo_.onDisconnectSet(this.path_, value, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
};
goog.exportProperty(fb.api.OnDisconnect.prototype, "set", fb.api.OnDisconnect.prototype.set);
fb.api.OnDisconnect.prototype.setWithPriority = function(value, priority, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.onDisconnect().setWithPriority", 2, 3, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.onDisconnect().setWithPriority", this.path_);
    fb.core.util.validation.validateFirebaseDataArg("Firebase.onDisconnect().setWithPriority", 1, value, this.path_, false);
    fb.core.util.validation.validatePriority("Firebase.onDisconnect().setWithPriority", 2, priority, false);
    fb.util.validation.validateCallback("Firebase.onDisconnect().setWithPriority", 3, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo_.onDisconnectSetWithPriority(this.path_, value, priority, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
};
goog.exportProperty(fb.api.OnDisconnect.prototype, "setWithPriority", fb.api.OnDisconnect.prototype.setWithPriority);
fb.api.OnDisconnect.prototype.update = function(objectToMerge, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.onDisconnect().update", 1, 2, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.onDisconnect().update", this.path_);
    if (goog.isArray(objectToMerge)) {
        var newObjectToMerge = {};
        for (var i = 0;i < objectToMerge.length;++i) {
            newObjectToMerge["" + i] = objectToMerge[i];
        }
        objectToMerge = newObjectToMerge;
        fb.core.util.warn("Passing an Array to Firebase.onDisconnect().update() is deprecated. Use set() if you want to overwrite the " + "existing data, or an Object with integer keys if you really do want to only update some of the children.");
    }
    fb.core.util.validation.validateFirebaseMergeDataArg("Firebase.onDisconnect().update", 1, objectToMerge, this.path_, false);
    fb.util.validation.validateCallback("Firebase.onDisconnect().update", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo_.onDisconnectUpdate(this.path_, objectToMerge, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
};
goog.exportProperty(fb.api.OnDisconnect.prototype, "update", fb.api.OnDisconnect.prototype.update);
goog.provide("fb.api.DataSnapshot");
goog.require("fb.core.snap");
goog.require("fb.core.util.SortedMap");
goog.require("fb.core.util.validation");
fb.api.DataSnapshot = function(node, ref, index) {
    this.node_ = node;
    this.query_ = ref;
    this.index_ = index;
};
fb.api.DataSnapshot.prototype.val = function() {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.val", 0, 0, arguments.length);
    return this.node_.val();
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "val", fb.api.DataSnapshot.prototype.val);
fb.api.DataSnapshot.prototype.exportVal = function() {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.exportVal", 0, 0, arguments.length);
    return this.node_.val(true);
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "exportVal", fb.api.DataSnapshot.prototype.exportVal);
fb.api.DataSnapshot.prototype.exists = function() {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.exists", 0, 0, arguments.length);
    return!this.node_.isEmpty();
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "exists", fb.api.DataSnapshot.prototype.exists);
fb.api.DataSnapshot.prototype.child = function(childPathString) {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.child", 0, 1, arguments.length);
    if (goog.isNumber(childPathString)) {
        childPathString = String(childPathString);
    }
    fb.core.util.validation.validatePathString("Firebase.DataSnapshot.child", 1, childPathString, false);
    var childPath = new fb.core.util.Path(childPathString);
    var childRef = this.query_.child(childPath);
    return new fb.api.DataSnapshot(this.node_.getChild(childPath), childRef, fb.core.snap.PriorityIndex);
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "child", fb.api.DataSnapshot.prototype.child);
fb.api.DataSnapshot.prototype.hasChild = function(childPathString) {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.hasChild", 1, 1, arguments.length);
    fb.core.util.validation.validatePathString("Firebase.DataSnapshot.hasChild", 1, childPathString, false);
    var childPath = new fb.core.util.Path(childPathString);
    return!this.node_.getChild(childPath).isEmpty();
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "hasChild", fb.api.DataSnapshot.prototype.hasChild);
fb.api.DataSnapshot.prototype.getPriority = function() {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.getPriority", 0, 0, arguments.length);
    return(this.node_.getPriority().val());
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "getPriority", fb.api.DataSnapshot.prototype.getPriority);
fb.api.DataSnapshot.prototype.forEach = function(action) {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.forEach", 1, 1, arguments.length);
    fb.util.validation.validateCallback("Firebase.DataSnapshot.forEach", 1, action, false);
    if (this.node_.isLeafNode()) {
        return false;
    }
    var childrenNode = (this.node_);
    var self = this;
    return!!childrenNode.forEachChild(this.index_, function(key, node) {
        return action(new fb.api.DataSnapshot(node, self.query_.child(key), fb.core.snap.PriorityIndex));
    });
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "forEach", fb.api.DataSnapshot.prototype.forEach);
fb.api.DataSnapshot.prototype.hasChildren = function() {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.hasChildren", 0, 0, arguments.length);
    if (this.node_.isLeafNode()) {
        return false;
    } else {
        return!this.node_.isEmpty();
    }
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "hasChildren", fb.api.DataSnapshot.prototype.hasChildren);
fb.api.DataSnapshot.prototype.name = function() {
    fb.core.util.warn("Firebase.DataSnapshot.name() being deprecated. " + "Please use Firebase.DataSnapshot.key() instead.");
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.name", 0, 0, arguments.length);
    return this.key();
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "name", fb.api.DataSnapshot.prototype.name);
fb.api.DataSnapshot.prototype.key = function() {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.key", 0, 0, arguments.length);
    return this.query_.key();
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "key", fb.api.DataSnapshot.prototype.key);
fb.api.DataSnapshot.prototype.numChildren = function() {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.numChildren", 0, 0, arguments.length);
    return this.node_.numChildren();
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "numChildren", fb.api.DataSnapshot.prototype.numChildren);
fb.api.DataSnapshot.prototype.ref = function() {
    fb.util.validation.validateArgCount("Firebase.DataSnapshot.ref", 0, 0, arguments.length);
    return this.query_;
};
goog.exportProperty(fb.api.DataSnapshot.prototype, "ref", fb.api.DataSnapshot.prototype.ref);
