goog.provide("fb.core.Repo");
goog.require("fb.api.DataSnapshot");
goog.require("fb.core.PersistentConnection");
goog.require("fb.core.ReadonlyRestClient");
goog.require("fb.core.SnapshotHolder");
goog.require("fb.core.SparseSnapshotTree");
goog.require("fb.core.SyncTree");
goog.require("fb.core.stats.StatsCollection");
goog.require("fb.core.stats.StatsListener");
goog.require("fb.core.stats.StatsManager");
goog.require("fb.core.stats.StatsReporter");
goog.require("fb.core.util");
goog.require("fb.core.util.ServerValues");
goog.require("fb.core.util.Tree");
goog.require("fb.core.view.EventQueue");
goog.require("fb.login.AuthenticationManager");
goog.require("fb.util.json");
goog.require("fb.util.jwt");
goog.require("goog.string");
fb.core.Repo = goog.defineClass(null, {constructor:function(repoInfo, forceRestClient) {
    this.repoInfo_ = repoInfo;
    this.stats_ = fb.core.stats.StatsManager.getCollection(repoInfo);
    this.statsListener_ = null;
    this.eventQueue_ = new fb.core.view.EventQueue;
    this.nextWriteId_ = 1;
    this.persistentConnection_ = null;
    this.server_;
    if (forceRestClient || fb.core.util.beingCrawled()) {
        this.server_ = new fb.core.ReadonlyRestClient(this.repoInfo_, goog.bind(this.onDataUpdate_, this));
        setTimeout(goog.bind(this.onConnectStatus_, this, true), 0);
    } else {
        this.persistentConnection_ = new fb.core.PersistentConnection(this.repoInfo_, goog.bind(this.onDataUpdate_, this), goog.bind(this.onConnectStatus_, this), goog.bind(this.onServerInfoUpdate_, this));
        this.server_ = this.persistentConnection_;
    }
    this.statsReporter_ = fb.core.stats.StatsManager.getOrCreateReporter(repoInfo, goog.bind(function() {
        return new fb.core.stats.StatsReporter(this.stats_, this.server_);
    }, this));
    this.transactions_init_();
    this.infoData_ = new fb.core.SnapshotHolder;
    var self = this;
    this.infoSyncTree_ = new fb.core.SyncTree({startListening:function(query, tag, currentHashFn, onComplete) {
        var infoEvents = [];
        var node = self.infoData_.getNode(query.path);
        if (!node.isEmpty()) {
            infoEvents = self.infoSyncTree_.applyServerOverwrite(query.path, node);
            setTimeout(function() {
                onComplete("ok");
            }, 0);
        }
        return infoEvents;
    }, stopListening:goog.nullFunction});
    this.updateInfo_("connected", false);
    this.onDisconnect_ = new fb.core.SparseSnapshotTree;
    this.auth = new fb.login.AuthenticationManager(repoInfo, goog.bind(this.server_.auth, this.server_), goog.bind(this.server_.unauth, this.server_), goog.bind(this.onAuthStatus_, this));
    this.dataUpdateCount = 0;
    this.interceptServerDataCallback_ = null;
    this.serverSyncTree_ = new fb.core.SyncTree({startListening:function(query, tag, currentHashFn, onComplete) {
        self.server_.listen(query, currentHashFn, tag, function(status, data) {
            var events = onComplete(status, data);
            self.eventQueue_.raiseEventsForChangedPath(query.path, events);
        });
        return[];
    }, stopListening:function(query, tag) {
        self.server_.unlisten(query, tag);
    }});
}, toString:function() {
    return(this.repoInfo_.secure ? "https://" : "http://") + this.repoInfo_.host;
}, name:function() {
    return this.repoInfo_.namespace;
}, serverTime:function() {
    var offsetNode = this.infoData_.getNode(new fb.core.util.Path(".info/serverTimeOffset"));
    var offset = (offsetNode.val()) || 0;
    return(new Date).getTime() + offset;
}, generateServerValues:function() {
    return fb.core.util.ServerValues.generateWithValues({"timestamp":this.serverTime()});
}, onDataUpdate_:function(pathString, data, isMerge, tag) {
    this.dataUpdateCount++;
    var path = new fb.core.util.Path(pathString);
    data = this.interceptServerDataCallback_ ? this.interceptServerDataCallback_(pathString, data) : data;
    var events = [];
    if (tag) {
        if (isMerge) {
            var taggedChildren = goog.object.map((data), function(raw) {
                return fb.core.snap.NodeFromJSON(raw);
            });
            events = this.serverSyncTree_.applyTaggedQueryMerge(path, taggedChildren, tag);
        } else {
            var taggedSnap = fb.core.snap.NodeFromJSON(data);
            events = this.serverSyncTree_.applyTaggedQueryOverwrite(path, taggedSnap, tag);
        }
    } else {
        if (isMerge) {
            var changedChildren = goog.object.map((data), function(raw) {
                return fb.core.snap.NodeFromJSON(raw);
            });
            events = this.serverSyncTree_.applyServerMerge(path, changedChildren);
        } else {
            var snap = fb.core.snap.NodeFromJSON(data);
            events = this.serverSyncTree_.applyServerOverwrite(path, snap);
        }
    }
    var affectedPath = path;
    if (events.length > 0) {
        affectedPath = this.rerunTransactions_(path);
    }
    this.eventQueue_.raiseEventsForChangedPath(affectedPath, events);
}, interceptServerData_:function(callback) {
    this.interceptServerDataCallback_ = callback;
}, onConnectStatus_:function(connectStatus) {
    this.updateInfo_("connected", connectStatus);
    if (connectStatus === false) {
        this.runOnDisconnectEvents_();
    }
}, onServerInfoUpdate_:function(updates) {
    var self = this;
    fb.core.util.each(updates, function(value, key) {
        self.updateInfo_(key, value);
    });
}, onAuthStatus_:function(authStatus) {
    this.updateInfo_("authenticated", authStatus);
}, updateInfo_:function(pathString, value) {
    var path = new fb.core.util.Path("/.info/" + pathString);
    var newNode = fb.core.snap.NodeFromJSON(value);
    this.infoData_.updateSnapshot(path, newNode);
    var events = this.infoSyncTree_.applyServerOverwrite(path, newNode);
    this.eventQueue_.raiseEventsForChangedPath(path, events);
}, getNextWriteId_:function() {
    return this.nextWriteId_++;
}, setWithPriority:function(path, newVal, newPriority, onComplete) {
    this.log_("set", {path:path.toString(), value:newVal, priority:newPriority});
    var serverValues = this.generateServerValues();
    var newNodeUnresolved = fb.core.snap.NodeFromJSON(newVal, newPriority);
    var newNode = fb.core.util.ServerValues.resolveDeferredValueSnapshot(newNodeUnresolved, serverValues);
    var writeId = this.getNextWriteId_();
    var events = this.serverSyncTree_.applyUserOverwrite(path, newNode, writeId, true);
    this.eventQueue_.queueEvents(events);
    var self = this;
    this.server_.put(path.toString(), newNodeUnresolved.val(true), function(status, errorReason) {
        var success = status === "ok";
        if (!success) {
            fb.core.util.warn("set at " + path + " failed: " + status);
        }
        var clearEvents = self.serverSyncTree_.ackUserWrite(writeId, !success);
        self.eventQueue_.raiseEventsForChangedPath(path, clearEvents);
        self.callOnCompleteCallback(onComplete, status, errorReason);
    });
    var affectedPath = this.abortTransactions_(path);
    this.rerunTransactions_(affectedPath);
    this.eventQueue_.raiseEventsForChangedPath(affectedPath, []);
}, update:function(path, childrenToMerge, onComplete) {
    this.log_("update", {path:path.toString(), value:childrenToMerge});
    var empty = true;
    var serverValues = this.generateServerValues();
    var changedChildren = {};
    goog.object.forEach(childrenToMerge, function(changedValue, changedKey) {
        empty = false;
        var newNodeUnresolved = fb.core.snap.NodeFromJSON(changedValue);
        changedChildren[changedKey] = fb.core.util.ServerValues.resolveDeferredValueSnapshot(newNodeUnresolved, serverValues);
    });
    if (!empty) {
        var writeId = this.getNextWriteId_();
        var events = this.serverSyncTree_.applyUserMerge(path, changedChildren, writeId);
        this.eventQueue_.queueEvents(events);
        var self = this;
        this.server_.merge(path.toString(), childrenToMerge, function(status, errorReason) {
            var success = status === "ok";
            if (!success) {
                fb.core.util.warn("update at " + path + " failed: " + status);
            }
            var clearEvents = self.serverSyncTree_.ackUserWrite(writeId, !success);
            var affectedPath = path;
            if (clearEvents.length > 0) {
                affectedPath = self.rerunTransactions_(path);
            }
            self.eventQueue_.raiseEventsForChangedPath(affectedPath, clearEvents);
            self.callOnCompleteCallback(onComplete, status, errorReason);
        });
        var affectedPath = this.abortTransactions_(path);
        this.rerunTransactions_(affectedPath);
        this.eventQueue_.raiseEventsForChangedPath(path, []);
    } else {
        fb.core.util.log("update() called with empty data.  Don't do anything.");
        this.callOnCompleteCallback(onComplete, "ok");
    }
}, runOnDisconnectEvents_:function() {
    this.log_("onDisconnectEvents");
    var self = this;
    var serverValues = this.generateServerValues();
    var resolvedOnDisconnectTree = fb.core.util.ServerValues.resolveDeferredValueTree(this.onDisconnect_, serverValues);
    var events = [];
    resolvedOnDisconnectTree.forEachTree(fb.core.util.Path.Empty, function(path, snap) {
        events = events.concat(self.serverSyncTree_.applyServerOverwrite(path, snap));
        var affectedPath = self.abortTransactions_(path);
        self.rerunTransactions_(affectedPath);
    });
    this.onDisconnect_ = new fb.core.SparseSnapshotTree;
    this.eventQueue_.raiseEventsForChangedPath(fb.core.util.Path.Empty, events);
}, onDisconnectCancel:function(path, onComplete) {
    var self = this;
    this.server_.onDisconnectCancel(path.toString(), function(status, errorReason) {
        if (status === "ok") {
            self.onDisconnect_.forget(path);
        }
        self.callOnCompleteCallback(onComplete, status, errorReason);
    });
}, onDisconnectSet:function(path, value, onComplete) {
    var self = this;
    var newNode = fb.core.snap.NodeFromJSON(value);
    this.server_.onDisconnectPut(path.toString(), newNode.val(true), function(status, errorReason) {
        if (status === "ok") {
            self.onDisconnect_.remember(path, newNode);
        }
        self.callOnCompleteCallback(onComplete, status, errorReason);
    });
}, onDisconnectSetWithPriority:function(path, value, priority, onComplete) {
    var self = this;
    var newNode = fb.core.snap.NodeFromJSON(value, priority);
    this.server_.onDisconnectPut(path.toString(), newNode.val(true), function(status, errorReason) {
        if (status === "ok") {
            self.onDisconnect_.remember(path, newNode);
        }
        self.callOnCompleteCallback(onComplete, status, errorReason);
    });
}, onDisconnectUpdate:function(path, childrenToMerge, onComplete) {
    var empty = true;
    for (var childName in childrenToMerge) {
        empty = false;
    }
    if (empty) {
        fb.core.util.log("onDisconnect().update() called with empty data.  Don't do anything.");
        this.callOnCompleteCallback(onComplete, "ok");
        return;
    }
    var self = this;
    this.server_.onDisconnectMerge(path.toString(), childrenToMerge, function(status, errorReason) {
        if (status === "ok") {
            for (var childName in childrenToMerge) {
                var newChildNode = fb.core.snap.NodeFromJSON(childrenToMerge[childName]);
                self.onDisconnect_.remember(path.child(childName), newChildNode);
            }
        }
        self.callOnCompleteCallback(onComplete, status, errorReason);
    });
}, addEventCallbackForQuery:function(query, eventRegistration) {
    var events;
    if (query.path.getFront() === ".info") {
        events = this.infoSyncTree_.addEventRegistration(query, eventRegistration);
    } else {
        events = this.serverSyncTree_.addEventRegistration(query, eventRegistration);
    }
    this.eventQueue_.raiseEventsAtPath(query.path, events);
}, removeEventCallbackForQuery:function(query, eventRegistration) {
    var events;
    if (query.path.getFront() === ".info") {
        events = this.infoSyncTree_.removeEventRegistration(query, eventRegistration);
    } else {
        events = this.serverSyncTree_.removeEventRegistration(query, eventRegistration);
    }
    this.eventQueue_.raiseEventsAtPath(query.path, events);
}, interrupt:function() {
    if (this.persistentConnection_) {
        this.persistentConnection_.interrupt();
    }
}, resume:function() {
    if (this.persistentConnection_) {
        this.persistentConnection_.resume();
    }
}, stats:function(showDelta) {
    if (typeof console === "undefined") {
        return;
    }
    var stats;
    if (showDelta) {
        if (!this.statsListener_) {
            this.statsListener_ = new fb.core.stats.StatsListener(this.stats_);
        }
        stats = this.statsListener_.get();
    } else {
        stats = this.stats_.get();
    }
    var longestName = goog.array.reduce(goog.object.getKeys(stats), function(previousValue, currentValue, index, array) {
        return Math.max(currentValue.length, previousValue);
    }, 0);
    for (var stat in stats) {
        var value = stats[stat];
        for (var i = stat.length;i < longestName + 2;i++) {
            stat += " ";
        }
        console.log(stat + value);
    }
}, statsIncrementCounter:function(metric) {
    this.stats_.incrementCounter(metric);
    this.statsReporter_.includeStat(metric);
}, log_:function(var_args) {
    var prefix = "";
    if (this.persistentConnection_) {
        prefix = this.persistentConnection_.id + ":";
    }
    fb.core.util.log(prefix, arguments);
}, callOnCompleteCallback:function(callback, status, errorReason) {
    if (callback) {
        fb.core.util.exceptionGuard(function() {
            if (status == "ok") {
                callback(null);
            } else {
                var code = (status || "error").toUpperCase();
                var message = code;
                if (errorReason) {
                    message += ": " + errorReason;
                }
                var error = new Error(message);
                error.code = code;
                callback(error);
            }
        });
    }
}});
goog.provide("fb.core.Repo_transaction");
goog.require("fb.core.Repo");
goog.require("fb.core.snap.PriorityIndex");
goog.require("fb.core.util");
fb.core.TransactionStatus = {RUN:1, SENT:2, COMPLETED:3, SENT_NEEDS_ABORT:4, NEEDS_ABORT:5};
fb.core.Repo.MAX_TRANSACTION_RETRIES_ = 25;
fb.core.Transaction;
fb.core.Repo.prototype.transactions_init_ = function() {
    this.transactionQueueTree_ = new fb.core.util.Tree;
};
fb.core.Repo.prototype.startTransaction = function(path, transactionUpdate, onComplete, applyLocally) {
    this.log_("transaction on " + path);
    var valueCallback = function() {
    };
    var watchRef = new Firebase(this, path);
    watchRef.on("value", valueCallback);
    var unwatcher = function() {
        watchRef.off("value", valueCallback);
    };
    var transaction = ({path:path, update:transactionUpdate, onComplete:onComplete, status:null, order:fb.core.util.LUIDGenerator(), applyLocally:applyLocally, retryCount:0, unwatcher:unwatcher, abortReason:null, currentWriteId:null, currentInputSnapshot:null, currentOutputSnapshotRaw:null, currentOutputSnapshotResolved:null});
    var currentState = this.getLatestState_(path);
    transaction.currentInputSnapshot = currentState;
    var newVal = transaction.update(currentState.val());
    if (!goog.isDef(newVal)) {
        transaction.unwatcher();
        transaction.currentOutputSnapshotRaw = null;
        transaction.currentOutputSnapshotResolved = null;
        if (transaction.onComplete) {
            var snapshot = new fb.api.DataSnapshot((transaction.currentInputSnapshot), new Firebase(this, transaction.path), fb.core.snap.PriorityIndex);
            transaction.onComplete(null, false, snapshot);
        }
    } else {
        fb.core.util.validation.validateFirebaseData("transaction failed: Data returned ", newVal, transaction.path);
        transaction.status = fb.core.TransactionStatus.RUN;
        var queueNode = this.transactionQueueTree_.subTree(path);
        var nodeQueue = queueNode.getValue() || [];
        nodeQueue.push(transaction);
        queueNode.setValue(nodeQueue);
        var priorityForNode;
        if (typeof newVal === "object" && newVal !== null && fb.util.obj.contains(newVal, ".priority")) {
            priorityForNode = fb.util.obj.get(newVal, ".priority");
            fb.core.util.assert(fb.core.util.validation.isValidPriority(priorityForNode), "Invalid priority returned by transaction. " + "Priority must be a valid string, finite number, server value, or null.");
        } else {
            var currentNode = this.serverSyncTree_.calcCompleteEventCache(path) || fb.core.snap.EMPTY_NODE;
            priorityForNode = currentNode.getPriority().val();
        }
        priorityForNode = (priorityForNode);
        var serverValues = this.generateServerValues();
        var newNodeUnresolved = fb.core.snap.NodeFromJSON(newVal, priorityForNode);
        var newNode = fb.core.util.ServerValues.resolveDeferredValueSnapshot(newNodeUnresolved, serverValues);
        transaction.currentOutputSnapshotRaw = newNodeUnresolved;
        transaction.currentOutputSnapshotResolved = newNode;
        transaction.currentWriteId = this.getNextWriteId_();
        var events = this.serverSyncTree_.applyUserOverwrite(path, newNode, transaction.currentWriteId, transaction.applyLocally);
        this.eventQueue_.raiseEventsForChangedPath(path, events);
        this.sendReadyTransactions_();
    }
};
fb.core.Repo.prototype.getLatestState_ = function(path, excludeSets) {
    return this.serverSyncTree_.calcCompleteEventCache(path, excludeSets) || fb.core.snap.EMPTY_NODE;
};
fb.core.Repo.prototype.sendReadyTransactions_ = function(opt_node) {
    var node = (opt_node || this.transactionQueueTree_);
    if (!opt_node) {
        this.pruneCompletedTransactionsBelowNode_(node);
    }
    if (node.getValue() !== null) {
        var queue = this.buildTransactionQueue_(node);
        fb.core.util.assert(queue.length > 0, "Sending zero length transaction queue");
        var allRun = goog.array.every(queue, function(transaction) {
            return transaction.status === fb.core.TransactionStatus.RUN;
        });
        if (allRun) {
            this.sendTransactionQueue_(node.path(), queue);
        }
    } else {
        if (node.hasChildren()) {
            var self = this;
            node.forEachChild(function(childNode) {
                self.sendReadyTransactions_(childNode);
            });
        }
    }
};
fb.core.Repo.prototype.sendTransactionQueue_ = function(path, queue) {
    var setsToIgnore = goog.array.map(queue, function(txn) {
        return txn.currentWriteId;
    });
    var latestState = this.getLatestState_(path, setsToIgnore);
    var snapToSend = latestState;
    var latestHash = latestState.hash();
    for (var i = 0;i < queue.length;i++) {
        var txn = queue[i];
        fb.core.util.assert(txn.status === fb.core.TransactionStatus.RUN, "tryToSendTransactionQueue_: items in queue should all be run.");
        txn.status = fb.core.TransactionStatus.SENT;
        txn.retryCount++;
        var relativePath = fb.core.util.Path.relativePath(path, txn.path);
        snapToSend = snapToSend.updateChild(relativePath, (txn.currentOutputSnapshotRaw));
    }
    var dataToSend = snapToSend.val(true);
    var pathToSend = path;
    var self = this;
    this.server_.put(pathToSend.toString(), dataToSend, function(status) {
        self.log_("transaction put response", {path:pathToSend.toString(), status:status});
        var events = [];
        if (status === "ok") {
            var callbacks = [];
            for (i = 0;i < queue.length;i++) {
                queue[i].status = fb.core.TransactionStatus.COMPLETED;
                events = events.concat(self.serverSyncTree_.ackUserWrite(queue[i].currentWriteId));
                if (queue[i].onComplete) {
                    var node = (queue[i].currentOutputSnapshotResolved);
                    var ref = new Firebase(self, queue[i].path);
                    var snapshot = new fb.api.DataSnapshot(node, ref, fb.core.snap.PriorityIndex);
                    callbacks.push(goog.bind(queue[i].onComplete, null, null, true, snapshot));
                }
                queue[i].unwatcher();
            }
            self.pruneCompletedTransactionsBelowNode_(self.transactionQueueTree_.subTree(path));
            self.sendReadyTransactions_();
            self.eventQueue_.raiseEventsForChangedPath(path, events);
            for (i = 0;i < callbacks.length;i++) {
                fb.core.util.exceptionGuard(callbacks[i]);
            }
        } else {
            if (status === "datastale") {
                for (i = 0;i < queue.length;i++) {
                    if (queue[i].status === fb.core.TransactionStatus.SENT_NEEDS_ABORT) {
                        queue[i].status = fb.core.TransactionStatus.NEEDS_ABORT;
                    } else {
                        queue[i].status = fb.core.TransactionStatus.RUN;
                    }
                }
            } else {
                fb.core.util.warn("transaction at " + pathToSend.toString() + " failed: " + status);
                for (i = 0;i < queue.length;i++) {
                    queue[i].status = fb.core.TransactionStatus.NEEDS_ABORT;
                    queue[i].abortReason = status;
                }
            }
            self.rerunTransactions_(path);
        }
    }, latestHash);
};
fb.core.Repo.prototype.rerunTransactions_ = function(changedPath) {
    var rootMostTransactionNode = this.getAncestorTransactionNode_(changedPath);
    var path = rootMostTransactionNode.path();
    var queue = this.buildTransactionQueue_(rootMostTransactionNode);
    this.rerunTransactionQueue_(queue, path);
    return path;
};
fb.core.Repo.prototype.rerunTransactionQueue_ = function(queue, path) {
    if (queue.length === 0) {
        return;
    }
    var callbacks = [];
    var events = [];
    var setsToIgnore = goog.array.map(queue, function(q) {
        return q.currentWriteId;
    });
    for (var i = 0;i < queue.length;i++) {
        var transaction = queue[i];
        var relativePath = fb.core.util.Path.relativePath(path, transaction.path);
        var abortTransaction = false, abortReason;
        fb.core.util.assert(relativePath !== null, "rerunTransactionsUnderNode_: relativePath should not be null.");
        if (transaction.status === fb.core.TransactionStatus.NEEDS_ABORT) {
            abortTransaction = true;
            abortReason = transaction.abortReason;
            events = events.concat(this.serverSyncTree_.ackUserWrite(transaction.currentWriteId, true));
        } else {
            if (transaction.status === fb.core.TransactionStatus.RUN) {
                if (transaction.retryCount >= fb.core.Repo.MAX_TRANSACTION_RETRIES_) {
                    abortTransaction = true;
                    abortReason = "maxretry";
                    events = events.concat(this.serverSyncTree_.ackUserWrite(transaction.currentWriteId, true));
                } else {
                    var currentNode = this.getLatestState_(transaction.path, setsToIgnore);
                    transaction.currentInputSnapshot = currentNode;
                    var newData = queue[i].update(currentNode.val());
                    if (goog.isDef(newData)) {
                        fb.core.util.validation.validateFirebaseData("transaction failed: Data returned ", newData, transaction.path);
                        var newDataNode = fb.core.snap.NodeFromJSON(newData);
                        var hasExplicitPriority = typeof newData === "object" && newData != null && fb.util.obj.contains(newData, ".priority");
                        if (!hasExplicitPriority) {
                            newDataNode = newDataNode.updatePriority(currentNode.getPriority());
                        }
                        var oldWriteId = transaction.currentWriteId;
                        var serverValues = this.generateServerValues();
                        var newNodeResolved = fb.core.util.ServerValues.resolveDeferredValueSnapshot(newDataNode, serverValues);
                        transaction.currentOutputSnapshotRaw = newDataNode;
                        transaction.currentOutputSnapshotResolved = newNodeResolved;
                        transaction.currentWriteId = this.getNextWriteId_();
                        goog.array.remove(setsToIgnore, oldWriteId);
                        events = events.concat(this.serverSyncTree_.applyUserOverwrite(transaction.path, newNodeResolved, transaction.currentWriteId, transaction.applyLocally));
                        events = events.concat(this.serverSyncTree_.ackUserWrite(oldWriteId, true));
                    } else {
                        abortTransaction = true;
                        abortReason = "nodata";
                        events = events.concat(this.serverSyncTree_.ackUserWrite(transaction.currentWriteId, true));
                    }
                }
            }
        }
        this.eventQueue_.raiseEventsForChangedPath(path, events);
        events = [];
        if (abortTransaction) {
            queue[i].status = fb.core.TransactionStatus.COMPLETED;
            (function(unwatcher) {
                setTimeout(unwatcher, Math.floor(0));
            })(queue[i].unwatcher);
            if (queue[i].onComplete) {
                if (abortReason === "nodata") {
                    var ref = new Firebase(this, queue[i].path);
                    var lastInput = (queue[i].currentInputSnapshot);
                    var snapshot = new fb.api.DataSnapshot(lastInput, ref, fb.core.snap.PriorityIndex);
                    callbacks.push(goog.bind(queue[i].onComplete, null, null, false, snapshot));
                } else {
                    callbacks.push(goog.bind(queue[i].onComplete, null, new Error(abortReason), false, null));
                }
            }
        }
    }
    this.pruneCompletedTransactionsBelowNode_(this.transactionQueueTree_);
    for (i = 0;i < callbacks.length;i++) {
        fb.core.util.exceptionGuard(callbacks[i]);
    }
    this.sendReadyTransactions_();
};
fb.core.Repo.prototype.getAncestorTransactionNode_ = function(path) {
    var front;
    var transactionNode = this.transactionQueueTree_;
    while ((front = path.getFront()) !== null && transactionNode.getValue() === null) {
        transactionNode = transactionNode.subTree(front);
        path = path.popFront();
    }
    return transactionNode;
};
fb.core.Repo.prototype.buildTransactionQueue_ = function(transactionNode) {
    var transactionQueue = [];
    this.aggregateTransactionQueuesForNode_(transactionNode, transactionQueue);
    transactionQueue.sort(function(a, b) {
        return a.order - b.order;
    });
    return transactionQueue;
};
fb.core.Repo.prototype.aggregateTransactionQueuesForNode_ = function(node, queue) {
    var nodeQueue = node.getValue();
    if (nodeQueue !== null) {
        for (var i = 0;i < nodeQueue.length;i++) {
            queue.push(nodeQueue[i]);
        }
    }
    var self = this;
    node.forEachChild(function(child) {
        self.aggregateTransactionQueuesForNode_(child, queue);
    });
};
fb.core.Repo.prototype.pruneCompletedTransactionsBelowNode_ = function(node) {
    var queue = node.getValue();
    if (queue) {
        var to = 0;
        for (var from = 0;from < queue.length;from++) {
            if (queue[from].status !== fb.core.TransactionStatus.COMPLETED) {
                queue[to] = queue[from];
                to++;
            }
        }
        queue.length = to;
        node.setValue(queue.length > 0 ? queue : null);
    }
    var self = this;
    node.forEachChild(function(childNode) {
        self.pruneCompletedTransactionsBelowNode_(childNode);
    });
};
fb.core.Repo.prototype.abortTransactions_ = function(path) {
    var affectedPath = this.getAncestorTransactionNode_(path).path();
    var transactionNode = this.transactionQueueTree_.subTree(path);
    var self = this;
    transactionNode.forEachAncestor(function(node) {
        self.abortTransactionsOnNode_(node);
    });
    this.abortTransactionsOnNode_(transactionNode);
    transactionNode.forEachDescendant(function(node) {
        self.abortTransactionsOnNode_(node);
    });
    return affectedPath;
};
fb.core.Repo.prototype.abortTransactionsOnNode_ = function(node) {
    var queue = node.getValue();
    if (queue !== null) {
        var callbacks = [];
        var events = [];
        var lastSent = -1;
        for (var i = 0;i < queue.length;i++) {
            if (queue[i].status === fb.core.TransactionStatus.SENT_NEEDS_ABORT) {
            } else {
                if (queue[i].status === fb.core.TransactionStatus.SENT) {
                    fb.core.util.assert(lastSent === i - 1, "All SENT items should be at beginning of queue.");
                    lastSent = i;
                    queue[i].status = fb.core.TransactionStatus.SENT_NEEDS_ABORT;
                    queue[i].abortReason = "set";
                } else {
                    fb.core.util.assert(queue[i].status === fb.core.TransactionStatus.RUN, "Unexpected transaction status in abort");
                    queue[i].unwatcher();
                    events = events.concat(this.serverSyncTree_.ackUserWrite(queue[i].currentWriteId, true));
                    if (queue[i].onComplete) {
                        var snapshot = null;
                        callbacks.push(goog.bind(queue[i].onComplete, null, new Error("set"), false, snapshot));
                    }
                }
            }
        }
        if (lastSent === -1) {
            node.setValue(null);
        } else {
            queue.length = lastSent + 1;
        }
        this.eventQueue_.raiseEventsForChangedPath(node.path(), events);
        for (i = 0;i < callbacks.length;i++) {
            fb.core.util.exceptionGuard(callbacks[i]);
        }
    }
};
goog.provide("fb.core.RepoManager");
goog.require("fb.core.Repo");
goog.require("fb.core.Repo_transaction");
goog.require("fb.util.obj");
fb.core.RepoManager = goog.defineClass(null, {constructor:function() {
    this.repos_ = {};
    this.useRestClient_ = false;
}, interrupt:function() {
    for (var repo in this.repos_) {
        this.repos_[repo].interrupt();
    }
}, resume:function() {
    for (var repo in this.repos_) {
        this.repos_[repo].resume();
    }
}, getRepo:function(repoInfo) {
    var repoHashString = repoInfo.toString();
    var repo = fb.util.obj.get(this.repos_, repoHashString);
    if (!repo) {
        repo = new fb.core.Repo(repoInfo, this.useRestClient_);
        this.repos_[repoHashString] = repo;
    }
    return repo;
}, forceRestClient:function() {
    this.useRestClient_ = true;
}});
goog.addSingletonGetter(fb.core.RepoManager);
goog.exportProperty(fb.core.RepoManager.prototype, "interrupt", fb.core.RepoManager.prototype.interrupt);
goog.exportProperty(fb.core.RepoManager.prototype, "resume", fb.core.RepoManager.prototype.resume);
