goog.provide("fb.core.view.ChildEventRegistration");
goog.provide("fb.core.view.EventRegistration");
goog.provide("fb.core.view.ValueEventRegistration");
goog.require("fb.api.DataSnapshot");
goog.require("fb.core.util");
goog.require("fb.core.view.CancelEvent");
goog.require("fb.core.view.DataEvent");
goog.require("goog.object");
fb.core.view.EventRegistration = function() {
};
fb.core.view.EventRegistration.prototype.respondsTo;
fb.core.view.EventRegistration.prototype.createEvent;
fb.core.view.EventRegistration.prototype.getEventRunner;
fb.core.view.EventRegistration.prototype.createCancelEvent;
fb.core.view.EventRegistration.prototype.matches;
fb.core.view.EventRegistration.prototype.hasAnyCallback;
fb.core.view.ValueEventRegistration = function(callback, cancelCallback, context) {
    this.callback_ = callback;
    this.cancelCallback_ = cancelCallback;
    this.context_ = context || null;
};
fb.core.view.ValueEventRegistration.prototype.respondsTo = function(eventType) {
    return eventType === "value";
};
fb.core.view.ValueEventRegistration.prototype.createEvent = function(change, query) {
    var index = query.getQueryParams().getIndex();
    return new fb.core.view.DataEvent("value", this, new fb.api.DataSnapshot(change.snapshotNode, query.ref(), index));
};
fb.core.view.ValueEventRegistration.prototype.getEventRunner = function(eventData) {
    var ctx = this.context_;
    if (eventData.getEventType() === "cancel") {
        fb.core.util.assert(this.cancelCallback_, "Raising a cancel event on a listener with no cancel callback");
        var cancelCB = this.cancelCallback_;
        return function() {
            cancelCB.call(ctx, eventData.error);
        };
    } else {
        var cb = this.callback_;
        return function() {
            cb.call(ctx, eventData.snapshot);
        };
    }
};
fb.core.view.ValueEventRegistration.prototype.createCancelEvent = function(error, path) {
    if (this.cancelCallback_) {
        return new fb.core.view.CancelEvent(this, error, path);
    } else {
        return null;
    }
};
fb.core.view.ValueEventRegistration.prototype.matches = function(other) {
    if (!(other instanceof fb.core.view.ValueEventRegistration)) {
        return false;
    } else {
        if (!other.callback_ || !this.callback_) {
            return true;
        } else {
            return other.callback_ === this.callback_ && other.context_ === this.context_;
        }
    }
};
fb.core.view.ValueEventRegistration.prototype.hasAnyCallback = function() {
    return this.callback_ !== null;
};
fb.core.view.ChildEventRegistration = function(callbacks, cancelCallback, opt_context) {
    this.callbacks_ = callbacks;
    this.cancelCallback_ = cancelCallback;
    this.context_ = opt_context;
};
fb.core.view.ChildEventRegistration.prototype.respondsTo = function(eventType) {
    var eventToCheck = eventType === "children_added" ? "child_added" : eventType;
    eventToCheck = eventToCheck === "children_removed" ? "child_removed" : eventToCheck;
    return goog.object.containsKey(this.callbacks_, eventToCheck);
};
fb.core.view.ChildEventRegistration.prototype.createCancelEvent = function(error, path) {
    if (this.cancelCallback_) {
        return new fb.core.view.CancelEvent(this, error, path);
    } else {
        return null;
    }
};
fb.core.view.ChildEventRegistration.prototype.createEvent = function(change, query) {
    fb.core.util.assert(change.childName != null, "Child events should have a childName.");
    var ref = query.ref().child((change.childName));
    var index = query.getQueryParams().getIndex();
    return new fb.core.view.DataEvent(change.type, this, new fb.api.DataSnapshot(change.snapshotNode, ref, index), change.prevName);
};
fb.core.view.ChildEventRegistration.prototype.getEventRunner = function(eventData) {
    var ctx = this.context_;
    if (eventData.getEventType() === "cancel") {
        fb.core.util.assert(this.cancelCallback_, "Raising a cancel event on a listener with no cancel callback");
        var cancelCB = this.cancelCallback_;
        return function() {
            cancelCB.call(ctx, eventData.error);
        };
    } else {
        var cb = this.callbacks_[eventData.eventType];
        return function() {
            cb.call(ctx, eventData.snapshot, eventData.prevName);
        };
    }
};
fb.core.view.ChildEventRegistration.prototype.matches = function(other) {
    if (other instanceof fb.core.view.ChildEventRegistration) {
        if (!this.callbacks_ || !other.callbacks_) {
            return true;
        } else {
            if (this.context_ === other.context_) {
                var otherCount = goog.object.getCount(other.callbacks_);
                var thisCount = goog.object.getCount(this.callbacks_);
                if (otherCount === thisCount) {
                    if (otherCount === 1) {
                        var otherKey = (goog.object.getAnyKey(other.callbacks_));
                        var thisKey = (goog.object.getAnyKey(this.callbacks_));
                        return thisKey === otherKey && (!other.callbacks_[otherKey] || !this.callbacks_[thisKey] || other.callbacks_[otherKey] === this.callbacks_[thisKey]);
                    } else {
                        return goog.object.every(this.callbacks_, function(cb, eventType) {
                            return other.callbacks_[eventType] === cb;
                        });
                    }
                }
            }
        }
    }
    return false;
};
fb.core.view.ChildEventRegistration.prototype.hasAnyCallback = function() {
    return this.callbacks_ !== null;
};
