goog.provide("Firebase");
goog.require("fb.api.INTERNAL");
goog.require("fb.api.OnDisconnect");
goog.require("fb.api.Query");
goog.require("fb.api.TEST_ACCESS");
goog.require("fb.api.TransactionResult");
goog.require("fb.constants");
goog.require("fb.core.Repo");
goog.require("fb.core.RepoManager");
goog.require("fb.core.util");
goog.require("fb.core.util.Path");
goog.require("fb.core.util.enableLogging");
goog.require("fb.core.util.nextPushId");
goog.require("fb.core.util.validation");
goog.require("fb.core.view.QueryParams");
goog.require("fb.login.Constants");
goog.require("fb.util.obj");
goog.require("fb.util.promise");
goog.require("fb.util.promise.Deferred");
goog.require("fb.util.validation");
function letMeUseMapAccessors() {
}
Firebase = goog.defineClass(fb.api.Query, {constructor:function(urlOrRepo, opt_pathOrContext) {
    var repo, path, repoManager;
    if (urlOrRepo instanceof fb.core.Repo) {
        repo = urlOrRepo;
        path = (opt_pathOrContext);
    } else {
        fb.util.validation.validateArgCount("new Firebase", 1, 2, arguments.length);
        var parsedUrl = fb.core.util.parseRepoInfo(arguments[0]), repoInfo = parsedUrl.repoInfo;
        fb.core.util.validation.validateUrl("new Firebase", 1, parsedUrl);
        if (opt_pathOrContext) {
            if (opt_pathOrContext instanceof fb.core.RepoManager) {
                repoManager = (opt_pathOrContext);
            } else {
                if (goog.isString(opt_pathOrContext)) {
                    repoManager = fb.core.RepoManager.getInstance();
                    repoInfo.persistenceKey = opt_pathOrContext;
                } else {
                    throw new Error("Expected a valid Firebase.Context for second argument to new Firebase()");
                }
            }
        } else {
            repoManager = fb.core.RepoManager.getInstance();
        }
        repo = repoManager.getRepo(repoInfo);
        path = parsedUrl.path;
    }
    fb.api.Query.call(this, repo, path, fb.core.view.QueryParams.DEFAULT, false);
    this.then = void 0;
    (this)["catch"] = void 0;
}, statics:{goOffline:function() {
    fb.util.validation.validateArgCount("Firebase.goOffline", 0, 0, arguments.length);
    fb.core.RepoManager.getInstance().interrupt();
}, goOnline:function() {
    fb.util.validation.validateArgCount("Firebase.goOnline", 0, 0, arguments.length);
    fb.core.RepoManager.getInstance().resume();
}, enableLogging:fb.core.util.enableLogging, ServerValue:{"TIMESTAMP":{".sv":"timestamp"}}, SDK_VERSION:fb.constants.CLIENT_VERSION, INTERNAL:fb.api.INTERNAL, Context:fb.core.RepoManager, TEST_ACCESS:fb.api.TEST_ACCESS}, name:function() {
    fb.core.util.warn("Firebase.name() being deprecated. Please use Firebase.key() instead.");
    fb.util.validation.validateArgCount("Firebase.name", 0, 0, arguments.length);
    return this.key();
}, key:function() {
    fb.util.validation.validateArgCount("Firebase.key", 0, 0, arguments.length);
    if (this.path.isEmpty()) {
        return null;
    } else {
        return this.path.getBack();
    }
}, child:function(pathString) {
    fb.util.validation.validateArgCount("Firebase.child", 1, 1, arguments.length);
    if (goog.isNumber(pathString)) {
        pathString = String(pathString);
    } else {
        if (!(pathString instanceof fb.core.util.Path)) {
            if (this.path.getFront() === null) {
                fb.core.util.validation.validateRootPathString("Firebase.child", 1, pathString, false);
            } else {
                fb.core.util.validation.validatePathString("Firebase.child", 1, pathString, false);
            }
        }
    }
    return new Firebase(this.repo, this.path.child(pathString));
}, parent:function() {
    fb.util.validation.validateArgCount("Firebase.parent", 0, 0, arguments.length);
    var parentPath = this.path.parent();
    return parentPath === null ? null : new Firebase(this.repo, parentPath);
}, root:function() {
    fb.util.validation.validateArgCount("Firebase.ref", 0, 0, arguments.length);
    var ref = this;
    while (ref.parent() !== null) {
        ref = ref.parent();
    }
    return ref;
}, set:function(newVal, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.set", 1, 2, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.set", this.path);
    fb.core.util.validation.validateFirebaseDataArg("Firebase.set", 1, newVal, this.path, false);
    fb.util.validation.validateCallback("Firebase.set", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.setWithPriority(this.path, newVal, null, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, update:function(objectToMerge, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.update", 1, 2, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.update", this.path);
    if (goog.isArray(objectToMerge)) {
        var newObjectToMerge = {};
        for (var i = 0;i < objectToMerge.length;++i) {
            newObjectToMerge["" + i] = objectToMerge[i];
        }
        objectToMerge = newObjectToMerge;
        fb.core.util.warn("Passing an Array to Firebase.update() is deprecated. " + "Use set() if you want to overwrite the existing data, or " + "an Object with integer keys if you really do want to " + "only update some of the children.");
    }
    fb.core.util.validation.validateFirebaseMergeDataArg("Firebase.update", 1, objectToMerge, this.path, false);
    fb.util.validation.validateCallback("Firebase.update", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.update(this.path, objectToMerge, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, setWithPriority:function(newVal, newPriority, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.setWithPriority", 2, 3, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.setWithPriority", this.path);
    fb.core.util.validation.validateFirebaseDataArg("Firebase.setWithPriority", 1, newVal, this.path, false);
    fb.core.util.validation.validatePriority("Firebase.setWithPriority", 2, newPriority, false);
    fb.util.validation.validateCallback("Firebase.setWithPriority", 3, opt_onComplete, true);
    if (this.key() === ".length" || this.key() === ".keys") {
        throw "Firebase.setWithPriority failed: " + this.key() + " is a read-only object.";
    }
    var deferred = new fb.util.promise.Deferred;
    this.repo.setWithPriority(this.path, newVal, newPriority, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, remove:function(opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.remove", 0, 1, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.remove", this.path);
    fb.util.validation.validateCallback("Firebase.remove", 1, opt_onComplete, true);
    return this.set(null, opt_onComplete);
}, transaction:function(transactionUpdate, opt_onComplete, opt_applyLocally) {
    fb.util.validation.validateArgCount("Firebase.transaction", 1, 3, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.transaction", this.path);
    fb.util.validation.validateCallback("Firebase.transaction", 1, transactionUpdate, false);
    fb.util.validation.validateCallback("Firebase.transaction", 2, opt_onComplete, true);
    fb.core.util.validation.validateBoolean("Firebase.transaction", 3, opt_applyLocally, true);
    if (this.key() === ".length" || this.key() === ".keys") {
        throw "Firebase.transaction failed: " + this.key() + " is a read-only object.";
    }
    if (typeof opt_applyLocally === "undefined") {
        opt_applyLocally = true;
    }
    var deferred = new fb.util.promise.Deferred;
    if (goog.isFunction(opt_onComplete)) {
        fb.util.promise.attachDummyErrorHandler(deferred.promise);
    }
    var promiseComplete = function(error, committed, snapshot) {
        if (error) {
            deferred.reject(error);
        } else {
            deferred.resolve(new fb.api.TransactionResult(committed, snapshot));
        }
        if (goog.isFunction(opt_onComplete)) {
            opt_onComplete(error, committed, snapshot);
        }
    };
    this.repo.startTransaction(this.path, transactionUpdate, promiseComplete, opt_applyLocally);
    return deferred.promise;
}, setPriority:function(priority, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.setPriority", 1, 2, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.setPriority", this.path);
    fb.core.util.validation.validatePriority("Firebase.setPriority", 1, priority, false);
    fb.util.validation.validateCallback("Firebase.setPriority", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.setWithPriority(this.path.child(".priority"), priority, null, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, push:function(opt_value, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.push", 0, 2, arguments.length);
    fb.core.util.validation.validateWritablePath("Firebase.push", this.path);
    fb.core.util.validation.validateFirebaseDataArg("Firebase.push", 1, opt_value, this.path, true);
    fb.util.validation.validateCallback("Firebase.push", 2, opt_onComplete, true);
    var now = this.repo.serverTime();
    var name = fb.core.util.nextPushId(now);
    var pushedRef = this.child(name);
    if (goog.isDefAndNotNull(opt_value)) {
        var self = this;
        var promise = pushedRef.set(opt_value, opt_onComplete).then(function() {
            return self.child(name);
        });
        pushedRef.then = goog.bind(promise.then, promise);
        (pushedRef)["catch"] = goog.bind(promise.then, promise, void 0);
        if (goog.isFunction(opt_onComplete)) {
            fb.util.promise.attachDummyErrorHandler(promise);
        }
    }
    return pushedRef;
}, onDisconnect:function() {
    fb.core.util.validation.validateWritablePath("Firebase.onDisconnect", this.path);
    return new fb.api.OnDisconnect(this.repo, this.path);
}, auth:function(cred, opt_onComplete, opt_onCancel) {
    fb.core.util.warn("FirebaseRef.auth() being deprecated. " + "Please use FirebaseRef.authWithCustomToken() instead.");
    fb.util.validation.validateArgCount("Firebase.auth", 1, 3, arguments.length);
    fb.core.util.validation.validateCredential("Firebase.auth", 1, cred, false);
    fb.util.validation.validateCallback("Firebase.auth", 2, opt_onComplete, true);
    fb.util.validation.validateCallback("Firebase.auth", 3, opt_onComplete, true);
    var clientOptions = {};
    clientOptions[fb.login.Constants.CLIENT_OPTION_SESSION_PERSISTENCE] = "none";
    var self = this;
    var deferred = new fb.util.promise.Deferred;
    self.repo.auth.authenticate(cred, {}, clientOptions, deferred.wrapCallback(opt_onComplete), opt_onCancel);
    return deferred.promise;
}, unauth:function(opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.unauth", 0, 1, arguments.length);
    fb.util.validation.validateCallback("Firebase.unauth", 1, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.unauthenticate(deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, getAuth:function() {
    fb.util.validation.validateArgCount("Firebase.getAuth", 0, 0, arguments.length);
    return this.repo.auth.getAuth();
}, onAuth:function(callback, opt_context) {
    fb.util.validation.validateArgCount("Firebase.onAuth", 1, 2, arguments.length);
    fb.util.validation.validateCallback("Firebase.onAuth", 1, callback, false);
    fb.util.validation.validateContextObject("Firebase.onAuth", 2, opt_context, true);
    this.repo.auth.on("auth_status", callback, opt_context);
}, offAuth:function(callback, opt_context) {
    fb.util.validation.validateArgCount("Firebase.offAuth", 1, 2, arguments.length);
    fb.util.validation.validateCallback("Firebase.offAuth", 1, callback, false);
    fb.util.validation.validateContextObject("Firebase.offAuth", 2, opt_context, true);
    this.repo.auth.off("auth_status", callback, opt_context);
}, authWithCustomToken:function(token, opt_onComplete, opt_options) {
    fb.util.validation.validateArgCount("Firebase.authWithCustomToken", 1, 3, arguments.length);
    if (arguments.length === 2 && fb.util.obj.isNonNullObject(opt_onComplete)) {
        opt_options = opt_onComplete;
        opt_onComplete = void 0;
    }
    fb.core.util.validation.validateCredential("Firebase.authWithCustomToken", 1, token, false);
    fb.util.validation.validateCallback("Firebase.authWithCustomToken", 2, opt_onComplete, true);
    fb.core.util.validation.validateObject("Firebase.authWithCustomToken", 3, opt_options, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.authenticate(token, {}, opt_options || {}, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, authWithOAuthPopup:function(provider, opt_onComplete, opt_options) {
    fb.util.validation.validateArgCount("Firebase.authWithOAuthPopup", 1, 3, arguments.length);
    if (arguments.length === 2 && fb.util.obj.isNonNullObject(opt_onComplete)) {
        opt_options = opt_onComplete;
        opt_onComplete = void 0;
    }
    fb.core.util.validation.validateAuthProvider("Firebase.authWithOAuthPopup", 1, provider, false);
    fb.util.validation.validateCallback("Firebase.authWithOAuthPopup", 2, opt_onComplete, true);
    fb.core.util.validation.validateObject("Firebase.authWithOAuthPopup", 3, opt_options, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.authWithPopup(provider, opt_options, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, authWithOAuthRedirect:function(provider, opt_onErr, opt_options) {
    fb.util.validation.validateArgCount("Firebase.authWithOAuthRedirect", 1, 3, arguments.length);
    if (arguments.length === 2 && fb.util.obj.isNonNullObject(opt_onErr)) {
        opt_options = opt_onErr;
        opt_onErr = void 0;
    }
    fb.core.util.validation.validateAuthProvider("Firebase.authWithOAuthRedirect", 1, provider, false);
    fb.util.validation.validateCallback("Firebase.authWithOAuthRedirect", 2, opt_onErr, false);
    fb.core.util.validation.validateObject("Firebase.authWithOAuthRedirect", 3, opt_options, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.authWithRedirect(provider, opt_options, deferred.wrapCallback(opt_onErr));
    return deferred.promise;
}, authWithOAuthToken:function(provider, params, opt_onComplete, opt_options) {
    fb.util.validation.validateArgCount("Firebase.authWithOAuthToken", 2, 4, arguments.length);
    if (arguments.length === 3 && fb.util.obj.isNonNullObject(opt_onComplete)) {
        opt_options = opt_onComplete;
        opt_onComplete = void 0;
    }
    fb.core.util.validation.validateAuthProvider("Firebase.authWithOAuthToken", 1, provider, false);
    fb.util.validation.validateCallback("Firebase.authWithOAuthToken", 3, opt_onComplete, true);
    fb.core.util.validation.validateObject("Firebase.authWithOAuthToken", 4, opt_options, true);
    var deferred = new fb.util.promise.Deferred;
    if (goog.isString(params)) {
        fb.core.util.validation.validateString("Firebase.authWithOAuthToken", 2, params, false);
        this.repo.auth.authWithCredential(provider + "/token", {"access_token":params}, opt_options, deferred.wrapCallback(opt_onComplete));
    } else {
        fb.core.util.validation.validateObject("Firebase.authWithOAuthToken", 2, params, false);
        this.repo.auth.authWithCredential(provider + "/token", params, opt_options, deferred.wrapCallback(opt_onComplete));
    }
    return deferred.promise;
}, authAnonymously:function(opt_onComplete, opt_options) {
    fb.util.validation.validateArgCount("Firebase.authAnonymously", 0, 2, arguments.length);
    if (arguments.length === 1 && fb.util.obj.isNonNullObject(opt_onComplete)) {
        opt_options = opt_onComplete;
        opt_onComplete = void 0;
    }
    fb.util.validation.validateCallback("Firebase.authAnonymously", 1, opt_onComplete, true);
    fb.core.util.validation.validateObject("Firebase.authAnonymously", 2, opt_options, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.authWithCredential("anonymous", {}, opt_options, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, authWithPassword:function(params, opt_onComplete, opt_options) {
    fb.util.validation.validateArgCount("Firebase.authWithPassword", 1, 3, arguments.length);
    if (arguments.length === 2 && fb.util.obj.isNonNullObject(opt_onComplete)) {
        opt_options = opt_onComplete;
        opt_onComplete = void 0;
    }
    fb.core.util.validation.validateObject("Firebase.authWithPassword", 1, params, false);
    fb.core.util.validation.validateObjectContainsKey("Firebase.authWithPassword", 1, params, "email", false, "string");
    fb.core.util.validation.validateObjectContainsKey("Firebase.authWithPassword", 1, params, "password", false, "string");
    fb.util.validation.validateCallback("Firebase.authWithPassword", 2, opt_onComplete, true);
    fb.core.util.validation.validateObject("Firebase.authWithPassword", 3, opt_options, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.authWithCredential("password", params, opt_options, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, createUser:function(params, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.createUser", 1, 2, arguments.length);
    fb.core.util.validation.validateObject("Firebase.createUser", 1, params, false);
    fb.core.util.validation.validateObjectContainsKey("Firebase.createUser", 1, params, "email", false, "string");
    fb.core.util.validation.validateObjectContainsKey("Firebase.createUser", 1, params, "password", false, "string");
    fb.util.validation.validateCallback("Firebase.createUser", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.createUser(params, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, removeUser:function(params, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.removeUser", 1, 2, arguments.length);
    fb.core.util.validation.validateObject("Firebase.removeUser", 1, params, false);
    fb.core.util.validation.validateObjectContainsKey("Firebase.removeUser", 1, params, "email", false, "string");
    fb.core.util.validation.validateObjectContainsKey("Firebase.removeUser", 1, params, "password", false, "string");
    fb.util.validation.validateCallback("Firebase.removeUser", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.removeUser(params, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, changePassword:function(params, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.changePassword", 1, 2, arguments.length);
    fb.core.util.validation.validateObject("Firebase.changePassword", 1, params, false);
    fb.core.util.validation.validateObjectContainsKey("Firebase.changePassword", 1, params, "email", false, "string");
    fb.core.util.validation.validateObjectContainsKey("Firebase.changePassword", 1, params, "oldPassword", false, "string");
    fb.core.util.validation.validateObjectContainsKey("Firebase.changePassword", 1, params, "newPassword", false, "string");
    fb.util.validation.validateCallback("Firebase.changePassword", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.changePassword(params, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, changeEmail:function(params, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.changeEmail", 1, 2, arguments.length);
    fb.core.util.validation.validateObject("Firebase.changeEmail", 1, params, false);
    fb.core.util.validation.validateObjectContainsKey("Firebase.changeEmail", 1, params, "oldEmail", false, "string");
    fb.core.util.validation.validateObjectContainsKey("Firebase.changeEmail", 1, params, "newEmail", false, "string");
    fb.core.util.validation.validateObjectContainsKey("Firebase.changeEmail", 1, params, "password", false, "string");
    fb.util.validation.validateCallback("Firebase.changeEmail", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.changeEmail(params, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}, resetPassword:function(params, opt_onComplete) {
    fb.util.validation.validateArgCount("Firebase.resetPassword", 1, 2, arguments.length);
    fb.core.util.validation.validateObject("Firebase.resetPassword", 1, params, false);
    fb.core.util.validation.validateObjectContainsKey("Firebase.resetPassword", 1, params, "email", false, "string");
    fb.util.validation.validateCallback("Firebase.resetPassword", 2, opt_onComplete, true);
    var deferred = new fb.util.promise.Deferred;
    this.repo.auth.resetPassword(params, deferred.wrapCallback(opt_onComplete));
    return deferred.promise;
}});
if (NODE_CLIENT) {
    module["exports"] = Firebase;
}
;
}
