goog.provide("fb.login.Constants");
fb.login.Constants = {SESSION_PERSISTENCE_KEY_PREFIX:"session", DEFAULT_SERVER_HOST:"auth.firebase.com", SERVER_HOST:"auth.firebase.com", API_VERSION:"v2", POPUP_PATH_TO_CHANNEL:"/auth/channel", POPUP_RELAY_FRAME_NAME:"__winchan_relay_frame", POPUP_CLOSE_CMD:"die", JSONP_CALLBACK_NAMESPACE:"__firebase_auth_jsonp", REDIR_REQUEST_ID_KEY:"redirect_request_id", REDIR_REQUEST_COMPLETION_KEY:"__firebase_request_key", REDIR_CLIENT_OPTIONS_KEY:"redirect_client_options", INTERNAL_REDIRECT_SENTINAL_PATH:"/blank/page.html",
    CLIENT_OPTION_SESSION_PERSISTENCE:"remember", CLIENT_OPTION_REDIRECT_TO:"redirectTo"};
goog.provide("fb.login.RequestInfo");
goog.require("fb.login.Constants");
fb.login.RequestInfo = function(opt_clientOptions, opt_transportOptions, opt_serverParams) {
    this.clientOptions = opt_clientOptions || {};
    this.transportOptions = opt_transportOptions || {};
    this.serverParams = opt_serverParams || {};
    if (!this.clientOptions[fb.login.Constants.CLIENT_OPTION_SESSION_PERSISTENCE]) {
        this.clientOptions[fb.login.Constants.CLIENT_OPTION_SESSION_PERSISTENCE] = "default";
    }
};
fb.login.RequestInfo.CLIENT_OPTIONS = [fb.login.Constants.CLIENT_OPTION_SESSION_PERSISTENCE, fb.login.Constants.CLIENT_OPTION_REDIRECT_TO];
fb.login.RequestInfo.fromParams = function(opt_params) {
    var clientOptions = {}, serverParams = {};
    fb.util.obj.foreach(opt_params || {}, function(param, value) {
        if (goog.array.contains(fb.login.RequestInfo.CLIENT_OPTIONS, param)) {
            clientOptions[param] = value;
        } else {
            serverParams[param] = value;
        }
    });
    return new fb.login.RequestInfo(clientOptions, {}, serverParams);
};
goog.provide("fb.login.SessionManager");
goog.require("fb.core.storage");
goog.require("fb.login.Constants");
goog.require("fb.util.json");
goog.require("fb.util.jwt");
fb.login.SessionManager = function(repoInfo, stores) {
    this.persistenceKey_ = [fb.login.Constants.SESSION_PERSISTENCE_KEY_PREFIX, repoInfo.persistenceKey, repoInfo.namespace].join(":");
    this.stores_ = stores;
};
fb.login.SessionManager.prototype.set = function(data, store) {
    if (!store) {
        if (this.stores_.length) {
            store = this.stores_[0];
        } else {
            throw new Error("fb.login.SessionManager : No storage options available!");
        }
    }
    store.set(this.persistenceKey_, data);
};
fb.login.SessionManager.prototype.get = function() {
    var sessions = goog.array.map(this.stores_, goog.bind(this.getFromStore_, this));
    sessions = goog.array.filter(sessions, function(session) {
        return session !== null;
    });
    goog.array.sort(sessions, function(a, b) {
        return fb.util.jwt.issuedAtTime(b["token"]) - fb.util.jwt.issuedAtTime(a["token"]);
    });
    if (sessions.length > 0) {
        return sessions.shift();
    }
    return null;
};
fb.login.SessionManager.prototype.getFromStore_ = function(store) {
    try {
        var session = store.get(this.persistenceKey_);
        if (session && session["token"]) {
            return session;
        }
    } catch (e) {
    }
    return null;
};
fb.login.SessionManager.prototype.clear = function(store) {
    var stores = store ? [store] : this.stores_, self = this;
    goog.array.forEach(this.stores_, function(store) {
        store.remove(self.persistenceKey_);
    });
};
goog.provide("fb.login.util.environment");
fb.login.util.environment.getUA = function() {
    if (typeof navigator !== "undefined" && typeof navigator["userAgent"] === "string") {
        return navigator["userAgent"];
    } else {
        return "";
    }
};
fb.login.util.environment.isMobileWrapper = function() {
    return fb.login.util.environment.isMobileCordova() || fb.login.util.environment.isMobileWindows() || fb.login.util.environment.isIosWebview();
};
fb.login.util.environment.isMobileCordova = function() {
    return typeof window !== "undefined" && !!(window["cordova"] || window["phonegap"] || window["PhoneGap"]) && /ios|iphone|ipod|ipad|android|blackberry|iemobile/i.test(fb.login.util.environment.getUA());
};
fb.login.util.environment.isReactNative = function() {
    return typeof navigator === "object" && navigator["product"] === "ReactNative";
};
fb.login.util.environment.isMobileWindows = function() {
    return typeof navigator !== "undefined" && (!!fb.login.util.environment.getUA().match(/Windows Phone/) || !!window["Windows"] && /^ms-appx:/.test(location.href));
};
fb.login.util.environment.isMobileFirefox = function() {
    var ua = fb.login.util.environment.getUA();
    return ua.indexOf("Fennec/") !== -1 || ua.indexOf("Firefox/") !== -1 && ua.indexOf("Android") !== -1;
};
fb.login.util.environment.isIosWebview = function() {
    var ua = fb.login.util.environment.getUA();
    return typeof navigator !== "undefined" && typeof window !== "undefined" && !!(ua.match(/(iPhone|iPod|iPad).*AppleWebKit(?!.*Safari)/i) || ua.match(/CriOS/) || ua.match(/Twitter for iPhone/) || ua.match(/FBAN\/FBIOS/) || window["navigator"]["standalone"]);
};
fb.login.util.environment.isHeadlessBrowser = function() {
    return!!fb.login.util.environment.getUA().match(/PhantomJS/);
};
fb.login.util.environment.isLocalFile = function() {
    return typeof location !== "undefined" && /^file:\//.test(location.href);
};
fb.login.util.environment.isIE = function() {
    var ua = fb.login.util.environment.getUA();
    return!!(ua.match(/MSIE/) || ua.match(/Trident/));
};
fb.login.util.environment.isIEVersionAtLeast = function(version) {
    var ua = fb.login.util.environment.getUA(), match;
    if (ua === "") {
        return false;
    }
    if (navigator["appName"] === "Microsoft Internet Explorer") {
        match = ua.match(/MSIE ([0-9]{1,}[\.0-9]{0,})/);
        if (match && match.length > 1) {
            return parseFloat(match[1]) >= version;
        }
    } else {
        if (ua.indexOf("Trident") > -1) {
            match = ua.match(/rv:([0-9]{2,2}[\.0-9]{0,})/);
            if (match && match.length > 1) {
                return parseFloat(match[1]) >= version;
            }
        }
    }
    return false;
};
goog.provide("fb.login.transports.util");
goog.require("fb.login.Constants");
goog.require("fb.util");
fb.login.transports.util.findRelay = function() {
    var loc = window["location"], frames = window["opener"]["frames"], i;
    for (i = frames.length - 1;i >= 0;i--) {
        try {
            if (frames[i]["location"]["protocol"] === window["location"]["protocol"] && frames[i]["location"]["host"] === window["location"]["host"] && frames[i]["name"] === fb.login.Constants.POPUP_RELAY_FRAME_NAME) {
                return frames[i];
            }
        } catch (e) {
        }
    }
    return null;
};
fb.login.transports.util.addListener = function(target, event, cb) {
    if (target["attachEvent"]) {
        target["attachEvent"]("on" + event, cb);
    } else {
        if (target["addEventListener"]) {
            target["addEventListener"](event, cb, false);
        }
    }
};
fb.login.transports.util.removeListener = function(target, event, cb) {
    if (target["detachEvent"]) {
        target["detachEvent"]("on" + event, cb);
    } else {
        if (target["removeEventListener"]) {
            target["removeEventListener"](event, cb, false);
        }
    }
};
fb.login.transports.util.extractOrigin = function(url) {
    if (!/^https?:\/\//.test(url)) {
        url = window["location"]["href"];
    }
    var m = /^(https?:\/\/[\-_a-zA-Z\.0-9:]+)/.exec(url);
    if (m) {
        return m[1];
    }
    return url;
};
fb.login.transports.util.extractRedirectCompletionHash = function(hashStr) {
    var hash = "";
    try {
        hashStr = hashStr.replace(/.*\?/, "");
        var hashObj = fb.util.querystringDecode(hashStr);
        if (hashObj && fb.util.obj.contains(hashObj, fb.login.Constants.REDIR_REQUEST_COMPLETION_KEY)) {
            hash = fb.util.obj.get(hashObj, fb.login.Constants.REDIR_REQUEST_COMPLETION_KEY);
        }
    } catch (e) {
    }
    return hash;
};
fb.login.transports.util.replaceRedirectCompletionHash = function() {
    try {
        var exp = new RegExp("&" + fb.login.Constants.REDIR_REQUEST_COMPLETION_KEY + "=([a-zA-z0-9]*)");
        document.location.hash = document.location.hash.replace(exp, "");
    } catch (e) {
    }
};
fb.login.transports.util.getBaseUrl = function() {
    var parsedUrl = fb.core.util.parseURL(fb.login.Constants.SERVER_HOST);
    return parsedUrl.scheme + "://" + parsedUrl.host + "/" + fb.login.Constants.API_VERSION;
};
fb.login.transports.util.getPopupChannelUrl = function(namespace) {
    return fb.login.transports.util.getBaseUrl() + "/" + namespace + fb.login.Constants.POPUP_PATH_TO_CHANNEL;
};
goog.provide("fb.login.Transport");
fb.login.Transport = function(options) {
};
fb.login.Transport.isAvailable = function() {
};
fb.login.Transport.prototype.open = function(url, params, onComplete) {
};
fb.login.Transport.prototype.classification = function() {
};
goog.provide("fb.login.transports.PopupReceiver");
goog.require("fb.login.Constants");
goog.require("fb.login.Transport");
goog.require("fb.login.transports.util");
goog.require("fb.login.util.environment");
goog.require("fb.util.json");
fb.login.transports.PopupReceiver = function(cb) {
    var self = this;
    this.cb = cb;
    this.targetOrigin = "*";
    if (fb.login.util.environment.isIEVersionAtLeast(8)) {
        this.messageTarget = this.inboundTarget = fb.login.transports.util.findRelay();
    } else {
        this.messageTarget = window["opener"];
        this.inboundTarget = window;
    }
    if (!self.messageTarget) {
        throw "Unable to find relay frame";
    }
    fb.login.transports.util.addListener(this.inboundTarget, "message", goog.bind(this.onMessage_, this));
    fb.login.transports.util.addListener(this.inboundTarget, "message", goog.bind(this.onDie_, this));
    try {
        this.doPost_({"a":"ready"});
    } catch (e) {
        fb.login.transports.util.addListener(this.messageTarget, "load", function(e) {
            self.doPost_({"a":"ready"});
        });
    }
    fb.login.transports.util.addListener(window, "unload", goog.bind(this.onUnload_, this));
};
fb.login.transports.PopupReceiver.prototype.doPost_ = function(msg) {
    msg = fb.util.json.stringify(msg);
    if (fb.login.util.environment.isIEVersionAtLeast(8)) {
        this.messageTarget["doPost"](msg, this.targetOrigin);
    } else {
        this.messageTarget["postMessage"](msg, this.targetOrigin);
    }
};
fb.login.transports.PopupReceiver.prototype.onMessage_ = function(e) {
    var self = this, d;
    try {
        d = fb.util.json.eval(e["data"]);
    } catch (err) {
    }
    if (!d || d["a"] !== "request") {
        return;
    }
    fb.login.transports.util.removeListener(window, "message", this.onMessage_);
    this.targetOrigin = e["origin"];
    if (this.cb) {
        setTimeout(function() {
            self.cb(self.targetOrigin, d["d"], function(response, forceKeepWindowOpen) {
                self.autoClose = !forceKeepWindowOpen;
                self.cb = undefined;
                self.doPost_({"a":"response", "d":response, "forceKeepWindowOpen":forceKeepWindowOpen});
            });
        }, 0);
    }
};
fb.login.transports.PopupReceiver.prototype.onUnload_ = function() {
    try {
        fb.login.transports.util.removeListener(this.inboundTarget, "message", this.onDie_);
    } catch (err) {
    }
    if (this.cb) {
        this.doPost_({"a":"error", "d":"unknown closed window"});
        this.cb = undefined;
    }
    try {
        window.close();
    } catch (err) {
    }
};
fb.login.transports.PopupReceiver.prototype.onDie_ = function(e) {
    if (this.autoClose && e["data"] === fb.login.Constants.POPUP_CLOSE_CMD) {
        try {
            window.close();
        } catch (err) {
        }
    }
};
goog.provide("fb.login.transports.Redirect");
goog.require("fb.constants");
goog.require("fb.core.storage");
goog.require("fb.login.Transport");
goog.require("fb.login.transports.util");
goog.require("fb.util");
fb.login.transports.Redirect = function(options) {
    this.requestId_ = goog.string.getRandomString() + goog.string.getRandomString() + goog.string.getRandomString();
    this.options_ = options;
};
fb.login.transports.Redirect.prototype.open = function(url, params, cb) {
    fb.core.storage.SessionStorage.set(fb.login.Constants.REDIR_REQUEST_ID_KEY, this.requestId_);
    fb.core.storage.SessionStorage.set(fb.login.Constants.REDIR_REQUEST_ID_KEY, this.requestId_);
    params["requestId"] = this.requestId_;
    params["redirectTo"] = params["redirectTo"] || window["location"]["href"];
    url += (/\?/.test(url) ? "" : "?") + fb.util.querystring(params);
    window["location"] = url;
};
fb.login.transports.Redirect["isAvailable"] = function() {
    return!NODE_CLIENT && !fb.login.util.environment.isLocalFile() && !fb.login.util.environment.isMobileCordova();
};
fb.login.transports.Redirect.prototype.classification = function() {
    return "redirect";
};
goog.provide("fb.login.Errors");
var errors = {"NETWORK_ERROR":"Unable to contact the Firebase server.", "SERVER_ERROR":"An unknown server error occurred.", "TRANSPORT_UNAVAILABLE":"There are no login transports available for the requested method.", "REQUEST_INTERRUPTED":"The browser redirected the page before the login request could complete.", "USER_CANCELLED":"The user cancelled authentication."};
fb.login.Errors.get = function(code) {
    var msg = fb.util.obj.get(errors, code);
    var e = new Error(msg, code);
    e["code"] = code;
    return e;
};
goog.provide("fb.login.transports.Popup");
goog.require("fb.core.util");
goog.require("fb.login.Constants");
goog.require("fb.login.Errors");
goog.require("fb.login.Transport");
goog.require("fb.login.transports.util");
goog.require("fb.login.util.environment");
goog.require("fb.util");
goog.require("fb.util.json");
fb.login.transports.Popup = function(options) {
    if (!options["window_features"] || fb.login.util.environment.isMobileFirefox()) {
        options["window_features"] = undefined;
    }
    if (!options["window_name"]) {
        options["window_name"] = "_blank";
    }
    this.options = options;
};
fb.login.transports.Popup.prototype.open = function(url, params, cb) {
    var self = this, isIE = fb.login.util.environment.isIEVersionAtLeast(8), iframe, messageTarget;
    if (!this.options["relay_url"]) {
        return cb(new Error("invalid arguments: origin of url and relay_url must match"));
    }
    var origin = fb.login.transports.util.extractOrigin(url);
    if (origin !== fb.login.transports.util.extractOrigin(self.options["relay_url"])) {
        if (cb) {
            setTimeout(function() {
                cb(new Error("invalid arguments: origin of url and relay_url must match"));
            }, 0);
        }
        return;
    }
    if (isIE) {
        iframe = document.createElement("iframe");
        iframe["setAttribute"]("src", self.options["relay_url"]);
        iframe["style"]["display"] = "none";
        iframe["setAttribute"]("name", fb.login.Constants.POPUP_RELAY_FRAME_NAME);
        document["body"]["appendChild"](iframe);
        messageTarget = iframe["contentWindow"];
    }
    url += (/\?/.test(url) ? "" : "?") + fb.util.querystring(params);
    var popup = window.open(url, self.options["window_name"], self.options["window_features"]);
    if (!messageTarget) {
        messageTarget = popup;
    }
    var closeInterval = setInterval(function() {
        if (popup && popup["closed"]) {
            cleanup(false);
            if (cb) {
                cb(fb.login.Errors.get("USER_CANCELLED"));
                cb = null;
            }
            return;
        }
    }, 500);
    var req = fb.util.json.stringify({"a":"request", "d":params});
    function cleanup(forceKeepWindowOpen) {
        if (iframe) {
            document["body"]["removeChild"](iframe);
            iframe = undefined;
        }
        if (closeInterval) {
            closeInterval = clearInterval(closeInterval);
        }
        fb.login.transports.util.removeListener(window, "message", onMessage);
        fb.login.transports.util.removeListener(window, "unload", cleanup);
        if (popup && !forceKeepWindowOpen) {
            try {
                popup["close"]();
            } catch (securityViolation) {
                messageTarget["postMessage"](fb.login.Constants.POPUP_CLOSE_CMD, origin);
            }
        }
        popup = messageTarget = undefined;
    }
    fb.login.transports.util.addListener(window, "unload", cleanup);
    function onMessage(e) {
        if (e["origin"] !== origin) {
            return;
        }
        try {
            var d = fb.util.json.eval(e["data"]);
            if (d["a"] === "ready") {
                messageTarget["postMessage"](req, origin);
            } else {
                if (d["a"] === "error") {
                    cleanup(false);
                    if (cb) {
                        cb(d["d"]);
                        cb = null;
                    }
                } else {
                    if (d["a"] === "response") {
                        cleanup(d["forceKeepWindowOpen"]);
                        if (cb) {
                            cb(null, d["d"]);
                            cb = null;
                        }
                    }
                }
            }
        } catch (err) {
        }
    }
    fb.login.transports.util.addListener(window, "message", onMessage);
};
fb.login.transports.Popup["isAvailable"] = function() {
    return!NODE_CLIENT && "postMessage" in window && !fb.login.util.environment.isLocalFile() && !fb.login.util.environment.isMobileWrapper() && !fb.login.util.environment.isHeadlessBrowser();
};
fb.login.transports.Popup.prototype.classification = function() {
    return "popup";
};
goog.provide("fb.login.transports.NodeHttp");
goog.require("fb.login.Constants");
goog.require("fb.login.Errors");
goog.require("fb.login.Transport");
goog.require("fb.login.transports.util");
goog.require("fb.login.util.environment");
goog.require("fb.util");
goog.require("fb.util.json");
fb.login.transports.NodeHttp = function(options) {
    if (!options["method"]) {
        options["method"] = "GET";
    }
    if (!options["headers"]) {
        options["headers"] = {};
    }
    if (!options["headers"]["content_type"]) {
        options["headers"]["content_type"] = "application/json";
    }
    options["headers"]["content_type"] = options["headers"]["content_type"].toLowerCase();
    this.options = options;
};
fb.login.transports.NodeHttp.prototype.open = function(url, params, cb) {
    var self = this, parsedUrl = fb.core.util.parseURL(url), http = parsedUrl.scheme === "http" ? require("http") : require("https"), method = this.options["method"], payload;
    var headers = {"Accept":"application/json;text/plain"};
    goog.object.extend(headers, this.options["headers"]);
    var requestOpts = {"host":parsedUrl.host.split(":")[0], "port":parsedUrl.port, "path":parsedUrl.pathString, "method":this.options["method"].toUpperCase()};
    if (method === "GET") {
        requestOpts["path"] += (/\?/.test(requestOpts["path"]) ? "" : "?") + fb.util.querystring(params);
        payload = null;
    } else {
        var contentType = this.options["headers"]["content_type"];
        if (contentType === "application/json") {
            payload = fb.util.json.stringify(params);
        }
        if (contentType === "application/x-www-form-urlencoded") {
            payload = fb.util.querystring(params);
        }
        headers["Content-Length"] = Buffer["byteLength"](payload, "utf8");
    }
    requestOpts["headers"] = headers;
    var req = http["request"](requestOpts, function(response) {
        var res = "";
        response["setEncoding"]("utf8");
        response["on"]("data", function(d) {
            res += d;
        });
        response["on"]("end", function() {
            try {
                res = fb.util.json.eval(res + "");
            } catch (e) {
            }
            if (cb) {
                cb(null, res);
                cb = null;
            }
        });
    });
    if (method !== "GET") {
        req["write"](payload);
    }
    req["on"]("error", function(e) {
        if (e && e["code"] && (e["code"] === "ENOTFOUND" || e["code"] === "ENETDOWN")) {
            cb(fb.login.Errors.get("NETWORK_ERROR"));
        } else {
            cb(fb.login.Errors.get("SERVER_ERROR"));
        }
        cb = null;
    });
    req["end"]();
};
fb.login.transports.NodeHttp["isAvailable"] = function() {
    return NODE_CLIENT;
};
fb.login.transports.NodeHttp.prototype.classification = function() {
    return "json";
};
goog.provide("fb.login.transports.XHR");
goog.require("fb.login.Constants");
goog.require("fb.login.Errors");
goog.require("fb.login.Transport");
goog.require("fb.login.transports.util");
goog.require("fb.login.util.environment");
goog.require("fb.util");
goog.require("fb.util.json");
fb.login.transports.XHR = function(options) {
    if (!options["method"]) {
        options["method"] = "GET";
    }
    if (!options["headers"]) {
        options["headers"] = {};
    }
    if (!options["headers"]["content_type"]) {
        options["headers"]["content_type"] = "application/json";
    }
    options["headers"]["content_type"] = options["headers"]["content_type"].toLowerCase();
    this.options = options;
};
fb.login.transports.XHR.prototype.open = function(url, params, cb) {
    var self = this;
    var xhr = new XMLHttpRequest, method = this.options["method"].toUpperCase(), payload;
    function handleInterrupt_(e) {
        if (cb) {
            cb(fb.login.Errors.get("REQUEST_INTERRUPTED"));
            cb = null;
        }
    }
    fb.login.transports.util.addListener(window, "beforeunload", handleInterrupt_);
    xhr.onreadystatechange = function() {
        if (cb && xhr.readyState === 4) {
            var res;
            if (xhr.status >= 200 && xhr.status < 300) {
                try {
                    res = fb.util.json.eval(xhr.responseText);
                } catch (e) {
                }
                cb(null, res);
            } else {
                if (xhr.status >= 500 && xhr.status < 600) {
                    cb(fb.login.Errors.get("SERVER_ERROR"));
                } else {
                    cb(fb.login.Errors.get("NETWORK_ERROR"));
                }
            }
            cb = null;
            fb.login.transports.util.removeListener(window, "beforeunload", handleInterrupt_);
        }
    };
    if (method === "GET") {
        url += (/\?/.test(url) ? "" : "?") + fb.util.querystring(params);
        payload = null;
    } else {
        var contentType = this.options["headers"]["content_type"];
        if (contentType === "application/json") {
            payload = fb.util.json.stringify(params);
        }
        if (contentType === "application/x-www-form-urlencoded") {
            payload = fb.util.querystring(params);
        }
    }
    xhr.open(method, url, true);
    var headers = {"X-Requested-With":"XMLHttpRequest", "Accept":"application/json;text/plain"};
    goog.object.extend(headers, this.options["headers"]);
    for (var header in headers) {
        xhr.setRequestHeader(header, headers[header]);
    }
    xhr.send(payload);
};
fb.login.transports.XHR["isAvailable"] = function() {
    return!NODE_CLIENT && !!window["XMLHttpRequest"] && (!fb.login.util.environment.isIE() || fb.login.util.environment.isIEVersionAtLeast(10));
};
fb.login.transports.XHR.prototype.classification = function() {
    return "json";
};
goog.provide("fb.login.transports.CordovaInAppBrowser");
goog.require("fb.core.util");
goog.require("fb.login.Constants");
goog.require("fb.login.Errors");
goog.require("fb.login.RequestInfo");
goog.require("fb.login.Transport");
goog.require("fb.login.transports.XHR");
goog.require("fb.login.transports.util");
goog.require("fb.login.util.environment");
goog.require("fb.util");
goog.require("fb.util.json");
fb.login.transports.CordovaInAppBrowser = function(options) {
    this.requestId_ = goog.string.getRandomString() + goog.string.getRandomString() + goog.string.getRandomString();
    this.options_ = options;
};
fb.login.transports.CordovaInAppBrowser.prototype.open = function(url, params, cb) {
    var self = this, parsedUrl = fb.core.util.parseURL(fb.login.Constants.SERVER_HOST), windowRef;
    function isSentinelPathMatch(url) {
        try {
            var a = document.createElement("a");
            a["href"] = url;
            return a["host"] === parsedUrl.host && a["pathname"] === fb.login.Constants.INTERNAL_REDIRECT_SENTINAL_PATH;
        } catch (e) {
        }
        return false;
    }
    function onClose_(e) {
        if (cb) {
            cb(fb.login.Errors.get("USER_CANCELLED"));
            cb = null;
        }
    }
    params["requestId"] = this.requestId_;
    params[fb.login.Constants.CLIENT_OPTION_REDIRECT_TO] = parsedUrl.scheme + "://" + parsedUrl.host + fb.login.Constants.INTERNAL_REDIRECT_SENTINAL_PATH;
    url += /\?/.test(url) ? "" : "?";
    url += fb.util.querystring(params);
    windowRef = window.open(url, "_blank", "location=no");
    if (!windowRef || !goog.isFunction(windowRef.addEventListener)) {
        cb(fb.login.Errors.get("TRANSPORT_UNAVAILABLE"));
        return;
    }
    windowRef.addEventListener("loadstart", function(e) {
        if (!e || !e["url"] || !isSentinelPathMatch(e["url"])) {
            return;
        }
        var reqKey = fb.login.transports.util.extractRedirectCompletionHash(e["url"]);
        windowRef.removeEventListener("exit", onClose_);
        windowRef.close();
        var path = "/auth/session";
        var requestInfo = new fb.login.RequestInfo(null, null, {"requestId":self.requestId_, "requestKey":reqKey});
        self.options_["requestWithCredential"](path, requestInfo, cb);
        cb = null;
    });
    windowRef.addEventListener("exit", onClose_);
};
fb.login.transports.CordovaInAppBrowser["isAvailable"] = function() {
    return!NODE_CLIENT && fb.login.util.environment.isMobileCordova();
};
fb.login.transports.CordovaInAppBrowser.prototype.classification = function() {
    return "redirect";
};
goog.provide("fb.login.transports.JSONP");
goog.require("fb.login.Constants");
goog.require("fb.login.Errors");
goog.require("fb.login.Transport");
goog.require("fb.login.transports.util");
goog.require("fb.util");
goog.require("fb.util.json");
fb.login.transports.JSONP = function(options) {
    if (!options["callback_parameter"]) {
        options["callback_parameter"] = "callback";
    }
    this.options = options;
    window[fb.login.Constants.JSONP_CALLBACK_NAMESPACE] = window[fb.login.Constants.JSONP_CALLBACK_NAMESPACE] || {};
};
fb.login.transports.JSONP.prototype.open = function(url, params, cb) {
    var id = "fn" + (new Date).getTime() + Math.floor(Math.random() * 99999);
    params[this.options["callback_parameter"]] = fb.login.Constants.JSONP_CALLBACK_NAMESPACE + "." + id;
    url += (/\?/.test(url) ? "" : "?") + fb.util.querystring(params);
    function handleInterrupt_(e) {
        if (cb) {
            cb(fb.login.Errors.get("REQUEST_INTERRUPTED"));
            cb = null;
        }
    }
    fb.login.transports.util.addListener(window, "beforeunload", handleInterrupt_);
    function cleanup_() {
        setTimeout(function() {
            window[fb.login.Constants.JSONP_CALLBACK_NAMESPACE][id] = undefined;
            if (goog.object.isEmpty(window[fb.login.Constants.JSONP_CALLBACK_NAMESPACE])) {
                window[fb.login.Constants.JSONP_CALLBACK_NAMESPACE] = undefined;
            }
            try {
                var el = document.getElementById(id);
                if (el) {
                    el.parentNode.removeChild(el);
                }
            } catch (e) {
            }
        }, 1);
        fb.login.transports.util.removeListener(window, "beforeunload", handleInterrupt_);
    }
    function onload_(res) {
        if (cb) {
            cb(null, res);
            cb = null;
        }
        cleanup_();
    }
    window[fb.login.Constants.JSONP_CALLBACK_NAMESPACE][id] = onload_;
    this.writeScriptTag_(id, url, cb);
};
fb.login.transports.JSONP.prototype.writeScriptTag_ = function(id, url, cb) {
    setTimeout(function() {
        try {
            var js = document.createElement("script");
            js.type = "text/javascript";
            js.id = id;
            js.async = true;
            js.src = url;
            js.onerror = function() {
                var el = document.getElementById(id);
                if (el !== null) {
                    el.parentNode.removeChild(el);
                }
                if (cb) {
                    cb(fb.login.Errors.get("NETWORK_ERROR"));
                }
            };
            var ref;
            var headElements = document.getElementsByTagName("head");
            if (!headElements || goog.array.isEmpty(headElements)) {
                ref = document.documentElement;
            } else {
                ref = headElements[0];
            }
            ref.appendChild(js);
        } catch (e) {
            if (cb) {
                cb(fb.login.Errors.get("NETWORK_ERROR"));
            }
        }
    }, 0);
};
fb.login.transports.JSONP["isAvailable"] = function() {
    return typeof document !== "undefined" && goog.isDefAndNotNull(document.createElement);
};
fb.login.transports.JSONP.prototype.classification = function() {
    return "json";
};
goog.provide("fb.login.AuthenticationManager");
goog.require("fb.constants");
goog.require("fb.core.storage");
goog.require("fb.core.util");
goog.require("fb.core.util.EventEmitter");
goog.require("fb.login.Constants");
goog.require("fb.login.Errors");
goog.require("fb.login.RequestInfo");
goog.require("fb.login.SessionManager");
goog.require("fb.login.transports.CordovaInAppBrowser");
goog.require("fb.login.transports.JSONP");
goog.require("fb.login.transports.NodeHttp");
goog.require("fb.login.transports.Popup");
goog.require("fb.login.transports.Redirect");
goog.require("fb.login.transports.XHR");
fb.login.AuthenticationManager = function(repoInfo, auth, unauth, onAuthStatus) {
    fb.core.util.EventEmitter.call(this, ["auth_status"]);
    this.repoInfo_ = repoInfo;
    this.authConn_ = auth;
    this.unauthConn_ = unauth;
    this.onAuthStatus_ = onAuthStatus;
    this.sessionManager_ = new fb.login.SessionManager(repoInfo, [fb.core.storage.PersistentStorage, fb.core.storage.SessionStorage]);
    this.authData_ = null;
    this.redirectRestart_ = false;
    this.resumeSession();
};
goog.inherits(fb.login.AuthenticationManager, fb.core.util.EventEmitter);
fb.login.AuthenticationManager.prototype.getAuth = function() {
    return this.authData_ || null;
};
fb.login.AuthenticationManager.prototype.resumeSession = function() {
    var redirectRequestId = fb.core.storage.SessionStorage.get(fb.login.Constants.REDIR_REQUEST_ID_KEY), self = this;
    if (redirectRequestId) {
        this.finishOAuthRedirectLogin_();
    }
    var session = this.sessionManager_.get();
    if (session && session["token"]) {
        this.updateAuthStatus_(session);
        this.authConn_(session["token"], function(status, data) {
            self.authOnComplete_(status, data, false, session["token"], session);
        }, function(cancelStatus, cancelReason) {
            self.authOnCancel_("resumeSession()", cancelStatus, cancelReason);
        });
    } else {
        this.updateAuthStatus_(null);
    }
};
fb.login.AuthenticationManager.prototype.authenticate = function(cred, userProfile, clientOptions, opt_onComplete, opt_onCancel) {
    if (this.repoInfo_.isDemoHost()) {
        fb.core.util.warn("Firebase authentication is not supported on demo Firebases (*.firebaseio-demo.com). " + "To secure your Firebase, create a production Firebase at https://www.firebase.com.");
    }
    var self = this;
    this.authConn_(cred, function(status, data) {
        self.authOnComplete_(status, data, true, cred, userProfile, clientOptions || {}, opt_onComplete);
    }, function(cancelStatus, cancelReason) {
        self.authOnCancel_("auth()", cancelStatus, cancelReason, opt_onCancel);
    });
};
fb.login.AuthenticationManager.prototype.unauthenticate = function(opt_onComplete) {
    var self = this;
    this.sessionManager_.clear();
    self.updateAuthStatus_(null);
    this.unauthConn_(function(status, errorReason) {
        if (status === "ok") {
            fb.core.util.callUserCallback(opt_onComplete, null);
        } else {
            var code = (status || "error").toUpperCase();
            var message = code;
            if (errorReason) {
                message += ": " + errorReason;
            }
            var error = new Error(message);
            error["code"] = code;
            fb.core.util.callUserCallback(opt_onComplete, error);
        }
    });
};
fb.login.AuthenticationManager.prototype.authOnComplete_ = function(status, authConnResult, newSession, cred, authData, opt_clientOptions, opt_onComplete) {
    if (status === "ok") {
        if (newSession) {
            var tokenPayload = authConnResult["auth"];
            authData["auth"] = tokenPayload;
            authData["expires"] = authConnResult["expires"];
            authData["token"] = fb.util.jwt.isValidFormat(cred) ? cred : "";
            var uid = null;
            if (tokenPayload && fb.util.obj.contains(tokenPayload, "uid")) {
                uid = fb.util.obj.get(tokenPayload, "uid");
            } else {
                if (fb.util.obj.contains(authData, "uid")) {
                    uid = fb.util.obj.get(authData, "uid");
                }
            }
            authData["uid"] = uid;
            var provider = "custom";
            if (tokenPayload && fb.util.obj.contains(tokenPayload, "provider")) {
                provider = fb.util.obj.get(tokenPayload, "provider");
            } else {
                if (fb.util.obj.contains(authData, "provider")) {
                    provider = fb.util.obj.get(authData, "provider");
                }
            }
            authData["provider"] = provider;
            this.sessionManager_.clear();
            if (fb.util.jwt.isValidFormat(cred)) {
                opt_clientOptions = opt_clientOptions || {};
                var store = fb.core.storage.PersistentStorage;
                if (opt_clientOptions[fb.login.Constants.CLIENT_OPTION_SESSION_PERSISTENCE] === "sessionOnly") {
                    store = fb.core.storage.SessionStorage;
                }
                if (opt_clientOptions[fb.login.Constants.CLIENT_OPTION_SESSION_PERSISTENCE] !== "none") {
                    this.sessionManager_.set(authData, store);
                }
            }
            this.updateAuthStatus_(authData);
        }
        fb.core.util.callUserCallback(opt_onComplete, null, authData);
        return;
    }
    this.handleBadAuthStatus_();
    var code = (status || "error").toUpperCase();
    var message = code;
    if (authConnResult) {
        message += ": " + authConnResult;
    }
    var error = new Error(message);
    error["code"] = code;
    fb.core.util.callUserCallback(opt_onComplete, error);
};
fb.login.AuthenticationManager.prototype.authOnCancel_ = function(fromFunction, cancelStatus, cancelReason, opt_onCancel) {
    fb.core.util.warn(fromFunction + " was canceled: " + cancelReason);
    this.handleBadAuthStatus_();
    var err = new Error(cancelReason);
    err["code"] = cancelStatus.toUpperCase();
    fb.core.util.callUserCallback(opt_onCancel, err);
};
fb.login.AuthenticationManager.prototype.handleBadAuthStatus_ = function() {
    this.sessionManager_.clear();
    this.updateAuthStatus_(null);
};
fb.login.AuthenticationManager.prototype.authWithCredential = function(provider, opt_params, opt_options, opt_onComplete) {
    this.checkServerSettingsOrThrow();
    var requestInfo = new fb.login.RequestInfo(opt_options || {}, {}, opt_params || {});
    var transports;
    if (NODE_CLIENT) {
        transports = [fb.login.transports.NodeHttp];
    } else {
        transports = [fb.login.transports.XHR, fb.login.transports.JSONP];
    }
    this.authWithTransports_(transports, "/auth/" + provider, requestInfo, opt_onComplete);
};
fb.login.AuthenticationManager.prototype.authWithPopup = function(provider, opt_params, opt_onComplete) {
    this.checkServerSettingsOrThrow();
    var transports = [fb.login.transports.Popup, fb.login.transports.CordovaInAppBrowser], requestInfo = fb.login.RequestInfo.fromParams(opt_params), width = 625, height = 625;
    if (provider === "anonymous" || provider === "password") {
        setTimeout(function() {
            fb.core.util.callUserCallback(opt_onComplete, fb.login.Errors.get("TRANSPORT_UNAVAILABLE"));
        }, 0);
        return;
    }
    requestInfo.transportOptions["window_features"] = "" + "menubar=yes," + "modal=yes," + "alwaysRaised=yes" + "location=yes," + "resizable=yes," + "scrollbars=yes," + "status=yes," + "height=" + height + "," + "width=" + width + "," + "top=" + (typeof screen === "object" ? (screen["height"] - height) * .5 : 0) + "," + "left=" + (typeof screen === "object" ? (screen["width"] - width) * .5 : 0);
    requestInfo.transportOptions["relay_url"] = fb.login.transports.util.getPopupChannelUrl(this.repoInfo_.namespace);
    requestInfo.transportOptions["requestWithCredential"] = goog.bind(this.requestWithCredential, this);
    this.authWithTransports_(transports, "/auth/" + provider, requestInfo, opt_onComplete);
};
fb.login.AuthenticationManager.prototype.authWithRedirect = function(provider, opt_params, opt_onErr) {
    this.checkServerSettingsOrThrow();
    var transports = [fb.login.transports.Redirect], requestInfo = fb.login.RequestInfo.fromParams(opt_params);
    if (provider === "anonymous" || provider === "firebase") {
        fb.core.util.callUserCallback(opt_onErr, fb.login.Errors.get("TRANSPORT_UNAVAILABLE"));
        return;
    }
    fb.core.storage.SessionStorage.set(fb.login.Constants.REDIR_CLIENT_OPTIONS_KEY, requestInfo.clientOptions);
    this.authWithTransports_(transports, "/auth/" + provider, requestInfo, opt_onErr);
};
fb.login.AuthenticationManager.prototype.finishOAuthRedirectLogin_ = function() {
    var redirectRequestId = fb.core.storage.SessionStorage.get(fb.login.Constants.REDIR_REQUEST_ID_KEY);
    if (redirectRequestId) {
        var clientOptions = fb.core.storage.SessionStorage.get(fb.login.Constants.REDIR_CLIENT_OPTIONS_KEY);
        fb.core.storage.SessionStorage.remove(fb.login.Constants.REDIR_REQUEST_ID_KEY);
        fb.core.storage.SessionStorage.remove(fb.login.Constants.REDIR_CLIENT_OPTIONS_KEY);
        var transports = [fb.login.transports.XHR, fb.login.transports.JSONP], serverParams = {"requestId":redirectRequestId, "requestKey":fb.login.transports.util.extractRedirectCompletionHash(document.location.hash)}, transportOptions = {}, requestInfo = new fb.login.RequestInfo(clientOptions, transportOptions, serverParams);
        this.redirectRestart_ = true;
        fb.login.transports.util.replaceRedirectCompletionHash();
        this.authWithTransports_(transports, "/auth/session", requestInfo, function(error) {
            this.redirectRestart_ = false;
        }.bind(this));
    }
};
fb.login.AuthenticationManager.prototype.createUser = function(params, opt_onComplete) {
    this.checkServerSettingsOrThrow();
    var path = "/users";
    var requestInfo = fb.login.RequestInfo.fromParams(params);
    requestInfo.serverParams["_method"] = "POST";
    this.requestWithCredential(path, requestInfo, function(err, res) {
        if (err) {
            fb.core.util.callUserCallback(opt_onComplete, err);
        } else {
            fb.core.util.callUserCallback(opt_onComplete, err, res);
        }
    });
};
fb.login.AuthenticationManager.prototype.removeUser = function(params, opt_onComplete) {
    var self = this;
    this.checkServerSettingsOrThrow();
    var path = "/users/" + encodeURIComponent(params["email"]);
    var requestInfo = fb.login.RequestInfo.fromParams(params);
    requestInfo.serverParams["_method"] = "DELETE";
    this.requestWithCredential(path, requestInfo, function(err, res) {
        if (!err && res && res["uid"]) {
            if (self.authData_ && self.authData_["uid"] && self.authData_["uid"] === res["uid"]) {
                self.unauthenticate();
            }
        }
        fb.core.util.callUserCallback(opt_onComplete, err);
    });
};
fb.login.AuthenticationManager.prototype.changePassword = function(params, opt_onComplete) {
    this.checkServerSettingsOrThrow();
    var path = "/users/" + encodeURIComponent(params["email"]) + "/password";
    var requestInfo = fb.login.RequestInfo.fromParams(params);
    requestInfo.serverParams["_method"] = "PUT";
    requestInfo.serverParams["password"] = params["newPassword"];
    this.requestWithCredential(path, requestInfo, function(err, res) {
        fb.core.util.callUserCallback(opt_onComplete, err);
    });
};
fb.login.AuthenticationManager.prototype.changeEmail = function(params, opt_onComplete) {
    this.checkServerSettingsOrThrow();
    var path = "/users/" + encodeURIComponent(params["oldEmail"]) + "/email";
    var requestInfo = fb.login.RequestInfo.fromParams(params);
    requestInfo.serverParams["_method"] = "PUT";
    requestInfo.serverParams["email"] = params["newEmail"];
    requestInfo.serverParams["password"] = params["password"];
    this.requestWithCredential(path, requestInfo, function(err, res) {
        fb.core.util.callUserCallback(opt_onComplete, err);
    });
};
fb.login.AuthenticationManager.prototype.resetPassword = function(params, opt_onComplete) {
    this.checkServerSettingsOrThrow();
    var path = "/users/" + encodeURIComponent(params["email"]) + "/password";
    var requestInfo = fb.login.RequestInfo.fromParams(params);
    requestInfo.serverParams["_method"] = "POST";
    this.requestWithCredential(path, requestInfo, function(err, res) {
        fb.core.util.callUserCallback(opt_onComplete, err);
    });
};
fb.login.AuthenticationManager.prototype.requestWithCredential = function(path, requestInfo, opt_onComplete) {
    var transports;
    if (NODE_CLIENT) {
        transports = [fb.login.transports.NodeHttp];
    } else {
        transports = [fb.login.transports.XHR, fb.login.transports.JSONP];
    }
    this.requestWithTransports_(transports, path, requestInfo, opt_onComplete);
};
fb.login.AuthenticationManager.prototype.authWithTransports_ = function(transports, path, requestInfo, opt_onComplete) {
    var self = this;
    this.requestWithTransports_(transports, path, requestInfo, function onLoginReturned(err, res) {
        if (err || !(res && res["token"] && res["uid"])) {
            fb.core.util.callUserCallback(opt_onComplete, err || fb.login.Errors.get("UNKNOWN_ERROR"));
        } else {
            res = (res);
            self.authenticate(res["token"], res, requestInfo.clientOptions, function(err, authData) {
                if (err) {
                    fb.core.util.callUserCallback(opt_onComplete, err);
                } else {
                    fb.core.util.callUserCallback(opt_onComplete, null, authData);
                }
            });
        }
    });
};
fb.login.AuthenticationManager.prototype.requestWithTransports_ = function(transports, path, requestInfo, opt_onComplete) {
    var availableTransports = goog.array.filter(transports, function(transport) {
        return typeof transport["isAvailable"] === "function" && transport["isAvailable"]();
    });
    if (availableTransports.length === 0) {
        setTimeout(function() {
            fb.core.util.callUserCallback(opt_onComplete, fb.login.Errors.get("TRANSPORT_UNAVAILABLE"));
        }, 0);
        return;
    }
    var transport = availableTransports.shift();
    var transportObj = new transport(requestInfo.transportOptions);
    var request = fb.util.obj.clone(requestInfo.serverParams);
    request["v"] = this.versionString();
    request["transport"] = transportObj.classification();
    request["suppress_status_codes"] = true;
    var url = fb.login.transports.util.getBaseUrl() + "/" + this.repoInfo_.namespace + path;
    transportObj.open(url, request, function onTransportReturned(err, res) {
        if (err) {
            fb.core.util.callUserCallback(opt_onComplete, err);
        } else {
            if (res && res["error"]) {
                var e = new Error(res["error"]["message"]);
                e["code"] = res["error"]["code"];
                e["details"] = res["error"]["details"];
                fb.core.util.callUserCallback(opt_onComplete, e);
            } else {
                fb.core.util.callUserCallback(opt_onComplete, null, res);
            }
        }
    });
};
fb.login.AuthenticationManager.prototype.updateAuthStatus_ = function(authData) {
    var stateChanged = this.authData_ !== null || authData !== null;
    this.authData_ = authData;
    if (stateChanged) {
        this.trigger("auth_status", authData);
    }
    this.onAuthStatus_(authData !== null);
};
fb.login.AuthenticationManager.prototype.getInitialEvent = function(event) {
    fb.core.util.assert(event === "auth_status", 'initial event must be of type "auth_status"');
    if (this.redirectRestart_) {
        return null;
    }
    return[this.authData_];
};
fb.login.AuthenticationManager.prototype.versionString = function() {
    return(NODE_CLIENT ? "node-" : "js-") + CLIENT_VERSION;
};
fb.login.AuthenticationManager.prototype.checkServerSettingsOrThrow = function() {
    if (this.repoInfo_.isCustomHost() && fb.login.Constants.SERVER_HOST === fb.login.Constants.DEFAULT_SERVER_HOST) {
        throw new Error("This custom Firebase server ('" + this.repoInfo_.domain + "') does not support delegated login.");
    }
};
