goog.provide("fb.realtime.Transport");
goog.require("fb.core.RepoInfo");
fb.realtime.Transport = function(connId, repoInfo, sessionId) {
};
fb.realtime.Transport.prototype.open = function(onMessage, onDisconnect) {
};
fb.realtime.Transport.prototype.start = function() {
};
fb.realtime.Transport.prototype.close = function() {
};
fb.realtime.Transport.prototype.send = function(data) {
};
fb.realtime.Transport.prototype.bytesReceived;
fb.realtime.Transport.prototype.bytesSent;
goog.provide("fb.realtime.Constants");
fb.realtime.Constants = {PROTOCOL_VERSION:"5", VERSION_PARAM:"v", TRANSPORT_SESSION_PARAM:"s", REFERER_PARAM:"r", FORGE_REF:"f", FORGE_DOMAIN:"firebaseio.com", LAST_SESSION_PARAM:"ls", WEBSOCKET:"websocket", LONG_POLLING:"long_polling"};
goog.provide("fb.realtime.polling.PacketReceiver");
fb.realtime.polling.PacketReceiver = function(onMessage) {
    this.onMessage_ = onMessage;
    this.pendingResponses = [];
    this.currentResponseNum = 0;
    this.closeAfterResponse = -1;
    this.onClose = null;
};
fb.realtime.polling.PacketReceiver.prototype.closeAfter = function(responseNum, callback) {
    this.closeAfterResponse = responseNum;
    this.onClose = callback;
    if (this.closeAfterResponse < this.currentResponseNum) {
        this.onClose();
        this.onClose = null;
    }
};
fb.realtime.polling.PacketReceiver.prototype.handleResponse = function(requestNum, data) {
    this.pendingResponses[requestNum] = data;
    while (this.pendingResponses[this.currentResponseNum]) {
        var toProcess = this.pendingResponses[this.currentResponseNum];
        delete this.pendingResponses[this.currentResponseNum];
        for (var i = 0;i < toProcess.length;++i) {
            if (toProcess[i]) {
                var self = this;
                fb.core.util.exceptionGuard(function() {
                    self.onMessage_(toProcess[i]);
                });
            }
        }
        if (this.currentResponseNum === this.closeAfterResponse) {
            if (this.onClose) {
                clearTimeout(this.onClose);
                this.onClose();
                this.onClose = null;
            }
            break;
        }
        this.currentResponseNum++;
    }
};
goog.provide("fb.realtime.BrowserPollConnection");
goog.require("fb.constants");
goog.require("fb.core.stats.StatsManager");
goog.require("fb.core.util");
goog.require("fb.core.util.CountedSet");
goog.require("fb.realtime.Constants");
goog.require("fb.realtime.Transport");
goog.require("fb.realtime.polling.PacketReceiver");
goog.require("fb.util.json");
var FIREBASE_LONGPOLL_START_PARAM = "start";
var FIREBASE_LONGPOLL_CLOSE_COMMAND = "close";
var FIREBASE_LONGPOLL_COMMAND_CB_NAME = "pLPCommand";
var FIREBASE_LONGPOLL_DATA_CB_NAME = "pRTLPCB";
var FIREBASE_LONGPOLL_ID_PARAM = "id";
var FIREBASE_LONGPOLL_PW_PARAM = "pw";
var FIREBASE_LONGPOLL_SERIAL_PARAM = "ser";
var FIREBASE_LONGPOLL_CALLBACK_ID_PARAM = "cb";
var FIREBASE_LONGPOLL_SEGMENT_NUM_PARAM = "seg";
var FIREBASE_LONGPOLL_SEGMENTS_IN_PACKET = "ts";
var FIREBASE_LONGPOLL_DATA_PARAM = "d";
var FIREBASE_LONGPOLL_DISCONN_FRAME_PARAM = "disconn";
var FIREBASE_LONGPOLL_DISCONN_FRAME_REQUEST_PARAM = "dframe";
var MAX_URL_DATA_SIZE = 1870;
var SEG_HEADER_SIZE = 30;
var MAX_PAYLOAD_SIZE = MAX_URL_DATA_SIZE - SEG_HEADER_SIZE;
var KEEPALIVE_REQUEST_INTERVAL = 25E3;
var LP_CONNECT_TIMEOUT = 3E4;
fb.realtime.BrowserPollConnection = function(connId, repoInfo, opt_transportSessionId, opt_lastSessionId) {
    this.connId = connId;
    this.log_ = fb.core.util.logWrapper(connId);
    this.repoInfo = repoInfo;
    this.bytesSent = 0;
    this.bytesReceived = 0;
    this.stats_ = fb.core.stats.StatsManager.getCollection(repoInfo);
    this.transportSessionId = opt_transportSessionId;
    this.everConnected_ = false;
    this.lastSessionId = opt_lastSessionId;
    this.urlFn = function(params) {
        return repoInfo.connectionURL(fb.realtime.Constants.LONG_POLLING, params);
    };
};
fb.realtime.BrowserPollConnection.prototype.open = function(onMessage, onDisconnect) {
    this.curSegmentNum = 0;
    this.onDisconnect_ = onDisconnect;
    this.myPacketOrderer = new fb.realtime.polling.PacketReceiver(onMessage);
    this.isClosed_ = false;
    var self = this;
    this.connectTimeoutTimer_ = setTimeout(function() {
        self.log_("Timed out trying to connect.");
        self.onClosed_();
        self.connectTimeoutTimer_ = null;
    }, Math.floor(LP_CONNECT_TIMEOUT));
    fb.core.util.executeWhenDOMReady(function() {
        if (self.isClosed_) {
            return;
        }
        self.scriptTagHolder = new FirebaseIFrameScriptHolder(function(command, arg1, arg2, arg3, arg4) {
            self.incrementIncomingBytes_(arguments);
            if (!self.scriptTagHolder) {
                return;
            }
            if (self.connectTimeoutTimer_) {
                clearTimeout(self.connectTimeoutTimer_);
                self.connectTimeoutTimer_ = null;
            }
            self.everConnected_ = true;
            if (command == FIREBASE_LONGPOLL_START_PARAM) {
                self.id = arg1;
                self.password = arg2;
            } else {
                if (command === FIREBASE_LONGPOLL_CLOSE_COMMAND) {
                    if (arg1) {
                        self.scriptTagHolder.sendNewPolls = false;
                        self.myPacketOrderer.closeAfter(arg1, function() {
                            self.onClosed_();
                        });
                    } else {
                        self.onClosed_();
                    }
                } else {
                    throw new Error("Unrecognized command received: " + command);
                }
            }
        }, function(pN, data) {
            self.incrementIncomingBytes_(arguments);
            self.myPacketOrderer.handleResponse(pN, data);
        }, function() {
            self.onClosed_();
        }, self.urlFn);
        var urlParams = {};
        urlParams[FIREBASE_LONGPOLL_START_PARAM] = "t";
        urlParams[FIREBASE_LONGPOLL_SERIAL_PARAM] = Math.floor(Math.random() * 1E8);
        if (self.scriptTagHolder.uniqueCallbackIdentifier) {
            urlParams[FIREBASE_LONGPOLL_CALLBACK_ID_PARAM] = self.scriptTagHolder.uniqueCallbackIdentifier;
        }
        urlParams[fb.realtime.Constants.VERSION_PARAM] = fb.realtime.Constants.PROTOCOL_VERSION;
        if (self.transportSessionId) {
            urlParams[fb.realtime.Constants.TRANSPORT_SESSION_PARAM] = self.transportSessionId;
        }
        if (self.lastSessionId) {
            urlParams[fb.realtime.Constants.LAST_SESSION_PARAM] = self.lastSessionId;
        }
        if (!NODE_CLIENT && typeof location !== "undefined" && location.href && location.href.indexOf(fb.realtime.Constants.FORGE_DOMAIN) !== -1) {
            urlParams[fb.realtime.Constants.REFERER_PARAM] = fb.realtime.Constants.FORGE_REF;
        }
        var connectURL = self.urlFn(urlParams);
        self.log_("Connecting via long-poll to " + connectURL);
        self.scriptTagHolder.addTag(connectURL, function() {
        });
    });
};
fb.realtime.BrowserPollConnection.prototype.start = function() {
    this.scriptTagHolder.startLongPoll(this.id, this.password);
    this.addDisconnectPingFrame(this.id, this.password);
};
fb.realtime.BrowserPollConnection.forceAllow = function() {
    fb.realtime.BrowserPollConnection.forceAllow_ = true;
};
fb.realtime.BrowserPollConnection.forceDisallow = function() {
    fb.realtime.BrowserPollConnection.forceDisallow_ = true;
};
fb.realtime.BrowserPollConnection["isAvailable"] = function() {
    return fb.realtime.BrowserPollConnection.forceAllow_ || !fb.realtime.BrowserPollConnection.forceDisallow_ && typeof document !== "undefined" && goog.isDefAndNotNull(document.createElement) && !fb.core.util.isChromeExtensionContentScript() && !fb.core.util.isWindowsStoreApp() && !NODE_CLIENT;
};
fb.realtime.BrowserPollConnection.prototype.markConnectionHealthy = function() {
};
fb.realtime.BrowserPollConnection.prototype.shutdown_ = function() {
    this.isClosed_ = true;
    if (this.scriptTagHolder) {
        this.scriptTagHolder.close();
        this.scriptTagHolder = null;
    }
    if (this.myDisconnFrame) {
        document.body.removeChild(this.myDisconnFrame);
        this.myDisconnFrame = null;
    }
    if (this.connectTimeoutTimer_) {
        clearTimeout(this.connectTimeoutTimer_);
        this.connectTimeoutTimer_ = null;
    }
};
fb.realtime.BrowserPollConnection.prototype.onClosed_ = function() {
    if (!this.isClosed_) {
        this.log_("Longpoll is closing itself");
        this.shutdown_();
        if (this.onDisconnect_) {
            this.onDisconnect_(this.everConnected_);
            this.onDisconnect_ = null;
        }
    }
};
fb.realtime.BrowserPollConnection.prototype.close = function() {
    if (!this.isClosed_) {
        this.log_("Longpoll is being closed.");
        this.shutdown_();
    }
};
fb.realtime.BrowserPollConnection.prototype.send = function(data) {
    var dataStr = fb.util.json.stringify(data);
    this.bytesSent += dataStr.length;
    this.stats_.incrementCounter("bytes_sent", dataStr.length);
    var base64data = fb.core.util.base64Encode(dataStr);
    var dataSegs = fb.core.util.splitStringBySize(base64data, MAX_PAYLOAD_SIZE);
    for (var i = 0;i < dataSegs.length;i++) {
        this.scriptTagHolder.enqueueSegment(this.curSegmentNum, dataSegs.length, dataSegs[i]);
        this.curSegmentNum++;
    }
};
fb.realtime.BrowserPollConnection.prototype.addDisconnectPingFrame = function(id, pw) {
    if (NODE_CLIENT) {
        return;
    }
    this.myDisconnFrame = document.createElement("iframe");
    var urlParams = {};
    urlParams[FIREBASE_LONGPOLL_DISCONN_FRAME_REQUEST_PARAM] = "t";
    urlParams[FIREBASE_LONGPOLL_ID_PARAM] = id;
    urlParams[FIREBASE_LONGPOLL_PW_PARAM] = pw;
    this.myDisconnFrame.src = this.urlFn(urlParams);
    this.myDisconnFrame.style.display = "none";
    document.body.appendChild(this.myDisconnFrame);
};
fb.realtime.BrowserPollConnection.prototype.incrementIncomingBytes_ = function(args) {
    var bytesReceived = fb.util.json.stringify(args).length;
    this.bytesReceived += bytesReceived;
    this.stats_.incrementCounter("bytes_received", bytesReceived);
};
function FirebaseIFrameScriptHolder(commandCB, onMessageCB, onDisconnectCB, urlFn) {
    this.urlFn = urlFn;
    this.onDisconnect = onDisconnectCB;
    this.outstandingRequests = new fb.core.util.CountedSet;
    this.pendingSegs = [];
    this.currentSerial = Math.floor(Math.random() * 1E8);
    this.sendNewPolls = true;
    if (!NODE_CLIENT) {
        this.uniqueCallbackIdentifier = fb.core.util.LUIDGenerator();
        window[FIREBASE_LONGPOLL_COMMAND_CB_NAME + this.uniqueCallbackIdentifier] = commandCB;
        window[FIREBASE_LONGPOLL_DATA_CB_NAME + this.uniqueCallbackIdentifier] = onMessageCB;
        this.myIFrame = this.createIFrame_();
        var script = "";
        if (this.myIFrame.src && this.myIFrame.src.substr(0, "javascript:".length) === "javascript:") {
            var currentDomain = document.domain;
            script = '<script>document.domain="' + currentDomain + '";\x3c/script>';
        }
        var iframeContents = "<html><body>" + script + "</body></html>";
        try {
            this.myIFrame.doc.open();
            this.myIFrame.doc.write(iframeContents);
            this.myIFrame.doc.close();
        } catch (e) {
            fb.core.util.log("frame writing exception");
            if (e.stack) {
                fb.core.util.log(e.stack);
            }
            fb.core.util.log(e);
        }
    } else {
        this.commandCB = commandCB;
        this.onMessageCB = onMessageCB;
    }
}
FirebaseIFrameScriptHolder.prototype.createIFrame_ = function() {
    var iframe = document.createElement("iframe");
    iframe.style.display = "none";
    if (document.body) {
        document.body.appendChild(iframe);
        try {
            var a = iframe.contentWindow.document;
            if (!a) {
                fb.core.util.log("No IE domain setting required");
            }
        } catch (e) {
            var domain = document.domain;
            iframe.src = "javascript:void((function(){document.open();document.domain='" + domain + "';document.close();})())";
        }
    } else {
        throw "Document body has not initialized. Wait to initialize Firebase until after the document is ready.";
    }
    if (iframe.contentDocument) {
        iframe.doc = iframe.contentDocument;
    } else {
        if (iframe.contentWindow) {
            iframe.doc = iframe.contentWindow.document;
        } else {
            if (iframe.document) {
                iframe.doc = iframe.document;
            }
        }
    }
    return iframe;
};
FirebaseIFrameScriptHolder.prototype.close = function() {
    this.alive = false;
    if (this.myIFrame) {
        this.myIFrame.doc.body.innerHTML = "";
        var self = this;
        setTimeout(function() {
            if (self.myIFrame !== null) {
                document.body.removeChild(self.myIFrame);
                self.myIFrame = null;
            }
        }, Math.floor(0));
    }
    if (NODE_CLIENT && this.myID) {
        var urlParams = {};
        urlParams[FIREBASE_LONGPOLL_DISCONN_FRAME_PARAM] = "t";
        urlParams[FIREBASE_LONGPOLL_ID_PARAM] = this.myID;
        urlParams[FIREBASE_LONGPOLL_PW_PARAM] = this.myPW;
        var theURL = this.urlFn(urlParams);
        FirebaseIFrameScriptHolder.nodeRestRequest(theURL);
    }
    var onDisconnect = this.onDisconnect;
    if (onDisconnect) {
        this.onDisconnect = null;
        onDisconnect();
    }
};
FirebaseIFrameScriptHolder.prototype.startLongPoll = function(id, pw) {
    this.myID = id;
    this.myPW = pw;
    this.alive = true;
    while (this.newRequest_()) {
    }
};
FirebaseIFrameScriptHolder.prototype.newRequest_ = function() {
    if (this.alive && this.sendNewPolls && this.outstandingRequests.count() < (this.pendingSegs.length > 0 ? 2 : 1)) {
        this.currentSerial++;
        var urlParams = {};
        urlParams[FIREBASE_LONGPOLL_ID_PARAM] = this.myID;
        urlParams[FIREBASE_LONGPOLL_PW_PARAM] = this.myPW;
        urlParams[FIREBASE_LONGPOLL_SERIAL_PARAM] = this.currentSerial;
        var theURL = this.urlFn(urlParams);
        var curDataString = "";
        var i = 0;
        while (this.pendingSegs.length > 0) {
            var nextSeg = this.pendingSegs[0];
            if (nextSeg.d.length + SEG_HEADER_SIZE + curDataString.length <= MAX_URL_DATA_SIZE) {
                var theSeg = this.pendingSegs.shift();
                curDataString = curDataString + "&" + FIREBASE_LONGPOLL_SEGMENT_NUM_PARAM + i + "=" + theSeg.seg + "&" + FIREBASE_LONGPOLL_SEGMENTS_IN_PACKET + i + "=" + theSeg.ts + "&" + FIREBASE_LONGPOLL_DATA_PARAM + i + "=" + theSeg.d;
                i++;
            } else {
                break;
            }
        }
        theURL = theURL + curDataString;
        this.addLongPollTag_(theURL, this.currentSerial);
        return true;
    } else {
        return false;
    }
};
FirebaseIFrameScriptHolder.prototype.enqueueSegment = function(segnum, totalsegs, data) {
    this.pendingSegs.push({seg:segnum, ts:totalsegs, d:data});
    if (this.alive) {
        this.newRequest_();
    }
};
FirebaseIFrameScriptHolder.prototype.addLongPollTag_ = function(url, serial) {
    var self = this;
    self.outstandingRequests.add(serial, 1);
    var doNewRequest = function() {
        self.outstandingRequests.remove(serial);
        self.newRequest_();
    };
    var keepaliveTimeout = setTimeout(doNewRequest, Math.floor(KEEPALIVE_REQUEST_INTERVAL));
    var readyStateCB = function() {
        clearTimeout(keepaliveTimeout);
        doNewRequest();
    };
    this.addTag(url, readyStateCB);
};
FirebaseIFrameScriptHolder.prototype.addTag = function(url, loadCB) {
    if (NODE_CLIENT) {
        this.doNodeLongPoll(url, loadCB);
    } else {
        var self = this;
        setTimeout(function() {
            try {
                if (!self.sendNewPolls) {
                    return;
                }
                var newScript = self.myIFrame.doc.createElement("script");
                newScript.type = "text/javascript";
                newScript.async = true;
                newScript.src = url;
                newScript.onload = newScript.onreadystatechange = function() {
                    var rstate = newScript.readyState;
                    if (!rstate || rstate === "loaded" || rstate === "complete") {
                        newScript.onload = newScript.onreadystatechange = null;
                        if (newScript.parentNode) {
                            newScript.parentNode.removeChild(newScript);
                        }
                        loadCB();
                    }
                };
                newScript.onerror = function() {
                    fb.core.util.log("Long-poll script failed to load: " + url);
                    self.sendNewPolls = false;
                    self.close();
                };
                self.myIFrame.doc.body.appendChild(newScript);
            } catch (e) {
            }
        }, Math.floor(1));
    }
};
if (typeof NODE_CLIENT !== "undefined" && NODE_CLIENT) {
    FirebaseIFrameScriptHolder.request = null;
    FirebaseIFrameScriptHolder.nodeRestRequest = function(req, onComplete) {
        if (!FirebaseIFrameScriptHolder.request) {
            FirebaseIFrameScriptHolder.request = (require("request"));
        }
        FirebaseIFrameScriptHolder.request(req, function(error, response, body) {
            if (error) {
                throw "Rest request for " + req.url + " failed.";
            }
            if (onComplete) {
                onComplete(body);
            }
        });
    };
    FirebaseIFrameScriptHolder.prototype.doNodeLongPoll = function(url, loadCB) {
        var self = this;
        FirebaseIFrameScriptHolder.nodeRestRequest({url:url, forever:true}, function(body) {
            self.evalBody(body);
            loadCB();
        });
    };
    FirebaseIFrameScriptHolder.prototype.evalBody = function(body) {
        eval("var jsonpCB = function(" + FIREBASE_LONGPOLL_COMMAND_CB_NAME + ", " + FIREBASE_LONGPOLL_DATA_CB_NAME + ") {" + body + "}");
        jsonpCB(this.commandCB, this.onMessageCB);
    };
}
;goog.provide("fb.realtime.WebSocketConnection");
goog.require("fb.constants");
goog.require("fb.core.stats.StatsManager");
goog.require("fb.core.storage");
goog.require("fb.core.util");
goog.require("fb.realtime.Constants");
goog.require("fb.realtime.Transport");
goog.require("fb.util.json");
var WEBSOCKET_MAX_FRAME_SIZE = 16384;
var WEBSOCKET_KEEPALIVE_INTERVAL = 45E3;
fb.WebSocket = null;
if (NODE_CLIENT) {
    goog.require("fb.core.util.NodePatches");
    fb.WebSocket = require("faye-websocket")["Client"];
} else {
    if (typeof MozWebSocket !== "undefined") {
        fb.WebSocket = MozWebSocket;
    } else {
        if (typeof WebSocket !== "undefined") {
            fb.WebSocket = WebSocket;
        }
    }
}
fb.realtime.WebSocketConnection = function(connId, repoInfo, opt_transportSessionId, opt_lastSessionId) {
    this.connId = connId;
    this.log_ = fb.core.util.logWrapper(this.connId);
    this.keepaliveTimer = null;
    this.frames = null;
    this.totalFrames = 0;
    this.bytesSent = 0;
    this.bytesReceived = 0;
    this.stats_ = fb.core.stats.StatsManager.getCollection(repoInfo);
    this.connURL = this.connectionURL_(repoInfo, opt_transportSessionId, opt_lastSessionId);
};
fb.realtime.WebSocketConnection.prototype.connectionURL_ = function(repoInfo, opt_transportSessionId, opt_lastSessionId) {
    var urlParams = {};
    urlParams[fb.realtime.Constants.VERSION_PARAM] = fb.realtime.Constants.PROTOCOL_VERSION;
    if (!NODE_CLIENT && typeof location !== "undefined" && location.href && location.href.indexOf(fb.realtime.Constants.FORGE_DOMAIN) !== -1) {
        urlParams[fb.realtime.Constants.REFERER_PARAM] = fb.realtime.Constants.FORGE_REF;
    }
    if (opt_transportSessionId) {
        urlParams[fb.realtime.Constants.TRANSPORT_SESSION_PARAM] = opt_transportSessionId;
    }
    if (opt_lastSessionId) {
        urlParams[fb.realtime.Constants.LAST_SESSION_PARAM] = opt_lastSessionId;
    }
    return repoInfo.connectionURL(fb.realtime.Constants.WEBSOCKET, urlParams);
};
fb.realtime.WebSocketConnection.prototype.open = function(onMess, onDisconn) {
    this.onDisconnect = onDisconn;
    this.onMessage = onMess;
    this.log_("Websocket connecting to " + this.connURL);
    this.everConnected_ = false;
    fb.core.storage.PersistentStorage.set("previous_websocket_failure", true);
    try {
        if (NODE_CLIENT) {
            var options = {"headers":{"User-Agent":"Firebase/" + fb.realtime.Constants.PROTOCOL_VERSION + "/" + CLIENT_VERSION + "/" + process.platform + "/Node"}};
            this.mySock = new fb.WebSocket(this.connURL, [], options);
        } else {
            this.mySock = new fb.WebSocket(this.connURL);
        }
    } catch (e) {
        this.log_("Error instantiating WebSocket.");
        var error = e.message || e.data;
        if (error) {
            this.log_(error);
        }
        this.onClosed_();
        return;
    }
    var self = this;
    this.mySock.onopen = function() {
        self.log_("Websocket connected.");
        self.everConnected_ = true;
    };
    this.mySock.onclose = function() {
        self.log_("Websocket connection was disconnected.");
        self.mySock = null;
        self.onClosed_();
    };
    this.mySock.onmessage = function(m) {
        self.handleIncomingFrame(m);
    };
    this.mySock.onerror = function(e) {
        self.log_("WebSocket error.  Closing connection.");
        var error = e.message || e.data;
        if (error) {
            self.log_(error);
        }
        self.onClosed_();
    };
};
fb.realtime.WebSocketConnection.prototype.start = function() {
};
fb.realtime.WebSocketConnection.forceDisallow = function() {
    fb.realtime.WebSocketConnection.forceDisallow_ = true;
};
fb.realtime.WebSocketConnection["isAvailable"] = function() {
    var isOldAndroid = false;
    if (typeof navigator !== "undefined" && navigator.userAgent) {
        var oldAndroidRegex = /Android ([0-9]{0,}\.[0-9]{0,})/;
        var oldAndroidMatch = navigator.userAgent.match(oldAndroidRegex);
        if (oldAndroidMatch && oldAndroidMatch.length > 1) {
            if (parseFloat(oldAndroidMatch[1]) < 4.4) {
                isOldAndroid = true;
            }
        }
    }
    return!isOldAndroid && fb.WebSocket !== null && !fb.realtime.WebSocketConnection.forceDisallow_;
};
fb.realtime.WebSocketConnection["responsesRequiredToBeHealthy"] = 2;
fb.realtime.WebSocketConnection["healthyTimeout"] = 3E4;
fb.realtime.WebSocketConnection.previouslyFailed = function() {
    return fb.core.storage.PersistentStorage.isInMemoryStorage || fb.core.storage.PersistentStorage.get("previous_websocket_failure") === true;
};
fb.realtime.WebSocketConnection.prototype.markConnectionHealthy = function() {
    fb.core.storage.PersistentStorage.remove("previous_websocket_failure");
};
fb.realtime.WebSocketConnection.prototype.appendFrame_ = function(data) {
    this.frames.push(data);
    if (this.frames.length == this.totalFrames) {
        var fullMess = this.frames.join("");
        this.frames = null;
        var jsonMess = fb.util.json.eval(fullMess);
        this.onMessage(jsonMess);
    }
};
fb.realtime.WebSocketConnection.prototype.handleNewFrameCount_ = function(frameCount) {
    this.totalFrames = frameCount;
    this.frames = [];
};
fb.realtime.WebSocketConnection.prototype.extractFrameCount_ = function(data) {
    fb.core.util.assert(this.frames === null, "We already have a frame buffer");
    if (data.length <= 6) {
        var frameCount = Number(data);
        if (!isNaN(frameCount)) {
            this.handleNewFrameCount_(frameCount);
            return null;
        }
    }
    this.handleNewFrameCount_(1);
    return data;
};
fb.realtime.WebSocketConnection.prototype.handleIncomingFrame = function(mess) {
    if (this.mySock === null) {
        return;
    }
    var data = mess["data"];
    this.bytesReceived += data.length;
    this.stats_.incrementCounter("bytes_received", data.length);
    this.resetKeepAlive();
    if (this.frames !== null) {
        this.appendFrame_(data);
    } else {
        var remainingData = this.extractFrameCount_(data);
        if (remainingData !== null) {
            this.appendFrame_(remainingData);
        }
    }
};
fb.realtime.WebSocketConnection.prototype.send = function(data) {
    this.resetKeepAlive();
    var dataStr = fb.util.json.stringify(data);
    this.bytesSent += dataStr.length;
    this.stats_.incrementCounter("bytes_sent", dataStr.length);
    var dataSegs = fb.core.util.splitStringBySize(dataStr, WEBSOCKET_MAX_FRAME_SIZE);
    if (dataSegs.length > 1) {
        this.sendString_(String(dataSegs.length));
    }
    for (var i = 0;i < dataSegs.length;i++) {
        this.sendString_(dataSegs[i]);
    }
};
fb.realtime.WebSocketConnection.prototype.shutdown_ = function() {
    this.isClosed_ = true;
    if (this.keepaliveTimer) {
        clearInterval(this.keepaliveTimer);
        this.keepaliveTimer = null;
    }
    if (this.mySock) {
        this.mySock.close();
        this.mySock = null;
    }
};
fb.realtime.WebSocketConnection.prototype.onClosed_ = function() {
    if (!this.isClosed_) {
        this.log_("WebSocket is closing itself");
        this.shutdown_();
        if (this.onDisconnect) {
            this.onDisconnect(this.everConnected_);
            this.onDisconnect = null;
        }
    }
};
fb.realtime.WebSocketConnection.prototype.close = function() {
    if (!this.isClosed_) {
        this.log_("WebSocket is being closed");
        this.shutdown_();
    }
};
fb.realtime.WebSocketConnection.prototype.resetKeepAlive = function() {
    var self = this;
    clearInterval(this.keepaliveTimer);
    this.keepaliveTimer = setInterval(function() {
        if (self.mySock) {
            self.sendString_("0");
        }
        self.resetKeepAlive();
    }, Math.floor(WEBSOCKET_KEEPALIVE_INTERVAL));
};
fb.realtime.WebSocketConnection.prototype.sendString_ = function(str) {
    try {
        this.mySock.send(str);
    } catch (e) {
        this.log_("Exception thrown from WebSocket.send():", e.message || e.data, "Closing connection.");
        setTimeout(goog.bind(this.onClosed_, this), 0);
    }
};
goog.require("fb.constants");
goog.require("fb.realtime.BrowserPollConnection");
goog.require("fb.realtime.Transport");
goog.provide("fb.realtime.TransportManager");
goog.require("fb.realtime.WebSocketConnection");
fb.realtime.TransportManager = function(repoInfo) {
    this.initTransports_(repoInfo);
};
fb.realtime.TransportManager.ALL_TRANSPORTS = [fb.realtime.BrowserPollConnection, fb.realtime.WebSocketConnection];
fb.realtime.TransportManager.prototype.initTransports_ = function(repoInfo) {
    var isWebSocketsAvailable = fb.realtime.WebSocketConnection && fb.realtime.WebSocketConnection["isAvailable"]();
    var isSkipPollConnection = isWebSocketsAvailable && !fb.realtime.WebSocketConnection.previouslyFailed();
    if (repoInfo.webSocketOnly) {
        if (!isWebSocketsAvailable) {
            fb.core.util.warn("wss:// URL used, but browser isn't known to support websockets.  Trying anyway.");
        }
        isSkipPollConnection = true;
    }
    if (isSkipPollConnection) {
        this.transports_ = [fb.realtime.WebSocketConnection];
    } else {
        var transports = this.transports_ = [];
        fb.core.util.each(fb.realtime.TransportManager.ALL_TRANSPORTS, function(i, transport) {
            if (transport && transport["isAvailable"]()) {
                transports.push(transport);
            }
        });
    }
};
fb.realtime.TransportManager.prototype.initialTransport = function() {
    if (this.transports_.length > 0) {
        return this.transports_[0];
    } else {
        throw new Error("No transports available");
    }
};
fb.realtime.TransportManager.prototype.upgradeTransport = function() {
    if (this.transports_.length > 1) {
        return this.transports_[1];
    } else {
        return null;
    }
};
goog.provide("fb.realtime.Connection");
goog.require("fb.core.storage");
goog.require("fb.core.util");
goog.require("fb.realtime.Constants");
goog.require("fb.realtime.TransportManager");
var UPGRADE_TIMEOUT = 6E4;
var DELAY_BEFORE_SENDING_EXTRA_REQUESTS = 5E3;
var BYTES_SENT_HEALTHY_OVERRIDE = 10 * 1024;
var BYTES_RECEIVED_HEALTHY_OVERRIDE = 100 * 1024;
var REALTIME_STATE_CONNECTING = 0;
var REALTIME_STATE_CONNECTED = 1;
var REALTIME_STATE_DISCONNECTED = 2;
var MESSAGE_TYPE = "t";
var MESSAGE_DATA = "d";
var CONTROL_SHUTDOWN = "s";
var CONTROL_RESET = "r";
var CONTROL_ERROR = "e";
var CONTROL_PONG = "o";
var SWITCH_ACK = "a";
var END_TRANSMISSION = "n";
var PING = "p";
var SERVER_HELLO = "h";
fb.realtime.Connection = function(connId, repoInfo, onMessage, onReady, onDisconnect, onKill, lastSessionId) {
    this.id = connId;
    this.log_ = fb.core.util.logWrapper("c:" + this.id + ":");
    this.onMessage_ = onMessage;
    this.onReady_ = onReady;
    this.onDisconnect_ = onDisconnect;
    this.onKill_ = onKill;
    this.repoInfo_ = repoInfo;
    this.pendingDataMessages = [];
    this.connectionCount = 0;
    this.transportManager_ = new fb.realtime.TransportManager(repoInfo);
    this.state_ = REALTIME_STATE_CONNECTING;
    this.lastSessionId = lastSessionId;
    this.log_("Connection created");
    this.start_();
};
fb.realtime.Connection.prototype.start_ = function() {
    var conn = this.transportManager_.initialTransport();
    this.conn_ = new conn(this.nextTransportId_(), this.repoInfo_, undefined, this.lastSessionId);
    this.primaryResponsesRequired_ = conn["responsesRequiredToBeHealthy"] || 0;
    var onMessageReceived = this.connReceiver_(this.conn_);
    var onConnectionLost = this.disconnReceiver_(this.conn_);
    this.tx_ = this.conn_;
    this.rx_ = this.conn_;
    this.secondaryConn_ = null;
    this.isHealthy_ = false;
    var self = this;
    setTimeout(function() {
        self.conn_ && self.conn_.open(onMessageReceived, onConnectionLost);
    }, Math.floor(0));
    var healthyTimeout_ms = conn["healthyTimeout"] || 0;
    if (healthyTimeout_ms > 0) {
        this.healthyTimeout_ = setTimeout(function() {
            self.healthyTimeout_ = null;
            if (!self.isHealthy_) {
                if (self.conn_ && self.conn_.bytesReceived > BYTES_RECEIVED_HEALTHY_OVERRIDE) {
                    self.log_("Connection exceeded healthy timeout but has received " + self.conn_.bytesReceived + " bytes.  Marking connection healthy.");
                    self.isHealthy_ = true;
                    self.conn_.markConnectionHealthy();
                } else {
                    if (self.conn_ && self.conn_.bytesSent > BYTES_SENT_HEALTHY_OVERRIDE) {
                        self.log_("Connection exceeded healthy timeout but has sent " + self.conn_.bytesSent + " bytes.  Leaving connection alive.");
                    } else {
                        self.log_("Closing unhealthy connection after timeout.");
                        self.close();
                    }
                }
            }
        }, Math.floor(healthyTimeout_ms));
    }
};
fb.realtime.Connection.prototype.nextTransportId_ = function() {
    return "c:" + this.id + ":" + this.connectionCount++;
};
fb.realtime.Connection.prototype.disconnReceiver_ = function(conn) {
    var self = this;
    return function(everConnected) {
        if (conn === self.conn_) {
            self.onConnectionLost_(everConnected);
        } else {
            if (conn === self.secondaryConn_) {
                self.log_("Secondary connection lost.");
                self.onSecondaryConnectionLost_();
            } else {
                self.log_("closing an old connection");
            }
        }
    };
};
fb.realtime.Connection.prototype.connReceiver_ = function(conn) {
    var self = this;
    return function(message) {
        if (self.state_ != REALTIME_STATE_DISCONNECTED) {
            if (conn === self.rx_) {
                self.onPrimaryMessageReceived_(message);
            } else {
                if (conn === self.secondaryConn_) {
                    self.onSecondaryMessageReceived_(message);
                } else {
                    self.log_("message on old connection");
                }
            }
        }
    };
};
fb.realtime.Connection.prototype.sendRequest = function(dataMsg) {
    var msg = {"t":"d", "d":dataMsg};
    this.sendData_(msg);
};
fb.realtime.Connection.prototype.tryCleanupConnection = function() {
    if (this.tx_ === this.secondaryConn_ && this.rx_ === this.secondaryConn_) {
        this.log_("cleaning up and promoting a connection: " + this.secondaryConn_.connId);
        this.conn_ = this.secondaryConn_;
        this.secondaryConn_ = null;
    }
};
fb.realtime.Connection.prototype.onSecondaryControl_ = function(controlData) {
    if (MESSAGE_TYPE in controlData) {
        var cmd = controlData[MESSAGE_TYPE];
        if (cmd === SWITCH_ACK) {
            this.upgradeIfSecondaryHealthy_();
        } else {
            if (cmd === CONTROL_RESET) {
                this.log_("Got a reset on secondary, closing it");
                this.secondaryConn_.close();
                if (this.tx_ === this.secondaryConn_ || this.rx_ === this.secondaryConn_) {
                    this.close();
                }
            } else {
                if (cmd === CONTROL_PONG) {
                    this.log_("got pong on secondary.");
                    this.secondaryResponsesRequired_--;
                    this.upgradeIfSecondaryHealthy_();
                }
            }
        }
    }
};
fb.realtime.Connection.prototype.onSecondaryMessageReceived_ = function(parsedData) {
    var layer = fb.core.util.requireKey("t", parsedData);
    var data = fb.core.util.requireKey("d", parsedData);
    if (layer == "c") {
        this.onSecondaryControl_(data);
    } else {
        if (layer == "d") {
            this.pendingDataMessages.push(data);
        } else {
            throw new Error("Unknown protocol layer: " + layer);
        }
    }
};
fb.realtime.Connection.prototype.upgradeIfSecondaryHealthy_ = function() {
    if (this.secondaryResponsesRequired_ <= 0) {
        this.log_("Secondary connection is healthy.");
        this.isHealthy_ = true;
        this.secondaryConn_.markConnectionHealthy();
        this.proceedWithUpgrade_();
    } else {
        this.log_("sending ping on secondary.");
        this.secondaryConn_.send({"t":"c", "d":{"t":PING, "d":{}}});
    }
};
fb.realtime.Connection.prototype.proceedWithUpgrade_ = function() {
    this.secondaryConn_.start();
    this.log_("sending client ack on secondary");
    this.secondaryConn_.send({"t":"c", "d":{"t":SWITCH_ACK, "d":{}}});
    this.log_("Ending transmission on primary");
    this.conn_.send({"t":"c", "d":{"t":END_TRANSMISSION, "d":{}}});
    this.tx_ = this.secondaryConn_;
    this.tryCleanupConnection();
};
fb.realtime.Connection.prototype.onPrimaryMessageReceived_ = function(parsedData) {
    var layer = fb.core.util.requireKey("t", parsedData);
    var data = fb.core.util.requireKey("d", parsedData);
    if (layer == "c") {
        this.onControl_(data);
    } else {
        if (layer == "d") {
            this.onDataMessage_(data);
        }
    }
};
fb.realtime.Connection.prototype.onDataMessage_ = function(message) {
    this.onPrimaryResponse_();
    this.onMessage_(message);
};
fb.realtime.Connection.prototype.onPrimaryResponse_ = function() {
    if (!this.isHealthy_) {
        this.primaryResponsesRequired_--;
        if (this.primaryResponsesRequired_ <= 0) {
            this.log_("Primary connection is healthy.");
            this.isHealthy_ = true;
            this.conn_.markConnectionHealthy();
        }
    }
};
fb.realtime.Connection.prototype.onControl_ = function(controlData) {
    var cmd = fb.core.util.requireKey(MESSAGE_TYPE, controlData);
    if (MESSAGE_DATA in controlData) {
        var payload = controlData[MESSAGE_DATA];
        if (cmd === SERVER_HELLO) {
            this.onHandshake_(payload);
        } else {
            if (cmd === END_TRANSMISSION) {
                this.log_("recvd end transmission on primary");
                this.rx_ = this.secondaryConn_;
                for (var i = 0;i < this.pendingDataMessages.length;++i) {
                    this.onDataMessage_(this.pendingDataMessages[i]);
                }
                this.pendingDataMessages = [];
                this.tryCleanupConnection();
            } else {
                if (cmd === CONTROL_SHUTDOWN) {
                    this.onConnectionShutdown_(payload);
                } else {
                    if (cmd === CONTROL_RESET) {
                        this.onReset_(payload);
                    } else {
                        if (cmd === CONTROL_ERROR) {
                            fb.core.util.error("Server Error: " + payload);
                        } else {
                            if (cmd === CONTROL_PONG) {
                                this.log_("got pong on primary.");
                                this.onPrimaryResponse_();
                                this.sendPingOnPrimaryIfNecessary_();
                            } else {
                                fb.core.util.error("Unknown control packet command: " + cmd);
                            }
                        }
                    }
                }
            }
        }
    }
};
fb.realtime.Connection.prototype.onHandshake_ = function(handshake) {
    var timestamp = handshake["ts"];
    var version = handshake["v"];
    var host = handshake["h"];
    this.sessionId = handshake["s"];
    this.repoInfo_.updateHost(host);
    if (this.state_ == REALTIME_STATE_CONNECTING) {
        this.conn_.start();
        this.onConnectionEstablished_(this.conn_, timestamp);
        if (fb.realtime.Constants.PROTOCOL_VERSION !== version) {
            fb.core.util.warn("Protocol version mismatch detected");
        }
        this.tryStartUpgrade_();
    }
};
fb.realtime.Connection.prototype.tryStartUpgrade_ = function() {
    var conn = this.transportManager_.upgradeTransport();
    if (conn) {
        this.startUpgrade_(conn);
    }
};
fb.realtime.Connection.prototype.startUpgrade_ = function(conn) {
    this.secondaryConn_ = new conn(this.nextTransportId_(), this.repoInfo_, this.sessionId);
    this.secondaryResponsesRequired_ = conn["responsesRequiredToBeHealthy"] || 0;
    var onMessage = this.connReceiver_(this.secondaryConn_);
    var onDisconnect = this.disconnReceiver_(this.secondaryConn_);
    this.secondaryConn_.open(onMessage, onDisconnect);
    var self = this;
    setTimeout(function() {
        if (self.secondaryConn_) {
            self.log_("Timed out trying to upgrade.");
            self.secondaryConn_.close();
        }
    }, Math.floor(UPGRADE_TIMEOUT));
};
fb.realtime.Connection.prototype.onReset_ = function(host) {
    this.log_("Reset packet received.  New host: " + host);
    this.repoInfo_.updateHost(host);
    if (this.state_ === REALTIME_STATE_CONNECTED) {
        this.close();
    } else {
        this.closeConnections_();
        this.start_();
    }
};
fb.realtime.Connection.prototype.onConnectionEstablished_ = function(conn, timestamp) {
    this.log_("Realtime connection established.");
    this.conn_ = conn;
    this.state_ = REALTIME_STATE_CONNECTED;
    if (this.onReady_) {
        this.onReady_(timestamp, this.sessionId);
        this.onReady_ = null;
    }
    var self = this;
    if (this.primaryResponsesRequired_ === 0) {
        this.log_("Primary connection is healthy.");
        this.isHealthy_ = true;
    } else {
        setTimeout(function() {
            self.sendPingOnPrimaryIfNecessary_();
        }, Math.floor(DELAY_BEFORE_SENDING_EXTRA_REQUESTS));
    }
};
fb.realtime.Connection.prototype.sendPingOnPrimaryIfNecessary_ = function() {
    if (!this.isHealthy_ && this.state_ === REALTIME_STATE_CONNECTED) {
        this.log_("sending ping on primary.");
        this.sendData_({"t":"c", "d":{"t":PING, "d":{}}});
    }
};
fb.realtime.Connection.prototype.onSecondaryConnectionLost_ = function() {
    var conn = this.secondaryConn_;
    this.secondaryConn_ = null;
    if (this.tx_ === conn || this.rx_ === conn) {
        this.close();
    }
};
fb.realtime.Connection.prototype.onConnectionLost_ = function(everConnected) {
    this.conn_ = null;
    if (!everConnected && this.state_ === REALTIME_STATE_CONNECTING) {
        this.log_("Realtime connection failed.");
        if (this.repoInfo_.isCacheableHost()) {
            fb.core.storage.PersistentStorage.remove("host:" + this.repoInfo_.host);
            this.repoInfo_.internalHost = this.repoInfo_.host;
        }
    } else {
        if (this.state_ === REALTIME_STATE_CONNECTED) {
            this.log_("Realtime connection lost.");
        }
    }
    this.close();
};
fb.realtime.Connection.prototype.onConnectionShutdown_ = function(reason) {
    this.log_("Connection shutdown command received. Shutting down...");
    if (this.onKill_) {
        this.onKill_(reason);
        this.onKill_ = null;
    }
    this.onDisconnect_ = null;
    this.close();
};
fb.realtime.Connection.prototype.sendData_ = function(data) {
    if (this.state_ !== REALTIME_STATE_CONNECTED) {
        throw "Connection is not connected";
    } else {
        this.tx_.send(data);
    }
};
fb.realtime.Connection.prototype.close = function() {
    if (this.state_ !== REALTIME_STATE_DISCONNECTED) {
        this.log_("Closing realtime connection.");
        this.state_ = REALTIME_STATE_DISCONNECTED;
        this.closeConnections_();
        if (this.onDisconnect_) {
            this.onDisconnect_();
            this.onDisconnect_ = null;
        }
    }
};
fb.realtime.Connection.prototype.closeConnections_ = function() {
    this.log_("Shutting down all connections");
    if (this.conn_) {
        this.conn_.close();
        this.conn_ = null;
    }
    if (this.secondaryConn_) {
        this.secondaryConn_.close();
        this.secondaryConn_ = null;
    }
    if (this.healthyTimeout_) {
        clearTimeout(this.healthyTimeout_);
        this.healthyTimeout_ = null;
    }
};
