goog.provide("fb.util.obj");
fb.util.obj.contains = function(obj, key) {
    return Object.prototype.hasOwnProperty.call(obj, key);
};
fb.util.obj.get = function(obj, key) {
    if (Object.prototype.hasOwnProperty.call(obj, key)) {
        return obj[key];
    }
};
fb.util.obj.foreach = function(obj, fn) {
    for (var key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            fn(key, obj[key]);
        }
    }
};
fb.util.obj.clone = function(obj) {
    var clone = {};
    fb.util.obj.foreach(obj, function(key, value) {
        clone[key] = value;
    });
    return clone;
};
fb.util.obj.isNonNullObject = function(obj) {
    return typeof obj === "object" && obj !== null;
};
goog.provide("fb.util");
goog.require("fb.util.obj");
fb.util.querystring = function(querystringParams) {
    var params = [];
    fb.util.obj.foreach(querystringParams, function(key, value) {
        if (goog.isArray(value)) {
            goog.array.forEach(value, function(arrayVal) {
                params.push(encodeURIComponent(key) + "=" + encodeURIComponent(arrayVal));
            });
        } else {
            params.push(encodeURIComponent(key) + "=" + encodeURIComponent(value));
        }
    });
    return params.length ? "&" + params.join("&") : "";
};
fb.util.querystringDecode = function(querystring) {
    var obj = {};
    var tokens = querystring.replace(/^\?/, "").split("&");
    goog.array.forEach(tokens, function(token) {
        if (token) {
            var key = token.split("=");
            obj[key[0]] = key[1];
        }
    });
    return obj;
};
goog.provide("fb.util.assert");
goog.provide("fb.util.assertionError");
goog.require("fb.constants");
fb.util.assert = function(assertion, message) {
    if (!assertion) {
        throw fb.util.assertionError(message);
    }
};
fb.util.assertionError = function(message) {
    return new Error("Firebase (" + fb.constants.CLIENT_VERSION + ") INTERNAL ASSERT FAILED: " + message);
};
goog.provide("fb.util.promise");
goog.provide("fb.util.promise.Deferred");
goog.provide("fb.util.promise.Promise");
goog.require("goog.Promise");
goog.scope(function() {
    fb.util.promise.Promise = goog.global.Promise || goog.Promise;
    fb.util.promise.Deferred = goog.defineClass(null, {constructor:function() {
        var self = this;
        this.resolve = null;
        this.reject = null;
        this.promise = new fb.util.promise.Promise(function(resolve, reject) {
            self.resolve = resolve;
            self.reject = reject;
        });
    }, wrapCallback:function(opt_nodeCallback) {
        var self = this;
        function meta(error, opt_value) {
            if (error) {
                self.reject(error);
            } else {
                self.resolve(opt_value);
            }
            if (goog.isFunction(opt_nodeCallback)) {
                fb.util.promise.attachDummyErrorHandler(self.promise);
                if (opt_nodeCallback.length === 1) {
                    opt_nodeCallback(error);
                } else {
                    opt_nodeCallback(error, opt_value);
                }
            }
        }
        return meta;
    }});
    fb.util.promise.attachDummyErrorHandler = function(promise) {
        promise.then(void 0, goog.nullFunction);
    };
});
goog.provide("fb.util.utf8");
goog.require("fb.util.assert");
fb.util.utf8.stringToByteArray = function(str) {
    var out = [], p = 0;
    for (var i = 0;i < str.length;i++) {
        var c = str.charCodeAt(i);
        if (c >= 55296 && c <= 56319) {
            var high = c - 55296;
            i++;
            fb.util.assert(i < str.length, "Surrogate pair missing trail surrogate.");
            var low = str.charCodeAt(i) - 56320;
            c = 65536 + (high << 10) + low;
        }
        if (c < 128) {
            out[p++] = c;
        } else {
            if (c < 2048) {
                out[p++] = c >> 6 | 192;
                out[p++] = c & 63 | 128;
            } else {
                if (c < 65536) {
                    out[p++] = c >> 12 | 224;
                    out[p++] = c >> 6 & 63 | 128;
                    out[p++] = c & 63 | 128;
                } else {
                    out[p++] = c >> 18 | 240;
                    out[p++] = c >> 12 & 63 | 128;
                    out[p++] = c >> 6 & 63 | 128;
                    out[p++] = c & 63 | 128;
                }
            }
        }
    }
    return out;
};
fb.util.utf8.stringLength = function(str) {
    var p = 0;
    for (var i = 0;i < str.length;i++) {
        var c = str.charCodeAt(i);
        if (c < 128) {
            p++;
        } else {
            if (c < 2048) {
                p += 2;
            } else {
                if (c >= 55296 && c <= 56319) {
                    p += 4;
                    i++;
                } else {
                    p += 3;
                }
            }
        }
    }
    return p;
};
goog.provide("fb.util.validation");
fb.util.validation.validateArgCount = function(fnName, minCount, maxCount, argCount) {
    var argError;
    if (argCount < minCount) {
        argError = "at least " + minCount;
    } else {
        if (argCount > maxCount) {
            argError = maxCount === 0 ? "none" : "no more than " + maxCount;
        }
    }
    if (argError) {
        var error = fnName + " failed: Was called with " + argCount + (argCount === 1 ? " argument." : " arguments.") + " Expects " + argError + ".";
        throw new Error(error);
    }
};
fb.util.validation.errorPrefix = function(fnName, argumentNumber, optional) {
    var argName = "";
    switch(argumentNumber) {
        case 1:
            argName = optional ? "first" : "First";
            break;
        case 2:
            argName = optional ? "second" : "Second";
            break;
        case 3:
            argName = optional ? "third" : "Third";
            break;
        case 4:
            argName = optional ? "fourth" : "Fourth";
            break;
        default:
            throw new Error("errorPrefix called with argumentNumber > 4.  Need to update it?");;
    }
    var error = fnName + " failed: ";
    error += argName + " argument ";
    return error;
};
fb.util.validation.validateNamespace = function(fnName, argumentNumber, namespace, optional) {
    if (optional && !goog.isDef(namespace)) {
        return;
    }
    if (!goog.isString(namespace)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "must be a valid firebase namespace.");
    }
};
fb.util.validation.validateCallback = function(fnName, argumentNumber, callback, optional) {
    if (optional && !goog.isDef(callback)) {
        return;
    }
    if (!goog.isFunction(callback)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "must be a valid function.");
    }
};
fb.util.validation.validateContextObject = function(fnName, argumentNumber, context, optional) {
    if (optional && !goog.isDef(context)) {
        return;
    }
    if (!goog.isObject(context) || context === null) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "must be a valid context object.");
    }
};
goog.provide("fb.util.json");
goog.require("goog.json");
fb.util.json.eval = function(str) {
    if (typeof JSON !== "undefined" && goog.isDef(JSON.parse)) {
        return JSON.parse(str);
    } else {
        return goog.json.parse(str);
    }
};
fb.util.json.stringify = function(data) {
    if (typeof JSON !== "undefined" && goog.isDef(JSON.stringify)) {
        return JSON.stringify(data);
    } else {
        return goog.json.serialize(data);
    }
};
