
import 'query.dart';
import 'snap.dart';
import 'tree.dart';
import 'sync.dart';

import 'dart:async';

abstract class Backend {

  Stream<Change> changes(Query query);

  Future<Node> put(Path<Name> path, Node data);

}


class MemoryBackend extends Backend {

  static final Map<String,MemoryBackend> _instances = {};

  Node _node = Node.nil;

  Map<Query,StreamController<Change>> _controllers = {};

  MemoryBackend._();

  factory MemoryBackend(String name) {
    return _instances.putIfAbsent(name, ()=>new MemoryBackend._());
  }


  @override
  Stream<Change> changes(Query query) {
    if (!query.queryParams.loadsAllData)
      throw new UnimplementedError();

    if (_controllers.containsKey(query))
      throw new StateError("Already listener registered for query $query.");

    var controller = _controllers[query] = new StreamController();

    controller.add(new Change.value(_node.subtree(query.path)));

    return controller.stream;
  }

  @override
  Future<Node> put(Path<Name> path, Node data) async {
    _node = _node.updateSubtree(path, data);
    _controllers.forEach((query, controller) {
      if (path.startsWith(query.path)) {
        controller.add(new Change.value(_node.subtree(query.path)));
      }
    });
    return data;
  }

}


