
import 'dart:collection';

class Pointer<T> {
  final T value;
  final Function callback;
  bool _released = false;
  Pointer(this.value, [this.callback]) {
    _referenceCounts[value] = (_referenceCounts[value]??0)+1;
    print("create pointer to $value (count=${_referenceCounts[value]})");
  }

  static final Map<Object,int> _referenceCounts = {};

  void release() {
    if (_released)
      throw new StateError("Pointer already released");
    var count = _referenceCounts[value]--;
    print("release pointer to $value (count=$count)");
    if (count==0) {
      _referenceCounts.remove(value);
      if (callback!=null) {
        callback();
      }
    }
    _released = true;
  }
}

class PointerMap<K,V> {

  final Map<K,V> _values = {};

  Pointer<V> operator [](Object key) =>
      _values.containsKey(key) ?
      new Pointer(_values[key], ()=>_values.remove(key)) : null;

  Pointer<V> putIfAbsent(K key, V ifAbsent()) =>
    new Pointer(_values.putIfAbsent(key,ifAbsent), ()=>_values.remove(key));

}