goog.provide("fb.core.SnapshotHolder");
fb.core.SnapshotHolder = function() {
    this.rootNode_ = fb.core.snap.EMPTY_NODE;
};
fb.core.SnapshotHolder.prototype.getNode = function(path) {
    return this.rootNode_.getChild(path);
};
fb.core.SnapshotHolder.prototype.updateSnapshot = function(path, newSnapshotNode) {
    this.rootNode_ = this.rootNode_.updateChild(path, newSnapshotNode);
};
if (goog.DEBUG) {
    fb.core.SnapshotHolder.prototype.toString = function() {
        return this.rootNode_.toString();
    };
}
;goog.provide("fb.core.view.CompleteChildSource");
fb.core.view.CompleteChildSource = function() {
};
fb.core.view.CompleteChildSource.prototype.getCompleteChild = function(childKey) {
};
fb.core.view.CompleteChildSource.prototype.getChildAfterChild = function(index, child, reverse) {
};
fb.core.view.NoCompleteChildSource_ = function() {
};
fb.core.view.NoCompleteChildSource_.prototype.getCompleteChild = function() {
    return null;
};
fb.core.view.NoCompleteChildSource_.prototype.getChildAfterChild = function() {
    return null;
};
fb.core.view.NO_COMPLETE_CHILD_SOURCE = new fb.core.view.NoCompleteChildSource_;
fb.core.view.WriteTreeCompleteChildSource = function(writes, viewCache, optCompleteServerCache) {
    this.writes_ = writes;
    this.viewCache_ = viewCache;
    this.optCompleteServerCache_ = optCompleteServerCache;
};
fb.core.view.WriteTreeCompleteChildSource.prototype.getCompleteChild = function(childKey) {
    var node = this.viewCache_.getEventCache();
    if (node.isCompleteForChild(childKey)) {
        return node.getNode().getImmediateChild(childKey);
    } else {
        var serverNode = this.optCompleteServerCache_ != null ? new fb.core.view.CacheNode(this.optCompleteServerCache_, true, false) : this.viewCache_.getServerCache();
        return this.writes_.calcCompleteChild(childKey, serverNode);
    }
};
fb.core.view.WriteTreeCompleteChildSource.prototype.getChildAfterChild = function(index, child, reverse) {
    var completeServerData = this.optCompleteServerCache_ != null ? this.optCompleteServerCache_ : this.viewCache_.getCompleteServerSnap();
    var nodes = this.writes_.calcIndexedSlice(completeServerData, child, 1, reverse, index);
    if (nodes.length === 0) {
        return null;
    } else {
        return nodes[0];
    }
};
goog.provide("fb.core.view.EventQueue");
fb.core.view.EventQueue = function() {
    this.eventLists_ = [];
    this.recursionDepth_ = 0;
};
fb.core.view.EventQueue.prototype.queueEvents = function(eventDataList) {
    var currList = null;
    for (var i = 0;i < eventDataList.length;i++) {
        var eventData = eventDataList[i];
        var eventPath = eventData.getPath();
        if (currList !== null && !eventPath.equals(currList.getPath())) {
            this.eventLists_.push(currList);
            currList = null;
        }
        if (currList === null) {
            currList = new fb.core.view.EventList(eventPath);
        }
        currList.add(eventData);
    }
    if (currList) {
        this.eventLists_.push(currList);
    }
};
fb.core.view.EventQueue.prototype.raiseEventsAtPath = function(path, eventDataList) {
    this.queueEvents(eventDataList);
    this.raiseQueuedEventsMatchingPredicate_(function(eventPath) {
        return eventPath.equals(path);
    });
};
fb.core.view.EventQueue.prototype.raiseEventsForChangedPath = function(changedPath, eventDataList) {
    this.queueEvents(eventDataList);
    this.raiseQueuedEventsMatchingPredicate_(function(eventPath) {
        return eventPath.contains(changedPath) || changedPath.contains(eventPath);
    });
};
fb.core.view.EventQueue.prototype.raiseQueuedEventsMatchingPredicate_ = function(predicate) {
    this.recursionDepth_++;
    var sentAll = true;
    for (var i = 0;i < this.eventLists_.length;i++) {
        var eventList = this.eventLists_[i];
        if (eventList) {
            var eventPath = eventList.getPath();
            if (predicate(eventPath)) {
                this.eventLists_[i].raise();
                this.eventLists_[i] = null;
            } else {
                sentAll = false;
            }
        }
    }
    if (sentAll) {
        this.eventLists_ = [];
    }
    this.recursionDepth_--;
};
fb.core.view.EventList = function(path) {
    this.path_ = path;
    this.events_ = [];
};
fb.core.view.EventList.prototype.add = function(eventData) {
    this.events_.push(eventData);
};
fb.core.view.EventList.prototype.raise = function() {
    for (var i = 0;i < this.events_.length;i++) {
        var eventData = this.events_[i];
        if (eventData !== null) {
            this.events_[i] = null;
            var eventFn = eventData.getEventRunner();
            if (fb.core.util.logger) {
                fb.core.util.log("event: " + eventData.toString());
            }
            fb.core.util.exceptionGuard(eventFn);
        }
    }
};
fb.core.view.EventList.prototype.getPath = function() {
    return this.path_;
};
goog.provide("fb.core.view.Change");
goog.provide("fb.core.view.Change.CHILD_ADDED");
goog.provide("fb.core.view.Change.CHILD_CHANGED");
goog.provide("fb.core.view.Change.CHILD_MOVED");
goog.provide("fb.core.view.Change.CHILD_REMOVED");
goog.provide("fb.core.view.Change.VALUE");
fb.core.view.Change = function(type, snapshotNode, opt_childName, opt_oldSnap, opt_prevName) {
    this.type = type;
    this.snapshotNode = snapshotNode;
    this.childName = opt_childName;
    this.oldSnap = opt_oldSnap;
    this.prevName = opt_prevName;
};
fb.core.view.Change.valueChange = function(snapshot) {
    return new fb.core.view.Change(fb.core.view.Change.VALUE, snapshot);
};
fb.core.view.Change.childAddedChange = function(childKey, snapshot) {
    return new fb.core.view.Change(fb.core.view.Change.CHILD_ADDED, snapshot, childKey);
};
fb.core.view.Change.childRemovedChange = function(childKey, snapshot) {
    return new fb.core.view.Change(fb.core.view.Change.CHILD_REMOVED, snapshot, childKey);
};
fb.core.view.Change.childChangedChange = function(childKey, newSnapshot, oldSnapshot) {
    return new fb.core.view.Change(fb.core.view.Change.CHILD_CHANGED, newSnapshot, childKey, oldSnapshot);
};
fb.core.view.Change.childMovedChange = function(childKey, snapshot) {
    return new fb.core.view.Change(fb.core.view.Change.CHILD_MOVED, snapshot, childKey);
};
fb.core.view.Change.prototype.changeWithPrevName = function(prevName) {
    return new fb.core.view.Change(this.type, this.snapshotNode, this.childName, this.oldSnap, prevName);
};
fb.core.view.Change.CHILD_ADDED = "child_added";
fb.core.view.Change.CHILD_REMOVED = "child_removed";
fb.core.view.Change.CHILD_CHANGED = "child_changed";
fb.core.view.Change.CHILD_MOVED = "child_moved";
fb.core.view.Change.VALUE = "value";
goog.provide("fb.core.view.CancelEvent");
goog.provide("fb.core.view.DataEvent");
goog.provide("fb.core.view.Event");
goog.require("fb.util.json");
fb.core.view.Event = function() {
};
fb.core.view.Event.prototype.getPath = goog.abstractMethod;
fb.core.view.Event.prototype.getEventType = goog.abstractMethod;
fb.core.view.Event.prototype.getEventRunner = goog.abstractMethod;
fb.core.view.Event.prototype.toString = goog.abstractMethod;
fb.core.view.DataEvent = function(eventType, eventRegistration, snapshot, opt_prevName) {
    this.eventRegistration = eventRegistration;
    this.snapshot = snapshot;
    this.prevName = opt_prevName;
    this.eventType = eventType;
};
fb.core.view.DataEvent.prototype.getPath = function() {
    var ref = this.snapshot.ref();
    if (this.eventType === "value") {
        return ref.path;
    } else {
        return ref.parent().path;
    }
};
fb.core.view.DataEvent.prototype.getEventType = function() {
    return this.eventType;
};
fb.core.view.DataEvent.prototype.getEventRunner = function() {
    return this.eventRegistration.getEventRunner(this);
};
fb.core.view.DataEvent.prototype.toString = function() {
    return this.getPath().toString() + ":" + this.eventType + ":" + fb.util.json.stringify(this.snapshot.exportVal());
};
fb.core.view.CancelEvent = function(eventRegistration, error, path) {
    this.eventRegistration = eventRegistration;
    this.error = error;
    this.path = path;
};
fb.core.view.CancelEvent.prototype.getPath = function() {
    return this.path;
};
fb.core.view.CancelEvent.prototype.getEventType = function() {
    return "cancel";
};
fb.core.view.CancelEvent.prototype.getEventRunner = function() {
    return this.eventRegistration.getEventRunner(this);
};
fb.core.view.CancelEvent.prototype.toString = function() {
    return this.path.toString() + ":cancel";
};
goog.provide("fb.core.view.CacheNode");
fb.core.view.CacheNode = function(node, fullyInitialized, filtered) {
    this.node_ = node;
    this.fullyInitialized_ = fullyInitialized;
    this.filtered_ = filtered;
};
fb.core.view.CacheNode.prototype.isFullyInitialized = function() {
    return this.fullyInitialized_;
};
fb.core.view.CacheNode.prototype.isFiltered = function() {
    return this.filtered_;
};
fb.core.view.CacheNode.prototype.isCompleteForPath = function(path) {
    if (path.isEmpty()) {
        return this.isFullyInitialized() && !this.filtered_;
    } else {
        var childKey = path.getFront();
        return this.isCompleteForChild(childKey);
    }
};
fb.core.view.CacheNode.prototype.isCompleteForChild = function(key) {
    return this.isFullyInitialized() && !this.filtered_ || this.node_.hasChild(key);
};
fb.core.view.CacheNode.prototype.getNode = function() {
    return this.node_;
};
goog.provide("fb.core.stats.StatsListener");
fb.core.stats.StatsListener = function(collection) {
    this.collection_ = collection;
    this.last_ = null;
};
fb.core.stats.StatsListener.prototype.get = function() {
    var newStats = this.collection_.get();
    var delta = goog.object.clone(newStats);
    if (this.last_) {
        for (var stat in this.last_) {
            delta[stat] = delta[stat] - this.last_[stat];
        }
    }
    this.last_ = newStats;
    return delta;
};
goog.provide("fb.core.stats.StatsReporter");
var FIRST_STATS_MIN_TIME = 10 * 1E3;
var FIRST_STATS_MAX_TIME = 30 * 1E3;
var REPORT_STATS_INTERVAL = 5 * 60 * 1E3;
fb.core.stats.StatsReporter = function(collection, connection) {
    this.statsToReport_ = {};
    this.statsListener_ = new fb.core.stats.StatsListener(collection);
    this.server_ = connection;
    var timeout = FIRST_STATS_MIN_TIME + (FIRST_STATS_MAX_TIME - FIRST_STATS_MIN_TIME) * Math.random();
    setTimeout(goog.bind(this.reportStats_, this), Math.floor(timeout));
};
fb.core.stats.StatsReporter.prototype.includeStat = function(stat) {
    this.statsToReport_[stat] = true;
};
fb.core.stats.StatsReporter.prototype.reportStats_ = function() {
    var stats = this.statsListener_.get();
    var reportedStats = {};
    var haveStatsToReport = false;
    for (var stat in stats) {
        if (stats[stat] > 0 && fb.util.obj.contains(this.statsToReport_, stat)) {
            reportedStats[stat] = stats[stat];
            haveStatsToReport = true;
        }
    }
    if (haveStatsToReport) {
        this.server_.reportStats(reportedStats);
    }
    setTimeout(goog.bind(this.reportStats_, this), Math.floor(Math.random() * 2 * REPORT_STATS_INTERVAL));
};
goog.provide("fb.core.stats.StatsCollection");
goog.require("fb.util.obj");
goog.require("goog.array");
goog.require("goog.object");
fb.core.stats.StatsCollection = function() {
    this.counters_ = {};
};
fb.core.stats.StatsCollection.prototype.incrementCounter = function(name, amount) {
    if (!goog.isDef(amount)) {
        amount = 1;
    }
    if (!fb.util.obj.contains(this.counters_, name)) {
        this.counters_[name] = 0;
    }
    this.counters_[name] += amount;
};
fb.core.stats.StatsCollection.prototype.get = function() {
    return goog.object.clone(this.counters_);
};
goog.provide("fb.core.stats.StatsManager");
goog.require("fb.core.stats.StatsCollection");
goog.require("fb.core.stats.StatsListener");
goog.require("fb.core.stats.StatsReporter");
fb.core.stats.StatsManager = {};
fb.core.stats.StatsManager.collections_ = {};
fb.core.stats.StatsManager.reporters_ = {};
fb.core.stats.StatsManager.getCollection = function(repoInfo) {
    var hashString = repoInfo.toString();
    if (!fb.core.stats.StatsManager.collections_[hashString]) {
        fb.core.stats.StatsManager.collections_[hashString] = new fb.core.stats.StatsCollection;
    }
    return fb.core.stats.StatsManager.collections_[hashString];
};
fb.core.stats.StatsManager.getOrCreateReporter = function(repoInfo, creatorFunction) {
    var hashString = repoInfo.toString();
    if (!fb.core.stats.StatsManager.reporters_[hashString]) {
        fb.core.stats.StatsManager.reporters_[hashString] = creatorFunction();
    }
    return fb.core.stats.StatsManager.reporters_[hashString];
};
goog.provide("fb.core.ServerActions");
fb.core.ServerActions = goog.defineClass(null, {listen:goog.abstractMethod, unlisten:goog.abstractMethod, put:goog.abstractMethod, merge:goog.abstractMethod, auth:goog.abstractMethod, unauth:goog.abstractMethod, onDisconnectPut:goog.abstractMethod, onDisconnectMerge:goog.abstractMethod, onDisconnectCancel:goog.abstractMethod, reportStats:goog.abstractMethod});
goog.provide("fb.core.snap.Node");
goog.provide("fb.core.snap.NamedNode");
fb.core.snap.Node = function() {
};
fb.core.snap.Node.prototype.isLeafNode;
fb.core.snap.Node.prototype.getPriority;
fb.core.snap.Node.prototype.updatePriority;
fb.core.snap.Node.prototype.getImmediateChild;
fb.core.snap.Node.prototype.getChild;
fb.core.snap.Node.prototype.getPredecessorChildName;
fb.core.snap.Node.prototype.updateImmediateChild;
fb.core.snap.Node.prototype.updateChild;
fb.core.snap.Node.prototype.hasChild;
fb.core.snap.Node.prototype.isEmpty;
fb.core.snap.Node.prototype.numChildren;
fb.core.snap.Node.prototype.forEachChild;
fb.core.snap.Node.prototype.val;
fb.core.snap.Node.prototype.hash;
fb.core.snap.Node.prototype.compareTo;
fb.core.snap.Node.prototype.equals;
fb.core.snap.Node.prototype.withIndex;
fb.core.snap.Node.prototype.isIndexed;
fb.core.snap.NamedNode = function(name, node) {
    this.name = name;
    this.node = node;
};
fb.core.snap.NamedNode.Wrap = function(name, node) {
    return new fb.core.snap.NamedNode(name, node);
};
goog.provide("fb.core.snap.comparators");
fb.core.snap.NAME_ONLY_COMPARATOR = function(left, right) {
    return fb.core.util.nameCompare(left.name, right.name);
};
fb.core.snap.NAME_COMPARATOR = function(left, right) {
    return fb.core.util.nameCompare(left, right);
};
goog.provide("fb.core.operation.Overwrite");
fb.core.operation.Overwrite = function(source, path, snap) {
    this.type = fb.core.OperationType.OVERWRITE;
    this.source = source;
    this.path = path;
    this.snap = snap;
};
fb.core.operation.Overwrite.prototype.operationForChild = function(childName) {
    if (this.path.isEmpty()) {
        return new fb.core.operation.Overwrite(this.source, fb.core.util.Path.Empty, this.snap.getImmediateChild(childName));
    } else {
        return new fb.core.operation.Overwrite(this.source, this.path.popFront(), this.snap);
    }
};
if (goog.DEBUG) {
    fb.core.operation.Overwrite.prototype.toString = function() {
        return "Operation(" + this.path + ": " + this.source.toString() + " overwrite: " + this.snap.toString() + ")";
    };
}
;goog.provide("fb.core.operation.ListenComplete");
fb.core.operation.ListenComplete = function(source, path) {
    this.type = fb.core.OperationType.LISTEN_COMPLETE;
    this.source = source;
    this.path = path;
};
fb.core.operation.ListenComplete.prototype.operationForChild = function(childName) {
    if (this.path.isEmpty()) {
        return new fb.core.operation.ListenComplete(this.source, fb.core.util.Path.Empty);
    } else {
        return new fb.core.operation.ListenComplete(this.source, this.path.popFront());
    }
};
if (goog.DEBUG) {
    fb.core.operation.ListenComplete.prototype.toString = function() {
        return "Operation(" + this.path + ": " + this.source.toString() + " listen_complete)";
    };
}
;goog.provide("fb.core.util.SortedMap");
goog.require("goog.array");
fb.Comparator;
fb.core.util.SortedMap = goog.defineClass(null, {constructor:function(comparator, opt_root) {
    this.comparator_ = comparator;
    this.root_ = opt_root ? opt_root : fb.core.util.SortedMap.EMPTY_NODE_;
}, insert:function(key, value) {
    return new fb.core.util.SortedMap(this.comparator_, this.root_.insert(key, value, this.comparator_).copy(null, null, fb.LLRBNode.BLACK, null, null));
}, remove:function(key) {
    return new fb.core.util.SortedMap(this.comparator_, this.root_.remove(key, this.comparator_).copy(null, null, fb.LLRBNode.BLACK, null, null));
}, get:function(key) {
    var cmp;
    var node = this.root_;
    while (!node.isEmpty()) {
        cmp = this.comparator_(key, node.key);
        if (cmp === 0) {
            return node.value;
        } else {
            if (cmp < 0) {
                node = node.left;
            } else {
                if (cmp > 0) {
                    node = node.right;
                }
            }
        }
    }
    return null;
}, getPredecessorKey:function(key) {
    var cmp, node = this.root_, rightParent = null;
    while (!node.isEmpty()) {
        cmp = this.comparator_(key, node.key);
        if (cmp === 0) {
            if (!node.left.isEmpty()) {
                node = node.left;
                while (!node.right.isEmpty()) {
                    node = node.right;
                }
                return node.key;
            } else {
                if (rightParent) {
                    return rightParent.key;
                } else {
                    return null;
                }
            }
        } else {
            if (cmp < 0) {
                node = node.left;
            } else {
                if (cmp > 0) {
                    rightParent = node;
                    node = node.right;
                }
            }
        }
    }
    throw new Error("Attempted to find predecessor key for a nonexistent key.  What gives?");
}, isEmpty:function() {
    return this.root_.isEmpty();
}, count:function() {
    return this.root_.count();
}, minKey:function() {
    return this.root_.minKey();
}, maxKey:function() {
    return this.root_.maxKey();
}, inorderTraversal:function(action) {
    return this.root_.inorderTraversal(action);
}, reverseTraversal:function(action) {
    return this.root_.reverseTraversal(action);
}, getIterator:function(opt_resultGenerator) {
    return new fb.core.util.SortedMapIterator(this.root_, null, this.comparator_, false, opt_resultGenerator);
}, getIteratorFrom:function(key, opt_resultGenerator) {
    return new fb.core.util.SortedMapIterator(this.root_, key, this.comparator_, false, opt_resultGenerator);
}, getReverseIteratorFrom:function(key, opt_resultGenerator) {
    return new fb.core.util.SortedMapIterator(this.root_, key, this.comparator_, true, opt_resultGenerator);
}, getReverseIterator:function(opt_resultGenerator) {
    return new fb.core.util.SortedMapIterator(this.root_, null, this.comparator_, true, opt_resultGenerator);
}});
fb.core.util.SortedMapIterator = goog.defineClass(null, {constructor:function(node, startKey, comparator, isReverse, opt_resultGenerator) {
    this.resultGenerator_ = opt_resultGenerator || null;
    this.isReverse_ = isReverse;
    this.nodeStack_ = [];
    var cmp = 1;
    while (!node.isEmpty()) {
        cmp = startKey ? comparator(node.key, startKey) : 1;
        if (isReverse) {
            cmp *= -1;
        }
        if (cmp < 0) {
            if (this.isReverse_) {
                node = node.left;
            } else {
                node = node.right;
            }
        } else {
            if (cmp === 0) {
                this.nodeStack_.push(node);
                break;
            } else {
                this.nodeStack_.push(node);
                if (this.isReverse_) {
                    node = node.right;
                } else {
                    node = node.left;
                }
            }
        }
    }
}, getNext:function() {
    if (this.nodeStack_.length === 0) {
        return null;
    }
    var node = this.nodeStack_.pop(), result;
    if (this.resultGenerator_) {
        result = this.resultGenerator_(node.key, node.value);
    } else {
        result = {key:node.key, value:node.value};
    }
    if (this.isReverse_) {
        node = node.left;
        while (!node.isEmpty()) {
            this.nodeStack_.push(node);
            node = node.right;
        }
    } else {
        node = node.right;
        while (!node.isEmpty()) {
            this.nodeStack_.push(node);
            node = node.left;
        }
    }
    return result;
}, hasNext:function() {
    return this.nodeStack_.length > 0;
}, peek:function() {
    if (this.nodeStack_.length === 0) {
        return null;
    }
    var node = goog.array.peek(this.nodeStack_);
    if (this.resultGenerator_) {
        return this.resultGenerator_(node.key, node.value);
    } else {
        return{key:node.key, value:node.value};
    }
}});
fb.LLRBNode = goog.defineClass(null, {constructor:function(key, value, color, opt_left, opt_right) {
    this.key = key;
    this.value = value;
    this.color = color != null ? color : fb.LLRBNode.RED;
    this.left = opt_left != null ? opt_left : fb.core.util.SortedMap.EMPTY_NODE_;
    this.right = opt_right != null ? opt_right : fb.core.util.SortedMap.EMPTY_NODE_;
}, statics:{RED:true, BLACK:false}, copy:function(key, value, color, left, right) {
    return new fb.LLRBNode(key != null ? key : this.key, value != null ? value : this.value, color != null ? color : this.color, left != null ? left : this.left, right != null ? right : this.right);
}, count:function() {
    return this.left.count() + 1 + this.right.count();
}, isEmpty:function() {
    return false;
}, inorderTraversal:function(action) {
    return this.left.inorderTraversal(action) || action(this.key, this.value) || this.right.inorderTraversal(action);
}, reverseTraversal:function(action) {
    return this.right.reverseTraversal(action) || action(this.key, this.value) || this.left.reverseTraversal(action);
}, min_:function() {
    if (this.left.isEmpty()) {
        return this;
    } else {
        return this.left.min_();
    }
}, minKey:function() {
    return this.min_().key;
}, maxKey:function() {
    if (this.right.isEmpty()) {
        return this.key;
    } else {
        return this.right.maxKey();
    }
}, insert:function(key, value, comparator) {
    var cmp, n;
    n = this;
    cmp = comparator(key, n.key);
    if (cmp < 0) {
        n = n.copy(null, null, null, n.left.insert(key, value, comparator), null);
    } else {
        if (cmp === 0) {
            n = n.copy(null, value, null, null, null);
        } else {
            n = n.copy(null, null, null, null, n.right.insert(key, value, comparator));
        }
    }
    return n.fixUp_();
}, removeMin_:function() {
    var n;
    if (this.left.isEmpty()) {
        return fb.core.util.SortedMap.EMPTY_NODE_;
    }
    n = this;
    if (!n.left.isRed_() && !n.left.left.isRed_()) {
        n = n.moveRedLeft_();
    }
    n = n.copy(null, null, null, n.left.removeMin_(), null);
    return n.fixUp_();
}, remove:function(key, comparator) {
    var n, smallest;
    n = this;
    if (comparator(key, n.key) < 0) {
        if (!n.left.isEmpty() && !n.left.isRed_() && !n.left.left.isRed_()) {
            n = n.moveRedLeft_();
        }
        n = n.copy(null, null, null, n.left.remove(key, comparator), null);
    } else {
        if (n.left.isRed_()) {
            n = n.rotateRight_();
        }
        if (!n.right.isEmpty() && !n.right.isRed_() && !n.right.left.isRed_()) {
            n = n.moveRedRight_();
        }
        if (comparator(key, n.key) === 0) {
            if (n.right.isEmpty()) {
                return fb.core.util.SortedMap.EMPTY_NODE_;
            } else {
                smallest = n.right.min_();
                n = n.copy(smallest.key, smallest.value, null, null, n.right.removeMin_());
            }
        }
        n = n.copy(null, null, null, null, n.right.remove(key, comparator));
    }
    return n.fixUp_();
}, isRed_:function() {
    return this.color;
}, fixUp_:function() {
    var n = this;
    if (n.right.isRed_() && !n.left.isRed_()) {
        n = n.rotateLeft_();
    }
    if (n.left.isRed_() && n.left.left.isRed_()) {
        n = n.rotateRight_();
    }
    if (n.left.isRed_() && n.right.isRed_()) {
        n = n.colorFlip_();
    }
    return n;
}, moveRedLeft_:function() {
    var n = this.colorFlip_();
    if (n.right.left.isRed_()) {
        n = n.copy(null, null, null, null, n.right.rotateRight_());
        n = n.rotateLeft_();
        n = n.colorFlip_();
    }
    return n;
}, moveRedRight_:function() {
    var n = this.colorFlip_();
    if (n.left.left.isRed_()) {
        n = n.rotateRight_();
        n = n.colorFlip_();
    }
    return n;
}, rotateLeft_:function() {
    var nl;
    nl = this.copy(null, null, fb.LLRBNode.RED, null, this.right.left);
    return this.right.copy(null, null, this.color, nl, null);
}, rotateRight_:function() {
    var nr;
    nr = this.copy(null, null, fb.LLRBNode.RED, this.left.right, null);
    return this.left.copy(null, null, this.color, null, nr);
}, colorFlip_:function() {
    var left, right;
    left = this.left.copy(null, null, !this.left.color, null, null);
    right = this.right.copy(null, null, !this.right.color, null, null);
    return this.copy(null, null, !this.color, left, right);
}, checkMaxDepth_:function() {
    var blackDepth;
    blackDepth = this.check_();
    if (Math.pow(2, blackDepth) <= this.count() + 1) {
        return true;
    } else {
        return false;
    }
}, check_:function() {
    var blackDepth;
    if (this.isRed_() && this.left.isRed_()) {
        throw new Error("Red node has red child(" + this.key + "," + this.value + ")");
    }
    if (this.right.isRed_()) {
        throw new Error("Right child of (" + this.key + "," + this.value + ") is red");
    }
    blackDepth = this.left.check_();
    if (blackDepth !== this.right.check_()) {
        throw new Error("Black depths differ");
    } else {
        return blackDepth + (this.isRed_() ? 0 : 1);
    }
}});
fb.LLRBEmptyNode = goog.defineClass(null, {constructor:function() {
}, copy:function() {
    return this;
}, insert:function(key, value, comparator) {
    return new fb.LLRBNode(key, value, null);
}, remove:function(key, comparator) {
    return this;
}, count:function() {
    return 0;
}, isEmpty:function() {
    return true;
}, inorderTraversal:function(action) {
    return false;
}, reverseTraversal:function(action) {
    return false;
}, minKey:function() {
    return null;
}, maxKey:function() {
    return null;
}, check_:function() {
    return 0;
}, isRed_:function() {
    return false;
}});
fb.core.util.SortedMap.EMPTY_NODE_ = new fb.LLRBEmptyNode;
goog.provide("fb.core.util.NodePatches");
(function() {
    if (NODE_CLIENT) {
        var version = process["version"];
        if (version === "v0.10.22" || version === "v0.10.23" || version === "v0.10.24") {
            var Writable = require("_stream_writable");
            Writable["prototype"]["write"] = function(chunk, encoding, cb) {
                var state = this["_writableState"];
                var ret = false;
                if (typeof encoding === "function") {
                    cb = encoding;
                    encoding = null;
                }
                if (Buffer["isBuffer"](chunk)) {
                    encoding = "buffer";
                } else {
                    if (!encoding) {
                        encoding = state["defaultEncoding"];
                    }
                }
                if (typeof cb !== "function") {
                    cb = function() {
                    };
                }
                if (state["ended"]) {
                    writeAfterEnd(this, state, cb);
                } else {
                    if (validChunk(this, state, chunk, cb)) {
                        ret = writeOrBuffer(this, state, chunk, encoding, cb);
                    }
                }
                return ret;
            };
            function writeAfterEnd(stream, state, cb) {
                var er = new Error("write after end");
                stream["emit"]("error", er);
                process["nextTick"](function() {
                    cb(er);
                });
            }
            function validChunk(stream, state, chunk, cb) {
                var valid = true;
                if (!Buffer["isBuffer"](chunk) && "string" !== typeof chunk && chunk !== null && chunk !== undefined && !state["objectMode"]) {
                    var er = new TypeError("Invalid non-string/buffer chunk");
                    stream["emit"]("error", er);
                    process["nextTick"](function() {
                        cb(er);
                    });
                    valid = false;
                }
                return valid;
            }
            function writeOrBuffer(stream, state, chunk, encoding, cb) {
                chunk = decodeChunk(state, chunk, encoding);
                if (Buffer["isBuffer"](chunk)) {
                    encoding = "buffer";
                }
                var len = state["objectMode"] ? 1 : chunk["length"];
                state["length"] += len;
                var ret = state["length"] < state["highWaterMark"];
                if (!ret) {
                    state["needDrain"] = true;
                }
                if (state["writing"]) {
                    state["buffer"]["push"](new WriteReq(chunk, encoding, cb));
                } else {
                    doWrite(stream, state, len, chunk, encoding, cb);
                }
                return ret;
            }
            function decodeChunk(state, chunk, encoding) {
                if (!state["objectMode"] && state["decodeStrings"] !== false && typeof chunk === "string") {
                    chunk = new Buffer(chunk, encoding);
                }
                return chunk;
            }
            function WriteReq(chunk, encoding, cb) {
                this["chunk"] = chunk;
                this["encoding"] = encoding;
                this["callback"] = cb;
            }
            function doWrite(stream, state, len, chunk, encoding, cb) {
                state["writelen"] = len;
                state["writecb"] = cb;
                state["writing"] = true;
                state["sync"] = true;
                stream["_write"](chunk, encoding, state["onwrite"]);
                state["sync"] = false;
            }
            var Duplex = require("_stream_duplex");
            Duplex["prototype"]["write"] = Writable["prototype"]["write"];
        }
    }
})();
goog.provide("fb.core.util.ServerValues");
fb.core.util.ServerValues.generateWithValues = function(values) {
    values = values || {};
    values["timestamp"] = values["timestamp"] || (new Date).getTime();
    return values;
};
fb.core.util.ServerValues.resolveDeferredValue = function(value, serverValues) {
    if (!value || typeof value !== "object") {
        return(value);
    } else {
        fb.core.util.assert(".sv" in value, "Unexpected leaf node or priority contents");
        return serverValues[value[".sv"]];
    }
};
fb.core.util.ServerValues.resolveDeferredValueTree = function(tree, serverValues) {
    var resolvedTree = new fb.core.SparseSnapshotTree;
    tree.forEachTree(new fb.core.util.Path(""), function(path, node) {
        resolvedTree.remember(path, fb.core.util.ServerValues.resolveDeferredValueSnapshot(node, serverValues));
    });
    return resolvedTree;
};
fb.core.util.ServerValues.resolveDeferredValueSnapshot = function(node, serverValues) {
    var rawPri = (node.getPriority().val()), priority = fb.core.util.ServerValues.resolveDeferredValue(rawPri, serverValues), newNode;
    if (node.isLeafNode()) {
        var leafNode = (node);
        var value = fb.core.util.ServerValues.resolveDeferredValue(leafNode.getValue(), serverValues);
        if (value !== leafNode.getValue() || priority !== leafNode.getPriority().val()) {
            return new fb.core.snap.LeafNode(value, fb.core.snap.NodeFromJSON(priority));
        } else {
            return node;
        }
    } else {
        var childrenNode = (node);
        newNode = childrenNode;
        if (priority !== childrenNode.getPriority().val()) {
            newNode = newNode.updatePriority(new fb.core.snap.LeafNode(priority));
        }
        childrenNode.forEachChild(fb.core.snap.PriorityIndex, function(childName, childNode) {
            var newChildNode = fb.core.util.ServerValues.resolveDeferredValueSnapshot(childNode, serverValues);
            if (newChildNode !== childNode) {
                newNode = newNode.updateImmediateChild(childName, newChildNode);
            }
        });
        return newNode;
    }
};
goog.provide("fb.core.storage.MemoryStorage");
goog.require("fb.util.obj");
goog.scope(function() {
    var obj = fb.util.obj;
    fb.core.storage.MemoryStorage = function() {
        this.cache_ = {};
    };
    var MemoryStorage = fb.core.storage.MemoryStorage;
    MemoryStorage.prototype.set = function(key, value) {
        if (value == null) {
            delete this.cache_[key];
        } else {
            this.cache_[key] = value;
        }
    };
    MemoryStorage.prototype.get = function(key) {
        if (obj.contains(this.cache_, key)) {
            return this.cache_[key];
        }
        return null;
    };
    MemoryStorage.prototype.remove = function(key) {
        delete this.cache_[key];
    };
    MemoryStorage.prototype.isInMemoryStorage = true;
});
goog.provide("fb.core.storage.DOMStorageWrapper");
goog.require("fb.util.obj");
goog.scope(function() {
    fb.core.storage.DOMStorageWrapper = function(domStorage) {
        this.domStorage_ = domStorage;
        this.prefix_ = "firebase:";
    };
    var DOMStorageWrapper = fb.core.storage.DOMStorageWrapper;
    DOMStorageWrapper.prototype.set = function(key, value) {
        if (value == null) {
            this.domStorage_.removeItem(this.prefixedName_(key));
        } else {
            this.domStorage_.setItem(this.prefixedName_(key), fb.util.json.stringify(value));
        }
    };
    DOMStorageWrapper.prototype.get = function(key) {
        var storedVal = this.domStorage_.getItem(this.prefixedName_(key));
        if (storedVal == null) {
            return null;
        } else {
            return fb.util.json.eval(storedVal);
        }
    };
    DOMStorageWrapper.prototype.remove = function(key) {
        this.domStorage_.removeItem(this.prefixedName_(key));
    };
    DOMStorageWrapper.prototype.isInMemoryStorage = false;
    DOMStorageWrapper.prototype.prefixedName_ = function(name) {
        return this.prefix_ + name;
    };
    DOMStorageWrapper.prototype.toString = function() {
        return this.domStorage_.toString();
    };
});
goog.provide("fb.core.storage");
goog.provide("fb.core.storage.PersistentStorage");
goog.provide("fb.core.storage.SessionStorage");
goog.require("fb.core.storage.DOMStorageWrapper");
goog.require("fb.core.storage.MemoryStorage");
fb.core.storage.createStoragefor = function(domStorageName) {
    try {
        if (typeof window !== "undefined" && typeof window[domStorageName] !== "undefined") {
            var domStorage = window[domStorageName];
            domStorage.setItem("firebase:sentinel", "cache");
            domStorage.removeItem("firebase:sentinel");
            return new fb.core.storage.DOMStorageWrapper(domStorage);
        }
    } catch (e) {
    }
    return new fb.core.storage.MemoryStorage;
};
fb.core.storage.PersistentStorage = fb.core.storage.createStoragefor("localStorage");
fb.core.storage.SessionStorage = fb.core.storage.createStoragefor("sessionStorage");
goog.provide("fb.core.RepoInfo");
goog.require("fb.core.storage");
fb.core.RepoInfo = function(host, secure, namespace, webSocketOnly, persistenceKey) {
    this.host = host.toLowerCase();
    this.domain = this.host.substr(this.host.indexOf(".") + 1);
    this.secure = secure;
    this.namespace = namespace;
    this.webSocketOnly = webSocketOnly;
    this.persistenceKey = persistenceKey || "";
    this.internalHost = fb.core.storage.PersistentStorage.get("host:" + host) || this.host;
};
fb.core.RepoInfo.prototype.needsQueryParam = function() {
    return this.host !== this.internalHost;
};
fb.core.RepoInfo.prototype.isCacheableHost = function() {
    return this.internalHost.substr(0, 2) === "s-";
};
fb.core.RepoInfo.prototype.isDemoHost = function() {
    return this.domain === "firebaseio-demo.com";
};
fb.core.RepoInfo.prototype.isCustomHost = function() {
    return this.domain !== "firebaseio.com" && this.domain !== "firebaseio-demo.com";
};
fb.core.RepoInfo.prototype.updateHost = function(newHost) {
    if (newHost !== this.internalHost) {
        this.internalHost = newHost;
        if (this.isCacheableHost()) {
            fb.core.storage.PersistentStorage.set("host:" + this.host, this.internalHost);
        }
    }
};
fb.core.RepoInfo.prototype.connectionURL = function(type, params) {
    fb.core.util.assert(typeof type === "string", "typeof type must == string");
    fb.core.util.assert(typeof params === "object", "typeof params must == object");
    var connURL;
    if (type === fb.realtime.Constants.WEBSOCKET) {
        connURL = (this.secure ? "wss://" : "ws://") + this.internalHost + "/.ws?";
    } else {
        if (type === fb.realtime.Constants.LONG_POLLING) {
            connURL = (this.secure ? "https://" : "http://") + this.internalHost + "/.lp?";
        } else {
            throw new Error("Unknown connection type: " + type);
        }
    }
    if (this.needsQueryParam()) {
        params["ns"] = this.namespace;
    }
    var pairs = [];
    goog.object.forEach(params, function(element, index, obj) {
        pairs.push(index + "=" + element);
    });
    return connURL + pairs.join("&");
};
fb.core.RepoInfo.prototype.toString = function() {
    var str = (this.secure ? "https://" : "http://") + this.host;
    if (this.persistenceKey) {
        str += "<" + this.persistenceKey + ">";
    }
    return str;
};
goog.provide("fb.core.util");
goog.provide("fb.core.util.LUIDGenerator");
goog.provide("fb.core.util.ObjectToUniqueKey");
goog.provide("fb.core.util.enableLogging");
goog.require("fb.constants");
goog.require("fb.core.RepoInfo");
goog.require("fb.core.storage.SessionStorage");
goog.require("fb.util.assert");
goog.require("fb.util.assertionError");
goog.require("fb.util.json");
goog.require("fb.util.utf8");
goog.require("goog.crypt.Sha1");
goog.require("goog.crypt.base64");
goog.require("goog.object");
goog.require("goog.string");
goog.forwardDeclare("fb.core.util.Path");
fb.core.util.LUIDGenerator = function() {
    var id = 1;
    return function() {
        return id++;
    };
}();
fb.core.util.assert = fb.util.assert;
fb.core.util.assertionError = fb.util.assertionError;
fb.core.util.assertWeak = function(assertion, message) {
    if (!assertion) {
        fb.core.util.error(message);
    }
};
fb.core.util.base64Encode = function(str) {
    var utf8Bytes = fb.util.utf8.stringToByteArray(str);
    return goog.crypt.base64.encodeByteArray(utf8Bytes, true);
};
fb.core.util.base64Decode = function(str) {
    try {
        if (fb.constants.NODE_CLIENT) {
            return(new Buffer(str, "base64")).toString("utf8");
        } else {
            if (typeof atob !== "undefined") {
                return atob(str);
            } else {
                return goog.crypt.base64.decodeString(str, true);
            }
        }
    } catch (e) {
        fb.core.util.log("base64Decode failed: ", e);
    }
    return null;
};
fb.core.util.sha1 = function(str) {
    var utf8Bytes = fb.util.utf8.stringToByteArray(str);
    var sha1 = new goog.crypt.Sha1;
    sha1.update(utf8Bytes);
    var sha1Bytes = sha1.digest();
    return goog.crypt.base64.encodeByteArray(sha1Bytes);
};
fb.core.util.buildLogMessage_ = function(var_args) {
    var message = "";
    for (var i = 0;i < arguments.length;i++) {
        if (goog.isArrayLike(arguments[i])) {
            message += fb.core.util.buildLogMessage_.apply(null, arguments[i]);
        } else {
            if (typeof arguments[i] === "object") {
                message += fb.util.json.stringify(arguments[i]);
            } else {
                message += arguments[i];
            }
        }
        message += " ";
    }
    return message;
};
fb.core.util.logger = null;
fb.core.util.firstLog_ = true;
fb.core.util.enableLogging = function(logger, opt_persistent) {
    fb.util.assert(!opt_persistent || (logger === true || logger === false), "Can't turn on custom loggers persistently.");
    if (logger === true) {
        if (typeof console !== "undefined") {
            if (typeof console.log === "function") {
                fb.core.util.logger = goog.bind(console.log, console);
            } else {
                if (typeof console.log === "object") {
                    fb.core.util.logger = function(message) {
                        console.log(message);
                    };
                }
            }
        }
        if (opt_persistent) {
            fb.core.storage.SessionStorage.set("logging_enabled", true);
        }
    } else {
        if (goog.isFunction(logger)) {
            fb.core.util.logger = logger;
        } else {
            fb.core.util.logger = null;
            fb.core.storage.SessionStorage.remove("logging_enabled");
        }
    }
};
fb.core.util.log = function(var_args) {
    if (fb.core.util.firstLog_ === true) {
        fb.core.util.firstLog_ = false;
        if (fb.core.util.logger === null && fb.core.storage.SessionStorage.get("logging_enabled") === true) {
            fb.core.util.enableLogging(true);
        }
    }
    if (fb.core.util.logger) {
        var message = fb.core.util.buildLogMessage_.apply(null, arguments);
        fb.core.util.logger(message);
    }
};
fb.core.util.logWrapper = function(prefix) {
    return function() {
        fb.core.util.log(prefix, arguments);
    };
};
fb.core.util.error = function(var_args) {
    if (typeof console !== "undefined") {
        var message = "FIREBASE INTERNAL ERROR: " + fb.core.util.buildLogMessage_.apply(null, arguments);
        if (typeof console.error !== "undefined") {
            console.error(message);
        } else {
            console.log(message);
        }
    }
};
fb.core.util.fatal = function(var_args) {
    var message = fb.core.util.buildLogMessage_.apply(null, arguments);
    throw new Error("FIREBASE FATAL ERROR: " + message);
};
fb.core.util.warn = function(var_args) {
    if (typeof console !== "undefined") {
        var message = "FIREBASE WARNING: " + fb.core.util.buildLogMessage_.apply(null, arguments);
        if (typeof console.warn !== "undefined") {
            console.warn(message);
        } else {
            console.log(message);
        }
    }
};
fb.core.util.warnIfPageIsSecure = function() {
    if (typeof window !== "undefined" && window.location && window.location.protocol && window.location.protocol.indexOf("https:") !== -1) {
        fb.core.util.warn("Insecure Firebase access from a secure page. " + "Please use https in calls to new Firebase().");
    }
};
fb.core.util.warnAboutUnsupportedMethod = function(methodName) {
    fb.core.util.warn(methodName + " is unsupported and will likely change soon.  " + "Please do not use.");
};
fb.core.util.parseRepoInfo = function(dataURL) {
    var parsedUrl = fb.core.util.parseURL(dataURL), namespace = parsedUrl.subdomain;
    if (parsedUrl.domain === "firebase") {
        fb.core.util.fatal(parsedUrl.host + " is no longer supported. " + "Please use <YOUR FIREBASE>.firebaseio.com instead");
    }
    if (!namespace || namespace == "undefined") {
        fb.core.util.fatal("Cannot parse Firebase url. " + "Please use https://<YOUR FIREBASE>.firebaseio.com");
    }
    if (!parsedUrl.secure) {
        fb.core.util.warnIfPageIsSecure();
    }
    var webSocketOnly = parsedUrl.scheme === "ws" || parsedUrl.scheme === "wss";
    return{repoInfo:new fb.core.RepoInfo(parsedUrl.host, parsedUrl.secure, namespace, webSocketOnly), path:new fb.core.util.Path(parsedUrl.pathString)};
};
fb.core.util.parseURL = function(dataURL) {
    var host = "", domain = "", subdomain = "", pathString = "";
    var secure = true, scheme = "https", port = 443;
    if (goog.isString(dataURL)) {
        var colonInd = dataURL.indexOf("//");
        if (colonInd >= 0) {
            scheme = dataURL.substring(0, colonInd - 1);
            dataURL = dataURL.substring(colonInd + 2);
        }
        var slashInd = dataURL.indexOf("/");
        if (slashInd === -1) {
            slashInd = dataURL.length;
        }
        host = dataURL.substring(0, slashInd);
        pathString = fb.core.util.decodePath(dataURL.substring(slashInd));
        var parts = host.split(".");
        if (parts.length === 3) {
            domain = parts[1];
            subdomain = parts[0].toLowerCase();
        } else {
            if (parts.length === 2) {
                domain = parts[0];
            }
        }
        colonInd = host.indexOf(":");
        if (colonInd >= 0) {
            secure = scheme === "https" || scheme === "wss";
            port = goog.string.parseInt(host.substring(colonInd + 1));
        }
    }
    return{host:host, port:port, domain:domain, subdomain:subdomain, secure:secure, scheme:scheme, pathString:pathString};
};
fb.core.util.decodePath = function(pathString) {
    var pathStringDecoded = "";
    var pieces = pathString.split("/");
    for (var i = 0;i < pieces.length;i++) {
        if (pieces[i].length > 0) {
            var piece = pieces[i];
            try {
                piece = goog.string.urlDecode(piece);
            } catch (e) {
            }
            pathStringDecoded += "/" + piece;
        }
    }
    return pathStringDecoded;
};
fb.core.util.isInvalidJSONNumber = function(data) {
    return goog.isNumber(data) && (data != data || data == Number.POSITIVE_INFINITY || data == Number.NEGATIVE_INFINITY);
};
fb.core.util.executeWhenDOMReady = function(fn) {
    if (NODE_CLIENT || document.readyState === "complete") {
        fn();
    } else {
        var called = false;
        var wrappedFn = function() {
            if (!document.body) {
                setTimeout(wrappedFn, Math.floor(10));
                return;
            }
            if (!called) {
                called = true;
                fn();
            }
        };
        if (document.addEventListener) {
            document.addEventListener("DOMContentLoaded", wrappedFn, false);
            window.addEventListener("load", wrappedFn, false);
        } else {
            if (document.attachEvent) {
                document.attachEvent("onreadystatechange", function() {
                    if (document.readyState === "complete") {
                        wrappedFn();
                    }
                });
                window.attachEvent("onload", wrappedFn);
            }
        }
    }
};
fb.core.util.MIN_NAME = "[MIN_NAME]";
fb.core.util.MAX_NAME = "[MAX_NAME]";
fb.core.util.nameCompare = function(a, b) {
    if (a === b) {
        return 0;
    } else {
        if (a === fb.core.util.MIN_NAME || b === fb.core.util.MAX_NAME) {
            return-1;
        } else {
            if (b === fb.core.util.MIN_NAME || a === fb.core.util.MAX_NAME) {
                return 1;
            } else {
                var aAsInt = fb.core.util.tryParseInt(a), bAsInt = fb.core.util.tryParseInt(b);
                if (aAsInt !== null) {
                    if (bAsInt !== null) {
                        return aAsInt - bAsInt == 0 ? a.length - b.length : aAsInt - bAsInt;
                    } else {
                        return-1;
                    }
                } else {
                    if (bAsInt !== null) {
                        return 1;
                    } else {
                        return a < b ? -1 : 1;
                    }
                }
            }
        }
    }
};
fb.core.util.stringCompare = function(a, b) {
    if (a === b) {
        return 0;
    } else {
        if (a < b) {
            return-1;
        } else {
            return 1;
        }
    }
};
fb.core.util.requireKey = function(key, obj) {
    if (obj && key in obj) {
        return obj[key];
    } else {
        throw new Error("Missing required key (" + key + ") in object: " + fb.util.json.stringify(obj));
    }
};
fb.core.util.ObjectToUniqueKey = function(obj) {
    if (typeof obj !== "object" || obj === null) {
        return fb.util.json.stringify(obj);
    }
    var keys = [];
    for (var k in obj) {
        keys.push(k);
    }
    keys.sort();
    var key = "{";
    for (var i = 0;i < keys.length;i++) {
        if (i !== 0) {
            key += ",";
        }
        key += fb.util.json.stringify(keys[i]);
        key += ":";
        key += fb.core.util.ObjectToUniqueKey(obj[keys[i]]);
    }
    key += "}";
    return key;
};
fb.core.util.splitStringBySize = function(str, segsize) {
    if (str.length <= segsize) {
        return[str];
    }
    var dataSegs = [];
    for (var c = 0;c < str.length;c += segsize) {
        if (c + segsize > str) {
            dataSegs.push(str.substring(c, str.length));
        } else {
            dataSegs.push(str.substring(c, c + segsize));
        }
    }
    return dataSegs;
};
fb.core.util.each = function(obj, fn) {
    if (goog.isArray(obj)) {
        for (var i = 0;i < obj.length;++i) {
            fn(i, obj[i]);
        }
    } else {
        goog.object.forEach(obj, fn);
    }
};
fb.core.util.bindCallback = function(callback, opt_context) {
    return opt_context ? goog.bind(callback, opt_context) : callback;
};
fb.core.util.doubleToIEEE754String = function(v) {
    fb.core.util.assert(!fb.core.util.isInvalidJSONNumber(v), "Invalid JSON number");
    var ebits = 11, fbits = 52;
    var bias = (1 << ebits - 1) - 1, s, e, f, ln, i, bits, str, bytes;
    if (v === 0) {
        e = 0;
        f = 0;
        s = 1 / v === -Infinity ? 1 : 0;
    } else {
        s = v < 0;
        v = Math.abs(v);
        if (v >= Math.pow(2, 1 - bias)) {
            ln = Math.min(Math.floor(Math.log(v) / Math.LN2), bias);
            e = ln + bias;
            f = Math.round(v * Math.pow(2, fbits - ln) - Math.pow(2, fbits));
        } else {
            e = 0;
            f = Math.round(v / Math.pow(2, 1 - bias - fbits));
        }
    }
    bits = [];
    for (i = fbits;i;i -= 1) {
        bits.push(f % 2 ? 1 : 0);
        f = Math.floor(f / 2);
    }
    for (i = ebits;i;i -= 1) {
        bits.push(e % 2 ? 1 : 0);
        e = Math.floor(e / 2);
    }
    bits.push(s ? 1 : 0);
    bits.reverse();
    str = bits.join("");
    var hexByteString = "";
    for (i = 0;i < 64;i += 8) {
        var hexByte = parseInt(str.substr(i, 8), 2).toString(16);
        if (hexByte.length === 1) {
            hexByte = "0" + hexByte;
        }
        hexByteString = hexByteString + hexByte;
    }
    return hexByteString.toLowerCase();
};
fb.core.util.isChromeExtensionContentScript = function() {
    return!!(typeof window === "object" && window["chrome"] && window["chrome"]["extension"] && !/^chrome/.test(window.location.href));
};
fb.core.util.isWindowsStoreApp = function() {
    return typeof Windows === "object" && typeof Windows.UI === "object";
};
fb.core.util.errorForServerCode = function(code) {
    var reason = "Unknown Error";
    if (code === "too_big") {
        reason = "The data requested exceeds the maximum size " + "that can be accessed with a single request.";
    } else {
        if (code == "permission_denied") {
            reason = "Client doesn't have permission to access the desired data.";
        } else {
            if (code == "unavailable") {
                reason = "The service is unavailable";
            }
        }
    }
    var error = new Error(code + ": " + reason);
    error.code = code.toUpperCase();
    return error;
};
fb.core.util.INTEGER_REGEXP_ = new RegExp("^-?\\d{1,10}$");
fb.core.util.tryParseInt = function(str) {
    if (fb.core.util.INTEGER_REGEXP_.test(str)) {
        var intVal = Number(str);
        if (intVal >= -2147483648 && intVal <= 2147483647) {
            return intVal;
        }
    }
    return null;
};
fb.core.util.exceptionGuard = function(fn) {
    try {
        fn();
    } catch (e) {
        setTimeout(function() {
            var stack = e.stack || "";
            fb.core.util.warn("Exception was thrown by user callback.", stack);
            throw e;
        }, Math.floor(0));
    }
};
fb.core.util.callUserCallback = function(opt_callback, var_args) {
    if (goog.isFunction(opt_callback)) {
        var args = Array.prototype.slice.call(arguments, 1);
        var newArgs = args.slice();
        fb.core.util.exceptionGuard(function() {
            opt_callback.apply(null, newArgs);
        });
    }
};
fb.core.util.beingCrawled = function() {
    var userAgent = typeof window === "object" && window["navigator"] && window["navigator"]["userAgent"] || "";
    return userAgent.search(/googlebot|google webmaster tools|bingbot|yahoo! slurp|baiduspider|yandexbot|duckduckbot/i) >= 0;
};
goog.provide("fb.util.jwt");
goog.require("fb.core.util");
goog.require("fb.util.json");
goog.require("fb.util.obj");
goog.require("goog.crypt.base64");
goog.require("goog.json");
fb.util.jwt.decode = function(token) {
    var header = {}, claims = {}, data = {}, signature = "";
    try {
        var parts = token.split(".");
        header = fb.util.json.eval(fb.core.util.base64Decode(parts[0]) || "");
        claims = fb.util.json.eval(fb.core.util.base64Decode(parts[1]) || "");
        signature = parts[2];
        data = claims["d"] || {};
        delete claims["d"];
    } catch (e) {
    }
    return{header:header, claims:claims, data:data, signature:signature};
};
fb.util.jwt.isValidTimestamp = function(token) {
    var claims = fb.util.jwt.decode(token).claims, now = Math.floor((new Date).getTime() / 1E3), validSince, validUntil;
    if (typeof claims === "object") {
        if (claims.hasOwnProperty("nbf")) {
            validSince = fb.util.obj.get(claims, "nbf");
        } else {
            if (claims.hasOwnProperty("iat")) {
                validSince = fb.util.obj.get(claims, "iat");
            }
        }
        if (claims.hasOwnProperty("exp")) {
            validUntil = fb.util.obj.get(claims, "exp");
        } else {
            validUntil = validSince + 86400;
        }
    }
    return now && validSince && validUntil && now >= validSince && now <= validUntil;
};
fb.util.jwt.issuedAtTime = function(token) {
    var claims = fb.util.jwt.decode(token).claims;
    if (typeof claims === "object" && claims.hasOwnProperty("iat")) {
        return fb.util.obj.get(claims, "iat");
    }
    return null;
};
fb.util.jwt.isValidFormat = function(token) {
    var decoded = fb.util.jwt.decode(token), claims = decoded.claims;
    return!!decoded.signature && !!claims && typeof claims === "object" && claims.hasOwnProperty("iat");
};
fb.util.jwt.isAdmin = function(token) {
    var claims = fb.util.jwt.decode(token).claims;
    return typeof claims === "object" && fb.util.obj.get(claims, "admin") === true;
};
goog.provide("fb.core.view.EventGenerator");
goog.require("fb.core.snap.NamedNode");
goog.require("fb.core.util");
fb.core.view.EventGenerator = function(query) {
    this.query_ = query;
    this.index_ = query.getQueryParams().getIndex();
};
fb.core.view.EventGenerator.prototype.generateEventsForChanges = function(changes, eventCache, eventRegistrations) {
    var events = [], self = this;
    var moves = [];
    goog.array.forEach(changes, function(change) {
        if (change.type === fb.core.view.Change.CHILD_CHANGED && self.index_.indexedValueChanged((change.oldSnap), change.snapshotNode)) {
            moves.push(fb.core.view.Change.childMovedChange((change.childName), change.snapshotNode));
        }
    });
    this.generateEventsForType_(events, fb.core.view.Change.CHILD_REMOVED, changes, eventRegistrations, eventCache);
    this.generateEventsForType_(events, fb.core.view.Change.CHILD_ADDED, changes, eventRegistrations, eventCache);
    this.generateEventsForType_(events, fb.core.view.Change.CHILD_MOVED, moves, eventRegistrations, eventCache);
    this.generateEventsForType_(events, fb.core.view.Change.CHILD_CHANGED, changes, eventRegistrations, eventCache);
    this.generateEventsForType_(events, fb.core.view.Change.VALUE, changes, eventRegistrations, eventCache);
    return events;
};
fb.core.view.EventGenerator.prototype.generateEventsForType_ = function(events, eventType, changes, registrations, eventCache) {
    var filteredChanges = goog.array.filter(changes, function(change) {
        return change.type === eventType;
    });
    var self = this;
    goog.array.sort(filteredChanges, goog.bind(this.compareChanges_, this));
    goog.array.forEach(filteredChanges, function(change) {
        var materializedChange = self.materializeSingleChange_(change, eventCache);
        goog.array.forEach(registrations, function(registration) {
            if (registration.respondsTo(change.type)) {
                events.push(registration.createEvent(materializedChange, self.query_));
            }
        });
    });
};
fb.core.view.EventGenerator.prototype.materializeSingleChange_ = function(change, eventCache) {
    if (change.type === "value" || change.type === "child_removed") {
        return change;
    } else {
        change.prevName = eventCache.getPredecessorChildName((change.childName), change.snapshotNode, this.index_);
        return change;
    }
};
fb.core.view.EventGenerator.prototype.compareChanges_ = function(a, b) {
    if (a.childName == null || b.childName == null) {
        throw fb.core.util.assertionError("Should only compare child_ events.");
    }
    var aWrapped = new fb.core.snap.NamedNode(a.childName, a.snapshotNode);
    var bWrapped = new fb.core.snap.NamedNode(b.childName, b.snapshotNode);
    return this.index_.compare(aWrapped, bWrapped);
};
goog.provide("fb.core.view.ChildChangeAccumulator");
goog.require("fb.core.util");
goog.require("fb.core.view.Change");
goog.require("fb.util.obj");
goog.require("goog.object");
fb.core.view.ChildChangeAccumulator = function() {
    this.changeMap_ = {};
};
fb.core.view.ChildChangeAccumulator.prototype.trackChildChange = function(change) {
    var Change = fb.core.view.Change;
    var type = change.type;
    var childKey = (change.childName);
    fb.core.util.assert(type == fb.core.view.Change.CHILD_ADDED || type == fb.core.view.Change.CHILD_CHANGED || type == fb.core.view.Change.CHILD_REMOVED, "Only child changes supported for tracking");
    fb.core.util.assert(childKey !== ".priority", "Only non-priority child changes can be tracked.");
    var oldChange = fb.util.obj.get(this.changeMap_, childKey);
    if (oldChange) {
        var oldType = oldChange.type;
        if (type == Change.CHILD_ADDED && oldType == Change.CHILD_REMOVED) {
            this.changeMap_[childKey] = Change.childChangedChange(childKey, change.snapshotNode, oldChange.snapshotNode);
        } else {
            if (type == Change.CHILD_REMOVED && oldType == Change.CHILD_ADDED) {
                delete this.changeMap_[childKey];
            } else {
                if (type == Change.CHILD_REMOVED && oldType == Change.CHILD_CHANGED) {
                    this.changeMap_[childKey] = Change.childRemovedChange(childKey, (oldChange.oldSnap));
                } else {
                    if (type == Change.CHILD_CHANGED && oldType == Change.CHILD_ADDED) {
                        this.changeMap_[childKey] = Change.childAddedChange(childKey, change.snapshotNode);
                    } else {
                        if (type == Change.CHILD_CHANGED && oldType == Change.CHILD_CHANGED) {
                            this.changeMap_[childKey] = Change.childChangedChange(childKey, change.snapshotNode, (oldChange.oldSnap));
                        } else {
                            throw fb.core.util.assertionError("Illegal combination of changes: " + change + " occurred after " + oldChange);
                        }
                    }
                }
            }
        }
    } else {
        this.changeMap_[childKey] = change;
    }
};
fb.core.view.ChildChangeAccumulator.prototype.getChanges = function() {
    return goog.object.getValues(this.changeMap_);
};
goog.provide("fb.core.view.filter.IndexedFilter");
goog.require("fb.core.util");
fb.core.view.filter.IndexedFilter = function(index) {
    this.index_ = index;
};
fb.core.view.filter.IndexedFilter.prototype.updateChild = function(snap, key, newChild, affectedPath, source, optChangeAccumulator) {
    var Change = fb.core.view.Change;
    fb.core.util.assert(snap.isIndexed(this.index_), "A node must be indexed if only a child is updated");
    var oldChild = snap.getImmediateChild(key);
    if (oldChild.getChild(affectedPath).equals(newChild.getChild(affectedPath))) {
        if (oldChild.isEmpty() == newChild.isEmpty()) {
            return snap;
        }
    }
    if (optChangeAccumulator != null) {
        if (newChild.isEmpty()) {
            if (snap.hasChild(key)) {
                optChangeAccumulator.trackChildChange(Change.childRemovedChange(key, oldChild));
            } else {
                fb.core.util.assert(snap.isLeafNode(), "A child remove without an old child only makes sense on a leaf node");
            }
        } else {
            if (oldChild.isEmpty()) {
                optChangeAccumulator.trackChildChange(Change.childAddedChange(key, newChild));
            } else {
                optChangeAccumulator.trackChildChange(Change.childChangedChange(key, newChild, oldChild));
            }
        }
    }
    if (snap.isLeafNode() && newChild.isEmpty()) {
        return snap;
    } else {
        return snap.updateImmediateChild(key, newChild).withIndex(this.index_);
    }
};
fb.core.view.filter.IndexedFilter.prototype.updateFullNode = function(oldSnap, newSnap, optChangeAccumulator) {
    var Change = fb.core.view.Change;
    if (optChangeAccumulator != null) {
        if (!oldSnap.isLeafNode()) {
            oldSnap.forEachChild(fb.core.snap.PriorityIndex, function(key, childNode) {
                if (!newSnap.hasChild(key)) {
                    optChangeAccumulator.trackChildChange(Change.childRemovedChange(key, childNode));
                }
            });
        }
        if (!newSnap.isLeafNode()) {
            newSnap.forEachChild(fb.core.snap.PriorityIndex, function(key, childNode) {
                if (oldSnap.hasChild(key)) {
                    var oldChild = oldSnap.getImmediateChild(key);
                    if (!oldChild.equals(childNode)) {
                        optChangeAccumulator.trackChildChange(Change.childChangedChange(key, childNode, oldChild));
                    }
                } else {
                    optChangeAccumulator.trackChildChange(Change.childAddedChange(key, childNode));
                }
            });
        }
    }
    return newSnap.withIndex(this.index_);
};
fb.core.view.filter.IndexedFilter.prototype.updatePriority = function(oldSnap, newPriority) {
    if (oldSnap.isEmpty()) {
        return fb.core.snap.EMPTY_NODE;
    } else {
        return oldSnap.updatePriority(newPriority);
    }
};
fb.core.view.filter.IndexedFilter.prototype.filtersNodes = function() {
    return false;
};
fb.core.view.filter.IndexedFilter.prototype.getIndexedFilter = function() {
    return this;
};
fb.core.view.filter.IndexedFilter.prototype.getIndex = function() {
    return this.index_;
};
goog.provide("fb.core.view.filter.RangedFilter");
goog.require("fb.core.view.filter.IndexedFilter");
fb.core.view.filter.RangedFilter = function(params) {
    this.indexedFilter_ = new fb.core.view.filter.IndexedFilter(params.getIndex());
    this.index_ = params.getIndex();
    this.startPost_ = this.getStartPost_(params);
    this.endPost_ = this.getEndPost_(params);
};
fb.core.view.filter.RangedFilter.prototype.getStartPost = function() {
    return this.startPost_;
};
fb.core.view.filter.RangedFilter.prototype.getEndPost = function() {
    return this.endPost_;
};
fb.core.view.filter.RangedFilter.prototype.matches = function(node) {
    return this.index_.compare(this.getStartPost(), node) <= 0 && this.index_.compare(node, this.getEndPost()) <= 0;
};
fb.core.view.filter.RangedFilter.prototype.updateChild = function(snap, key, newChild, affectedPath, source, optChangeAccumulator) {
    if (!this.matches(new fb.core.snap.NamedNode(key, newChild))) {
        newChild = fb.core.snap.EMPTY_NODE;
    }
    return this.indexedFilter_.updateChild(snap, key, newChild, affectedPath, source, optChangeAccumulator);
};
fb.core.view.filter.RangedFilter.prototype.updateFullNode = function(oldSnap, newSnap, optChangeAccumulator) {
    if (newSnap.isLeafNode()) {
        newSnap = fb.core.snap.EMPTY_NODE;
    }
    var filtered = newSnap.withIndex(this.index_);
    filtered = filtered.updatePriority(fb.core.snap.EMPTY_NODE);
    var self = this;
    newSnap.forEachChild(fb.core.snap.PriorityIndex, function(key, childNode) {
        if (!self.matches(new fb.core.snap.NamedNode(key, childNode))) {
            filtered = filtered.updateImmediateChild(key, fb.core.snap.EMPTY_NODE);
        }
    });
    return this.indexedFilter_.updateFullNode(oldSnap, filtered, optChangeAccumulator);
};
fb.core.view.filter.RangedFilter.prototype.updatePriority = function(oldSnap, newPriority) {
    return oldSnap;
};
fb.core.view.filter.RangedFilter.prototype.filtersNodes = function() {
    return true;
};
fb.core.view.filter.RangedFilter.prototype.getIndexedFilter = function() {
    return this.indexedFilter_;
};
fb.core.view.filter.RangedFilter.prototype.getIndex = function() {
    return this.index_;
};
fb.core.view.filter.RangedFilter.prototype.getStartPost_ = function(params) {
    if (params.hasStart()) {
        var startName = params.getIndexStartName();
        return params.getIndex().makePost(params.getIndexStartValue(), startName);
    } else {
        return params.getIndex().minPost();
    }
};
fb.core.view.filter.RangedFilter.prototype.getEndPost_ = function(params) {
    if (params.hasEnd()) {
        var endName = params.getIndexEndName();
        return params.getIndex().makePost(params.getIndexEndValue(), endName);
    } else {
        return params.getIndex().maxPost();
    }
};
goog.provide("fb.core.view.filter.NodeFilter");
goog.require("fb.core.view.ChildChangeAccumulator");
goog.require("fb.core.view.CompleteChildSource");
fb.core.view.filter.NodeFilter = function() {
};
fb.core.view.filter.NodeFilter.prototype.updateChild = function(snap, key, newChild, affectedPath, source, optChangeAccumulator) {
};
fb.core.view.filter.NodeFilter.prototype.updateFullNode = function(oldSnap, newSnap, optChangeAccumulator) {
};
fb.core.view.filter.NodeFilter.prototype.updatePriority = function(oldSnap, newPriority) {
};
fb.core.view.filter.NodeFilter.prototype.filtersNodes = function() {
};
fb.core.view.filter.NodeFilter.prototype.getIndexedFilter = function() {
};
fb.core.view.filter.NodeFilter.prototype.getIndex = function() {
};
goog.provide("fb.core.view.filter.LimitedFilter");
goog.require("fb.core.snap.NamedNode");
goog.require("fb.core.view.filter.RangedFilter");
goog.require("fb.core.util");
fb.core.view.filter.LimitedFilter = function(params) {
    this.rangedFilter_ = new fb.core.view.filter.RangedFilter(params);
    this.index_ = params.getIndex();
    this.limit_ = params.getLimit();
    this.reverse_ = !params.isViewFromLeft();
};
fb.core.view.filter.LimitedFilter.prototype.updateChild = function(snap, key, newChild, affectedPath, source, optChangeAccumulator) {
    if (!this.rangedFilter_.matches(new fb.core.snap.NamedNode(key, newChild))) {
        newChild = fb.core.snap.EMPTY_NODE;
    }
    if (snap.getImmediateChild(key).equals(newChild)) {
        return snap;
    } else {
        if (snap.numChildren() < this.limit_) {
            return this.rangedFilter_.getIndexedFilter().updateChild(snap, key, newChild, affectedPath, source, optChangeAccumulator);
        } else {
            return this.fullLimitUpdateChild_(snap, key, newChild, source, optChangeAccumulator);
        }
    }
};
fb.core.view.filter.LimitedFilter.prototype.updateFullNode = function(oldSnap, newSnap, optChangeAccumulator) {
    var filtered;
    if (newSnap.isLeafNode() || newSnap.isEmpty()) {
        filtered = fb.core.snap.EMPTY_NODE.withIndex(this.index_);
    } else {
        if (this.limit_ * 2 < newSnap.numChildren() && newSnap.isIndexed(this.index_)) {
            filtered = fb.core.snap.EMPTY_NODE.withIndex(this.index_);
            var iterator;
            newSnap = (newSnap);
            if (this.reverse_) {
                iterator = newSnap.getReverseIteratorFrom(this.rangedFilter_.getEndPost(), this.index_);
            } else {
                iterator = newSnap.getIteratorFrom(this.rangedFilter_.getStartPost(), this.index_);
            }
            var count = 0;
            while (iterator.hasNext() && count < this.limit_) {
                var next = iterator.getNext();
                var inRange;
                if (this.reverse_) {
                    inRange = this.index_.compare(this.rangedFilter_.getStartPost(), next) <= 0;
                } else {
                    inRange = this.index_.compare(next, this.rangedFilter_.getEndPost()) <= 0;
                }
                if (inRange) {
                    filtered = filtered.updateImmediateChild(next.name, next.node);
                    count++;
                } else {
                    break;
                }
            }
        } else {
            filtered = newSnap.withIndex(this.index_);
            filtered = (filtered.updatePriority(fb.core.snap.EMPTY_NODE));
            var startPost;
            var endPost;
            var cmp;
            if (this.reverse_) {
                iterator = filtered.getReverseIterator(this.index_);
                startPost = this.rangedFilter_.getEndPost();
                endPost = this.rangedFilter_.getStartPost();
                var indexCompare = this.index_.getCompare();
                cmp = function(a, b) {
                    return indexCompare(b, a);
                };
            } else {
                iterator = filtered.getIterator(this.index_);
                startPost = this.rangedFilter_.getStartPost();
                endPost = this.rangedFilter_.getEndPost();
                cmp = this.index_.getCompare();
            }
            count = 0;
            var foundStartPost = false;
            while (iterator.hasNext()) {
                next = iterator.getNext();
                if (!foundStartPost && cmp(startPost, next) <= 0) {
                    foundStartPost = true;
                }
                inRange = foundStartPost && count < this.limit_ && cmp(next, endPost) <= 0;
                if (inRange) {
                    count++;
                } else {
                    filtered = filtered.updateImmediateChild(next.name, fb.core.snap.EMPTY_NODE);
                }
            }
        }
    }
    return this.rangedFilter_.getIndexedFilter().updateFullNode(oldSnap, filtered, optChangeAccumulator);
};
fb.core.view.filter.LimitedFilter.prototype.updatePriority = function(oldSnap, newPriority) {
    return oldSnap;
};
fb.core.view.filter.LimitedFilter.prototype.filtersNodes = function() {
    return true;
};
fb.core.view.filter.LimitedFilter.prototype.getIndexedFilter = function() {
    return this.rangedFilter_.getIndexedFilter();
};
fb.core.view.filter.LimitedFilter.prototype.getIndex = function() {
    return this.index_;
};
fb.core.view.filter.LimitedFilter.prototype.fullLimitUpdateChild_ = function(snap, childKey, childSnap, source, optChangeAccumulator) {
    var Change = fb.core.view.Change;
    var cmp;
    if (this.reverse_) {
        var indexCmp = this.index_.getCompare();
        cmp = function(a, b) {
            return indexCmp(b, a);
        };
    } else {
        cmp = this.index_.getCompare();
    }
    var oldEventCache = (snap);
    fb.core.util.assert(oldEventCache.numChildren() == this.limit_, "");
    var newChildNamedNode = new fb.core.snap.NamedNode(childKey, childSnap);
    var windowBoundary = (this.reverse_ ? oldEventCache.getFirstChild(this.index_) : oldEventCache.getLastChild(this.index_));
    var inRange = this.rangedFilter_.matches(newChildNamedNode);
    if (oldEventCache.hasChild(childKey)) {
        var oldChildSnap = oldEventCache.getImmediateChild(childKey);
        var nextChild = source.getChildAfterChild(this.index_, windowBoundary, this.reverse_);
        while (nextChild != null && (nextChild.name == childKey || oldEventCache.hasChild(nextChild.name))) {
            nextChild = source.getChildAfterChild(this.index_, nextChild, this.reverse_);
        }
        var compareNext = nextChild == null ? 1 : cmp(nextChild, newChildNamedNode);
        var remainsInWindow = inRange && !childSnap.isEmpty() && compareNext >= 0;
        if (remainsInWindow) {
            if (optChangeAccumulator != null) {
                optChangeAccumulator.trackChildChange(Change.childChangedChange(childKey, childSnap, oldChildSnap));
            }
            return oldEventCache.updateImmediateChild(childKey, childSnap);
        } else {
            if (optChangeAccumulator != null) {
                optChangeAccumulator.trackChildChange(Change.childRemovedChange(childKey, oldChildSnap));
            }
            var newEventCache = oldEventCache.updateImmediateChild(childKey, fb.core.snap.EMPTY_NODE);
            var nextChildInRange = nextChild != null && this.rangedFilter_.matches(nextChild);
            if (nextChildInRange) {
                if (optChangeAccumulator != null) {
                    optChangeAccumulator.trackChildChange(Change.childAddedChange(nextChild.name, nextChild.node));
                }
                return newEventCache.updateImmediateChild(nextChild.name, nextChild.node);
            } else {
                return newEventCache;
            }
        }
    } else {
        if (childSnap.isEmpty()) {
            return snap;
        } else {
            if (inRange) {
                if (cmp(windowBoundary, newChildNamedNode) >= 0) {
                    if (optChangeAccumulator != null) {
                        optChangeAccumulator.trackChildChange(Change.childRemovedChange(windowBoundary.name, windowBoundary.node));
                        optChangeAccumulator.trackChildChange(Change.childAddedChange(childKey, childSnap));
                    }
                    return oldEventCache.updateImmediateChild(childKey, childSnap).updateImmediateChild(windowBoundary.name, fb.core.snap.EMPTY_NODE);
                } else {
                    return snap;
                }
            } else {
                return snap;
            }
        }
    }
};
goog.provide("fb.core.view.ViewProcessor");
goog.require("fb.core.view.CompleteChildSource");
goog.require("fb.core.util");
fb.core.view.ProcessorResult = function(viewCache, changes) {
    this.viewCache = viewCache;
    this.changes = changes;
};
fb.core.view.ViewProcessor = function(filter) {
    this.filter_ = filter;
};
fb.core.view.ViewProcessor.prototype.assertIndexed = function(viewCache) {
    fb.core.util.assert(viewCache.getEventCache().getNode().isIndexed(this.filter_.getIndex()), "Event snap not indexed");
    fb.core.util.assert(viewCache.getServerCache().getNode().isIndexed(this.filter_.getIndex()), "Server snap not indexed");
};
fb.core.view.ViewProcessor.prototype.applyOperation = function(oldViewCache, operation, writesCache, optCompleteCache) {
    var accumulator = new fb.core.view.ChildChangeAccumulator;
    var newViewCache, filterServerNode;
    if (operation.type === fb.core.OperationType.OVERWRITE) {
        var overwrite = (operation);
        if (overwrite.source.fromUser) {
            newViewCache = this.applyUserOverwrite_(oldViewCache, overwrite.path, overwrite.snap, writesCache, optCompleteCache, accumulator);
        } else {
            fb.core.util.assert(overwrite.source.fromServer, "Unknown source.");
            filterServerNode = overwrite.source.tagged || oldViewCache.getServerCache().isFiltered() && !overwrite.path.isEmpty();
            newViewCache = this.applyServerOverwrite_(oldViewCache, overwrite.path, overwrite.snap, writesCache, optCompleteCache, filterServerNode, accumulator);
        }
    } else {
        if (operation.type === fb.core.OperationType.MERGE) {
            var merge = (operation);
            if (merge.source.fromUser) {
                newViewCache = this.applyUserMerge_(oldViewCache, merge.path, merge.children, writesCache, optCompleteCache, accumulator);
            } else {
                fb.core.util.assert(merge.source.fromServer, "Unknown source.");
                filterServerNode = merge.source.tagged || oldViewCache.getServerCache().isFiltered();
                newViewCache = this.applyServerMerge_(oldViewCache, merge.path, merge.children, writesCache, optCompleteCache, filterServerNode, accumulator);
            }
        } else {
            if (operation.type === fb.core.OperationType.ACK_USER_WRITE) {
                var ackUserWrite = (operation);
                if (!ackUserWrite.revert) {
                    newViewCache = this.ackUserWrite_(oldViewCache, ackUserWrite.path, ackUserWrite.affectedTree, writesCache, optCompleteCache, accumulator);
                } else {
                    newViewCache = this.revertUserWrite_(oldViewCache, ackUserWrite.path, writesCache, optCompleteCache, accumulator);
                }
            } else {
                if (operation.type === fb.core.OperationType.LISTEN_COMPLETE) {
                    newViewCache = this.listenComplete_(oldViewCache, operation.path, writesCache, optCompleteCache, accumulator);
                } else {
                    throw fb.core.util.assertionError("Unknown operation type: " + operation.type);
                }
            }
        }
    }
    var changes = accumulator.getChanges();
    this.maybeAddValueEvent_(oldViewCache, newViewCache, changes);
    return new fb.core.view.ProcessorResult(newViewCache, changes);
};
fb.core.view.ViewProcessor.prototype.maybeAddValueEvent_ = function(oldViewCache, newViewCache, accumulator) {
    var eventSnap = newViewCache.getEventCache();
    if (eventSnap.isFullyInitialized()) {
        var isLeafOrEmpty = eventSnap.getNode().isLeafNode() || eventSnap.getNode().isEmpty();
        var oldCompleteSnap = oldViewCache.getCompleteEventSnap();
        if (accumulator.length > 0 || !oldViewCache.getEventCache().isFullyInitialized() || isLeafOrEmpty && !eventSnap.getNode().equals((oldCompleteSnap)) || !eventSnap.getNode().getPriority().equals(oldCompleteSnap.getPriority())) {
            accumulator.push(fb.core.view.Change.valueChange((newViewCache.getCompleteEventSnap())));
        }
    }
};
fb.core.view.ViewProcessor.prototype.generateEventCacheAfterServerEvent_ = function(viewCache, changePath, writesCache, source, accumulator) {
    var oldEventSnap = viewCache.getEventCache();
    if (writesCache.shadowingWrite(changePath) != null) {
        return viewCache;
    } else {
        var newEventCache, serverNode;
        if (changePath.isEmpty()) {
            fb.core.util.assert(viewCache.getServerCache().isFullyInitialized(), "If change path is empty, we must have complete server data");
            if (viewCache.getServerCache().isFiltered()) {
                var serverCache = viewCache.getCompleteServerSnap();
                var completeChildren = serverCache instanceof fb.core.snap.ChildrenNode ? serverCache : fb.core.snap.EMPTY_NODE;
                var completeEventChildren = writesCache.calcCompleteEventChildren(completeChildren);
                newEventCache = this.filter_.updateFullNode(viewCache.getEventCache().getNode(), completeEventChildren, accumulator);
            } else {
                var completeNode = (writesCache.calcCompleteEventCache(viewCache.getCompleteServerSnap()));
                newEventCache = this.filter_.updateFullNode(viewCache.getEventCache().getNode(), completeNode, accumulator);
            }
        } else {
            var childKey = changePath.getFront();
            if (childKey == ".priority") {
                fb.core.util.assert(changePath.getLength() == 1, "Can't have a priority with additional path components");
                var oldEventNode = oldEventSnap.getNode();
                serverNode = viewCache.getServerCache().getNode();
                var updatedPriority = writesCache.calcEventCacheAfterServerOverwrite(changePath, oldEventNode, serverNode);
                if (updatedPriority != null) {
                    newEventCache = this.filter_.updatePriority(oldEventNode, updatedPriority);
                } else {
                    newEventCache = oldEventSnap.getNode();
                }
            } else {
                var childChangePath = changePath.popFront();
                var newEventChild;
                if (oldEventSnap.isCompleteForChild(childKey)) {
                    serverNode = viewCache.getServerCache().getNode();
                    var eventChildUpdate = writesCache.calcEventCacheAfterServerOverwrite(changePath, oldEventSnap.getNode(), serverNode);
                    if (eventChildUpdate != null) {
                        newEventChild = oldEventSnap.getNode().getImmediateChild(childKey).updateChild(childChangePath, eventChildUpdate);
                    } else {
                        newEventChild = oldEventSnap.getNode().getImmediateChild(childKey);
                    }
                } else {
                    newEventChild = writesCache.calcCompleteChild(childKey, viewCache.getServerCache());
                }
                if (newEventChild != null) {
                    newEventCache = this.filter_.updateChild(oldEventSnap.getNode(), childKey, newEventChild, childChangePath, source, accumulator);
                } else {
                    newEventCache = oldEventSnap.getNode();
                }
            }
        }
        return viewCache.updateEventSnap(newEventCache, oldEventSnap.isFullyInitialized() || changePath.isEmpty(), this.filter_.filtersNodes());
    }
};
fb.core.view.ViewProcessor.prototype.applyServerOverwrite_ = function(oldViewCache, changePath, changedSnap, writesCache, optCompleteCache, filterServerNode, accumulator) {
    var oldServerSnap = oldViewCache.getServerCache();
    var newServerCache;
    var serverFilter = filterServerNode ? this.filter_ : this.filter_.getIndexedFilter();
    if (changePath.isEmpty()) {
        newServerCache = serverFilter.updateFullNode(oldServerSnap.getNode(), changedSnap, null);
    } else {
        if (serverFilter.filtersNodes() && !oldServerSnap.isFiltered()) {
            var newServerNode = oldServerSnap.getNode().updateChild(changePath, changedSnap);
            newServerCache = serverFilter.updateFullNode(oldServerSnap.getNode(), newServerNode, null);
        } else {
            var childKey = changePath.getFront();
            if (!oldServerSnap.isCompleteForPath(changePath) && changePath.getLength() > 1) {
                return oldViewCache;
            }
            var childChangePath = changePath.popFront();
            var childNode = oldServerSnap.getNode().getImmediateChild(childKey);
            var newChildNode = childNode.updateChild(childChangePath, changedSnap);
            if (childKey == ".priority") {
                newServerCache = serverFilter.updatePriority(oldServerSnap.getNode(), newChildNode);
            } else {
                newServerCache = serverFilter.updateChild(oldServerSnap.getNode(), childKey, newChildNode, childChangePath, fb.core.view.NO_COMPLETE_CHILD_SOURCE, null);
            }
        }
    }
    var newViewCache = oldViewCache.updateServerSnap(newServerCache, oldServerSnap.isFullyInitialized() || changePath.isEmpty(), serverFilter.filtersNodes());
    var source = new fb.core.view.WriteTreeCompleteChildSource(writesCache, newViewCache, optCompleteCache);
    return this.generateEventCacheAfterServerEvent_(newViewCache, changePath, writesCache, source, accumulator);
};
fb.core.view.ViewProcessor.prototype.applyUserOverwrite_ = function(oldViewCache, changePath, changedSnap, writesCache, optCompleteCache, accumulator) {
    var oldEventSnap = oldViewCache.getEventCache();
    var newViewCache, newEventCache;
    var source = new fb.core.view.WriteTreeCompleteChildSource(writesCache, oldViewCache, optCompleteCache);
    if (changePath.isEmpty()) {
        newEventCache = this.filter_.updateFullNode(oldViewCache.getEventCache().getNode(), changedSnap, accumulator);
        newViewCache = oldViewCache.updateEventSnap(newEventCache, true, this.filter_.filtersNodes());
    } else {
        var childKey = changePath.getFront();
        if (childKey === ".priority") {
            newEventCache = this.filter_.updatePriority(oldViewCache.getEventCache().getNode(), changedSnap);
            newViewCache = oldViewCache.updateEventSnap(newEventCache, oldEventSnap.isFullyInitialized(), oldEventSnap.isFiltered());
        } else {
            var childChangePath = changePath.popFront();
            var oldChild = oldEventSnap.getNode().getImmediateChild(childKey);
            var newChild;
            if (childChangePath.isEmpty()) {
                newChild = changedSnap;
            } else {
                var childNode = source.getCompleteChild(childKey);
                if (childNode != null) {
                    if (childChangePath.getBack() === ".priority" && childNode.getChild((childChangePath.parent())).isEmpty()) {
                        newChild = childNode;
                    } else {
                        newChild = childNode.updateChild(childChangePath, changedSnap);
                    }
                } else {
                    newChild = fb.core.snap.EMPTY_NODE;
                }
            }
            if (!oldChild.equals(newChild)) {
                var newEventSnap = this.filter_.updateChild(oldEventSnap.getNode(), childKey, newChild, childChangePath, source, accumulator);
                newViewCache = oldViewCache.updateEventSnap(newEventSnap, oldEventSnap.isFullyInitialized(), this.filter_.filtersNodes());
            } else {
                newViewCache = oldViewCache;
            }
        }
    }
    return newViewCache;
};
fb.core.view.ViewProcessor.cacheHasChild_ = function(viewCache, childKey) {
    return viewCache.getEventCache().isCompleteForChild(childKey);
};
fb.core.view.ViewProcessor.prototype.applyUserMerge_ = function(viewCache, path, changedChildren, writesCache, serverCache, accumulator) {
    var self = this;
    var curViewCache = viewCache;
    changedChildren.foreach(function(relativePath, childNode) {
        var writePath = path.child(relativePath);
        if (fb.core.view.ViewProcessor.cacheHasChild_(viewCache, writePath.getFront())) {
            curViewCache = self.applyUserOverwrite_(curViewCache, writePath, childNode, writesCache, serverCache, accumulator);
        }
    });
    changedChildren.foreach(function(relativePath, childNode) {
        var writePath = path.child(relativePath);
        if (!fb.core.view.ViewProcessor.cacheHasChild_(viewCache, writePath.getFront())) {
            curViewCache = self.applyUserOverwrite_(curViewCache, writePath, childNode, writesCache, serverCache, accumulator);
        }
    });
    return curViewCache;
};
fb.core.view.ViewProcessor.prototype.applyMerge_ = function(node, merge) {
    merge.foreach(function(relativePath, childNode) {
        node = node.updateChild(relativePath, childNode);
    });
    return node;
};
fb.core.view.ViewProcessor.prototype.applyServerMerge_ = function(viewCache, path, changedChildren, writesCache, serverCache, filterServerNode, accumulator) {
    if (viewCache.getServerCache().getNode().isEmpty() && !viewCache.getServerCache().isFullyInitialized()) {
        return viewCache;
    }
    var curViewCache = viewCache;
    var viewMergeTree;
    if (path.isEmpty()) {
        viewMergeTree = changedChildren;
    } else {
        viewMergeTree = fb.core.util.ImmutableTree.Empty.setTree(path, changedChildren);
    }
    var serverNode = viewCache.getServerCache().getNode();
    var self = this;
    viewMergeTree.children.inorderTraversal(function(childKey, childTree) {
        if (serverNode.hasChild(childKey)) {
            var serverChild = viewCache.getServerCache().getNode().getImmediateChild(childKey);
            var newChild = self.applyMerge_(serverChild, childTree);
            curViewCache = self.applyServerOverwrite_(curViewCache, new fb.core.util.Path(childKey), newChild, writesCache, serverCache, filterServerNode, accumulator);
        }
    });
    viewMergeTree.children.inorderTraversal(function(childKey, childMergeTree) {
        var isUnknownDeepMerge = !viewCache.getServerCache().isCompleteForChild(childKey) && childMergeTree.value == null;
        if (!serverNode.hasChild(childKey) && !isUnknownDeepMerge) {
            var serverChild = viewCache.getServerCache().getNode().getImmediateChild(childKey);
            var newChild = self.applyMerge_(serverChild, childMergeTree);
            curViewCache = self.applyServerOverwrite_(curViewCache, new fb.core.util.Path(childKey), newChild, writesCache, serverCache, filterServerNode, accumulator);
        }
    });
    return curViewCache;
};
fb.core.view.ViewProcessor.prototype.ackUserWrite_ = function(viewCache, ackPath, affectedTree, writesCache, optCompleteCache, accumulator) {
    if (writesCache.shadowingWrite(ackPath) != null) {
        return viewCache;
    }
    var filterServerNode = viewCache.getServerCache().isFiltered();
    var serverCache = viewCache.getServerCache();
    if (affectedTree.value != null) {
        if (ackPath.isEmpty() && serverCache.isFullyInitialized() || serverCache.isCompleteForPath(ackPath)) {
            return this.applyServerOverwrite_(viewCache, ackPath, serverCache.getNode().getChild(ackPath), writesCache, optCompleteCache, filterServerNode, accumulator);
        } else {
            if (ackPath.isEmpty()) {
                var changedChildren = (fb.core.util.ImmutableTree.Empty);
                serverCache.getNode().forEachChild(fb.core.snap.KeyIndex, function(name, node) {
                    changedChildren = changedChildren.set(new fb.core.util.Path(name), node);
                });
                return this.applyServerMerge_(viewCache, ackPath, changedChildren, writesCache, optCompleteCache, filterServerNode, accumulator);
            } else {
                return viewCache;
            }
        }
    } else {
        var changedChildren = (fb.core.util.ImmutableTree.Empty);
        affectedTree.foreach(function(mergePath, value) {
            var serverCachePath = ackPath.child(mergePath);
            if (serverCache.isCompleteForPath(serverCachePath)) {
                changedChildren = changedChildren.set(mergePath, serverCache.getNode().getChild(serverCachePath));
            }
        });
        return this.applyServerMerge_(viewCache, ackPath, changedChildren, writesCache, optCompleteCache, filterServerNode, accumulator);
    }
};
fb.core.view.ViewProcessor.prototype.revertUserWrite_ = function(viewCache, path, writesCache, optCompleteServerCache, accumulator) {
    var complete;
    if (writesCache.shadowingWrite(path) != null) {
        return viewCache;
    } else {
        var source = new fb.core.view.WriteTreeCompleteChildSource(writesCache, viewCache, optCompleteServerCache);
        var oldEventCache = viewCache.getEventCache().getNode();
        var newEventCache;
        if (path.isEmpty() || path.getFront() === ".priority") {
            var newNode;
            if (viewCache.getServerCache().isFullyInitialized()) {
                newNode = writesCache.calcCompleteEventCache(viewCache.getCompleteServerSnap());
            } else {
                var serverChildren = viewCache.getServerCache().getNode();
                fb.core.util.assert(serverChildren instanceof fb.core.snap.ChildrenNode, "serverChildren would be complete if leaf node");
                newNode = writesCache.calcCompleteEventChildren((serverChildren));
            }
            newNode = (newNode);
            newEventCache = this.filter_.updateFullNode(oldEventCache, newNode, accumulator);
        } else {
            var childKey = path.getFront();
            var newChild = writesCache.calcCompleteChild(childKey, viewCache.getServerCache());
            if (newChild == null && viewCache.getServerCache().isCompleteForChild(childKey)) {
                newChild = oldEventCache.getImmediateChild(childKey);
            }
            if (newChild != null) {
                newEventCache = this.filter_.updateChild(oldEventCache, childKey, newChild, path.popFront(), source, accumulator);
            } else {
                if (viewCache.getEventCache().getNode().hasChild(childKey)) {
                    newEventCache = this.filter_.updateChild(oldEventCache, childKey, fb.core.snap.EMPTY_NODE, path.popFront(), source, accumulator);
                } else {
                    newEventCache = oldEventCache;
                }
            }
            if (newEventCache.isEmpty() && viewCache.getServerCache().isFullyInitialized()) {
                complete = writesCache.calcCompleteEventCache(viewCache.getCompleteServerSnap());
                if (complete.isLeafNode()) {
                    newEventCache = this.filter_.updateFullNode(newEventCache, complete, accumulator);
                }
            }
        }
        complete = viewCache.getServerCache().isFullyInitialized() || writesCache.shadowingWrite(fb.core.util.Path.Empty) != null;
        return viewCache.updateEventSnap(newEventCache, complete, this.filter_.filtersNodes());
    }
};
fb.core.view.ViewProcessor.prototype.listenComplete_ = function(viewCache, path, writesCache, serverCache, accumulator) {
    var oldServerNode = viewCache.getServerCache();
    var newViewCache = viewCache.updateServerSnap(oldServerNode.getNode(), oldServerNode.isFullyInitialized() || path.isEmpty(), oldServerNode.isFiltered());
    return this.generateEventCacheAfterServerEvent_(newViewCache, path, writesCache, fb.core.view.NO_COMPLETE_CHILD_SOURCE, accumulator);
};
goog.provide("fb.core.snap.Index");
goog.provide("fb.core.snap.KeyIndex");
goog.provide("fb.core.snap.PathIndex");
goog.provide("fb.core.snap.PriorityIndex");
goog.provide("fb.core.snap.ValueIndex");
goog.require("fb.core.util");
fb.core.snap.Index = function() {
};
fb.core.snap.Index.FallbackType;
fb.core.snap.Index.Fallback = {};
fb.core.snap.Index.prototype.compare = goog.abstractMethod;
fb.core.snap.Index.prototype.isDefinedOn = goog.abstractMethod;
fb.core.snap.Index.prototype.getCompare = function() {
    return goog.bind(this.compare, this);
};
fb.core.snap.Index.prototype.indexedValueChanged = function(oldNode, newNode) {
    var oldWrapped = new fb.core.snap.NamedNode(fb.core.util.MIN_NAME, oldNode);
    var newWrapped = new fb.core.snap.NamedNode(fb.core.util.MIN_NAME, newNode);
    return this.compare(oldWrapped, newWrapped) !== 0;
};
fb.core.snap.Index.prototype.minPost = function() {
    return fb.core.snap.NamedNode.MIN;
};
fb.core.snap.Index.prototype.maxPost = goog.abstractMethod;
fb.core.snap.Index.prototype.makePost = goog.abstractMethod;
fb.core.snap.Index.prototype.toString = goog.abstractMethod;
fb.core.snap.PathIndex = function(indexPath) {
    fb.core.snap.Index.call(this);
    fb.core.util.assert(!indexPath.isEmpty() && indexPath.getFront() !== ".priority", "Can't create PathIndex with empty path or .priority key");
    this.indexPath_ = indexPath;
};
goog.inherits(fb.core.snap.PathIndex, fb.core.snap.Index);
fb.core.snap.PathIndex.prototype.extractChild = function(snap) {
    return snap.getChild(this.indexPath_);
};
fb.core.snap.PathIndex.prototype.isDefinedOn = function(node) {
    return!node.getChild(this.indexPath_).isEmpty();
};
fb.core.snap.PathIndex.prototype.compare = function(a, b) {
    var aChild = this.extractChild(a.node);
    var bChild = this.extractChild(b.node);
    var indexCmp = aChild.compareTo(bChild);
    if (indexCmp === 0) {
        return fb.core.util.nameCompare(a.name, b.name);
    } else {
        return indexCmp;
    }
};
fb.core.snap.PathIndex.prototype.makePost = function(indexValue, name) {
    var valueNode = fb.core.snap.NodeFromJSON(indexValue);
    var node = fb.core.snap.EMPTY_NODE.updateChild(this.indexPath_, valueNode);
    return new fb.core.snap.NamedNode(name, node);
};
fb.core.snap.PathIndex.prototype.maxPost = function() {
    var node = fb.core.snap.EMPTY_NODE.updateChild(this.indexPath_, fb.core.snap.MAX_NODE);
    return new fb.core.snap.NamedNode(fb.core.util.MAX_NAME, node);
};
fb.core.snap.PathIndex.prototype.toString = function() {
    return this.indexPath_.slice().join("/");
};
fb.core.snap.PriorityIndex_ = function() {
    fb.core.snap.Index.call(this);
};
goog.inherits(fb.core.snap.PriorityIndex_, fb.core.snap.Index);
fb.core.snap.PriorityIndex_.prototype.compare = function(a, b) {
    var aPriority = a.node.getPriority();
    var bPriority = b.node.getPriority();
    var indexCmp = aPriority.compareTo(bPriority);
    if (indexCmp === 0) {
        return fb.core.util.nameCompare(a.name, b.name);
    } else {
        return indexCmp;
    }
};
fb.core.snap.PriorityIndex_.prototype.isDefinedOn = function(node) {
    return!node.getPriority().isEmpty();
};
fb.core.snap.PriorityIndex_.prototype.indexedValueChanged = function(oldNode, newNode) {
    return!oldNode.getPriority().equals(newNode.getPriority());
};
fb.core.snap.PriorityIndex_.prototype.minPost = function() {
    return fb.core.snap.NamedNode.MIN;
};
fb.core.snap.PriorityIndex_.prototype.maxPost = function() {
    return new fb.core.snap.NamedNode(fb.core.util.MAX_NAME, new fb.core.snap.LeafNode("[PRIORITY-POST]", fb.core.snap.MAX_NODE));
};
fb.core.snap.PriorityIndex_.prototype.makePost = function(indexValue, name) {
    var priorityNode = fb.core.snap.NodeFromJSON(indexValue);
    return new fb.core.snap.NamedNode(name, new fb.core.snap.LeafNode("[PRIORITY-POST]", priorityNode));
};
fb.core.snap.PriorityIndex_.prototype.toString = function() {
    return ".priority";
};
fb.core.snap.PriorityIndex = new fb.core.snap.PriorityIndex_;
fb.core.snap.KeyIndex_ = function() {
    fb.core.snap.Index.call(this);
};
goog.inherits(fb.core.snap.KeyIndex_, fb.core.snap.Index);
fb.core.snap.KeyIndex_.prototype.compare = function(a, b) {
    return fb.core.util.nameCompare(a.name, b.name);
};
fb.core.snap.KeyIndex_.prototype.isDefinedOn = function(node) {
    throw fb.core.util.assertionError("KeyIndex.isDefinedOn not expected to be called.");
};
fb.core.snap.KeyIndex_.prototype.indexedValueChanged = function(oldNode, newNode) {
    return false;
};
fb.core.snap.KeyIndex_.prototype.minPost = function() {
    return fb.core.snap.NamedNode.MIN;
};
fb.core.snap.KeyIndex_.prototype.maxPost = function() {
    return new fb.core.snap.NamedNode(fb.core.util.MAX_NAME, fb.core.snap.EMPTY_NODE);
};
fb.core.snap.KeyIndex_.prototype.makePost = function(indexValue, name) {
    fb.core.util.assert(goog.isString(indexValue), "KeyIndex indexValue must always be a string.");
    return new fb.core.snap.NamedNode((indexValue), fb.core.snap.EMPTY_NODE);
};
fb.core.snap.KeyIndex_.prototype.toString = function() {
    return ".key";
};
fb.core.snap.KeyIndex = new fb.core.snap.KeyIndex_;
fb.core.snap.ValueIndex_ = function() {
    fb.core.snap.Index.call(this);
};
goog.inherits(fb.core.snap.ValueIndex_, fb.core.snap.Index);
fb.core.snap.ValueIndex_.prototype.compare = function(a, b) {
    var indexCmp = a.node.compareTo(b.node);
    if (indexCmp === 0) {
        return fb.core.util.nameCompare(a.name, b.name);
    } else {
        return indexCmp;
    }
};
fb.core.snap.ValueIndex_.prototype.isDefinedOn = function(node) {
    return true;
};
fb.core.snap.ValueIndex_.prototype.indexedValueChanged = function(oldNode, newNode) {
    return!oldNode.equals(newNode);
};
fb.core.snap.ValueIndex_.prototype.minPost = function() {
    return fb.core.snap.NamedNode.MIN;
};
fb.core.snap.ValueIndex_.prototype.maxPost = function() {
    return fb.core.snap.NamedNode.MAX;
};
fb.core.snap.ValueIndex_.prototype.makePost = function(indexValue, name) {
    var valueNode = fb.core.snap.NodeFromJSON(indexValue);
    return new fb.core.snap.NamedNode(name, valueNode);
};
fb.core.snap.ValueIndex_.prototype.toString = function() {
    return ".value";
};
fb.core.snap.ValueIndex = new fb.core.snap.ValueIndex_;
goog.provide("fb.core.view.QueryParams");
goog.require("fb.core.snap.Index");
goog.require("fb.core.snap.PriorityIndex");
goog.require("fb.core.util");
goog.require("fb.core.view.filter.IndexedFilter");
goog.require("fb.core.view.filter.LimitedFilter");
goog.require("fb.core.view.filter.NodeFilter");
goog.require("fb.core.view.filter.RangedFilter");
fb.core.view.QueryParams = function() {
    this.limitSet_ = false;
    this.startSet_ = false;
    this.startNameSet_ = false;
    this.endSet_ = false;
    this.endNameSet_ = false;
    this.limit_ = 0;
    this.viewFrom_ = "";
    this.indexStartValue_ = null;
    this.indexStartName_ = "";
    this.indexEndValue_ = null;
    this.indexEndName_ = "";
    this.index_ = fb.core.snap.PriorityIndex;
};
fb.core.view.QueryParams.WIRE_PROTOCOL_CONSTANTS_ = {INDEX_START_VALUE:"sp", INDEX_START_NAME:"sn", INDEX_END_VALUE:"ep", INDEX_END_NAME:"en", LIMIT:"l", VIEW_FROM:"vf", VIEW_FROM_LEFT:"l", VIEW_FROM_RIGHT:"r", INDEX:"i"};
fb.core.view.QueryParams.REST_QUERY_CONSTANTS_ = {ORDER_BY:"orderBy", PRIORITY_INDEX:"$priority", VALUE_INDEX:"$value", KEY_INDEX:"$key", START_AT:"startAt", END_AT:"endAt", LIMIT_TO_FIRST:"limitToFirst", LIMIT_TO_LAST:"limitToLast"};
fb.core.view.QueryParams.DEFAULT = new fb.core.view.QueryParams;
fb.core.view.QueryParams.prototype.hasStart = function() {
    return this.startSet_;
};
fb.core.view.QueryParams.prototype.isViewFromLeft = function() {
    if (this.viewFrom_ === "") {
        return this.startSet_;
    } else {
        return this.viewFrom_ === fb.core.view.QueryParams.WIRE_PROTOCOL_CONSTANTS_.VIEW_FROM_LEFT;
    }
};
fb.core.view.QueryParams.prototype.getIndexStartValue = function() {
    fb.core.util.assert(this.startSet_, "Only valid if start has been set");
    return this.indexStartValue_;
};
fb.core.view.QueryParams.prototype.getIndexStartName = function() {
    fb.core.util.assert(this.startSet_, "Only valid if start has been set");
    if (this.startNameSet_) {
        return this.indexStartName_;
    } else {
        return fb.core.util.MIN_NAME;
    }
};
fb.core.view.QueryParams.prototype.hasEnd = function() {
    return this.endSet_;
};
fb.core.view.QueryParams.prototype.getIndexEndValue = function() {
    fb.core.util.assert(this.endSet_, "Only valid if end has been set");
    return this.indexEndValue_;
};
fb.core.view.QueryParams.prototype.getIndexEndName = function() {
    fb.core.util.assert(this.endSet_, "Only valid if end has been set");
    if (this.endNameSet_) {
        return this.indexEndName_;
    } else {
        return fb.core.util.MAX_NAME;
    }
};
fb.core.view.QueryParams.prototype.hasLimit = function() {
    return this.limitSet_;
};
fb.core.view.QueryParams.prototype.hasAnchoredLimit = function() {
    return this.limitSet_ && this.viewFrom_ !== "";
};
fb.core.view.QueryParams.prototype.getLimit = function() {
    fb.core.util.assert(this.limitSet_, "Only valid if limit has been set");
    return this.limit_;
};
fb.core.view.QueryParams.prototype.getIndex = function() {
    return this.index_;
};
fb.core.view.QueryParams.prototype.copy_ = function() {
    var copy = new fb.core.view.QueryParams;
    copy.limitSet_ = this.limitSet_;
    copy.limit_ = this.limit_;
    copy.startSet_ = this.startSet_;
    copy.indexStartValue_ = this.indexStartValue_;
    copy.startNameSet_ = this.startNameSet_;
    copy.indexStartName_ = this.indexStartName_;
    copy.endSet_ = this.endSet_;
    copy.indexEndValue_ = this.indexEndValue_;
    copy.endNameSet_ = this.endNameSet_;
    copy.indexEndName_ = this.indexEndName_;
    copy.index_ = this.index_;
    return copy;
};
fb.core.view.QueryParams.prototype.limit = function(newLimit) {
    var newParams = this.copy_();
    newParams.limitSet_ = true;
    newParams.limit_ = newLimit;
    newParams.viewFrom_ = "";
    return newParams;
};
fb.core.view.QueryParams.prototype.limitToFirst = function(newLimit) {
    var newParams = this.copy_();
    newParams.limitSet_ = true;
    newParams.limit_ = newLimit;
    newParams.viewFrom_ = fb.core.view.QueryParams.WIRE_PROTOCOL_CONSTANTS_.VIEW_FROM_LEFT;
    return newParams;
};
fb.core.view.QueryParams.prototype.limitToLast = function(newLimit) {
    var newParams = this.copy_();
    newParams.limitSet_ = true;
    newParams.limit_ = newLimit;
    newParams.viewFrom_ = fb.core.view.QueryParams.WIRE_PROTOCOL_CONSTANTS_.VIEW_FROM_RIGHT;
    return newParams;
};
fb.core.view.QueryParams.prototype.startAt = function(indexValue, key) {
    var newParams = this.copy_();
    newParams.startSet_ = true;
    if (!goog.isDef(indexValue)) {
        indexValue = null;
    }
    newParams.indexStartValue_ = indexValue;
    if (key != null) {
        newParams.startNameSet_ = true;
        newParams.indexStartName_ = key;
    } else {
        newParams.startNameSet_ = false;
        newParams.indexStartName_ = "";
    }
    return newParams;
};
fb.core.view.QueryParams.prototype.endAt = function(indexValue, key) {
    var newParams = this.copy_();
    newParams.endSet_ = true;
    if (!goog.isDef(indexValue)) {
        indexValue = null;
    }
    newParams.indexEndValue_ = indexValue;
    if (goog.isDef(key)) {
        newParams.endNameSet_ = true;
        newParams.indexEndName_ = key;
    } else {
        newParams.startEndSet_ = false;
        newParams.indexEndName_ = "";
    }
    return newParams;
};
fb.core.view.QueryParams.prototype.orderBy = function(index) {
    var newParams = this.copy_();
    newParams.index_ = index;
    return newParams;
};
fb.core.view.QueryParams.prototype.getQueryObject = function() {
    var WIRE_PROTOCOL_CONSTANTS = fb.core.view.QueryParams.WIRE_PROTOCOL_CONSTANTS_;
    var obj = {};
    if (this.startSet_) {
        obj[WIRE_PROTOCOL_CONSTANTS.INDEX_START_VALUE] = this.indexStartValue_;
        if (this.startNameSet_) {
            obj[WIRE_PROTOCOL_CONSTANTS.INDEX_START_NAME] = this.indexStartName_;
        }
    }
    if (this.endSet_) {
        obj[WIRE_PROTOCOL_CONSTANTS.INDEX_END_VALUE] = this.indexEndValue_;
        if (this.endNameSet_) {
            obj[WIRE_PROTOCOL_CONSTANTS.INDEX_END_NAME] = this.indexEndName_;
        }
    }
    if (this.limitSet_) {
        obj[WIRE_PROTOCOL_CONSTANTS.LIMIT] = this.limit_;
        var viewFrom = this.viewFrom_;
        if (viewFrom === "") {
            if (this.isViewFromLeft()) {
                viewFrom = WIRE_PROTOCOL_CONSTANTS.VIEW_FROM_LEFT;
            } else {
                viewFrom = WIRE_PROTOCOL_CONSTANTS.VIEW_FROM_RIGHT;
            }
        }
        obj[WIRE_PROTOCOL_CONSTANTS.VIEW_FROM] = viewFrom;
    }
    if (this.index_ !== fb.core.snap.PriorityIndex) {
        obj[WIRE_PROTOCOL_CONSTANTS.INDEX] = this.index_.toString();
    }
    return obj;
};
fb.core.view.QueryParams.prototype.loadsAllData = function() {
    return!(this.startSet_ || this.endSet_ || this.limitSet_);
};
fb.core.view.QueryParams.prototype.isDefault = function() {
    return this.loadsAllData() && this.index_ == fb.core.snap.PriorityIndex;
};
fb.core.view.QueryParams.prototype.getNodeFilter = function() {
    if (this.loadsAllData()) {
        return new fb.core.view.filter.IndexedFilter(this.getIndex());
    } else {
        if (this.hasLimit()) {
            return new fb.core.view.filter.LimitedFilter(this);
        } else {
            return new fb.core.view.filter.RangedFilter(this);
        }
    }
};
fb.core.view.QueryParams.prototype.toRestQueryStringParameters = function() {
    var REST_CONSTANTS = fb.core.view.QueryParams.REST_QUERY_CONSTANTS_;
    var qs = {};
    if (this.isDefault()) {
        return qs;
    }
    var orderBy;
    if (this.index_ === fb.core.snap.PriorityIndex) {
        orderBy = REST_CONSTANTS.PRIORITY_INDEX;
    } else {
        if (this.index_ === fb.core.snap.ValueIndex) {
            orderBy = REST_CONSTANTS.VALUE_INDEX;
        } else {
            if (this.index_ === fb.core.snap.KeyIndex) {
                orderBy = REST_CONSTANTS.KEY_INDEX;
            } else {
                fb.core.util.assert(this.index_ instanceof fb.core.snap.PathIndex, "Unrecognized index type!");
                orderBy = this.index_.toString();
            }
        }
    }
    qs[REST_CONSTANTS.ORDER_BY] = fb.util.json.stringify(orderBy);
    if (this.startSet_) {
        qs[REST_CONSTANTS.START_AT] = fb.util.json.stringify(this.indexStartValue_);
        if (this.startNameSet_) {
            qs[REST_CONSTANTS.START_AT] += "," + fb.util.json.stringify(this.indexStartName_);
        }
    }
    if (this.endSet_) {
        qs[REST_CONSTANTS.END_AT] = fb.util.json.stringify(this.indexEndValue_);
        if (this.endNameSet_) {
            qs[REST_CONSTANTS.END_AT] += "," + fb.util.json.stringify(this.indexEndName_);
        }
    }
    if (this.limitSet_) {
        if (this.isViewFromLeft()) {
            qs[REST_CONSTANTS.LIMIT_TO_FIRST] = this.limit_;
        } else {
            qs[REST_CONSTANTS.LIMIT_TO_LAST] = this.limit_;
        }
    }
    return qs;
};
if (goog.DEBUG) {
    fb.core.view.QueryParams.prototype.toString = function() {
        return fb.util.json.stringify(this.getQueryObject());
    };
}
;goog.provide("fb.core.snap.IndexMap");
goog.require("fb.core.snap.Index");
goog.require("fb.core.util");
fb.core.snap.IndexMap = function(indexes, indexSet) {
    this.indexes_ = indexes;
    this.indexSet_ = indexSet;
};
fb.core.snap.IndexMap.prototype.get = function(indexKey) {
    var sortedMap = fb.util.obj.get(this.indexes_, indexKey);
    if (!sortedMap) {
        throw new Error("No index defined for " + indexKey);
    }
    if (sortedMap === fb.core.snap.Index.Fallback) {
        return null;
    } else {
        return sortedMap;
    }
};
fb.core.snap.IndexMap.prototype.hasIndex = function(indexDefinition) {
    return goog.object.contains(this.indexSet_, indexDefinition.toString());
};
fb.core.snap.IndexMap.prototype.addIndex = function(indexDefinition, existingChildren) {
    fb.core.util.assert(indexDefinition !== fb.core.snap.KeyIndex, "KeyIndex always exists and isn't meant to be added to the IndexMap.");
    var childList = [];
    var sawIndexedValue = false;
    var iter = existingChildren.getIterator(fb.core.snap.NamedNode.Wrap);
    var next = iter.getNext();
    while (next) {
        sawIndexedValue = sawIndexedValue || indexDefinition.isDefinedOn(next.node);
        childList.push(next);
        next = iter.getNext();
    }
    var newIndex;
    if (sawIndexedValue) {
        newIndex = fb.core.snap.buildChildSet(childList, indexDefinition.getCompare());
    } else {
        newIndex = fb.core.snap.Index.Fallback;
    }
    var indexName = indexDefinition.toString();
    var newIndexSet = goog.object.clone(this.indexSet_);
    newIndexSet[indexName] = indexDefinition;
    var newIndexes = goog.object.clone(this.indexes_);
    newIndexes[indexName] = newIndex;
    return new fb.core.snap.IndexMap(newIndexes, newIndexSet);
};
fb.core.snap.IndexMap.prototype.addToIndexes = function(namedNode, existingChildren) {
    var self = this;
    var newIndexes = goog.object.map(this.indexes_, function(indexedChildren, indexName) {
        var index = fb.util.obj.get(self.indexSet_, indexName);
        fb.core.util.assert(index, "Missing index implementation for " + indexName);
        if (indexedChildren === fb.core.snap.Index.Fallback) {
            if (index.isDefinedOn(namedNode.node)) {
                var childList = [];
                var iter = existingChildren.getIterator(fb.core.snap.NamedNode.Wrap);
                var next = iter.getNext();
                while (next) {
                    if (next.name != namedNode.name) {
                        childList.push(next);
                    }
                    next = iter.getNext();
                }
                childList.push(namedNode);
                return fb.core.snap.buildChildSet(childList, index.getCompare());
            } else {
                return fb.core.snap.Index.Fallback;
            }
        } else {
            var existingSnap = existingChildren.get(namedNode.name);
            var newChildren = indexedChildren;
            if (existingSnap) {
                newChildren = newChildren.remove(new fb.core.snap.NamedNode(namedNode.name, existingSnap));
            }
            return newChildren.insert(namedNode, namedNode.node);
        }
    });
    return new fb.core.snap.IndexMap(newIndexes, this.indexSet_);
};
fb.core.snap.IndexMap.prototype.removeFromIndexes = function(namedNode, existingChildren) {
    var newIndexes = goog.object.map(this.indexes_, function(indexedChildren) {
        if (indexedChildren === fb.core.snap.Index.Fallback) {
            return indexedChildren;
        } else {
            var existingSnap = existingChildren.get(namedNode.name);
            if (existingSnap) {
                return indexedChildren.remove(new fb.core.snap.NamedNode(namedNode.name, existingSnap));
            } else {
                return indexedChildren;
            }
        }
    });
    return new fb.core.snap.IndexMap(newIndexes, this.indexSet_);
};
fb.core.snap.IndexMap.Default = new fb.core.snap.IndexMap({".priority":fb.core.snap.Index.Fallback}, {".priority":fb.core.snap.PriorityIndex});
goog.provide("fb.core.snap.LeafNode");
goog.require("fb.core.snap.Node");
goog.require("fb.core.util");
fb.core.snap.LeafNode = goog.defineClass(null, {constructor:function(value, opt_priorityNode) {
    this.value_ = value;
    fb.core.util.assert(goog.isDef(this.value_) && this.value_ !== null, "LeafNode shouldn't be created with null/undefined value.");
    this.priorityNode_ = opt_priorityNode || fb.core.snap.EMPTY_NODE;
    fb.core.snap.validatePriorityNode(this.priorityNode_);
    this.lazyHash_ = null;
}, statics:{VALUE_TYPE_ORDER:["object", "boolean", "number", "string"]}, isLeafNode:function() {
    return true;
}, getPriority:function() {
    return this.priorityNode_;
}, updatePriority:function(newPriorityNode) {
    return new fb.core.snap.LeafNode(this.value_, newPriorityNode);
}, getImmediateChild:function(childName) {
    if (childName === ".priority") {
        return this.priorityNode_;
    } else {
        return fb.core.snap.EMPTY_NODE;
    }
}, getChild:function(path) {
    if (path.isEmpty()) {
        return this;
    } else {
        if (path.getFront() === ".priority") {
            return this.priorityNode_;
        } else {
            return fb.core.snap.EMPTY_NODE;
        }
    }
}, hasChild:function() {
    return false;
}, getPredecessorChildName:function(childName, childNode) {
    return null;
}, updateImmediateChild:function(childName, newChildNode) {
    if (childName === ".priority") {
        return this.updatePriority(newChildNode);
    } else {
        if (newChildNode.isEmpty() && childName !== ".priority") {
            return this;
        } else {
            return fb.core.snap.EMPTY_NODE.updateImmediateChild(childName, newChildNode).updatePriority(this.priorityNode_);
        }
    }
}, updateChild:function(path, newChildNode) {
    var front = path.getFront();
    if (front === null) {
        return newChildNode;
    } else {
        if (newChildNode.isEmpty() && front !== ".priority") {
            return this;
        } else {
            fb.core.util.assert(front !== ".priority" || path.getLength() === 1, ".priority must be the last token in a path");
            return this.updateImmediateChild(front, fb.core.snap.EMPTY_NODE.updateChild(path.popFront(), newChildNode));
        }
    }
}, isEmpty:function() {
    return false;
}, numChildren:function() {
    return 0;
}, forEachChild:function(index, action) {
    return false;
}, val:function(opt_exportFormat) {
    if (opt_exportFormat && !this.getPriority().isEmpty()) {
        return{".value":this.getValue(), ".priority":this.getPriority().val()};
    } else {
        return this.getValue();
    }
}, hash:function() {
    if (this.lazyHash_ === null) {
        var toHash = "";
        if (!this.priorityNode_.isEmpty()) {
            toHash += "priority:" + fb.core.snap.priorityHashText((this.priorityNode_.val())) + ":";
        }
        var type = typeof this.value_;
        toHash += type + ":";
        if (type === "number") {
            toHash += fb.core.util.doubleToIEEE754String((this.value_));
        } else {
            toHash += this.value_;
        }
        this.lazyHash_ = fb.core.util.sha1(toHash);
    }
    return(this.lazyHash_);
}, getValue:function() {
    return this.value_;
}, compareTo:function(other) {
    if (other === fb.core.snap.EMPTY_NODE) {
        return 1;
    } else {
        if (other instanceof fb.core.snap.ChildrenNode) {
            return-1;
        } else {
            fb.core.util.assert(other.isLeafNode(), "Unknown node type");
            return this.compareToLeafNode_((other));
        }
    }
}, compareToLeafNode_:function(otherLeaf) {
    var otherLeafType = typeof otherLeaf.value_;
    var thisLeafType = typeof this.value_;
    var otherIndex = goog.array.indexOf(fb.core.snap.LeafNode.VALUE_TYPE_ORDER, otherLeafType);
    var thisIndex = goog.array.indexOf(fb.core.snap.LeafNode.VALUE_TYPE_ORDER, thisLeafType);
    fb.core.util.assert(otherIndex >= 0, "Unknown leaf type: " + otherLeafType);
    fb.core.util.assert(thisIndex >= 0, "Unknown leaf type: " + thisLeafType);
    if (otherIndex === thisIndex) {
        if (thisLeafType === "object") {
            return 0;
        } else {
            if (this.value_ < otherLeaf.value_) {
                return-1;
            } else {
                if (this.value_ === otherLeaf.value_) {
                    return 0;
                } else {
                    return 1;
                }
            }
        }
    } else {
        return thisIndex - otherIndex;
    }
}, withIndex:function() {
    return this;
}, isIndexed:function() {
    return true;
}, equals:function(other) {
    if (other === this) {
        return true;
    } else {
        if (other.isLeafNode()) {
            var otherLeaf = (other);
            return this.value_ === otherLeaf.value_ && this.priorityNode_.equals(otherLeaf.priorityNode_);
        } else {
            return false;
        }
    }
}});
if (goog.DEBUG) {
    fb.core.snap.LeafNode.prototype.toString = function() {
        return fb.util.json.stringify(this.val(true));
    };
}
;goog.provide("fb.core.snap.ChildrenNode");
goog.require("fb.core.snap.IndexMap");
goog.require("fb.core.snap.LeafNode");
goog.require("fb.core.snap.NamedNode");
goog.require("fb.core.snap.Node");
goog.require("fb.core.snap.PriorityIndex");
goog.require("fb.core.snap.comparators");
goog.require("fb.core.util");
goog.require("fb.core.util.SortedMap");
fb.core.snap.ChildrenNode = function(children, priorityNode, indexMap) {
    this.children_ = children;
    this.priorityNode_ = priorityNode;
    if (this.priorityNode_) {
        fb.core.snap.validatePriorityNode(this.priorityNode_);
    }
    if (children.isEmpty()) {
        fb.core.util.assert(!this.priorityNode_ || this.priorityNode_.isEmpty(), "An empty node cannot have a priority");
    }
    this.indexMap_ = indexMap;
    this.lazyHash_ = null;
};
fb.core.snap.ChildrenNode.prototype.isLeafNode = function() {
    return false;
};
fb.core.snap.ChildrenNode.prototype.getPriority = function() {
    return this.priorityNode_ || fb.core.snap.EMPTY_NODE;
};
fb.core.snap.ChildrenNode.prototype.updatePriority = function(newPriorityNode) {
    if (this.children_.isEmpty()) {
        return this;
    } else {
        return new fb.core.snap.ChildrenNode(this.children_, newPriorityNode, this.indexMap_);
    }
};
fb.core.snap.ChildrenNode.prototype.getImmediateChild = function(childName) {
    if (childName === ".priority") {
        return this.getPriority();
    } else {
        var child = this.children_.get(childName);
        return child === null ? fb.core.snap.EMPTY_NODE : child;
    }
};
fb.core.snap.ChildrenNode.prototype.getChild = function(path) {
    var front = path.getFront();
    if (front === null) {
        return this;
    }
    return this.getImmediateChild(front).getChild(path.popFront());
};
fb.core.snap.ChildrenNode.prototype.hasChild = function(childName) {
    return this.children_.get(childName) !== null;
};
fb.core.snap.ChildrenNode.prototype.updateImmediateChild = function(childName, newChildNode) {
    fb.core.util.assert(newChildNode, "We should always be passing snapshot nodes");
    if (childName === ".priority") {
        return this.updatePriority(newChildNode);
    } else {
        var namedNode = new fb.core.snap.NamedNode(childName, newChildNode);
        var newChildren, newIndexMap, newPriority;
        if (newChildNode.isEmpty()) {
            newChildren = this.children_.remove(childName);
            newIndexMap = this.indexMap_.removeFromIndexes(namedNode, this.children_);
        } else {
            newChildren = this.children_.insert(childName, newChildNode);
            newIndexMap = this.indexMap_.addToIndexes(namedNode, this.children_);
        }
        newPriority = newChildren.isEmpty() ? fb.core.snap.EMPTY_NODE : this.priorityNode_;
        return new fb.core.snap.ChildrenNode(newChildren, newPriority, newIndexMap);
    }
};
fb.core.snap.ChildrenNode.prototype.updateChild = function(path, newChildNode) {
    var front = path.getFront();
    if (front === null) {
        return newChildNode;
    } else {
        fb.core.util.assert(path.getFront() !== ".priority" || path.getLength() === 1, ".priority must be the last token in a path");
        var newImmediateChild = this.getImmediateChild(front).updateChild(path.popFront(), newChildNode);
        return this.updateImmediateChild(front, newImmediateChild);
    }
};
fb.core.snap.ChildrenNode.prototype.isEmpty = function() {
    return this.children_.isEmpty();
};
fb.core.snap.ChildrenNode.prototype.numChildren = function() {
    return this.children_.count();
};
fb.core.snap.ChildrenNode.INTEGER_REGEXP_ = /^(0|[1-9]\d*)$/;
fb.core.snap.ChildrenNode.prototype.val = function(opt_exportFormat) {
    if (this.isEmpty()) {
        return null;
    }
    var obj = {};
    var numKeys = 0, maxKey = 0, allIntegerKeys = true;
    this.forEachChild(fb.core.snap.PriorityIndex, function(key, childNode) {
        obj[key] = childNode.val(opt_exportFormat);
        numKeys++;
        if (allIntegerKeys && fb.core.snap.ChildrenNode.INTEGER_REGEXP_.test(key)) {
            maxKey = Math.max(maxKey, Number(key));
        } else {
            allIntegerKeys = false;
        }
    });
    if (!opt_exportFormat && allIntegerKeys && maxKey < 2 * numKeys) {
        var array = [];
        for (var key in obj) {
            array[key] = obj[key];
        }
        return array;
    } else {
        if (opt_exportFormat && !this.getPriority().isEmpty()) {
            obj[".priority"] = this.getPriority().val();
        }
        return obj;
    }
};
fb.core.snap.ChildrenNode.prototype.hash = function() {
    if (this.lazyHash_ === null) {
        var toHash = "";
        if (!this.getPriority().isEmpty()) {
            toHash += "priority:" + fb.core.snap.priorityHashText((this.getPriority().val())) + ":";
        }
        this.forEachChild(fb.core.snap.PriorityIndex, function(key, childNode) {
            var childHash = childNode.hash();
            if (childHash !== "") {
                toHash += ":" + key + ":" + childHash;
            }
        });
        this.lazyHash_ = toHash === "" ? "" : fb.core.util.sha1(toHash);
    }
    return this.lazyHash_;
};
fb.core.snap.ChildrenNode.prototype.getPredecessorChildName = function(childName, childNode, index) {
    var idx = this.resolveIndex_(index);
    if (idx) {
        var predecessor = idx.getPredecessorKey(new fb.core.snap.NamedNode(childName, childNode));
        return predecessor ? predecessor.name : null;
    } else {
        return this.children_.getPredecessorKey(childName);
    }
};
fb.core.snap.ChildrenNode.prototype.getFirstChildName = function(indexDefinition) {
    var idx = this.resolveIndex_(indexDefinition);
    if (idx) {
        var minKey = idx.minKey();
        return minKey && minKey.name;
    } else {
        return this.children_.minKey();
    }
};
fb.core.snap.ChildrenNode.prototype.getFirstChild = function(indexDefinition) {
    var minKey = this.getFirstChildName(indexDefinition);
    if (minKey) {
        return new fb.core.snap.NamedNode(minKey, this.children_.get(minKey));
    } else {
        return null;
    }
};
fb.core.snap.ChildrenNode.prototype.getLastChildName = function(indexDefinition) {
    var idx = this.resolveIndex_(indexDefinition);
    if (idx) {
        var maxKey = idx.maxKey();
        return maxKey && maxKey.name;
    } else {
        return this.children_.maxKey();
    }
};
fb.core.snap.ChildrenNode.prototype.getLastChild = function(indexDefinition) {
    var maxKey = this.getLastChildName(indexDefinition);
    if (maxKey) {
        return new fb.core.snap.NamedNode(maxKey, this.children_.get(maxKey));
    } else {
        return null;
    }
};
fb.core.snap.ChildrenNode.prototype.forEachChild = function(index, action) {
    var idx = this.resolveIndex_(index);
    if (idx) {
        return idx.inorderTraversal(function(wrappedNode) {
            return action(wrappedNode.name, wrappedNode.node);
        });
    } else {
        return this.children_.inorderTraversal(action);
    }
};
fb.core.snap.ChildrenNode.prototype.getIterator = function(indexDefinition) {
    return this.getIteratorFrom(indexDefinition.minPost(), indexDefinition);
};
fb.core.snap.ChildrenNode.prototype.getIteratorFrom = function(startPost, indexDefinition) {
    var idx = this.resolveIndex_(indexDefinition);
    if (idx) {
        return idx.getIteratorFrom(startPost, function(key) {
            return key;
        });
    } else {
        var iterator = this.children_.getIteratorFrom(startPost.name, fb.core.snap.NamedNode.Wrap);
        var next = iterator.peek();
        while (next != null && indexDefinition.compare(next, startPost) < 0) {
            iterator.getNext();
            next = iterator.peek();
        }
        return iterator;
    }
};
fb.core.snap.ChildrenNode.prototype.getReverseIterator = function(indexDefinition) {
    return this.getReverseIteratorFrom(indexDefinition.maxPost(), indexDefinition);
};
fb.core.snap.ChildrenNode.prototype.getReverseIteratorFrom = function(endPost, indexDefinition) {
    var idx = this.resolveIndex_(indexDefinition);
    if (idx) {
        return idx.getReverseIteratorFrom(endPost, function(key) {
            return key;
        });
    } else {
        var iterator = this.children_.getReverseIteratorFrom(endPost.name, fb.core.snap.NamedNode.Wrap);
        var next = iterator.peek();
        while (next != null && indexDefinition.compare(next, endPost) > 0) {
            iterator.getNext();
            next = iterator.peek();
        }
        return iterator;
    }
};
fb.core.snap.ChildrenNode.prototype.compareTo = function(other) {
    if (this.isEmpty()) {
        if (other.isEmpty()) {
            return 0;
        } else {
            return-1;
        }
    } else {
        if (other.isLeafNode() || other.isEmpty()) {
            return 1;
        } else {
            if (other === fb.core.snap.MAX_NODE) {
                return-1;
            } else {
                return 0;
            }
        }
    }
};
fb.core.snap.ChildrenNode.prototype.withIndex = function(indexDefinition) {
    if (indexDefinition === fb.core.snap.KeyIndex || this.indexMap_.hasIndex(indexDefinition)) {
        return this;
    } else {
        var newIndexMap = this.indexMap_.addIndex(indexDefinition, this.children_);
        return new fb.core.snap.ChildrenNode(this.children_, this.priorityNode_, newIndexMap);
    }
};
fb.core.snap.ChildrenNode.prototype.isIndexed = function(index) {
    return index === fb.core.snap.KeyIndex || this.indexMap_.hasIndex(index);
};
fb.core.snap.ChildrenNode.prototype.equals = function(other) {
    if (other === this) {
        return true;
    } else {
        if (other.isLeafNode()) {
            return false;
        } else {
            var otherChildrenNode = (other);
            if (!this.getPriority().equals(otherChildrenNode.getPriority())) {
                return false;
            } else {
                if (this.children_.count() === otherChildrenNode.children_.count()) {
                    var thisIter = this.getIterator(fb.core.snap.PriorityIndex);
                    var otherIter = otherChildrenNode.getIterator(fb.core.snap.PriorityIndex);
                    var thisCurrent = thisIter.getNext();
                    var otherCurrent = otherIter.getNext();
                    while (thisCurrent && otherCurrent) {
                        if (thisCurrent.name !== otherCurrent.name || !thisCurrent.node.equals(otherCurrent.node)) {
                            return false;
                        }
                        thisCurrent = thisIter.getNext();
                        otherCurrent = otherIter.getNext();
                    }
                    return thisCurrent === null && otherCurrent === null;
                } else {
                    return false;
                }
            }
        }
    }
};
fb.core.snap.ChildrenNode.prototype.resolveIndex_ = function(indexDefinition) {
    if (indexDefinition === fb.core.snap.KeyIndex) {
        return null;
    } else {
        return this.indexMap_.get(indexDefinition.toString());
    }
};
if (goog.DEBUG) {
    fb.core.snap.ChildrenNode.prototype.toString = function() {
        return fb.util.json.stringify(this.val(true));
    };
}
;goog.provide("fb.core.snap");
goog.require("fb.core.snap.ChildrenNode");
goog.require("fb.core.snap.IndexMap");
goog.require("fb.core.snap.LeafNode");
goog.require("fb.core.util");
var USE_HINZE = true;
fb.core.snap.NodeFromJSON = function(json, opt_priority) {
    if (json === null) {
        return fb.core.snap.EMPTY_NODE;
    }
    var priority = null;
    if (typeof json === "object" && ".priority" in json) {
        priority = json[".priority"];
    } else {
        if (typeof opt_priority !== "undefined") {
            priority = opt_priority;
        }
    }
    fb.core.util.assert(priority === null || typeof priority === "string" || typeof priority === "number" || typeof priority === "object" && ".sv" in priority, "Invalid priority type found: " + typeof priority);
    if (typeof json === "object" && ".value" in json && json[".value"] !== null) {
        json = json[".value"];
    }
    if (typeof json !== "object" || ".sv" in json) {
        var jsonLeaf = (json);
        return new fb.core.snap.LeafNode(jsonLeaf, fb.core.snap.NodeFromJSON(priority));
    }
    if (!(json instanceof Array) && USE_HINZE) {
        var children = [];
        var childrenHavePriority = false;
        var hinzeJsonObj = (json);
        fb.util.obj.foreach(hinzeJsonObj, function(key, child) {
            if (typeof key !== "string" || key.substring(0, 1) !== ".") {
                var childNode = fb.core.snap.NodeFromJSON(hinzeJsonObj[key]);
                if (!childNode.isEmpty()) {
                    childrenHavePriority = childrenHavePriority || !childNode.getPriority().isEmpty();
                    children.push(new fb.core.snap.NamedNode(key, childNode));
                }
            }
        });
        if (children.length == 0) {
            return fb.core.snap.EMPTY_NODE;
        }
        var childSet = (fb.core.snap.buildChildSet(children, fb.core.snap.NAME_ONLY_COMPARATOR, function(namedNode) {
            return namedNode.name;
        }, fb.core.snap.NAME_COMPARATOR));
        if (childrenHavePriority) {
            var sortedChildSet = fb.core.snap.buildChildSet(children, fb.core.snap.PriorityIndex.getCompare());
            return new fb.core.snap.ChildrenNode(childSet, fb.core.snap.NodeFromJSON(priority), new fb.core.snap.IndexMap({".priority":sortedChildSet}, {".priority":fb.core.snap.PriorityIndex}));
        } else {
            return new fb.core.snap.ChildrenNode(childSet, fb.core.snap.NodeFromJSON(priority), fb.core.snap.IndexMap.Default);
        }
    } else {
        var node = fb.core.snap.EMPTY_NODE;
        var jsonObj = (json);
        goog.object.forEach(jsonObj, function(childData, key) {
            if (fb.util.obj.contains(jsonObj, key)) {
                if (key.substring(0, 1) !== ".") {
                    var childNode = fb.core.snap.NodeFromJSON(childData);
                    if (childNode.isLeafNode() || !childNode.isEmpty()) {
                        node = node.updateImmediateChild(key, childNode);
                    }
                }
            }
        });
        return node.updatePriority(fb.core.snap.NodeFromJSON(priority));
    }
};
var LOG_2 = Math.log(2);
fb.core.snap.Base12Num = function(length) {
    var logBase2 = function(num) {
        return parseInt(Math.log(num) / LOG_2, 10);
    };
    var bitMask = function(bits) {
        return parseInt(Array(bits + 1).join("1"), 2);
    };
    this.count = logBase2(length + 1);
    this.current_ = this.count - 1;
    var mask = bitMask(this.count);
    this.bits_ = length + 1 & mask;
};
fb.core.snap.Base12Num.prototype.nextBitIsOne = function() {
    var result = !(this.bits_ & 1 << this.current_);
    this.current_--;
    return result;
};
fb.core.snap.buildChildSet = function(childList, cmp, keyFn, mapSortFn) {
    childList.sort(cmp);
    var buildBalancedTree = function(low, high) {
        var length = high - low;
        if (length == 0) {
            return null;
        } else {
            if (length == 1) {
                var namedNode = childList[low];
                var key = keyFn ? keyFn(namedNode) : namedNode;
                return new fb.LLRBNode(key, namedNode.node, fb.LLRBNode.BLACK, null, null);
            } else {
                var middle = parseInt(length / 2, 10) + low;
                var left = buildBalancedTree(low, middle);
                var right = buildBalancedTree(middle + 1, high);
                namedNode = childList[middle];
                key = keyFn ? keyFn(namedNode) : namedNode;
                return new fb.LLRBNode(key, namedNode.node, fb.LLRBNode.BLACK, left, right);
            }
        }
    };
    var buildFrom12Array = function(base12) {
        var node = null;
        var root = null;
        var index = childList.length;
        var buildPennant = function(chunkSize, color) {
            var low = index - chunkSize;
            var high = index;
            index -= chunkSize;
            var childTree = buildBalancedTree(low + 1, high);
            var namedNode = childList[low];
            var key = keyFn ? keyFn(namedNode) : namedNode;
            attachPennant(new fb.LLRBNode(key, namedNode.node, color, null, childTree));
        };
        var attachPennant = function(pennant) {
            if (node) {
                node.left = pennant;
                node = pennant;
            } else {
                root = pennant;
                node = pennant;
            }
        };
        for (var i = 0;i < base12.count;++i) {
            var isOne = base12.nextBitIsOne();
            var chunkSize = Math.pow(2, base12.count - (i + 1));
            if (isOne) {
                buildPennant(chunkSize, fb.LLRBNode.BLACK);
            } else {
                buildPennant(chunkSize, fb.LLRBNode.BLACK);
                buildPennant(chunkSize, fb.LLRBNode.RED);
            }
        }
        return root;
    };
    var base12 = new fb.core.snap.Base12Num(childList.length);
    var root = buildFrom12Array(base12);
    if (root !== null) {
        return new fb.core.util.SortedMap(mapSortFn || cmp, root);
    } else {
        return new fb.core.util.SortedMap(mapSortFn || cmp);
    }
};
fb.core.snap.priorityHashText = function(priority) {
    if (typeof priority === "number") {
        return "number:" + fb.core.util.doubleToIEEE754String(priority);
    } else {
        return "string:" + priority;
    }
};
fb.core.snap.validatePriorityNode = function(priorityNode) {
    if (priorityNode.isLeafNode()) {
        var val = priorityNode.val();
        fb.core.util.assert(typeof val === "string" || typeof val === "number" || typeof val === "object" && fb.util.obj.contains(val, ".sv"), "Priority must be a string or number.");
    } else {
        fb.core.util.assert(priorityNode === fb.core.snap.MAX_NODE || priorityNode.isEmpty(), "priority of unexpected type.");
    }
    fb.core.util.assert(priorityNode === fb.core.snap.MAX_NODE || priorityNode.getPriority().isEmpty(), "Priority nodes can't have a priority of their own.");
};
fb.core.snap.EMPTY_NODE = new fb.core.snap.ChildrenNode(new fb.core.util.SortedMap(fb.core.snap.NAME_COMPARATOR), null, fb.core.snap.IndexMap.Default);
fb.core.snap.MAX_NODE_ = function() {
    fb.core.snap.ChildrenNode.call(this, new fb.core.util.SortedMap(fb.core.snap.NAME_COMPARATOR), fb.core.snap.EMPTY_NODE, fb.core.snap.IndexMap.Default);
};
goog.inherits(fb.core.snap.MAX_NODE_, fb.core.snap.ChildrenNode);
fb.core.snap.MAX_NODE_.prototype.compareTo = function(other) {
    if (other === this) {
        return 0;
    } else {
        return 1;
    }
};
fb.core.snap.MAX_NODE_.prototype.equals = function(other) {
    return other === this;
};
fb.core.snap.MAX_NODE_.prototype.getPriority = function() {
    return this;
};
fb.core.snap.MAX_NODE_.prototype.getImmediateChild = function(childName) {
    return fb.core.snap.EMPTY_NODE;
};
fb.core.snap.MAX_NODE_.prototype.isEmpty = function() {
    return false;
};
fb.core.snap.MAX_NODE = new fb.core.snap.MAX_NODE_;
fb.core.snap.NamedNode.MIN = new fb.core.snap.NamedNode(fb.core.util.MIN_NAME, fb.core.snap.EMPTY_NODE);
fb.core.snap.NamedNode.MAX = new fb.core.snap.NamedNode(fb.core.util.MAX_NAME, fb.core.snap.MAX_NODE);
goog.provide("fb.core.view.ViewCache");
goog.require("fb.core.view.CacheNode");
goog.require("fb.core.snap");
fb.core.view.ViewCache = function(eventCache, serverCache) {
    this.eventCache_ = eventCache;
    this.serverCache_ = serverCache;
};
fb.core.view.ViewCache.Empty = new fb.core.view.ViewCache(new fb.core.view.CacheNode(fb.core.snap.EMPTY_NODE, false, false), new fb.core.view.CacheNode(fb.core.snap.EMPTY_NODE, false, false));
fb.core.view.ViewCache.prototype.updateEventSnap = function(eventSnap, complete, filtered) {
    return new fb.core.view.ViewCache(new fb.core.view.CacheNode(eventSnap, complete, filtered), this.serverCache_);
};
fb.core.view.ViewCache.prototype.updateServerSnap = function(serverSnap, complete, filtered) {
    return new fb.core.view.ViewCache(this.eventCache_, new fb.core.view.CacheNode(serverSnap, complete, filtered));
};
fb.core.view.ViewCache.prototype.getEventCache = function() {
    return this.eventCache_;
};
fb.core.view.ViewCache.prototype.getCompleteEventSnap = function() {
    return this.eventCache_.isFullyInitialized() ? this.eventCache_.getNode() : null;
};
fb.core.view.ViewCache.prototype.getServerCache = function() {
    return this.serverCache_;
};
fb.core.view.ViewCache.prototype.getCompleteServerSnap = function() {
    return this.serverCache_.isFullyInitialized() ? this.serverCache_.getNode() : null;
};
goog.provide("fb.core.view.View");
goog.require("fb.core.view.EventGenerator");
goog.require("fb.core.view.ViewCache");
goog.require("fb.core.view.ViewProcessor");
goog.require("fb.core.util");
fb.core.view.View = function(query, initialViewCache) {
    this.query_ = query;
    var params = query.getQueryParams();
    var indexFilter = new fb.core.view.filter.IndexedFilter(params.getIndex());
    var filter = params.getNodeFilter();
    this.processor_ = new fb.core.view.ViewProcessor(filter);
    var initialServerCache = initialViewCache.getServerCache();
    var initialEventCache = initialViewCache.getEventCache();
    var serverSnap = indexFilter.updateFullNode(fb.core.snap.EMPTY_NODE, initialServerCache.getNode(), null);
    var eventSnap = filter.updateFullNode(fb.core.snap.EMPTY_NODE, initialEventCache.getNode(), null);
    var newServerCache = new fb.core.view.CacheNode(serverSnap, initialServerCache.isFullyInitialized(), indexFilter.filtersNodes());
    var newEventCache = new fb.core.view.CacheNode(eventSnap, initialEventCache.isFullyInitialized(), filter.filtersNodes());
    this.viewCache_ = new fb.core.view.ViewCache(newEventCache, newServerCache);
    this.eventRegistrations_ = [];
    this.eventGenerator_ = new fb.core.view.EventGenerator(query);
};
fb.core.view.View.prototype.getQuery = function() {
    return this.query_;
};
fb.core.view.View.prototype.getServerCache = function() {
    return this.viewCache_.getServerCache().getNode();
};
fb.core.view.View.prototype.getCompleteServerCache = function(path) {
    var cache = this.viewCache_.getCompleteServerSnap();
    if (cache) {
        if (this.query_.getQueryParams().loadsAllData() || !path.isEmpty() && !cache.getImmediateChild(path.getFront()).isEmpty()) {
            return cache.getChild(path);
        }
    }
    return null;
};
fb.core.view.View.prototype.isEmpty = function() {
    return this.eventRegistrations_.length === 0;
};
fb.core.view.View.prototype.addEventRegistration = function(eventRegistration) {
    this.eventRegistrations_.push(eventRegistration);
};
fb.core.view.View.prototype.removeEventRegistration = function(eventRegistration, cancelError) {
    var cancelEvents = [];
    if (cancelError) {
        fb.core.util.assert(eventRegistration == null, "A cancel should cancel all event registrations.");
        var path = this.query_.path;
        goog.array.forEach(this.eventRegistrations_, function(registration) {
            cancelError = (cancelError);
            var maybeEvent = registration.createCancelEvent(cancelError, path);
            if (maybeEvent) {
                cancelEvents.push(maybeEvent);
            }
        });
    }
    if (eventRegistration) {
        var remaining = [];
        for (var i = 0;i < this.eventRegistrations_.length;++i) {
            var existing = this.eventRegistrations_[i];
            if (!existing.matches(eventRegistration)) {
                remaining.push(existing);
            } else {
                if (eventRegistration.hasAnyCallback()) {
                    remaining = remaining.concat(this.eventRegistrations_.slice(i + 1));
                    break;
                }
            }
        }
        this.eventRegistrations_ = remaining;
    } else {
        this.eventRegistrations_ = [];
    }
    return cancelEvents;
};
fb.core.view.View.prototype.applyOperation = function(operation, writesCache, optCompleteServerCache) {
    if (operation.type === fb.core.OperationType.MERGE && operation.source.queryId !== null) {
        fb.core.util.assert(this.viewCache_.getCompleteServerSnap(), "We should always have a full cache before handling merges");
        fb.core.util.assert(this.viewCache_.getCompleteEventSnap(), "Missing event cache, even though we have a server cache");
    }
    var oldViewCache = this.viewCache_;
    var result = this.processor_.applyOperation(oldViewCache, operation, writesCache, optCompleteServerCache);
    this.processor_.assertIndexed(result.viewCache);
    fb.core.util.assert(result.viewCache.getServerCache().isFullyInitialized() || !oldViewCache.getServerCache().isFullyInitialized(), "Once a server snap is complete, it should never go back");
    this.viewCache_ = result.viewCache;
    return this.generateEventsForChanges_(result.changes, result.viewCache.getEventCache().getNode(), null);
};
fb.core.view.View.prototype.getInitialEvents = function(registration) {
    var eventSnap = this.viewCache_.getEventCache();
    var initialChanges = [];
    if (!eventSnap.getNode().isLeafNode()) {
        var eventNode = (eventSnap.getNode());
        eventNode.forEachChild(fb.core.snap.PriorityIndex, function(key, childNode) {
            initialChanges.push(fb.core.view.Change.childAddedChange(key, childNode));
        });
    }
    if (eventSnap.isFullyInitialized()) {
        initialChanges.push(fb.core.view.Change.valueChange(eventSnap.getNode()));
    }
    return this.generateEventsForChanges_(initialChanges, eventSnap.getNode(), registration);
};
fb.core.view.View.prototype.generateEventsForChanges_ = function(changes, eventCache, opt_eventRegistration) {
    var registrations = opt_eventRegistration ? [opt_eventRegistration] : this.eventRegistrations_;
    return this.eventGenerator_.generateEventsForChanges(changes, eventCache, registrations);
};
goog.provide("fb.core.operation.Merge");
goog.require("fb.core.util");
fb.core.operation.Merge = function(source, path, children) {
    this.type = fb.core.OperationType.MERGE;
    this.source = source;
    this.path = path;
    this.children = children;
};
fb.core.operation.Merge.prototype.operationForChild = function(childName) {
    if (this.path.isEmpty()) {
        var childTree = this.children.subtree(new fb.core.util.Path(childName));
        if (childTree.isEmpty()) {
            return null;
        } else {
            if (childTree.value) {
                return new fb.core.operation.Overwrite(this.source, fb.core.util.Path.Empty, childTree.value);
            } else {
                return new fb.core.operation.Merge(this.source, fb.core.util.Path.Empty, childTree);
            }
        }
    } else {
        fb.core.util.assert(this.path.getFront() === childName, "Can't get a merge for a child not on the path of the operation");
        return new fb.core.operation.Merge(this.source, this.path.popFront(), this.children);
    }
};
if (goog.DEBUG) {
    fb.core.operation.Merge.prototype.toString = function() {
        return "Operation(" + this.path + ": " + this.source.toString() + " merge: " + this.children.toString() + ")";
    };
}
;goog.provide("fb.core.ReadonlyRestClient");
goog.require("fb.core.util");
goog.require("fb.util");
goog.require("fb.util.json");
goog.require("fb.util.jwt");
goog.require("fb.util.obj");
fb.core.ReadonlyRestClient = goog.defineClass(null, {constructor:function(repoInfo, onDataUpdate) {
    this.log_ = fb.core.util.logWrapper("p:rest:");
    this.repoInfo_ = repoInfo;
    this.onDataUpdate_ = onDataUpdate;
    this.credential_ = null;
    this.listens_ = {};
}, listen:function(query, currentHashFn, tag, onComplete) {
    var pathString = query.path.toString();
    this.log_("Listen called for " + pathString + " " + query.queryIdentifier());
    var listenId = fb.core.ReadonlyRestClient.getListenId_(query, tag);
    var thisListen = new Object;
    this.listens_[listenId] = thisListen;
    var queryStringParamaters = query.getQueryParams().toRestQueryStringParameters();
    var self = this;
    this.restRequest_(pathString + ".json", queryStringParamaters, function(error, result) {
        var data = result;
        if (error === 404) {
            data = null;
            error = null;
        }
        if (error === null) {
            self.onDataUpdate_(pathString, data, false, tag);
        }
        if (fb.util.obj.get(self.listens_, listenId) === thisListen) {
            var status;
            if (!error) {
                status = "ok";
            } else {
                if (error == 401) {
                    status = "permission_denied";
                } else {
                    status = "rest_error:" + error;
                }
            }
            onComplete(status, null);
        }
    });
}, unlisten:function(query, tag) {
    var listenId = fb.core.ReadonlyRestClient.getListenId_(query, tag);
    delete this.listens_[listenId];
}, auth:function(cred, opt_callback, opt_cancelCallback) {
    this.credential_ = cred;
    var res = fb.util.jwt.decode(cred);
    var auth = res.data;
    var expires = res.claims && res.claims["exp"];
    if (opt_callback) {
        opt_callback("ok", {"auth":auth, "expires":expires});
    }
}, unauth:function(onComplete) {
    this.credential_ = null;
    onComplete("ok", null);
}, onDisconnectPut:function(pathString, data, opt_onComplete) {
}, onDisconnectMerge:function(pathString, data, opt_onComplete) {
}, onDisconnectCancel:function(pathString, opt_onComplete) {
}, put:function(pathString, data, opt_onComplete, opt_hash) {
}, merge:function(pathString, data, onComplete, opt_hash) {
}, reportStats:function(stats) {
}, restRequest_:function(pathString, queryStringParameters, callback) {
    queryStringParameters = queryStringParameters || {};
    queryStringParameters["format"] = "export";
    if (this.credential_) {
        queryStringParameters["auth"] = this.credential_;
    }
    var url = (this.repoInfo_.secure ? "https://" : "http://") + this.repoInfo_.host + pathString + "?" + fb.util.querystring(queryStringParameters);
    this.log_("Sending REST request for " + url);
    var xhr = new XMLHttpRequest;
    var self = this;
    xhr.onreadystatechange = function() {
        if (callback && xhr.readyState === 4) {
            self.log_("REST Response for " + url + " received. status:", xhr.status, "response:", xhr.responseText);
            var res = null;
            if (xhr.status >= 200 && xhr.status < 300) {
                try {
                    res = fb.util.json.eval(xhr.responseText);
                } catch (e) {
                    fb.core.util.warn("Failed to parse JSON response for " + url + ": " + xhr.responseText);
                }
                callback(null, res);
            } else {
                if (xhr.status !== 401 && xhr.status !== 404) {
                    fb.core.util.warn("Got unsuccessful REST response for " + url + " Status: " + xhr.status);
                }
                callback(xhr.status);
            }
            callback = null;
        }
    };
    xhr.open("GET", url, true);
    xhr.send();
}, statics:{getListenId_:function(query, opt_tag) {
    if (goog.isDef(opt_tag)) {
        return "tag$" + opt_tag;
    } else {
        fb.core.util.assert(query.getQueryParams().isDefault(), "should have a tag if it's not a default query.");
        return query.path.toString();
    }
}}});
goog.provide("fb.core.util.EventEmitter");
goog.require("fb.core.util");
goog.require("goog.array");
fb.core.util.EventEmitter = goog.defineClass(null, {constructor:function(allowedEvents) {
    fb.core.util.assert(goog.isArray(allowedEvents) && allowedEvents.length > 0, "Requires a non-empty array");
    this.allowedEvents_ = allowedEvents;
    this.listeners_ = {};
}, getInitialEvent:goog.abstractMethod, trigger:function(eventType, var_args) {
    var listeners = goog.array.clone(this.listeners_[eventType] || []);
    for (var i = 0;i < listeners.length;i++) {
        listeners[i].callback.apply(listeners[i].context, Array.prototype.slice.call(arguments, 1));
    }
}, on:function(eventType, callback, context) {
    this.validateEventType_(eventType);
    this.listeners_[eventType] = this.listeners_[eventType] || [];
    this.listeners_[eventType].push({callback:callback, context:context});
    var eventData = this.getInitialEvent(eventType);
    if (eventData) {
        callback.apply(context, eventData);
    }
}, off:function(eventType, callback, context) {
    this.validateEventType_(eventType);
    var listeners = this.listeners_[eventType] || [];
    for (var i = 0;i < listeners.length;i++) {
        if (listeners[i].callback === callback && (!context || context === listeners[i].context)) {
            listeners.splice(i, 1);
            return;
        }
    }
}, validateEventType_:function(eventType) {
    fb.core.util.assert(goog.array.find(this.allowedEvents_, function(et) {
        return et === eventType;
    }), "Unknown event: " + eventType);
}});
goog.provide("fb.core.util.nextPushId");
goog.require("fb.core.util");
fb.core.util.nextPushId = function() {
    var PUSH_CHARS = "-0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz";
    var lastPushTime = 0;
    var lastRandChars = [];
    return function(now) {
        var duplicateTime = now === lastPushTime;
        lastPushTime = now;
        var timeStampChars = new Array(8);
        for (var i = 7;i >= 0;i--) {
            timeStampChars[i] = PUSH_CHARS.charAt(now % 64);
            now = Math.floor(now / 64);
        }
        fb.core.util.assert(now === 0, "Cannot push at time == 0");
        var id = timeStampChars.join("");
        if (!duplicateTime) {
            for (i = 0;i < 12;i++) {
                lastRandChars[i] = Math.floor(Math.random() * 64);
            }
        } else {
            for (i = 11;i >= 0 && lastRandChars[i] === 63;i--) {
                lastRandChars[i] = 0;
            }
            lastRandChars[i]++;
        }
        for (i = 0;i < 12;i++) {
            id += PUSH_CHARS.charAt(lastRandChars[i]);
        }
        fb.core.util.assert(id.length === 20, "nextPushId: Length should be 20.");
        return id;
    };
}();
goog.provide("fb.core.util.OnlineMonitor");
goog.require("fb.core.util");
goog.require("fb.core.util.EventEmitter");
fb.core.util.OnlineMonitor = goog.defineClass(fb.core.util.EventEmitter, {constructor:function() {
    fb.core.util.EventEmitter.call(this, ["online"]);
    this.online_ = true;
    if (typeof window !== "undefined" && typeof window.addEventListener !== "undefined") {
        var self = this;
        window.addEventListener("online", function() {
            if (!self.online_) {
                self.online_ = true;
                self.trigger("online", true);
            }
        }, false);
        window.addEventListener("offline", function() {
            if (self.online_) {
                self.online_ = false;
                self.trigger("online", false);
            }
        }, false);
    }
}, getInitialEvent:function(eventType) {
    fb.core.util.assert(eventType === "online", "Unknown event type: " + eventType);
    return[this.online_];
}, currentlyOnline:function() {
    return this.online_;
}});
goog.addSingletonGetter(fb.core.util.OnlineMonitor);
goog.provide("fb.core.util.VisibilityMonitor");
goog.require("fb.core.util");
goog.require("fb.core.util.EventEmitter");
fb.core.util.VisibilityMonitor = goog.defineClass(fb.core.util.EventEmitter, {constructor:function() {
    fb.core.util.EventEmitter.call(this, ["visible"]);
    var hidden, visibilityChange;
    if (typeof document !== "undefined" && typeof document.addEventListener !== "undefined") {
        if (typeof document["hidden"] !== "undefined") {
            visibilityChange = "visibilitychange";
            hidden = "hidden";
        } else {
            if (typeof document["mozHidden"] !== "undefined") {
                visibilityChange = "mozvisibilitychange";
                hidden = "mozHidden";
            } else {
                if (typeof document["msHidden"] !== "undefined") {
                    visibilityChange = "msvisibilitychange";
                    hidden = "msHidden";
                } else {
                    if (typeof document["webkitHidden"] !== "undefined") {
                        visibilityChange = "webkitvisibilitychange";
                        hidden = "webkitHidden";
                    }
                }
            }
        }
    }
    this.visible_ = true;
    if (visibilityChange) {
        var self = this;
        document.addEventListener(visibilityChange, function() {
            var visible = !document[hidden];
            if (visible !== self.visible_) {
                self.visible_ = visible;
                self.trigger("visible", visible);
            }
        }, false);
    }
}, getInitialEvent:function(eventType) {
    fb.core.util.assert(eventType === "visible", "Unknown event type: " + eventType);
    return[this.visible_];
}});
goog.addSingletonGetter(fb.core.util.VisibilityMonitor);
goog.provide("fb.core.util.Path");
goog.provide("fb.core.util.ValidationPath");
goog.require("fb.core.util");
goog.require("fb.util.utf8");
goog.require("goog.string");
fb.core.util.Path = goog.defineClass(null, {constructor:function(pathOrString, opt_pieceNum) {
    if (arguments.length == 1) {
        this.pieces_ = pathOrString.split("/");
        var copyTo = 0;
        for (var i = 0;i < this.pieces_.length;i++) {
            if (this.pieces_[i].length > 0) {
                this.pieces_[copyTo] = this.pieces_[i];
                copyTo++;
            }
        }
        this.pieces_.length = copyTo;
        this.pieceNum_ = 0;
    } else {
        this.pieces_ = pathOrString;
        this.pieceNum_ = opt_pieceNum;
    }
}, getFront:function() {
    if (this.pieceNum_ >= this.pieces_.length) {
        return null;
    }
    return this.pieces_[this.pieceNum_];
}, getLength:function() {
    return this.pieces_.length - this.pieceNum_;
}, popFront:function() {
    var pieceNum = this.pieceNum_;
    if (pieceNum < this.pieces_.length) {
        pieceNum++;
    }
    return new fb.core.util.Path(this.pieces_, pieceNum);
}, getBack:function() {
    if (this.pieceNum_ < this.pieces_.length) {
        return this.pieces_[this.pieces_.length - 1];
    }
    return null;
}, toString:function() {
    var pathString = "";
    for (var i = this.pieceNum_;i < this.pieces_.length;i++) {
        if (this.pieces_[i] !== "") {
            pathString += "/" + this.pieces_[i];
        }
    }
    return pathString || "/";
}, toUrlEncodedString:function() {
    var pathString = "";
    for (var i = this.pieceNum_;i < this.pieces_.length;i++) {
        if (this.pieces_[i] !== "") {
            pathString += "/" + goog.string.urlEncode(this.pieces_[i]);
        }
    }
    return pathString || "/";
}, slice:function(opt_begin) {
    var begin = opt_begin || 0;
    return this.pieces_.slice(this.pieceNum_ + begin);
}, parent:function() {
    if (this.pieceNum_ >= this.pieces_.length) {
        return null;
    }
    var pieces = [];
    for (var i = this.pieceNum_;i < this.pieces_.length - 1;i++) {
        pieces.push(this.pieces_[i]);
    }
    return new fb.core.util.Path(pieces, 0);
}, child:function(childPathObj) {
    var pieces = [];
    for (var i = this.pieceNum_;i < this.pieces_.length;i++) {
        pieces.push(this.pieces_[i]);
    }
    if (childPathObj instanceof fb.core.util.Path) {
        for (i = childPathObj.pieceNum_;i < childPathObj.pieces_.length;i++) {
            pieces.push(childPathObj.pieces_[i]);
        }
    } else {
        var childPieces = childPathObj.split("/");
        for (i = 0;i < childPieces.length;i++) {
            if (childPieces[i].length > 0) {
                pieces.push(childPieces[i]);
            }
        }
    }
    return new fb.core.util.Path(pieces, 0);
}, isEmpty:function() {
    return this.pieceNum_ >= this.pieces_.length;
}, statics:{relativePath:function(outerPath, innerPath) {
    var outer = outerPath.getFront(), inner = innerPath.getFront();
    if (outer === null) {
        return innerPath;
    } else {
        if (outer === inner) {
            return fb.core.util.Path.relativePath(outerPath.popFront(), innerPath.popFront());
        } else {
            throw new Error("INTERNAL ERROR: innerPath (" + innerPath + ") is not within " + "outerPath (" + outerPath + ")");
        }
    }
}, comparePaths:function(left, right) {
    var leftKeys = left.slice();
    var rightKeys = right.slice();
    for (var i = 0;i < leftKeys.length && i < rightKeys.length;i++) {
        var cmp = fb.core.util.nameCompare(leftKeys[i], rightKeys[i]);
        if (cmp !== 0) {
            return cmp;
        }
    }
    if (leftKeys.length === rightKeys.length) {
        return 0;
    }
    return leftKeys.length < rightKeys.length ? -1 : 1;
}}, equals:function(other) {
    if (this.getLength() !== other.getLength()) {
        return false;
    }
    for (var i = this.pieceNum_, j = other.pieceNum_;i <= this.pieces_.length;i++, j++) {
        if (this.pieces_[i] !== other.pieces_[j]) {
            return false;
        }
    }
    return true;
}, contains:function(other) {
    var i = this.pieceNum_;
    var j = other.pieceNum_;
    if (this.getLength() > other.getLength()) {
        return false;
    }
    while (i < this.pieces_.length) {
        if (this.pieces_[i] !== other.pieces_[j]) {
            return false;
        }
        ++i;
        ++j;
    }
    return true;
}});
fb.core.util.Path.Empty = new fb.core.util.Path("");
fb.core.util.ValidationPath = goog.defineClass(null, {constructor:function(path, errorPrefix) {
    this.parts_ = path.slice();
    this.byteLength_ = Math.max(1, this.parts_.length);
    this.errorPrefix_ = errorPrefix;
    for (var i = 0;i < this.parts_.length;i++) {
        this.byteLength_ += fb.util.utf8.stringLength(this.parts_[i]);
    }
    this.checkValid_();
}, statics:{MAX_PATH_DEPTH:32, MAX_PATH_LENGTH_BYTES:768}, push:function(child) {
    if (this.parts_.length > 0) {
        this.byteLength_ += 1;
    }
    this.parts_.push(child);
    this.byteLength_ += fb.util.utf8.stringLength(child);
    this.checkValid_();
}, pop:function() {
    var last = this.parts_.pop();
    this.byteLength_ -= fb.util.utf8.stringLength(last);
    if (this.parts_.length > 0) {
        this.byteLength_ -= 1;
    }
}, checkValid_:function() {
    if (this.byteLength_ > fb.core.util.ValidationPath.MAX_PATH_LENGTH_BYTES) {
        throw new Error(this.errorPrefix_ + "has a key path longer than " + fb.core.util.ValidationPath.MAX_PATH_LENGTH_BYTES + " bytes (" + this.byteLength_ + ").");
    }
    if (this.parts_.length > fb.core.util.ValidationPath.MAX_PATH_DEPTH) {
        throw new Error(this.errorPrefix_ + "path specified exceeds the maximum depth that can be written (" + fb.core.util.ValidationPath.MAX_PATH_DEPTH + ") or object contains a cycle " + this.toErrorString());
    }
}, toErrorString:function() {
    if (this.parts_.length == 0) {
        return "";
    }
    return "in property '" + this.parts_.join(".") + "'";
}});
goog.provide("fb.core.util.ImmutableTree");
goog.require("fb.core.util");
goog.require("fb.core.util.Path");
goog.require("fb.core.util.SortedMap");
goog.require("fb.util.json");
goog.require("fb.util.obj");
goog.require("goog.object");
fb.core.util.ImmutableTree = goog.defineClass(null, {constructor:function(value, opt_children) {
    this.value = value;
    this.children = opt_children || fb.core.util.ImmutableTree.EmptyChildren_;
}, statics:{EmptyChildren_:new fb.core.util.SortedMap(fb.core.util.stringCompare), fromObject:function(obj) {
    var tree = fb.core.util.ImmutableTree.Empty;
    goog.object.forEach(obj, function(childSnap, childPath) {
        tree = tree.set(new fb.core.util.Path(childPath), childSnap);
    });
    return tree;
}}, isEmpty:function() {
    return this.value === null && this.children.isEmpty();
}, findRootMostMatchingPathAndValue:function(relativePath, predicate) {
    if (this.value != null && predicate(this.value)) {
        return{path:fb.core.util.Path.Empty, value:this.value};
    } else {
        if (relativePath.isEmpty()) {
            return null;
        } else {
            var front = relativePath.getFront();
            var child = this.children.get(front);
            if (child !== null) {
                var childExistingPathAndValue = child.findRootMostMatchingPathAndValue(relativePath.popFront(), predicate);
                if (childExistingPathAndValue != null) {
                    var fullPath = (new fb.core.util.Path(front)).child(childExistingPathAndValue.path);
                    return{path:fullPath, value:childExistingPathAndValue.value};
                } else {
                    return null;
                }
            } else {
                return null;
            }
        }
    }
}, findRootMostValueAndPath:function(relativePath) {
    return this.findRootMostMatchingPathAndValue(relativePath, function() {
        return true;
    });
}, subtree:function(relativePath) {
    if (relativePath.isEmpty()) {
        return this;
    } else {
        var front = relativePath.getFront();
        var childTree = this.children.get(front);
        if (childTree !== null) {
            return childTree.subtree(relativePath.popFront());
        } else {
            return fb.core.util.ImmutableTree.Empty;
        }
    }
}, set:function(relativePath, toSet) {
    if (relativePath.isEmpty()) {
        return new fb.core.util.ImmutableTree(toSet, this.children);
    } else {
        var front = relativePath.getFront();
        var child = this.children.get(front) || fb.core.util.ImmutableTree.Empty;
        var newChild = child.set(relativePath.popFront(), toSet);
        var newChildren = this.children.insert(front, newChild);
        return new fb.core.util.ImmutableTree(this.value, newChildren);
    }
}, remove:function(relativePath) {
    if (relativePath.isEmpty()) {
        if (this.children.isEmpty()) {
            return fb.core.util.ImmutableTree.Empty;
        } else {
            return new fb.core.util.ImmutableTree(null, this.children);
        }
    } else {
        var front = relativePath.getFront();
        var child = this.children.get(front);
        if (child) {
            var newChild = child.remove(relativePath.popFront());
            var newChildren;
            if (newChild.isEmpty()) {
                newChildren = this.children.remove(front);
            } else {
                newChildren = this.children.insert(front, newChild);
            }
            if (this.value === null && newChildren.isEmpty()) {
                return fb.core.util.ImmutableTree.Empty;
            } else {
                return new fb.core.util.ImmutableTree(this.value, newChildren);
            }
        } else {
            return this;
        }
    }
}, get:function(relativePath) {
    if (relativePath.isEmpty()) {
        return this.value;
    } else {
        var front = relativePath.getFront();
        var child = this.children.get(front);
        if (child) {
            return child.get(relativePath.popFront());
        } else {
            return null;
        }
    }
}, setTree:function(relativePath, newTree) {
    if (relativePath.isEmpty()) {
        return newTree;
    } else {
        var front = relativePath.getFront();
        var child = this.children.get(front) || fb.core.util.ImmutableTree.Empty;
        var newChild = child.setTree(relativePath.popFront(), newTree);
        var newChildren;
        if (newChild.isEmpty()) {
            newChildren = this.children.remove(front);
        } else {
            newChildren = this.children.insert(front, newChild);
        }
        return new fb.core.util.ImmutableTree(this.value, newChildren);
    }
}, fold:function(fn) {
    return this.fold_(fb.core.util.Path.Empty, fn);
}, fold_:function(pathSoFar, fn) {
    var accum = {};
    this.children.inorderTraversal(function(childKey, childTree) {
        accum[childKey] = childTree.fold_(pathSoFar.child(childKey), fn);
    });
    return fn(pathSoFar, this.value, accum);
}, findOnPath:function(path, f) {
    return this.findOnPath_(path, fb.core.util.Path.Empty, f);
}, findOnPath_:function(pathToFollow, pathSoFar, f) {
    var result = this.value ? f(pathSoFar, this.value) : false;
    if (result) {
        return result;
    } else {
        if (pathToFollow.isEmpty()) {
            return null;
        } else {
            var front = pathToFollow.getFront();
            var nextChild = this.children.get(front);
            if (nextChild) {
                return nextChild.findOnPath_(pathToFollow.popFront(), pathSoFar.child(front), f);
            } else {
                return null;
            }
        }
    }
}, foreachOnPath:function(path, f) {
    return this.foreachOnPath_(path, fb.core.util.Path.Empty, f);
}, foreachOnPath_:function(pathToFollow, currentRelativePath, f) {
    if (pathToFollow.isEmpty()) {
        return this;
    } else {
        if (this.value) {
            f(currentRelativePath, this.value);
        }
        var front = pathToFollow.getFront();
        var nextChild = this.children.get(front);
        if (nextChild) {
            return nextChild.foreachOnPath_(pathToFollow.popFront(), currentRelativePath.child(front), f);
        } else {
            return fb.core.util.ImmutableTree.Empty;
        }
    }
}, foreach:function(f) {
    this.foreach_(fb.core.util.Path.Empty, f);
}, foreach_:function(currentRelativePath, f) {
    this.children.inorderTraversal(function(childName, childTree) {
        childTree.foreach_(currentRelativePath.child(childName), f);
    });
    if (this.value) {
        f(currentRelativePath, this.value);
    }
}, foreachChild:function(f) {
    this.children.inorderTraversal(function(childName, childTree) {
        if (childTree.value) {
            f(childName, childTree.value);
        }
    });
}});
fb.core.util.ImmutableTree.Empty = new fb.core.util.ImmutableTree(null);
if (goog.DEBUG) {
    fb.core.util.ImmutableTree.prototype.toString = function() {
        var json = {};
        this.foreach(function(relativePath, value) {
            var pathString = relativePath.toString();
            json[pathString] = value.toString();
        });
        return fb.util.json.stringify(json);
    };
}
;goog.provide("fb.core.operation.AckUserWrite");
goog.require("fb.core.util.ImmutableTree");
fb.core.operation.AckUserWrite = function(path, affectedTree, revert) {
    this.type = fb.core.OperationType.ACK_USER_WRITE;
    this.source = fb.core.OperationSource.User;
    this.path = path;
    this.affectedTree = affectedTree;
    this.revert = revert;
};
fb.core.operation.AckUserWrite.prototype.operationForChild = function(childName) {
    if (!this.path.isEmpty()) {
        fb.core.util.assert(this.path.getFront() === childName, "operationForChild called for unrelated child.");
        return new fb.core.operation.AckUserWrite(this.path.popFront(), this.affectedTree, this.revert);
    } else {
        if (this.affectedTree.value != null) {
            fb.core.util.assert(this.affectedTree.children.isEmpty(), "affectedTree should not have overlapping affected paths.");
            return this;
        } else {
            var childTree = this.affectedTree.subtree(new fb.core.util.Path(childName));
            return new fb.core.operation.AckUserWrite(fb.core.util.Path.Empty, childTree, this.revert);
        }
    }
};
if (goog.DEBUG) {
    fb.core.operation.AckUserWrite.prototype.toString = function() {
        return "Operation(" + this.path + ": " + this.source.toString() + " ack write revert=" + this.revert + " affectedTree=" + this.affectedTree + ")";
    };
}
;goog.provide("fb.core.Operation");
goog.require("fb.core.operation.AckUserWrite");
goog.require("fb.core.operation.Merge");
goog.require("fb.core.operation.Overwrite");
goog.require("fb.core.operation.ListenComplete");
goog.require("fb.core.util");
fb.core.OperationType = {OVERWRITE:0, MERGE:1, ACK_USER_WRITE:2, LISTEN_COMPLETE:3};
fb.core.Operation = function() {
};
fb.core.Operation.prototype.source;
fb.core.Operation.prototype.type;
fb.core.Operation.prototype.path;
fb.core.Operation.prototype.operationForChild = goog.abstractMethod;
fb.core.OperationSource = function(fromUser, fromServer, queryId, tagged) {
    this.fromUser = fromUser;
    this.fromServer = fromServer;
    this.queryId = queryId;
    this.tagged = tagged;
    fb.core.util.assert(!tagged || fromServer, "Tagged queries must be from server.");
};
fb.core.OperationSource.User = new fb.core.OperationSource(true, false, null, false);
fb.core.OperationSource.Server = new fb.core.OperationSource(false, true, null, false);
fb.core.OperationSource.forServerTaggedQuery = function(queryId) {
    return new fb.core.OperationSource(false, true, queryId, true);
};
if (goog.DEBUG) {
    fb.core.OperationSource.prototype.toString = function() {
        return this.fromUser ? "user" : this.tagged ? "server(queryID=" + this.queryId + ")" : "server";
    };
}
;goog.provide("fb.core.CompoundWrite");
goog.require("fb.core.snap.Node");
goog.require("fb.core.util");
goog.require("fb.core.util.ImmutableTree");
fb.core.CompoundWrite = function(writeTree) {
    this.writeTree_ = writeTree;
};
fb.core.CompoundWrite.Empty = new fb.core.CompoundWrite((new fb.core.util.ImmutableTree(null)));
fb.core.CompoundWrite.prototype.addWrite = function(path, node) {
    if (path.isEmpty()) {
        return new fb.core.CompoundWrite(new fb.core.util.ImmutableTree(node));
    } else {
        var rootmost = this.writeTree_.findRootMostValueAndPath(path);
        if (rootmost != null) {
            var rootMostPath = rootmost.path, value = rootmost.value;
            var relativePath = fb.core.util.Path.relativePath(rootMostPath, path);
            value = value.updateChild(relativePath, node);
            return new fb.core.CompoundWrite(this.writeTree_.set(rootMostPath, value));
        } else {
            var subtree = new fb.core.util.ImmutableTree(node);
            var newWriteTree = this.writeTree_.setTree(path, subtree);
            return new fb.core.CompoundWrite(newWriteTree);
        }
    }
};
fb.core.CompoundWrite.prototype.addWrites = function(path, updates) {
    var newWrite = this;
    fb.util.obj.foreach(updates, function(childKey, node) {
        newWrite = newWrite.addWrite(path.child(childKey), node);
    });
    return newWrite;
};
fb.core.CompoundWrite.prototype.removeWrite = function(path) {
    if (path.isEmpty()) {
        return fb.core.CompoundWrite.Empty;
    } else {
        var newWriteTree = this.writeTree_.setTree(path, fb.core.util.ImmutableTree.Empty);
        return new fb.core.CompoundWrite(newWriteTree);
    }
};
fb.core.CompoundWrite.prototype.hasCompleteWrite = function(path) {
    return this.getCompleteNode(path) != null;
};
fb.core.CompoundWrite.prototype.getCompleteNode = function(path) {
    var rootmost = this.writeTree_.findRootMostValueAndPath(path);
    if (rootmost != null) {
        return this.writeTree_.get(rootmost.path).getChild(fb.core.util.Path.relativePath(rootmost.path, path));
    } else {
        return null;
    }
};
fb.core.CompoundWrite.prototype.getCompleteChildren = function() {
    var children = [];
    var node = this.writeTree_.value;
    if (node != null) {
        if (!node.isLeafNode()) {
            node = (node);
            node.forEachChild(fb.core.snap.PriorityIndex, function(childName, childNode) {
                children.push(new fb.core.snap.NamedNode(childName, childNode));
            });
        }
    } else {
        this.writeTree_.children.inorderTraversal(function(childName, childTree) {
            if (childTree.value != null) {
                children.push(new fb.core.snap.NamedNode(childName, childTree.value));
            }
        });
    }
    return children;
};
fb.core.CompoundWrite.prototype.childCompoundWrite = function(path) {
    if (path.isEmpty()) {
        return this;
    } else {
        var shadowingNode = this.getCompleteNode(path);
        if (shadowingNode != null) {
            return new fb.core.CompoundWrite(new fb.core.util.ImmutableTree(shadowingNode));
        } else {
            return new fb.core.CompoundWrite(this.writeTree_.subtree(path));
        }
    }
};
fb.core.CompoundWrite.prototype.isEmpty = function() {
    return this.writeTree_.isEmpty();
};
fb.core.CompoundWrite.prototype.apply = function(node) {
    return fb.core.CompoundWrite.applySubtreeWrite_(fb.core.util.Path.Empty, this.writeTree_, node);
};
fb.core.CompoundWrite.applySubtreeWrite_ = function(relativePath, writeTree, node) {
    if (writeTree.value != null) {
        return node.updateChild(relativePath, writeTree.value);
    } else {
        var priorityWrite = null;
        writeTree.children.inorderTraversal(function(childKey, childTree) {
            if (childKey === ".priority") {
                fb.core.util.assert(childTree.value !== null, "Priority writes must always be leaf nodes");
                priorityWrite = childTree.value;
            } else {
                node = fb.core.CompoundWrite.applySubtreeWrite_(relativePath.child(childKey), childTree, node);
            }
        });
        if (!node.getChild(relativePath).isEmpty() && priorityWrite !== null) {
            node = node.updateChild(relativePath.child(".priority"), (priorityWrite));
        }
        return node;
    }
};
goog.provide("fb.core.WriteTree");
goog.require("fb.core.CompoundWrite");
goog.require("fb.core.util");
goog.require("fb.core.view.CacheNode");
fb.core.WriteRecord;
fb.core.WriteTree = function() {
    this.visibleWrites_ = (fb.core.CompoundWrite.Empty);
    this.allWrites_ = [];
    this.lastWriteId_ = -1;
};
fb.core.WriteTree.prototype.childWrites = function(path) {
    return new fb.core.WriteTreeRef(path, this);
};
fb.core.WriteTree.prototype.addOverwrite = function(path, snap, writeId, visible) {
    fb.core.util.assert(writeId > this.lastWriteId_, "Stacking an older write on top of newer ones");
    if (!goog.isDef(visible)) {
        visible = true;
    }
    this.allWrites_.push({path:path, snap:snap, writeId:writeId, visible:visible});
    if (visible) {
        this.visibleWrites_ = this.visibleWrites_.addWrite(path, snap);
    }
    this.lastWriteId_ = writeId;
};
fb.core.WriteTree.prototype.addMerge = function(path, changedChildren, writeId) {
    fb.core.util.assert(writeId > this.lastWriteId_, "Stacking an older merge on top of newer ones");
    this.allWrites_.push({path:path, children:changedChildren, writeId:writeId, visible:true});
    this.visibleWrites_ = this.visibleWrites_.addWrites(path, changedChildren);
    this.lastWriteId_ = writeId;
};
fb.core.WriteTree.prototype.getWrite = function(writeId) {
    for (var i = 0;i < this.allWrites_.length;i++) {
        var record = this.allWrites_[i];
        if (record.writeId === writeId) {
            return record;
        }
    }
    return null;
};
fb.core.WriteTree.prototype.removeWrite = function(writeId) {
    var idx = goog.array.findIndex(this.allWrites_, function(s) {
        return s.writeId === writeId;
    });
    fb.core.util.assert(idx >= 0, "removeWrite called with nonexistent writeId.");
    var writeToRemove = this.allWrites_[idx];
    this.allWrites_.splice(idx, 1);
    var removedWriteWasVisible = writeToRemove.visible;
    var removedWriteOverlapsWithOtherWrites = false;
    var i = this.allWrites_.length - 1;
    while (removedWriteWasVisible && i >= 0) {
        var currentWrite = this.allWrites_[i];
        if (currentWrite.visible) {
            if (i >= idx && this.recordContainsPath_(currentWrite, writeToRemove.path)) {
                removedWriteWasVisible = false;
            } else {
                if (writeToRemove.path.contains(currentWrite.path)) {
                    removedWriteOverlapsWithOtherWrites = true;
                }
            }
        }
        i--;
    }
    if (!removedWriteWasVisible) {
        return false;
    } else {
        if (removedWriteOverlapsWithOtherWrites) {
            this.resetTree_();
            return true;
        } else {
            if (writeToRemove.snap) {
                this.visibleWrites_ = this.visibleWrites_.removeWrite(writeToRemove.path);
            } else {
                var children = writeToRemove.children;
                var self = this;
                goog.object.forEach(children, function(childSnap, childName) {
                    self.visibleWrites_ = self.visibleWrites_.removeWrite(writeToRemove.path.child(childName));
                });
            }
            return true;
        }
    }
};
fb.core.WriteTree.prototype.getCompleteWriteData = function(path) {
    return this.visibleWrites_.getCompleteNode(path);
};
fb.core.WriteTree.prototype.calcCompleteEventCache = function(treePath, completeServerCache, writeIdsToExclude, includeHiddenWrites) {
    if (!writeIdsToExclude && !includeHiddenWrites) {
        var shadowingNode = this.visibleWrites_.getCompleteNode(treePath);
        if (shadowingNode != null) {
            return shadowingNode;
        } else {
            var subMerge = this.visibleWrites_.childCompoundWrite(treePath);
            if (subMerge.isEmpty()) {
                return completeServerCache;
            } else {
                if (completeServerCache == null && !subMerge.hasCompleteWrite(fb.core.util.Path.Empty)) {
                    return null;
                } else {
                    var layeredCache = completeServerCache || fb.core.snap.EMPTY_NODE;
                    return subMerge.apply(layeredCache);
                }
            }
        }
    } else {
        var merge = this.visibleWrites_.childCompoundWrite(treePath);
        if (!includeHiddenWrites && merge.isEmpty()) {
            return completeServerCache;
        } else {
            if (!includeHiddenWrites && completeServerCache == null && !merge.hasCompleteWrite(fb.core.util.Path.Empty)) {
                return null;
            } else {
                var filter = function(write) {
                    return(write.visible || includeHiddenWrites) && (!writeIdsToExclude || !goog.array.contains(writeIdsToExclude, write.writeId)) && (write.path.contains(treePath) || treePath.contains(write.path));
                };
                var mergeAtPath = fb.core.WriteTree.layerTree_(this.allWrites_, filter, treePath);
                layeredCache = completeServerCache || fb.core.snap.EMPTY_NODE;
                return mergeAtPath.apply(layeredCache);
            }
        }
    }
};
fb.core.WriteTree.prototype.calcCompleteEventChildren = function(treePath, completeServerChildren) {
    var completeChildren = fb.core.snap.EMPTY_NODE;
    var topLevelSet = this.visibleWrites_.getCompleteNode(treePath);
    if (topLevelSet) {
        if (!topLevelSet.isLeafNode()) {
            topLevelSet.forEachChild(fb.core.snap.PriorityIndex, function(childName, childSnap) {
                completeChildren = completeChildren.updateImmediateChild(childName, childSnap);
            });
        }
        return completeChildren;
    } else {
        if (completeServerChildren) {
            var merge = this.visibleWrites_.childCompoundWrite(treePath);
            completeServerChildren.forEachChild(fb.core.snap.PriorityIndex, function(childName, childNode) {
                var node = merge.childCompoundWrite(new fb.core.util.Path(childName)).apply(childNode);
                completeChildren = completeChildren.updateImmediateChild(childName, node);
            });
            goog.array.forEach(merge.getCompleteChildren(), function(namedNode) {
                completeChildren = completeChildren.updateImmediateChild(namedNode.name, namedNode.node);
            });
            return completeChildren;
        } else {
            merge = this.visibleWrites_.childCompoundWrite(treePath);
            goog.array.forEach(merge.getCompleteChildren(), function(namedNode) {
                completeChildren = completeChildren.updateImmediateChild(namedNode.name, namedNode.node);
            });
            return completeChildren;
        }
    }
};
fb.core.WriteTree.prototype.calcEventCacheAfterServerOverwrite = function(treePath, childPath, existingEventSnap, existingServerSnap) {
    fb.core.util.assert(existingEventSnap || existingServerSnap, "Either existingEventSnap or existingServerSnap must exist");
    var path = treePath.child(childPath);
    if (this.visibleWrites_.hasCompleteWrite(path)) {
        return null;
    } else {
        var childMerge = this.visibleWrites_.childCompoundWrite(path);
        if (childMerge.isEmpty()) {
            return existingServerSnap.getChild(childPath);
        } else {
            return childMerge.apply(existingServerSnap.getChild(childPath));
        }
    }
};
fb.core.WriteTree.prototype.calcCompleteChild = function(treePath, childKey, existingServerSnap) {
    var path = treePath.child(childKey);
    var shadowingNode = this.visibleWrites_.getCompleteNode(path);
    if (shadowingNode != null) {
        return shadowingNode;
    } else {
        if (existingServerSnap.isCompleteForChild(childKey)) {
            var childMerge = this.visibleWrites_.childCompoundWrite(path);
            return childMerge.apply(existingServerSnap.getNode().getImmediateChild(childKey));
        } else {
            return null;
        }
    }
};
fb.core.WriteTree.prototype.shadowingWrite = function(path) {
    return this.visibleWrites_.getCompleteNode(path);
};
fb.core.WriteTree.prototype.calcIndexedSlice = function(treePath, completeServerData, startPost, count, reverse, index) {
    var toIterate;
    var merge = this.visibleWrites_.childCompoundWrite(treePath);
    var shadowingNode = merge.getCompleteNode(fb.core.util.Path.Empty);
    if (shadowingNode != null) {
        toIterate = shadowingNode;
    } else {
        if (completeServerData != null) {
            toIterate = merge.apply(completeServerData);
        } else {
            return[];
        }
    }
    toIterate = toIterate.withIndex(index);
    if (!toIterate.isEmpty() && !toIterate.isLeafNode()) {
        var nodes = [];
        var cmp = index.getCompare();
        var iter = reverse ? toIterate.getReverseIteratorFrom(startPost, index) : toIterate.getIteratorFrom(startPost, index);
        var next = iter.getNext();
        while (next && nodes.length < count) {
            if (cmp(next, startPost) !== 0) {
                nodes.push(next);
            }
            next = iter.getNext();
        }
        return nodes;
    } else {
        return[];
    }
};
fb.core.WriteTree.prototype.recordContainsPath_ = function(writeRecord, path) {
    if (writeRecord.snap) {
        return writeRecord.path.contains(path);
    } else {
        return!!goog.object.findKey(writeRecord.children, function(childSnap, childName) {
            return writeRecord.path.child(childName).contains(path);
        });
    }
};
fb.core.WriteTree.prototype.resetTree_ = function() {
    this.visibleWrites_ = fb.core.WriteTree.layerTree_(this.allWrites_, fb.core.WriteTree.DefaultFilter_, fb.core.util.Path.Empty);
    if (this.allWrites_.length > 0) {
        this.lastWriteId_ = this.allWrites_[this.allWrites_.length - 1].writeId;
    } else {
        this.lastWriteId_ = -1;
    }
};
fb.core.WriteTree.DefaultFilter_ = function(write) {
    return write.visible;
};
fb.core.WriteTree.layerTree_ = function(writes, filter, treeRoot) {
    var compoundWrite = fb.core.CompoundWrite.Empty;
    for (var i = 0;i < writes.length;++i) {
        var write = writes[i];
        if (filter(write)) {
            var writePath = write.path;
            var relativePath;
            if (write.snap) {
                if (treeRoot.contains(writePath)) {
                    relativePath = fb.core.util.Path.relativePath(treeRoot, writePath);
                    compoundWrite = compoundWrite.addWrite(relativePath, write.snap);
                } else {
                    if (writePath.contains(treeRoot)) {
                        relativePath = fb.core.util.Path.relativePath(writePath, treeRoot);
                        compoundWrite = compoundWrite.addWrite(fb.core.util.Path.Empty, write.snap.getChild(relativePath));
                    } else {
                    }
                }
            } else {
                if (write.children) {
                    if (treeRoot.contains(writePath)) {
                        relativePath = fb.core.util.Path.relativePath(treeRoot, writePath);
                        compoundWrite = compoundWrite.addWrites(relativePath, write.children);
                    } else {
                        if (writePath.contains(treeRoot)) {
                            relativePath = fb.core.util.Path.relativePath(writePath, treeRoot);
                            if (relativePath.isEmpty()) {
                                compoundWrite = compoundWrite.addWrites(fb.core.util.Path.Empty, write.children);
                            } else {
                                var child = fb.util.obj.get(write.children, relativePath.getFront());
                                if (child) {
                                    var deepNode = child.getChild(relativePath.popFront());
                                    compoundWrite = compoundWrite.addWrite(fb.core.util.Path.Empty, deepNode);
                                }
                            }
                        } else {
                        }
                    }
                } else {
                    throw fb.core.util.assertionError("WriteRecord should have .snap or .children");
                }
            }
        }
    }
    return compoundWrite;
};
fb.core.WriteTreeRef = function(path, writeTree) {
    this.treePath_ = path;
    this.writeTree_ = writeTree;
};
fb.core.WriteTreeRef.prototype.calcCompleteEventCache = function(completeServerCache, writeIdsToExclude, includeHiddenWrites) {
    return this.writeTree_.calcCompleteEventCache(this.treePath_, completeServerCache, writeIdsToExclude, includeHiddenWrites);
};
fb.core.WriteTreeRef.prototype.calcCompleteEventChildren = function(completeServerChildren) {
    return this.writeTree_.calcCompleteEventChildren(this.treePath_, completeServerChildren);
};
fb.core.WriteTreeRef.prototype.calcEventCacheAfterServerOverwrite = function(path, existingEventSnap, existingServerSnap) {
    return this.writeTree_.calcEventCacheAfterServerOverwrite(this.treePath_, path, existingEventSnap, existingServerSnap);
};
fb.core.WriteTreeRef.prototype.shadowingWrite = function(path) {
    return this.writeTree_.shadowingWrite(this.treePath_.child(path));
};
fb.core.WriteTreeRef.prototype.calcIndexedSlice = function(completeServerData, startPost, count, reverse, index) {
    return this.writeTree_.calcIndexedSlice(this.treePath_, completeServerData, startPost, count, reverse, index);
};
fb.core.WriteTreeRef.prototype.calcCompleteChild = function(childKey, existingServerCache) {
    return this.writeTree_.calcCompleteChild(this.treePath_, childKey, existingServerCache);
};
fb.core.WriteTreeRef.prototype.child = function(childName) {
    return new fb.core.WriteTreeRef(this.treePath_.child(childName), this.writeTree_);
};
goog.provide("fb.core.util.Tree");
goog.require("fb.core.util");
goog.require("fb.core.util.Path");
goog.require("fb.util.obj");
goog.require("goog.object");
fb.core.util.TreeNode = goog.defineClass(null, {constructor:function() {
    this.children = {};
    this.childCount = 0;
    this.value = null;
}});
fb.core.util.Tree = goog.defineClass(null, {constructor:function(opt_name, opt_parent, opt_node) {
    this.name_ = opt_name ? opt_name : "";
    this.parent_ = opt_parent ? opt_parent : null;
    this.node_ = opt_node ? opt_node : new fb.core.util.TreeNode;
}, subTree:function(pathObj) {
    var path = pathObj instanceof fb.core.util.Path ? pathObj : new fb.core.util.Path(pathObj);
    var child = this, next;
    while ((next = path.getFront()) !== null) {
        var childNode = fb.util.obj.get(child.node_.children, next) || new fb.core.util.TreeNode;
        child = new fb.core.util.Tree(next, child, childNode);
        path = path.popFront();
    }
    return child;
}, getValue:function() {
    return this.node_.value;
}, setValue:function(value) {
    fb.core.util.assert(typeof value !== "undefined", "Cannot set value to undefined");
    this.node_.value = value;
    this.updateParents_();
}, clear:function() {
    this.node_.value = null;
    this.node_.children = {};
    this.node_.childCount = 0;
    this.updateParents_();
}, hasChildren:function() {
    return this.node_.childCount > 0;
}, isEmpty:function() {
    return this.getValue() === null && !this.hasChildren();
}, forEachChild:function(action) {
    var self = this;
    goog.object.forEach(this.node_.children, function(childTree, child) {
        action(new fb.core.util.Tree(child, self, childTree));
    });
}, forEachDescendant:function(action, opt_includeSelf, opt_childrenFirst) {
    if (opt_includeSelf && !opt_childrenFirst) {
        action(this);
    }
    this.forEachChild(function(child) {
        child.forEachDescendant(action, true, opt_childrenFirst);
    });
    if (opt_includeSelf && opt_childrenFirst) {
        action(this);
    }
}, forEachAncestor:function(action, opt_includeSelf) {
    var node = opt_includeSelf ? this : this.parent();
    while (node !== null) {
        if (action(node)) {
            return true;
        }
        node = node.parent();
    }
    return false;
}, forEachImmediateDescendantWithValue:function(action) {
    this.forEachChild(function(child) {
        if (child.getValue() !== null) {
            action(child);
        } else {
            child.forEachImmediateDescendantWithValue(action);
        }
    });
}, path:function() {
    return new fb.core.util.Path(this.parent_ === null ? this.name_ : this.parent_.path() + "/" + this.name_);
}, name:function() {
    return this.name_;
}, parent:function() {
    return this.parent_;
}, updateParents_:function() {
    if (this.parent_ !== null) {
        this.parent_.updateChild_(this.name_, this);
    }
}, updateChild_:function(childName, child) {
    var childEmpty = child.isEmpty();
    var childExists = fb.util.obj.contains(this.node_.children, childName);
    if (childEmpty && childExists) {
        delete this.node_.children[childName];
        this.node_.childCount--;
        this.updateParents_();
    } else {
        if (!childEmpty && !childExists) {
            this.node_.children[childName] = child.node_;
            this.node_.childCount++;
            this.updateParents_();
        }
    }
}});
goog.provide("fb.core.util.validation");
goog.require("fb.core.util");
goog.require("fb.core.util.Path");
goog.require("fb.core.util.ValidationPath");
goog.require("fb.util.obj");
goog.require("fb.util.utf8");
goog.require("fb.util.validation");
fb.core.util.validation = {INVALID_KEY_REGEX_:/[\[\].#$\/\u0000-\u001F\u007F]/, INVALID_PATH_REGEX_:/[\[\].#$\u0000-\u001F\u007F]/, VALID_AUTH_PROVIDER:/^[a-zA-Z][a-zA-Z._\-+]+$/, MAX_LEAF_SIZE_:10 * 1024 * 1024, isValidKey:function(key) {
    return goog.isString(key) && key.length !== 0 && !fb.core.util.validation.INVALID_KEY_REGEX_.test(key);
}, isValidPathString:function(pathString) {
    return goog.isString(pathString) && pathString.length !== 0 && !fb.core.util.validation.INVALID_PATH_REGEX_.test(pathString);
}, isValidRootPathString:function(pathString) {
    if (pathString) {
        pathString = pathString.replace(/^\/*\.info(\/|$)/, "/");
    }
    return fb.core.util.validation.isValidPathString(pathString);
}, isValidPriority:function(priority) {
    return priority === null || goog.isString(priority) || goog.isNumber(priority) && !fb.core.util.isInvalidJSONNumber(priority) || goog.isObject(priority) && fb.util.obj.contains(priority, ".sv");
}, validateFirebaseDataArg:function(fnName, argumentNumber, data, path, optional) {
    if (optional && !goog.isDef(data)) {
        return;
    }
    fb.core.util.validation.validateFirebaseData(fb.util.validation.errorPrefix(fnName, argumentNumber, optional), data, path);
}, validateFirebaseData:function(errorPrefix, data, path) {
    if (path instanceof fb.core.util.Path) {
        path = new fb.core.util.ValidationPath(path, errorPrefix);
    }
    if (!goog.isDef(data)) {
        throw new Error(errorPrefix + "contains undefined " + path.toErrorString());
    }
    if (goog.isFunction(data)) {
        throw new Error(errorPrefix + "contains a function " + path.toErrorString() + " with contents: " + data.toString());
    }
    if (fb.core.util.isInvalidJSONNumber(data)) {
        throw new Error(errorPrefix + "contains " + data.toString() + " " + path.toErrorString());
    }
    if (goog.isString(data) && data.length > fb.core.util.validation.MAX_LEAF_SIZE_ / 3 && fb.util.utf8.stringLength(data) > fb.core.util.validation.MAX_LEAF_SIZE_) {
        throw new Error(errorPrefix + "contains a string greater than " + fb.core.util.validation.MAX_LEAF_SIZE_ + " utf8 bytes " + path.toErrorString() + " ('" + data.substring(0, 50) + "...')");
    }
    if (goog.isObject(data)) {
        var hasDotValue = false, hasActualChild = false;
        fb.util.obj.foreach(data, function(key, value) {
            if (key === ".value") {
                hasDotValue = true;
            } else {
                if (key !== ".priority" && key !== ".sv") {
                    hasActualChild = true;
                    if (!fb.core.util.validation.isValidKey(key)) {
                        throw new Error(errorPrefix + " contains an invalid key (" + key + ") " + path.toErrorString() + ".  Keys must be non-empty strings " + 'and can\'t contain ".", "#", "$", "/", "[", or "]"');
                    }
                }
            }
            path.push(key);
            fb.core.util.validation.validateFirebaseData(errorPrefix, value, path);
            path.pop();
        });
        if (hasDotValue && hasActualChild) {
            throw new Error(errorPrefix + ' contains ".value" child ' + path.toErrorString() + " in addition to actual children.");
        }
    }
}, validateFirebaseMergePaths:function(errorPrefix, mergePaths) {
    var i, curPath;
    for (i = 0;i < mergePaths.length;i++) {
        curPath = mergePaths[i];
        var keys = curPath.slice();
        for (var j = 0;j < keys.length;j++) {
            if (keys[j] === ".priority" && j === keys.length - 1) {
            } else {
                if (!fb.core.util.validation.isValidKey(keys[j])) {
                    throw new Error(errorPrefix + "contains an invalid key (" + keys[j] + ") in path " + curPath.toString() + ". Keys must be non-empty strings " + 'and can\'t contain ".", "#", "$", "/", "[", or "]"');
                }
            }
        }
    }
    mergePaths.sort(fb.core.util.Path.comparePaths);
    var prevPath = null;
    for (i = 0;i < mergePaths.length;i++) {
        curPath = mergePaths[i];
        if (prevPath !== null && prevPath.contains(curPath)) {
            throw new Error(errorPrefix + "contains a path " + prevPath.toString() + " that is ancestor of another path " + curPath.toString());
        }
        prevPath = curPath;
    }
}, validateFirebaseMergeDataArg:function(fnName, argumentNumber, data, path, optional) {
    if (optional && !goog.isDef(data)) {
        return;
    }
    var errorPrefix = fb.util.validation.errorPrefix(fnName, argumentNumber, optional);
    if (!goog.isObject(data) || goog.isArray(data)) {
        throw new Error(errorPrefix + " must be an object containing the children to replace.");
    }
    var mergePaths = [];
    fb.util.obj.foreach(data, function(key, value) {
        var curPath = new fb.core.util.Path(key);
        fb.core.util.validation.validateFirebaseData(errorPrefix, value, path.child(curPath));
        if (curPath.getBack() === ".priority") {
            if (!fb.core.util.validation.isValidPriority(value)) {
                throw new Error(errorPrefix + "contains an invalid value for '" + curPath.toString() + "', which must be a valid " + "Firebase priority (a string, finite number, server value, or null).");
            }
        }
        mergePaths.push(curPath);
    });
    fb.core.util.validation.validateFirebaseMergePaths(errorPrefix, mergePaths);
}, validatePriority:function(fnName, argumentNumber, priority, optional) {
    if (optional && !goog.isDef(priority)) {
        return;
    }
    if (fb.core.util.isInvalidJSONNumber(priority)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "is " + priority.toString() + ", but must be a valid Firebase priority (a string, finite number, " + "server value, or null).");
    }
    if (!fb.core.util.validation.isValidPriority(priority)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "must be a valid Firebase priority " + "(a string, finite number, server value, or null).");
    }
}, validateEventType:function(fnName, argumentNumber, eventType, optional) {
    if (optional && !goog.isDef(eventType)) {
        return;
    }
    switch(eventType) {
        case "value":
            ;
        case "child_added":
            ;
        case "child_removed":
            ;
        case "child_changed":
            ;
        case "child_moved":
            break;
        default:
            throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + 'must be a valid event type: "value", "child_added", "child_removed", ' + '"child_changed", or "child_moved".');;
    }
}, validateKey:function(fnName, argumentNumber, key, optional) {
    if (optional && !goog.isDef(key)) {
        return;
    }
    if (!fb.core.util.validation.isValidKey(key)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + 'was an invalid key: "' + key + '".  Firebase keys must be non-empty strings and ' + 'can\'t contain ".", "#", "$", "/", "[", or "]").');
    }
}, validatePathString:function(fnName, argumentNumber, pathString, optional) {
    if (optional && !goog.isDef(pathString)) {
        return;
    }
    if (!fb.core.util.validation.isValidPathString(pathString)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + 'was an invalid path: "' + pathString + '". Paths must be non-empty strings and ' + 'can\'t contain ".", "#", "$", "[", or "]"');
    }
}, validateRootPathString:function(fnName, argumentNumber, pathString, optional) {
    if (pathString) {
        pathString = pathString.replace(/^\/*\.info(\/|$)/, "/");
    }
    fb.core.util.validation.validatePathString(fnName, argumentNumber, pathString, optional);
}, validateWritablePath:function(fnName, path) {
    if (path.getFront() === ".info") {
        throw new Error(fnName + " failed: Can't modify data under /.info/");
    }
}, validateUrl:function(fnName, argumentNumber, parsedUrl) {
    var pathString = parsedUrl.path.toString();
    if (!goog.isString(parsedUrl.repoInfo.host) || parsedUrl.repoInfo.host.length === 0 || !fb.core.util.validation.isValidKey(parsedUrl.repoInfo.namespace) || pathString.length !== 0 && !fb.core.util.validation.isValidRootPathString(pathString)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, false) + "must be a valid firebase URL and " + 'the path can\'t contain ".", "#", "$", "[", or "]".');
    }
}, validateCredential:function(fnName, argumentNumber, cred, optional) {
    if (optional && !goog.isDef(cred)) {
        return;
    }
    if (!goog.isString(cred)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "must be a valid credential (a string).");
    }
}, validateBoolean:function(fnName, argumentNumber, bool, optional) {
    if (optional && !goog.isDef(bool)) {
        return;
    }
    if (!goog.isBoolean(bool)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "must be a boolean.");
    }
}, validateString:function(fnName, argumentNumber, string, optional) {
    if (optional && !goog.isDef(string)) {
        return;
    }
    if (!goog.isString(string)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "must be a valid string.");
    }
}, validateAuthProvider:function(fnName, argumentNumber, provider, optional) {
    if (optional && !goog.isDef(provider)) {
        return;
    }
    fb.core.util.validation.validateString(fnName, argumentNumber, provider, optional);
    if (!fb.core.util.validation.VALID_AUTH_PROVIDER.test(provider)) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "'" + provider + "' is not a valid authentication provider.");
    }
}, validateObject:function(fnName, argumentNumber, obj, optional) {
    if (optional && !goog.isDef(obj)) {
        return;
    }
    if (!goog.isObject(obj) || obj === null) {
        throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + "must be a valid object.");
    }
}, validateObjectContainsKey:function(fnName, argumentNumber, obj, key, optional, opt_type) {
    var objectContainsKey = goog.isObject(obj) && fb.util.obj.contains(obj, key);
    if (!objectContainsKey) {
        if (optional) {
            return;
        } else {
            throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + 'must contain the key "' + key + '"');
        }
    }
    if (opt_type) {
        var val = fb.util.obj.get(obj, key);
        if (opt_type === "number" && !goog.isNumber(val) || opt_type === "string" && !goog.isString(val) || opt_type === "boolean" && !goog.isBoolean(val) || opt_type === "function" && !goog.isFunction(val) || opt_type === "object" && !goog.isObject(val)) {
            if (optional) {
                throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + 'contains invalid value for key "' + key + '" (must be of type "' + opt_type + '")');
            } else {
                throw new Error(fb.util.validation.errorPrefix(fnName, argumentNumber, optional) + 'must contain the key "' + key + '" with type "' + opt_type + '"');
            }
        }
    }
}};
goog.provide("fb.core.util.CountedSet");
goog.require("fb.core.util");
goog.require("fb.util.obj");
goog.require("goog.object");
fb.core.util.CountedSet = goog.defineClass(null, {constructor:function() {
    this.set = {};
}, add:function(item, val) {
    this.set[item] = val !== null ? val : true;
}, contains:function(key) {
    return fb.util.obj.contains(this.set, key);
}, get:function(item) {
    return this.contains(item) ? this.set[item] : undefined;
}, remove:function(item) {
    delete this.set[item];
}, clear:function() {
    this.set = {};
}, isEmpty:function() {
    return goog.object.isEmpty(this.set);
}, count:function() {
    return goog.object.getCount(this.set);
}, each:function(fn) {
    goog.object.forEach(this.set, function(v, k) {
        fn(k, v);
    });
}, keys:function() {
    var keys = [];
    goog.object.forEach(this.set, function(v, k) {
        keys.push(k);
    });
    return keys;
}});
goog.provide("fb.core.SparseSnapshotTree");
goog.require("fb.core.snap.Node");
goog.require("fb.core.snap.PriorityIndex");
goog.require("fb.core.util.CountedSet");
goog.require("fb.core.util.Path");
fb.core.SparseSnapshotTree = function() {
    this.value_ = null;
    this.children_ = null;
};
fb.core.SparseSnapshotTree.prototype.find = function(path) {
    if (this.value_ != null) {
        return this.value_.getChild(path);
    } else {
        if (!path.isEmpty() && this.children_ != null) {
            var childKey = path.getFront();
            path = path.popFront();
            if (this.children_.contains(childKey)) {
                var childTree = this.children_.get(childKey);
                return childTree.find(path);
            } else {
                return null;
            }
        } else {
            return null;
        }
    }
};
fb.core.SparseSnapshotTree.prototype.remember = function(path, data) {
    if (path.isEmpty()) {
        this.value_ = data;
        this.children_ = null;
    } else {
        if (this.value_ !== null) {
            this.value_ = this.value_.updateChild(path, data);
        } else {
            if (this.children_ == null) {
                this.children_ = new fb.core.util.CountedSet;
            }
            var childKey = path.getFront();
            if (!this.children_.contains(childKey)) {
                this.children_.add(childKey, new fb.core.SparseSnapshotTree);
            }
            var child = this.children_.get(childKey);
            path = path.popFront();
            child.remember(path, data);
        }
    }
};
fb.core.SparseSnapshotTree.prototype.forget = function(path) {
    if (path.isEmpty()) {
        this.value_ = null;
        this.children_ = null;
        return true;
    } else {
        if (this.value_ !== null) {
            if (this.value_.isLeafNode()) {
                return false;
            } else {
                var value = this.value_;
                this.value_ = null;
                var self = this;
                value.forEachChild(fb.core.snap.PriorityIndex, function(key, tree) {
                    self.remember(new fb.core.util.Path(key), tree);
                });
                return this.forget(path);
            }
        } else {
            if (this.children_ !== null) {
                var childKey = path.getFront();
                path = path.popFront();
                if (this.children_.contains(childKey)) {
                    var safeToRemove = this.children_.get(childKey).forget(path);
                    if (safeToRemove) {
                        this.children_.remove(childKey);
                    }
                }
                if (this.children_.isEmpty()) {
                    this.children_ = null;
                    return true;
                } else {
                    return false;
                }
            } else {
                return true;
            }
        }
    }
};
fb.core.SparseSnapshotTree.prototype.forEachTree = function(prefixPath, func) {
    if (this.value_ !== null) {
        func(prefixPath, this.value_);
    } else {
        this.forEachChild(function(key, tree) {
            var path = new fb.core.util.Path(prefixPath.toString() + "/" + key);
            tree.forEachTree(path, func);
        });
    }
};
fb.core.SparseSnapshotTree.prototype.forEachChild = function(func) {
    if (this.children_ !== null) {
        this.children_.each(function(key, tree) {
            func(key, tree);
        });
    }
};
